%% PRINTSCREETEST : Prints the principal inertias
%
% ARGUMENTS   : S       -> Dispersion matrix (S, R, or Rp)
%               r       -> Number of factor to retain
%               Output  -> Optional input argument. If it's present, display the result in
%                         this object. If not, the results are displayed in matlab workspace
%                         window.
%               Output2 -> Another optional input argument. If it's present, display the
%                          result in latex code in this object.
%
% Last update: Urbano Lorenzo-Seva 21/04/2015
%

function PrintScreeTest(S,r,Output,Output2);

[k,tmp]=size(S);

[Q,D]=ed(S);
f0=diag(D);
f=[];
for i=1:k,
    if i==1,
       f=[f0(i) f0(i)/sum(f0) f0(i)/sum(f0)];
    else,
       f=[f; [f0(i) f0(i)/sum(f0) (f(i-1,3) + f0(i)/sum(f0))]];
    end;       
end;

if nargin==3
    MyPrint(Output,' ');
    MyPrint(Output,'EXPLAINED VARIANCE BASED ON EIGENVALUES (SCREE TEST)');
    MyPrint(Output,' ');
    
    MyPrint(Output,'Factor    Eigenvalue   Proportion of   Cumulative Proportion    Scree plot');
    MyPrint(Output,'                       Variance        of Variance');
    MyPrint(Output,' ');
    for i=1:k,
        buff1=sprintf('%3.0f       %6.4f       %6.4f          ',i,f(i,1),f(i,2)); 
        if i<=r,
           buff2=sprintf('%6.4f                   ',f(i,3));   
        else,
           buff2='                         '; 
        end;
        ast=floor(f(i,2)/max(f(:,2))*40);
        if f(i,2) > 0.01,
            buff3='*';
            for j=1:ast,
                buff3=[buff3 '*'];
            end;
        else,
            buff3='*';
        end;
        buff4 = sprintf('%s%s%s',buff1,buff2,buff3);
        MyPrint(Output,buff4);    
    end;
    MyPrint(Output,' ');
    MyPrint(Output,' ');
    
    
elseif nargin==2
    disp(' ');
    disp('EXPLAINED VARIANCE BASED ON EIGENVALUES (SCREE TEST)');
    disp(' ');

    disp('Factor    Eigenvalue   Proportion of   Cumulative Proportion    Scree plot');
    disp('                       Variance        of Variance');
    disp(' ');
    for i=1:k,
        buff1=sprintf('%3.0f       %6.4f       %6.4f          ',i,f(i,1),f(i,2)); 
        if i<=r,
           buff2=sprintf('%6.4f                   ',f(i,3));   
        else,
           buff2='                         '; 
        end;
        ast=floor(f(i,2)/max(f(:,2))*40);
        if f(i,2) > 0.01,
            buff3='*';
            for j=1:ast,
                buff3=[buff3 '*'];
            end;
        else,
            buff3='*';
        end;
        buff4 = sprintf('%s%s%s',buff1,buff2,buff3);
        disp(buff4);    
    end;
    disp(' ');
    disp(' ');
    
elseif nargin == 4
    
    %%%%%%%%%%%% TEXT %%%%%%%%%%%%%
    MyPrint(Output,' ');
    MyPrint(Output,'EXPLAINED VARIANCE BASED ON EIGENVALUES (SCREE TEST)');
    MyPrint(Output,' ');
    
    MyPrint(Output,'Factor    Eigenvalue   Proportion of   Cumulative Proportion    Scree plot');
    MyPrint(Output,'                       Variance        of Variance');
    MyPrint(Output,' ');
    for i=1:k,
        buff1=sprintf('%3.0f       %6.4f       %6.4f          ',i,f(i,1),f(i,2)); 
        if i<=r,
           buff2=sprintf('%6.4f                   ',f(i,3));   
        else,
           buff2='                         '; 
        end;
        ast=floor(f(i,2)/max(f(:,2))*40);
        if f(i,2) > 0.01,
            buff3='*';
            for j=1:ast,
                buff3=[buff3 '*'];
            end;
        else,
            buff3='*';
        end;
        buff4 = sprintf('%s%s%s',buff1,buff2,buff3);
        MyPrint(Output,buff4);    
    end;
    MyPrint(Output,' ');
    MyPrint(Output,'-----------------------------------------------------------------------');
    MyPrint(Output,' ');
    
    %%%%%%%%%%%% LATEX %%%%%%%%%%%%
    
    %Taula
    MyPrint(Output2,'\begin{center}');
    MyPrint(Output2,'\begin{longtable}[H]{l S S S}');
    MyPrint(Output2,'\caption{Explained variance based on Eigenvalues (Scree Test)}\\');
    MyPrint(Output2,'\toprule');
    MyPrint(Output2,'\multicolumn{1}{l}{\textbf{Factor}} & ');
    MyPrint(Output2,'\multicolumn{1}{c}{\textbf{Eigenvalue}} & ');
    MyPrint(Output2,'\multicolumn{1}{c}{\textbf{Proportion of Variance}} & ');
    MyPrint(Output2,'\multicolumn{1}{p{4cm}}{\textbf{Cumulative Proportion of Variance}} \\');
    MyPrint(Output2,'\midrule');
    MyPrint(Output2,'\endfirsthead');
    MyPrint(Output2,'\multicolumn{4}{c}%');
    MyPrint(Output2,'{\tablename\ \thetable\ -- \textit{(Continued from previous page)}} \\');
    MyPrint(Output2,'\toprule');
    MyPrint(Output2,'\multicolumn{1}{l}{\textbf{Factor}} & ');
    MyPrint(Output2,'\multicolumn{1}{c}{\textbf{Eigenvalue}} & ');
    MyPrint(Output2,'\multicolumn{1}{c}{\textbf{Proportion of Variance}} & ');
    MyPrint(Output2,'\multicolumn{1}{p{4cm}}{\textbf{Cumulative Proportion of Variance}} \\');
    MyPrint(Output2,'\midrule');
    MyPrint(Output2,'\endhead');
    MyPrint(Output2,'\bottomrule \multicolumn{4}{r}{\textit{Continued on next page}} \\');
    MyPrint(Output2,'\endfoot');
    MyPrint(Output2,'\bottomrule');
    MyPrint(Output2,'\endlastfoot');
%     
%     MyPrint(Output2,'\begin{table}[H]');
%     MyPrint(Output2,'\caption{Explained variance based on Eigenvalues (Scree Test)}');
%     MyPrint(Output2,'\label{taula:PA}');
%     MyPrint(Output2,'\centering');
%     MyPrint(Output2,'\begin{tabular}{l c c c}');
%     MyPrint(Output2,'\hline');
%     MyPrint(Output2,'Variable & Eigenvalue & Proportion of Variance & Cumulative Proportion of Variance \\');
%     MyPrint(Output2,'\hline');
    buff3='\\';
    for i=1:k,
        buff1=sprintf('%3.0f & %6.4f & %6.4f',i,f(i,1),f(i,2)); 
        if i<=r,
           buff2=sprintf(' & %6.4f',f(i,3));   
        else,
           buff2=''; 
        end;
        buff4 = sprintf('%s%s%s',buff1,buff2,buff3);
        MyPrint(Output2,buff4);
    end;
%     MyPrint(Output2,'\end{tabular}');
%     MyPrint(Output2,'\end{table}');
    MyPrint(Output2,'\end{longtable}');
    MyPrint(Output2,'\end{center}');
    
    
    %Grafic
    MyPrint(Output2,'\begin{figure}[H]');
    MyPrint(Output2,'\centering');
    MyPrint(Output2,'\begin{tikzpicture}');
    MyPrint(Output2,'\usetikzlibrary{patterns}');
    MyPrint(Output2,' \begin{axis}[xbar,');
    MyPrint(Output2,'  xlabel=$Percentage\ of\ Variance$,');
    MyPrint(Output2,'  ylabel=$Variables$,');
    MyPrint(Output2,'  width=0.8\textwidth,');
    if k<=15
        MyPrint(Output2,'  height=0.6\textwidth,');
    elseif k>15 && k<=25
        MyPrint(Output2,'  height=0.7\textwidth,');
    elseif k>25 && k<=40
        MyPrint(Output2,'  height=0.8\textwidth,');
    elseif k>40 
        MyPrint(Output2,'  height=1\textwidth,');
    end;
    MyPrint(Output2,'  xmin=0,');
    max_prop=max(f(:,2));
    buff = sprintf('  xmax=%2.0f,',(max_prop*100)+1);  
    MyPrint(Output2, buff);
    MyPrint(Output2,'  ymin=0,');
    buff=sprintf('  ymax=%2.0f,',k+1);
    MyPrint(Output2, buff);
    MyPrint(Output2,'  y dir=reverse,');
    MyPrint(Output2,'  bar width=5pt,');
    buff = sprintf('  ytick={%2.0f,...,1}',k);
    MyPrint(Output2, buff);
    MyPrint(Output2,'] ');
    MyPrint(Output2,'\addplot');
    MyPrint(Output2,'[draw=blue,pattern=horizontal lines light blue]');
    MyPrint(Output2,'coordinates {');
    for i=1:k
        buff=sprintf('(%3.1f,%2.0f)',(f(i,2)*100),i);
        MyPrint(Output2,buff);
    end;
    MyPrint(Output2,'};');
%     MyPrint('\addplot [red,sharp plot]');
%     MyPrint('coordinates {');
%     buff=sprintf('(%2.0f,%2.0f) (%2.0f,0)};',a,b,a);
%     MyPrint(Output2,buff);
    MyPrint(Output2,'\end{axis}');
    MyPrint(Output2,'\end{tikzpicture}');
    MyPrint(Output2,'\caption{Scree Test}');
    MyPrint(Output2,'\end{figure}');
    
end;



