%% COBLIMIN: Consensus Oblimin Rotation
%
% ARGUMENTS   : A     -> supermatrix with different sets of columns.
%               p     -> number of sets.
%               m     -> vector with column orders of A(k) .
%               gamma -> The gamma parameter
%               conv  -> convergence criterion.
%               nrs   -> number of random starts.
%
% RETURN VALUE: T    -> supermatrix with T(1)...T(p).
%               PHI  -> Inter-factor correlation matrix
%               Po   -> Consensus loading matrix
%               fr   -> function value.
%               ir   -> number of iterations.
%
% DEPENDENCIES: ccc.m
%               gpanew.m
%               oblimin.m
%               part.m
%               ssq.m
%

function [T,PHI,Po,fr,ir] = coblimin(A,p,m,gamma,conv,nrs);

   n = size(A);

   [Tr,fr,ir]=gpanew(A,p,m,conv,nrs+20);  
   SA = zeros(n(1),m(1));
   for k=1:p,
       SA= SA + part(A,k,m)*Tr(:,(k-1)*m+1:k*m);
   end;

   SA = SA ./ p;   

   [Po,PHI] = oblimin(SA,gamma,conv,nrs);

   T = [];
   for k=1:p,
       SA = part(A,k,m);
       To = (SA'*SA)^(-1) * SA' * Po;
       T = [T To];
   end;
