%% CUNROTATED: Consensus unrotated generalized procrustes analysis; (MAXDIFF algorithm; TB 1977)
%
% ARGUMENTS   : A    -> supermatrix with different sets of columns.
%               p    -> number of sets.
%               m    -> vector with column orders of A(k) .
%               conv -> convergence criterion.
%               nrs  -> number of random starts.
%
% RETURN VALUE: Tr  -> supermatrix with T(1)...T(p).
%               Pv  -> Conssensus pattern matrix
%               fr  -> function value.
%               ir  -> number of iterations.
%
% DEPENDENCIES: gpanew.m
%               part.m
%               ssq.m

function [Tr,P,fr,ir] = cunrotated(A,p,m,conv,nrs);

   n = size(A);
   
   [Tr,fr,ir]=gpanew(A,p,m,conv,nrs);  
   SA = zeros(n(1),m(1));
   for k=1:p,
       SA= SA + part(A,k,m(k))*Tr(:,(k-1)*m(k)+1:k*m(k));
    end;
    
   P = SA ./ p;   
