%% CVARIM: Produces varimax function value and rotation matrix T.
%
% ARGUMENTS   : A    -> supermatrix with different sets of columns.
%               p    -> number of sets.
%               m    -> vector with column orders of A(k) .
%               conv -> convergence criterion.
%               nrs  -> number of random starts.
%
% RETURN VALUE: T   -> supermatrix with T(1)...T(p).
%               Pv  -> Conssensus pattern matrix
%               fr  -> function value.
%               ir  -> number of iterations.
%
% DEPENDENCIES: gpanew.m
%               part.m
%               ssq.m
%               varim.m
%

function [T,Pv,fr,ir] = cvarim(A,p,m,conv,nrs);

   n = size(A);
   
   [Tr,fr,ir]=gpanew(A,p,m,conv,nrs);  
   SA = zeros(n(1),m(1));
   for k=1:p,
       SA= SA + part(A,k,m(k))*Tr(:,(k-1)*m(k)+1:k*m(k));
    end;
    
   CA = SA ./ p;   

   Tv = varim(CA,conv,nrs);
   Pv = CA*Tv;
  
   T = [];
   for k=1:p,
       Tvi = Tr(:,(k-1)*m+1:k*m)*Tv;
       T = [T Tvi];
   end;
