%% GLB : Greater Lower Bound routine
%
% ARGUMENTS   : C     -> Covariance/correlation matrix
%               conv  -> Convergence criterion
%               T     -> matrix for start (can be omitted)
%
% RETURN VALUE: gamma -> Optimal communalities for each variable
%               iter  -> Number of iterations
%               T     -> Last matrix
%
% DEPENDENCIES: nrm.m
%               tr.m
%
% MODIFICATED FOR PSYCHOLOGICAL TEST TOOLBOX
%
% November 2016, David Navarro-Gonzalez
%

function [gamma,iter,T]=glb(C,conv,T);

[m,m]=size(C);
if nargin<=2
  T=nrm(randn(m,m));	% T denotes T_prime in Ten Berge, et al. 81
end;
C0=C-diag(diag(C));

labda0=0;
iter=0;
f=tr(T*C*T');
fold=f+2*conv;
labda=-10000;
extra=0;
iterextra=0;
gamma=zeros(m,1);
contador=0;
while ((fold-f>conv) | (labda<-.1*conv))
    if isnan(f)
        errordlg('An error has occurred when calculating the convergence criterion of the dispersion matrix. The analysis can not continue','Psychological Test Toolbox');
        figHandles = get(0,'Children');
        set(figHandles,'Pointer','arrow');
        close(figHandles(2));
        close(figHandles(3));
        error('An error has occurred when calculating the convergence criterion of the dispersion matrix. The analysis can not continue');
    end;
  fold=f;
  iter=iter+1;
  iterextra=iterextra+1;
  for i=1:m
    yi=T*C0(:,i);
    delta=(yi'*yi)^.5;
    if delta==0
      T(:,i)=nrm(T(:,i));
    end;
    if ((delta>0) & (delta<C(i,i)))
      T(:,i)=-yi/delta;
    end;
    if delta>=C(i,i)
      T(:,i)=-yi/C(i,i);
    end;
  end;
  f=tr(T*C*T');
  %if iter/100==round(iter/100), fprintf('function value = %12.8f after %g iters \n',f,iter);end;
  if (fold-f)<=conv & (extra==0)		% if function value converged, check smallest eigenvalue
    tt=sum(T.^2);    
    w=tt<=1.0000000001;
    gamma=diag(C);
    gg=diag(C0*T'*T*C0);
    gamma(w)=gg(w).^.5;
    labda=min(eig(C0+diag(gamma)));
    if labda<-.1*conv
      extra=1;
      iterextra=0;
    end;
  end;
  if i==27
      check_ok=1;
  end;

  if ((extra==1) & (round((iterextra+1)/10)==(iterextra+1)/10))
    tt=sum(T.^2);    
    w=tt<=1.0000000001;
    gamma=diag(C);
    gg=diag(C0*T'*T*C0);
    gamma(w)=gg(w).^.5;
    labda0=labda;
    labda=min(eig(C0+diag(gamma)));	% reevaluate labda only once every ten full cylces
    if abs(labda-labda0)<.0001*abs(labda0)
      %disp('Too small improvements; restart T');
      T=nrm(randn(m,m));	
      extra=0;
      extraiter=0;
      f=tr(T*C*T');
    end;
  end;
  
 % [fold f conv]
  
end;     
tt=sum(T.^2);    
w=tt<=1.0000000001;
gamma=diag(C);
gg=diag(C0*T'*T*C0);
gamma(w)=gg(w).^.5;
labda=min(eig(C0+diag(gamma)));	% reevaluate labda only once every ten full cylces
if labda<-.1*conv,
    beep,
    disp('too negative eigenvalue');
    disp(labda);
end;
%disp('check1');
%disp( C0*T'+diag(gamma)*T' );
%disp('check2; eigenvalues of C0+Gamma');
%eig(C0+diag(gamma))	% reevaluate labda only once every ten full cylces



