%% GPANEW : Special purpose program for simall (generalized procrustes analysis); (MAXDIFF algorithm; TB 1977)
%		EQUAL COLUMN NUMBERS
%
% ARGUMENTS   : A     -> supermatrix with different sets of columns.
%               p     -> number of sets.
%               m     -> vector with column orders of A(k) .
%               conv  -> convergence criterion.
%               nrs   -> number of random starts.
%
% RETURN VALUE: T     -> supermatrix with T(1)...T(p).
%               fr    -> function value.
%               ir    -> number of iterations.
%
% DEPENDENCIES: part.m
%               ssq.m
%

function [Tr,fr,ir] = gpanew(A,p,m,conv,nrs)

minm=min(m);
[n,cclear] = size(A); clear cclear;     % n is row order of A

% I=eye(minm); Tsup=[]; for k=1:p Tsup=[Tsup;I(:,1:m(k))]; end;

for nos=1:nrs,
  % Random start
%  Tsup=[]; for k=1:p Tsup=[Tsup; orth(rand(m(k),minm)-.5)]; end;
  I=eye(minm); Tsup=[]; for k=1:p Tsup=[Tsup; I(:,1:m(k))];  end;

  % Y becomes rotated A

  Y=[];
  for k=1:p,
    Y=[Y part(A,k,minm)*Tsup((k-1)*minm+1:k*minm,:)];
  end;

  % Z
  Z=zeros(n,minm);
  for k=1:p Z=Z+part(Y,k,minm)/p; end;

  f=0;
  for k=1:p
    f=f+ssq( Y(:,(k-1)*minm+1:k*minm) - Z );
  end;

  fold=f+2*conv*f;
  iter=0;
  while abs(fold-f)>conv*f
    fold=f;

    for k=1:p

      % Y contains rotated A
      Ak=part(A,k,minm); 			% A_k 
      H=A*Tsup-Ak*Tsup((k-1)*minm+1:k*minm,:);	% sum l~=k A_l T_l

      [P,D,Q]=svd(Ak'*H);
      U=P*Q';
      Tsup((k-1)*minm+1:k*minm,:)=U;
      Y(:,(k-1)*minm+1:k*minm)=Ak*U;

      Z=zeros(n,minm);
      for k=1:p Z=Z+part(Y,k,minm)/p; end;
      f=0;
      for k=1:p,f=f+ssq( part(Y,k,minm) - Z );end;

    end;
    iter=iter+1;
  end;
% disp([iter f]);
  if nos==1
    fr=f;ir=iter;Trr=Tsup;
  elseif f < fr
    fr=f;ir=iter;Trr=Tsup;
  end;
end;

% hieronder: toevoeging alleen om in simall te kunnen gebruiken!
Tr=[];
for k=1:p
  Tr=[Tr Trr((k-1)*minm+1:k*minm,:)];
end;
