%% MRFA : Compute Minimum Rank Factor Analysis
%
% ARGUMENTS   : SIGMA -> covariance matrix
%               r     -> dimensionality
%               conv1 -> overall convergence criterion
%               conv2 -> inner iterations (glb) convergence criterion
%               start -> random starts
%
% RETURN VALUE: gam   -> optimal communalities
%               f     -> mrfa function value
%
% DEPENDENCIES: glb.m
%               ed.m
%               nrm.m
%               tr.m
%

function [gam, f] = mrfa (SIGMA, r, nstarts, conv1, conv2);

[m,m]=size(SIGMA);

% initializations

%nstarts=2;
func=zeros(nstarts,1);
GAM=zeros(m,nstarts);

for ana=1:nstarts

 gam=rand(m,1);
 T=nrm(randn(m,m));

 % computation start function value

 SIGMARED=SIGMA-diag(diag(SIGMA))+diag(gam);
 [K,L]=ed(SIGMARED);
 l=diag(L);
 f=sum(l(r+1:m));
 %fprintf(' Unexplained Common Variance at start is %12.8f \n',f);
 fold=f+2*conv1*f;
 iter=0;

 while (((fold-f)>f*conv1) && (f>.01*conv1)) || (iter<2);
  
  fold=f;
  iter=iter+1;
  if r<m-1,
      w=(sum((K(:,r+1:m).^2)')').^.5;
  end;
  if r==m-1,
      w=(K(:,r+1:m).^2).^.5;
  end;

  Sigma=(w*w').*SIGMA;
  [gam,initer,T]=glb(Sigma,conv2,T);	% glb step
  gam=gam./(w.^2);
  SIGMARED=SIGMA-diag(diag(SIGMA))+diag(gam);
  [K,L]=ed(SIGMARED);
  l=diag(L);

  %%URBA (MARCH, 2009)
  tmp = size(l);
  if (tmp(1) < m),
    gam==-1;
    f=-1;
    return;
  end;
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% correction of gamma`s because of negative eigenvalues 

  if l(m)<0
    gam=gam-l(m);  
    l=l-l(m);
    if l(m)<-.1,fprintf('correction factor= %12.8f \n',l(m));end;
  end;

  f=sum(l(r+1:m));
% fprintf('mrfa value is %12.8f after %g iters (and %g inner iters) \n',f,iter,initer);
 end;
 func(ana)=f;
 GAM(:,ana)=gam;
end;

[f,mi]=min(func);
gam=GAM(:,mi);

return