%% ORROWN : Order B from A by Index of Congruence by Tuckey
%
% ARGUMENTS   : A   -> First Matrix
%               B   -> Second Matrix
%               PHI -> Correlation Matrix between factors
%
% RETURN VALUE: L   -> B in the new order
%               D   -> PHI in the new orde
%

function [L, D] = orrown(A,B,PHI)

   C = abs(congru(A,B));

   [n,n] = size(C);

   maximo = zeros(n,3);
   for h=1:n,
       for i=1:n,
           for j=1:n,
               if abs(maximo(h,1)) < abs(C(i,j)),
                  maximo(h,1) = C(i,j); 
                  maximo(h,2) = i; 
                  maximo(h,3) = j; 
                end; 
           end;
       end;
       for i=1:n,
           C(maximo(h,2),i) = 0;
       end;       
       for i=1:n,
           C(i,maximo(h,3)) = 0;
       end;
  end;

   L = [];
   for i=1:n,
     for j=1:n,
         if maximo(j,2) == i,
            L = [L B(:,maximo(j,3))];
         end;
      end;
   end;

   C = [];
   for i=1:n,
     for j=1:n,
         if maximo(j,2) == i,
            C = [C PHI(:,maximo(j,3))];
         end;
      end;
   end;

  D = [];
   for i=1:n,
     for j=1:n,
         if maximo(j,2) == i,
            D = [D; C(maximo(j,3),:)];
         end;
      end;
   end;

  c=diag(congru(A,L));
  I = eye(n);
  for i=1:n,
      if c(i) < 0, I(i,i) = -1; end;
  end;
  L = L*I;
  D = D*I;
  D = D'*I;

