%% REWLSF : WLS regression person-fit analysis for a single individual according  to the 2PM
% tt-> vector theta para el sujeto en el test
% filat-> patrn de respuestas para el sujeto en el test
%

function [inte,slo,q,ncrit,mst,msw,msb] = rewlsf(filat,tt,lam,thres,sige)

[n,m]= size (lam);
espe=zeros(1,n);
varcon=zeros(1,n);

% Computes the alpha=0.001 critical value of F

fc = finv(0.99,2,n-2);

% Computes item expectations and conditional variances 
for i=1:n,
 [tmp1,tmp2]=espeitf(tt,lam(i,:),thres(i,1),sige(i,1));
 espe(i)=tmp1;
 varcon(i)=(tmp2);
end;
secon=sqrt(varcon);

% Computes the WLS weights and transforms the original variables
weight=ones(1,n)./secon;
wfilat=weight.*filat;
wespe=weight.*espe;

% Obtains the regression estimates, predicted scores and, se
tmp3=mean(wfilat);
tmp4=mean(wespe);
tmp5=cov(wfilat,wespe,1);
tmp6=tmp5(1,2);
tmp7=tmp5(2,2);
slo=tmp6./tmp7;
inte=tmp3-(slo.*tmp4);
wpred=(inte*ones(1,n))+(slo*wespe);
vartot=var(wfilat',1);
varpred=slo.*slo.*var(wespe',1);
varerr=(n/(n-2))*(vartot-varpred);

% Computes if the fitted regression significantly departs from the
% model expected regression
T=[ones(n,1),wespe'];
vecdif=[-inte,(1-slo)];
q=vecdif*T'*T*vecdif';
ncrit=2*varerr*fc;

% Obtains the mean square decomposition
d1=(wfilat-wespe);
d2=(wfilat-wpred);
d3=(wpred-wespe);
mst=(1/n)*d1*d1';
msw=(1/n)*d2*d2';
msb=(1/n)*d3*d3';
return;

