%% salient : number of salient loadings as the basis for determining the number of factors
% ref = Gorsuch 1997 J Pers Assess p 545
%
% ARGUMENTS   : R        -> Dispersion Matrix
%               salvalue -> a loading > abs(.40) and that is the highest loading for the variable
%               numsals  -> Enter the minimum # of salient loadings for there to be "a factor" 
%

function nfSAL = salient(R,salvalue,numsals);

% Number of eigenvalues > 1
eigvals = sort(eig(R));

nfactors = 0;

for nev = 1:size(eigvals,1)
    if eigvals(nev,:) > 1,  
       nfactors = nfactors + 1; 
   end;
end;

[B,A] = pcaR(R,nfactors);

if nfactors > 1; 
    Tv = varim(A,0.00001,10);
    A = A*Tv;
end;

rowmax = max(abs(A),[],2);

nfSAL = 0;

[r,m]=size(A);
for lupec = 1:m,
   nsalients = 0;
   for luper = 1:r,
       if abs(A(luper,lupec)) >= salvalue  &  abs(A(luper,lupec)) == rowmax(luper,1),
          nsalients = nsalients + 1;
      end;
   end;
   if nsalients >= numsals,
      nfSAL = nfSAL + 1;
   end;
end;
