%% STEMLEAFPLOT_RTF : Plots stem and leaf plot in rtf style (modification of the original stemleafplot.m)
%
% INPUT
%   Output : Where the plot is going to be printed
%   V   : Array of numerical inputs (NaN values are ignored)
%
% OPTION
%   P   : Leaf precision (defined as integer power of 10)
%         Stem precision (by default) is 10^(P+1).
%         P is automatically rounded at the beginning of the function.
%         Leaf and stem units are printed at the bottom of the graph.
%         Examples: P = -3 rounds V to the nearest 10^-3 = 0.001
%                   P = 3 rounds V to the nearest 10^3 = 1000
%         [DEFAULT: P = 0]
%
% OUTPUT
%   Command window output
%
% Jered Wells
% 01/28/2011
% jered [dot] wells [at] duke [dot] edu
%
% v1.2 (02/14/2012)
%

function stemleafplot_rtf(Output,v,p)

if ~isnumeric(v); error 'Input V must be numeric'; end
if ~exist('p','var'); p = 0; elseif isempty(p); p = 0; end
if ~isnumeric(p); error 'Input P must be an integer'; end
p = round(p);


% Condition V
v = v(~isnan(v));
v = v(:);
v = round(v,p);

% Organize stems and leaves
allstems = floor(v./10^(p+1));
allleaves = round(abs(v./10^p));

nstems = allstems(allstems<0)+1; 	% Negative stems
nstems = nstems(:);
pstems = allstems(~(allstems<0));	% Positive stems
pstems = pstems(:);

nleaves = allleaves(allstems<0);    % Negative leaves
nleaves = nleaves(:);
pleaves = allleaves(~(allstems<0)); % Negative leaves
pleaves = pleaves(:);

dig = ceil(max(log10(abs(allstems))))+1;    % Max # of digits in stem
form = strcat(['%' num2str(dig+1) 'i']);    % Format string for SPRINTF

% Plot negative stems
if ~isempty(nstems)
    for ii = min(nstems(:)):0
        strstem = sprintf(form,ii);
        if ii==0; strstem(end-1:end) = '-0'; end
        strleaves = sprintf('%1i',mod(sort(nleaves(nstems==ii)),10));
        s = strcat([strstem ' |' strleaves]);
        MyPrint(Output,s);
        MyPrint(Output,'\par');
    end % NSTEMS
end % IF

% Plot positive stems
if ~isempty(pstems)
    for ii = 0:max(pstems(:))
        strstem = sprintf(form,ii);
        strleaves = sprintf('%1i',mod(sort(pleaves(pstems==ii)),10));
        s = strcat([strstem ' |' strleaves]);
        MyPrint(Output,s)
        MyPrint(Output,'\par');
    end % PSTEMS
end % IF

end 
