%% TARROT_OBL : Oblique semi-specified target rotation
%
% Browne, M.W. Oblique rotation to a partially  specified target.
% British Journal of Mathematical and Statistical Psychology,
% 25, 1972, pp. 207-212.
%
% ARGUMENTS   : F      ->  Factor solution to be rotated
%               B      ->  Hypothetical pattern for F (9:  Non specified values in the target)
%               conver -> Convergence index
%
% RETURN VALUE: FM  ->  Factor solution rotated (pattern)
%               C   ->   Correlation between factors
%     
            
function [FM, C] = tarrot_obl (F,B,conver)

  FM = F;

  COMAX = 1.0-conver;
  NEMIT = 10;
  TOLNE = 1;
  J1 = 1; J2 = 1;

  [NV,NF] = size(FM);
  ind  = zeros(NV,NF);
  indv = -1*ones(NV,1);
  keas = zeros(NV,1);
  
  H = diag(FM*FM');
  C = eye(NF);
  
%  //C *** CONSTRUCT IND,KEAS ***
%//                                      C *** CONSTRUCT IND,KEAS ***
      NUN = 0;
      for J = 1:NF,
          ist = 1;
          for I = 1:NV,
              if (abs(B(I,J)) ~= 9.0), 
                  ind(ist,J)=I;
                  if (B(I,J) ~= 0),
                      keas(J) = 1;
                  end;
                  ist=ist+1;
              end;
              indv(J) = ist-1;
              NUN = NUN + ist;
    %	      //if (ist < (r-1))   error ----> columna poco definida.
          end;
      end;
      FNUN = NUN;
      
% *** INITIATE FOR OUTER CYCLE ***
  CRITP = 0;
  for J = 1:NF,
		iend = indv(J);
		if (iend >0),
	      for ia=1:iend,
			I = ind(ia,J);
            CRITP = CRITP + (FM(I,J) - B(I,J))^2;
          end;
        end;
  end;

  KDON = 1; NAT = 1; NIT = 0;
  while (KDON ~=0 ),                                  % LINE 255 EN BROWNE
        if (NAT == 1),
%                   *** CHECK ORDER FOR OPTIMALITY ***
% ------------------------------ METHOD 1 ------------------------------
%            INTERCHANGE COLUMNS OF T SO AS TO MINIMIZE THE TRACE
           INTER = 0;
% *** GET T ***
           for J = 1:NF,
              iend = indv(J);
              SBBI = 0;
              if (iend>0),
                  for ia=1:iend,
                    I = ind(ia,J);  
                    SBBI = SBBI + B(I,J)^2;
                  end;
              end;
              for L = 1:NF,
                SFFI=0;
                SFBI=0;
                for ia=1:iend,
                    I = ind(ia,J); 
                    SFFI = SFFI + FM(I,L)^2;
                    SFBI = SFBI + FM(I,L)*B(I,J);
                end;
                T(J,L) = SBBI + SFFI - 2.0*abs(SFBI);
              end;
           end;
           CRITB = trace(T);

% *** FIND THE SMALLEST VALUE IN MATRIX T
           ITE = 1;
           JTE = 1;
           for J=1:NF,
             TMIN = 1.00;
             for L=1:NF,
               for K=1:NF,
                 if (T(K,L) < TMIN),
                   TMIN = T(K,L);
                   ITE = K;
                   JTE = L;
                 end;
               end;
             end;
             if (ITE ~= JTE),
% *** INTERCHANGE COLUMNS ITE AND JTE OF FM AND T ***
               INTER = INTER + 1;
               for I=1:NV,
                 TEMP = FM(I,ITE);
                 FM(I,ITE) = FM(I,JTE);
                 FM(I,JTE) = TEMP;
               end;
               for I=1:NF,
                 TEMP = C(I,ITE);
                 C(I,ITE) = C(I,JTE);
                 C(I,JTE) = TEMP;
                 TEMP = T(I,ITE);
                 T(I,ITE) = T(I,JTE);
                 T(I,JTE) = TEMP;
               end;
               for I=1:NF,
                 TEMP = C(ITE,I);
                 C(ITE,I) = C(JTE,I);
                 C(JTE,I) = TEMP;
               end;
             end;
% *** SET ROW AND COLUMN ITE TO 1.0D20
             for I=1:NF,
               T(ITE,I) = 1.0;
               T(I,ITE) = 1.0;
             end;
           end;                       %LINE  370 EN BROWNE

% ------------------------------ METHOD 2 ------------------------------
%             MINIMIZE THE DIAGONAL FOR 2X2 SUBMATRICES OF T
           if (INTER > 0),
% *** GET NEW T ***
             for J = 1:NF,
               SBBI = 0;
               iend = indv(J);
               if (iend > 0),
                  for ia=1:iend,
                     I = ind(ia,J); 
                     SBBI = SBBI + B(I,J)^2;
                  end;
               end;
               for L = 1:NF,
                 SFFI = 0;
                 SFBI = 0;
                 if (iend>0),
                     for ia=1:iend,
                       I = ind(ia,J); 
                       SFFI = SFFI + FM(I,L)^2;
                       SFBI = SFBI + FM(I,L)*B(I,J);
                     end;
                 end;
                 T(J,L) = SBBI + SFFI - 2*abs(SFBI);
               end;
             end;
           end;                             %LINE 450 EN BROWNE

% *** OBTAIN THE DIFFERENCE BETWEEN THE SUM OF THE DIAGONAL ELEMENTS AND
%     THE SUM OF THE OFF-DIAGONAL ELEMMENTS FOR SUBMATRICES OF T
           J = 1;
           while (J <= (NF*NF)),
             TEMP = 0;
             KTE = 1;
             LTE = 1;
             for K = 1:NF,
               for L = 1:K,
                 if (L ~= K),
                    TEMB = T(K,L) + T(L,K) - T(K,K) - T(L,L);
                    if (TEMP > TEMB),
                       TEMP = TEMB;
                       KTE = K;
                       LTE = L;
                    end;
                 end;
               end;
             end;

             if (TEMP < 0),
% *** INTERCHANGE COLUMNS KTE AND LTE OF FM AND B ***
               INTER = INTER + 1;
               for I = 1:NV,
                 TEMP = FM(I,KTE);
                 FM(I,KTE) = FM(I,LTE);
                 FM(I,LTE) = TEMP;
               end;
               for I=1:NF,
                 TEMP = C(I,KTE);
                 C(I,KTE) = C(I,LTE);
                 C(I,LTE) = TEMP;
                 TEMP = T(I,KTE);
                 T(I,KTE) = T(I,LTE);
                 T(I,LTE) = TEMP;
               end;
               for I=1:NF,
                 TEMP = C(KTE,I);
                 C(KTE,I) = C(LTE,I);
                 C(LTE,I) = TEMP;
               end;
             else
               J = NF*NF + 1;
             end;
             J = J + 1;
           end;                         %LINE  500  EN BROWNE

           for J = 1:NF,
             SFBI = 0;
             iend = indv(J);
             if (iend>0),
                 for ia=1:iend,
                     I = ind(ia,J);
                   SFBI = SFBI + FM(I,J)*B(I,J);
                 end;
             end;
             if (SFBI < 0 ),
% * REFLECT
               for I=1:NV,
                 FM(I,J) = (-1) * FM(I,J);
               end;
               for I=1:NF,
                 C(I,J) = (-1) * C(I,J);
                 C(J,I) = (-1) * C(J,I);
               end;
             end;
           end;
% *** CALCULATE DEVIATION CRITERION
           if (INTER ~= 0),
              CRIT = trace(T);
           end;

        end;                            %LINE 560 en BROWNE

        KDON  = 0;
        COLAG = -1.0;
        COSMA =  1.0;
        for J1 = 1:NF,
            SFFI = 0;
            SFBI = 0;
             iend = indv(J);
             if (iend>0),
                for ia=1:iend,
                     I = ind(ia,J);
                     SFFI = SFFI + FM(I,J1)*FM(I,J1);
                     if (keas(J1) ~= 0),
                         SFBI = SFBI + FM(I,J1)*B(I,J1);
                     end;
                end;
             end;
             if keas(J1) == 0,
                 SFBI = SFBI + FM(I,J1);
             end;

            if (SFBI < 0),
% *** REFLECT ***
               SFBI = (-1) * SFBI;
               for I = 1:NV,
                   FM(I,J1) = (-1)* FM(I,J1);
               end;
               for I = 1:NF,
                   C(I,J1) = (-1)*C(I,J1);
                   C(J1,I) = (-1)*C(J1,I);
               end;
            end;

             if keas(J1) == 0,
                 SFBI = 0;
             end;

            for J2 = 1:NF,
              if (J2 ~= J1),
                  lend = indv(J2);
                  C12  = C(J1,J2);
                  OMCS = 1.0 - C12*C12;
                  SFFMBL = 0;
                  SFFL = 0;
                  if lend > 0,
                      for la = 1:lend,
                          L = ind(la,J2);
                          SFFMBL = SFFMBL + FM(L,J1)*(FM(L,J2) - B(L,J2));
                          SFFL = SFFL + FM(L,J1)*FM(L,J1);
                      end;
                  end;

                  rot = 1;
                  if (SFBI == 0),
% *** LINEAR EQUATION ***
                     TEMP = SFFMBL - (C12*SFFI);
                     TEMB = SFFI + SFFL;
                     if (TEMB == 0),
                        rot = 0;
                     else
                        DEL = TEMP/TEMB;
                     end;
                  else
% *** NEWTON ITERATION . INITIAL APPROXIMATION ***
                     DELB = 0;
                     if (SFFL ~= 0), DELB=SFFMBL/SFFL; end;
                     XKAY2 = SFBI/SFFI;
                     TEMP = XKAY2 - OMCS;
                     TEMP = max([TEMP 0]);
                     if (TEMP ~= 0), TEMP = sqrt(TEMP); end;
                     TEMC = DELB + C12;
                     if (TEMC < 0 ), TEMP = (-1) * TEMP; end;
                     DELA = (-1) * C12 + TEMP;

                     if (SFFL == 0),
                        DEL=DELA;
                     else

                        if (NIT > 0),
                            if ((DELA*DELB) > 0 ),
                               TEMB = 2*SFFL;
                               if (TEMP ~= 0),
                                  TEMC = OMCS/XKAY2;
                               else
                                  TEMC = sqrt(XKAY2/OMCS);
                               end;
                               TEMP = 2.0D0*SFFI*(1.0D0 - TEMC);
                               DEL = (TEMP*DELA + TEMB*DELB)/(TEMP + TEMB);
                            else
                                DEL  = 0;
                            end;
                        else
                           TEMB = 2*SFFL;
                           if (TEMP ~= 0),
                              TEMC = OMCS/XKAY2;
                           else
                              TEMC = sqrt(XKAY2/OMCS);
                           end;
                           TEMP = 2*SFFI*(1 - TEMC);
                           DEL = (TEMP*DELA + TEMB*DELB)/(TEMP + TEMB);

                        end;                      % LINE 660 EN BROWNE

% *** NEWTON ITERATION ***
                        NEWIT = 0;
                        TA2 = 2*(SFFI + SFFL);
                        A1  = 2*(C12*SFFI - SFFMBL);
                        AST = (-2)*SFBI;
                        TEMP = 1/(1 + 2*C12*DEL + DEL*DEL);
                        TEMB = sqrt(TEMP);
                        TEMC = DEL + C12;
                        QPD = TA2*DEL + A1 + AST*TEMC*TEMB;
                        TEMB = TEMB*TEMP;
                        QPPD = TA2 + AST*OMCS*TEMB;
                        DEN = QPPD;
% *** THIRD DERIVATIVE USED . THIS COULD BE OMITTED ***
                        QPPPD = (-3)*AST*OMCS*TEMC*TEMB*TEMP;
                        DEN = DEN - ((QPD*QPPPD)/(2*QPPD));
                        DELINC  = (-1) * QPD/DEN;
                        DEL = DEL + DELINC;
                        NEWIT = NEWIT + 1;
                     end;
                  end;                                % LINE 700 EN BROWNE
                  if (rot == 1),

% *** DELTA IS NOW AVAILABLE ***
                     GAM = sqrt((DEL*DEL + 2*C12*DEL) + 1);
                     TECOS = (1 + C12*DEL)/GAM;
                     COLAG = max([COLAG TECOS]);
                     COSMA = min([COSMA TECOS]);
                     if (TECOS < COMAX), KDON=KDON+1; end;
% *** ROTATE FM AND UPDATE C ***
                     for I = 1:NV,
                         TEMP = FM(I,J1);
                         FM(I,J2) = (-1)*TEMP*DEL + FM(I,J2);
                         FM(I,J1) = TEMP*GAM;
                     end;
                     for I = 1:NF,
                         C(J1,I) = (C(J1,I) + C(J2,I)*DEL)/GAM;
                         C(I,J1) = C(J1,I);
                     end;
                     C(J1,J1) = 1;
                     SFFI = SFFI*GAM*GAM;
                     SFBI = SFBI*GAM;
                  end;
              end;                                    % LINE 730 EN BROWNE
            end;
        end;                                          % LINE 740 EN BROWNE


% *** CALCULATE CRITERION . END OF CYCLE ***
      CRIT = 0;
      for J = 1:NF,
        iend = indv(J);
        if iend > 0;
           for ia = 1:iend,
               I = ind(ia,J);
              CRIT = CRIT + (FM(I,J) - B(I,J))^2;
           end;
        end;
      end;
      if (CRIT > CRITP),
          NIT  = NIT+1;
          KDON = 0;
      else
          CRITP = CRIT;
          NIT = NIT + 1;
%          NAT = NIT - (NIT/5)*5;
%          if (NIT <= 2), NAT=1; end;
          NAT = 0;
      end;

      if NIT < 10,
          KDON = 1;
      end;
      
  end;                                                % LINE 800 EN BROWNE

  
% *** CHECK
  TEMP = 0;
  for I = 1:NV,
    for JA = 1:NF,
        for JB = 1:NF,
          H(I) = H(I) - FM(I,JA)*C(JA,JB)*FM(I,JB);
      end;
    end;
    TEMP = max([TEMP abs(H(I))] );
  end;
  buff = sprintf ('ACCURACY CHECK  %12.10f',TEMP);
%  disp(buff);
  CRIT = sqrt(CRIT/FNUN);

  return;
