%% TETRACHORIC : Computes TETRACHORIC matrix 
%
% ARGUMENTS   : X     -> Binary Matrix (N x m)
%
% RETURN VALUE: R     -> Tetrachoric matrix
%               THRES -> Variable's Thresholds
%               CON   -> Super Matrix of contingency tables
%

function [R,THRES,CON]=tetrachoric(X);
[N,m] = size(X);

R=eye(m);
THRES=zeros(m,1);
CON = [];

pairs = m*(m+1)/2;


for i=1:(m-1),
   for j=i+1:m,
       Xi = [X(:,i),X(:,j)];
       TC  = tableconti2(Xi);
       CON = [CON; TC];
       
       A = TC(1,1);
       B = TC(1,2);
       C = TC(2,1);
       D = TC(2,2);
       [r,thres] = tetra(A,B,C,D);
       rp=corrcoef(Xi);
       s= sign(rp(2,1));
       R(i,j) = s*r;
       R(j,i) = s*r;
    end;
    THRES(i) = thres(1);
end;
THRES(m) = thres(2);

return;
