%% WVARIM : Produces varimax function value and rotation matrix T with Cureton anf Mulaik weights
% Based on Nevels (1986). See also Pascal source.
%
% ARGUMENTS   : F    -> matrix to be rotated.
%               conv -> convergence criterion.
%               nrs  -> number of random starts.
%
% RETURN VALUE: Tr   -> rotation matrix.
%               fr   -> varimax function value.
%               ir   -> number of iterations.
%

function [Tr,W,fr,ir] = wvarim(F1,conv,nrs);

  [n,m]=size(F1);

  [Q,D]=ed(F1*F1');
  F = Q(:,1:m)*sqrt(D(1:m,1:m));
%  ['LOADING MATRIX (PCA)']
%  F
  
  tau=diag(sign(F(:,1)));
  H = diag(diag(F*F'))^(-1/2);
  G = tau*H*F;
  
  c = sqrt(1/m);
  acos_c = acos(c)*360/(2*pi);

  A = G;

  W=[];
  for i=1:n,
   aij= A(i,1);
   acos_a = acos(aij)*360/(2*pi);
   if aij < c,
    wi = cos( (( acos_a - acos_c ) / ( 90 - acos_c ) * 90)*2*pi/360 )^2 + 0.001;
   else
    wi = cos( (( acos_c - acos_a ) / (      acos_c ) * 90)*2*pi/360 )^2 + 0.001;
   end; 

   W = [W; wi];
  end;

  W = diag(W);
  A = W*A;

%  ['ROW-NORMALIZED LOADING MATRIX']
%  [H*F]
  
%  ['REFLECTED ROW-NORMALIZED LOADING MATRIX & WEIGHTS']
%  [G diag(W)]

%['WEIGHTED REFLECTED ROW-NORMALIZED LOADING MATRIX']
%  A

[m,r] = size(A);
for nos=1:nrs+1,
   
  if nos == 1, T = eye(r);
  else         T = orth(rand(r,r) - .5); end;
  
  B=A*T;
  f=ssq((B.*B)-ones(m,1)*mean(B.*B));
  fold=f-2*conv*f;
  if f==0, fold=-conv; end;
  iter=0;
  while f-fold>f*conv
    fold=f;iter=iter+1;
    for i=1:r
      for j=i+1:r
        x=B(:,i);y=B(:,j);
        xx=T(:,i);yy=T(:,j);
        u=x.^2-y.^2;v=2*x.*y;
        u=u-ones(m,1)*mean(u);v=v-ones(m,1)*mean(v);
        a=2*sum(u.*v);b=sum(u.^2)-sum(v.^2);c=(a^2+b^2)^.5;
        e = (u'*u) - (v'*v);
        if a>=0; signe=1; end;
        if a<0; signe=-1; end;
        if c<.00000000001
          disp(' No rotation anymore');
           cosi=1;sin=0;
           fr = 0; ir = 0; Tr = 0;
           Tr = eye(3);
           return; 
        end;
        if c>=.00000000001
          vvv=-signe*((b+c)/(2*c))^.5;
          sin=(.5-.5*vvv)^.5;cosi=(.5+.5*vvv)^.5;
        end;
        v=cosi*x-sin*y;w=cosi*y+sin*x;
        vv=cosi*xx-sin*yy;ww=cosi*yy+sin*xx;
        if vvv>=0       % prevent permutation of columns
          B(:,i)=v;B(:,j)=w;T(:,i)=vv;T(:,j)=ww;
        end;
        if vvv<0
          B(:,j)=v;B(:,i)=w;T(:,j)=vv;T(:,i)=ww;
        end;
      end;
    end;
    f = ssq((B.*B)-ones(m,1)*mean(B.*B));
  end;
  if nos == 1
    fr = f; ir = iter; Tr = T;
  elseif f > fr
    fr = f; ir = iter; Tr = T;
  end;
end;

%['ROTATED WEIGHTED REFLECTED  ROW-NORMALIZED LOADING MATRIX']
%B

%['ROTATED UNWEIGHTED REFLECTED ROW-NORMALIZED LOADING MATRIX']
%W\B

%['ROTATED UNWEIGHTED UNREFLECTED ROW-NORMALIZED LOADING MATRIX']
%tau\(W\B)

%['ROTATED UNWEIGHTED UNREFLECTED UNROW-NORMALIZED LOADING MATRIX']
%tau\H\(W\B)

I = eye(r,r);
A= F*Tr;
for i=1:r,
    if sum(A(:,i)) <0,
       I(i,i) = -1;
    end;
end;

Tr = Tr*I;
A= F*Tr;

Tr = (F1'*F1)^(-1) * F1' * A;

return
