function  varargout = PsychologicalTestToolbox(varargin)
%% Psychological Test Toolbox v 1.11
%
% This program is designed for computing factor analysis controlling two
% response bias: Social Desirability and Acquiescence.
%
% For detailed info about the program, please go to:
% http://psico.fcep.urv.cat/utilitats/PsychologicalTestToolbox
%
% Licensing:
%   This code is distributed under the GNU General Public License version
%   3.
%
% Authors:
%   David Navarro-Gonzalez
%   Andreu Vigil-Colet
%   Pere Joan Ferrando
%   Urbano Lorenzo-Seva
%
% Universitat Rovira i Virgili
% December, 2019
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

addpath lib 
warning off 
warning off MATLAB:uitabgroup:OldVersion

% Data structure

% Path of raw data
data.path_X=[];

% Path of dispersion matrix provided by user
data.path_S=[];

% Path of file containing labels
data.path_labels=[];

% Path of file containing target matrix
data.path_target=[];

%Original number of participants (with and without missing data)
data.N_original=[];

%Number of participants (without missing data)
data.N=[];

% Number of variables
data.n=[];

% If there're some missing data (user defined)
data.missings_on=[];

% Missings value assigned
data.missings_value =[];

% Document name
data.name=[];

% Document path
data.path=[];

% If the user provides the matrix
data.matrix_user_on=[];

% Matrix with all the items
data.X=[];

% Correlation matrix, defined by the user
data.S=[];

% Variable labels
data.labels=[];

% Variable labels of the selected items
data.labelsa=[];

% If the labels are provided by the user
data.labels_user_on=0;

% Target rotation matrix
data.H=[];

% Matrix with the selected items
data.Xa=[];

% Matrix ready for the factorial analysis, only including the selected items
% and placing first the SD items
data.X_a=[];

% User defined matrix, only containing the selected items
data.Sa=[];

% Matrix ready for the factorial analysis, only including the selected items
% and placing first the SD items (user defined matrix)
data.S_a=[];

% Included items index
data.inclosos=[];

% Excluded items index
data.exclosos=[];

% Content items index
data.items_contingut=[];

% SD items index
data.items_sd=[];

% Missings position, composed by 0 (no missing value) and 1 (missing value)
data.missings_position=[];

% Vector with SD items number and content items [G(1) G(2)]
data.G=[];

% Type of matrix computed for the following analysis
data.Matrix=2; %Pearson by default

% If the user wants factorial socres
data.pfactorial=0;  %By default NO

% Rotation type for the factorial analysis
data.Rotation=2; % Oblique by default

% Set the letter for the menus
data.font = ChooseFont('Helvetica','Arial','MS Sans Serif');

% If the save output will be in latex mode
data.latex_on=0;    %By default, plain text

% If the save output will be in rtf style
data.rtf_on=0;      %By default, plain text

% If the save output will be in latex mode
data.latex2_on=0;   %By default, plain text

% If the save output will be in rtf style
data.rtf2_on=0;     %By default, plain text

% Polychoric matrix is calculated
data.Rp_all=[];

%If Acquiescence control function is enabled (NO by default)
data.checking_AQ=0;

%If Social Desirability control function is enabled (NO by default)
data.checking_DS=0;

% If graded model will be used when computing factor scores (NO by
% default)
data.graded=0;

% If graded model will be used, how many computational nodes will be used
% (between 7 and 40), none by default
data.n_nodes=7;

% A marker for warning about losing data when loading a new analysis
data.Bandera=0; % NO Warning by default

% Code variable with the version of the program
data.version='4A1E26E9-F2CF-425B-A794-B1A00APTT108';

% Actual date
buff1=clock;
c=fix(buff1);
month={};
month{1}='January';month{2}='February';month{3}='March';month{4}='April';month{5}='May';month{6}='June';
month{7}='July';month{8}='August';month{9}='September';month{10}='October';month{11}='November';month{12}='December';
actual_month=month{c(2)};
buff=sprintf('%.0f-%s-%.0f, %.0f:%.0f',c(3),actual_month,c(1),c(4),c(5));
data.actual_date=buff;

%%%% Output structure %%%%
Printer.text=[];
Printer.latex=[];
Printer.rtf=[];

% Available at any time
setappdata(0,'data',data);
setappdata(0,'Printer',Printer);

finestra.N=760;
finestra.M=600;
pantalla=get(0,'ScreenSize');
finestra.coordenades=[(pantalla(3)/2)-(finestra.N/2)  (pantalla(4)/2)-(finestra.M/2) finestra.N finestra.M];

%Figure creation
handles.fig = figure('Units','Pixels',...
    'color' , [0.941 0.941 0.941],...
    'Position',finestra.coordenades);
tmp=get(0,'FactoryUIControlBackgroundColor');
set(handles.fig, 'MenuBar','none','Name','Psychological Test Toolbox','Toolbar','none','NumberTitle','off','resize','off');

%Creating MENUS
%Menu File
handles.MenuFile = uimenu('Label','File');
handles.MenuFile_OpenData = uimenu(handles.MenuFile,...
                                    'checked','off',...
                                    'Label','Open Data');
handles.MenuFile_Open = uimenu(handles.MenuFile,...
                                    'checked','off',...
                                    'Label','Open analysis',...
                                    'Separator','on');
handles.MenuFile_Save = uimenu(handles.MenuFile,...
                                    'checked','off',...
                                    'Label','Save analysis');
handles.MenuFile_Scoring_Utility = uimenu(handles.MenuFile,...
                                    'checked','off',...
                                    'Label','Launch Scoring Utility',...
                                    'Separator','on');                                
handles.MenuFile_Save_Matrices = uimenu(handles.MenuFile,...
                                    'checked','off',...
                                    'Label','Save Variance/Covariance matrices',...
                                    'Separator','on'); 
handles.MenuFile_ClearData = uimenu(handles.MenuFile,...
                                'checked','off',...
                                'Label','Clear data',...
                                'Separator','on');
handles.MenuFile_Quit = uimenu(handles.MenuFile,...
                                'checked','off',...
                                'Label','Quit',...
                                'Separator','on');
                            
%Menu Help
handles.MenuHelp = uimenu('Label','Help');
handles.MenuHelp_About = uimenu(handles.MenuHelp,...
                                    'checked','off',...
                                    'Label','About Psychological Test Toolbox');
handles.MenuHelp_Help = uimenu(handles.MenuHelp,...
                                    'checked','off',...
                                    'Label','Help');                             

%%Tab agrupation
handles.hTabGroup = uitabgroup('parent',handles.fig);


%%TAB I PANEL of the front page
handles.tab(1) = uitab(handles.hTabGroup,'title','Front Page','Units','pixels');
handles.panel_tab1=uipanel('parent',handles.tab(1),'Units','pixels','BackgroundColor', [.784 .949 .98]);

%AXES for the logo
logo.axes = axes('parent',handles.panel_tab1,...
   'Position',[0.2 0.15 0.6 0.6],...
   'XGrid','off',...
    'YGrid','off',...
    'Layer','top');
axis(logo.axes,'equal','off',[0 120 0 100 0 130]);

%DRAW LOGO
logo.fac = [1 2 3 4; ...
    2 6 7 3; ...
    4 3 7 8; ...
    1 5 8 4; ...
    1 2 6 5; ...
    5 6 7 8];
%(x,y,z,t);
%x,y,z defining position
% t defining size
logo.cube=zeros(8,3,21);
%groups of 7
%group 1
logo.cube(:,:,1)=DefineCube(15,60,60,8);

logo.cube(:,:,2)=DefineCube(35,15,90,5);
logo.cube(:,:,3)=DefineCube(35,55,30,5);
logo.cube(:,:,4)=DefineCube(35,25,60,5);
logo.cube(:,:,5)=DefineCube(35,25,15,5);
logo.cube(:,:,6)=DefineCube(35,55,105,5);
logo.cube(:,:,7)=DefineCube(35,75,75,5);
logo.cube(:,:,8)=DefineCube(35,95,45,5);
%group 2
logo.cube(:,:,9)=DefineCube(60,15,60,5);
logo.cube(:,:,10)=DefineCube(60,55,25,8);
logo.cube(:,:,11)=DefineCube(60,55,75,5);
logo.cube(:,:,12)=DefineCube(60,85,95,8);
logo.cube(:,:,13)=DefineCube(60,95,45,5);
%group 3
logo.cube(:,:,14)=DefineCube(85,5,45,5);
logo.cube(:,:,15)=DefineCube(85,75,90,5);
logo.cube(:,:,16)=DefineCube(85,25,75,5);
logo.cube(:,:,17)=DefineCube(85,45,15,5);
logo.cube(:,:,18)=DefineCube(85,55,105,5);
logo.cube(:,:,19)=DefineCube(85,95,60,5);
logo.cube(:,:,20)=DefineCube(85,65,30,5);

logo.cube(:,:,21)=DefineCube(105,50,60,8);
logo.xv = 50;
logo.yv = 25;
logo.inc= -10;
patch('Faces',logo.fac,'Vertices',logo.cube(:,:,1),'FaceColor',[0 0 1]);  % patch function
material shiny;
% alpha('color');
alphamap('rampdown');
for i=2:8
    patch('Faces',logo.fac,'Vertices',logo.cube(:,:,i),'FaceColor',[.8 0 .8]);  % patch function
    material shiny;
%     alpha('color');
    alphamap('rampdown');
end;
for i=9:13
    patch('Faces',logo.fac,'Vertices',logo.cube(:,:,i),'FaceColor',[1 0 0]);  % patch function
    material shiny;
%     alpha('color');
    alphamap('rampdown');
end;
for i=14:20
    patch('Faces',logo.fac,'Vertices',logo.cube(:,:,i),'FaceColor',[.8 .8 0]);  % patch function
    material shiny;
%     alpha('color');
    alphamap('rampdown');
end;
patch('Faces',logo.fac,'Vertices',logo.cube(:,:,21),'FaceColor',[0 1 0]);  % patch function
material shiny;
alpha 0.5
alphamap('rampdown');
view(51,26);
%SAVE LOGO
setappdata(0,'logo',logo);


%TITLE
handles.text01=uicontrol(handles.panel_tab1,'Style','text',...
    'Units','pixels',...
    'position',[0 500 760 50],...
    'FontSize',28,...
    'fontname',char(data.font),...
    'ForegroundColor',[.2 .4 .6],...
    'HorizontalAlignment','center',...
    'FontWeight','bold',...
    'BackgroundColor', [.784 .949 .98],...
    'String','Psychological Test Toolbox');

%AUTHORS
handles.text021=uicontrol(handles.panel_tab1,'Style','text',...
    'Units','pixels',...
    'position',[0 450 760 50],...
    'FontSize',14,...
    'fontname',char(data.font),...
    'ForegroundColor',[.2 .2 .2],...
    'HorizontalAlignment','center',...
    'BackgroundColor', [.784 .949 .98],...
    'String','David Navarro-Gonzalez, Andreu Vigil-Colet,');

handles.text022=uicontrol(handles.panel_tab1,'Style','text',...
    'Units','pixels',...
    'position',[0 425 760 50],...
    'FontSize',14,...
    'fontname',char(data.font),...
    'ForegroundColor',[.2 .2 .2],...
    'HorizontalAlignment','center',...
    'BackgroundColor', [.784 .949 .98],...
    'String','Pere J. Ferrando, Urbano Lorenzo-Seva');

%Release
handles.text03=uicontrol(handles.panel_tab1,'Style','text',...
    'Units','pixels',...
    'position',[0 395 760 50],...
    'FontSize',12,...
    'fontname',char(data.font),...
    'ForegroundColor',[.2 .2 .2],...
    'HorizontalAlignment','center',...
    'BackgroundColor', [.784 .949 .98],...
    'String','Release Version 1.11');
%YEAR
handles.text04=uicontrol(handles.panel_tab1,'Style','text',...
    'Units','pixels',...
    'position',[0 400 760 20],...
    'FontSize',12,...
    'fontname',char(data.font),...
    'ForegroundColor',[.2 .2 .2],...
    'HorizontalAlignment','center',...
    'BackgroundColor', [.784 .949 .98],...
    'String','January 2020');
%URV
handles.text05=uicontrol(handles.panel_tab1,'Style','text',...
    'Units','pixels',...
    'position',[0 20 760 40],...
    'FontSize',12,...
    'fontname',char(data.font),...
    'ForegroundColor',[.2 .2 .2],...
    'HorizontalAlignment','center',...
    'BackgroundColor', [.784 .949 .98],...
    'String','Research Center for Behavior Assessment');

handles.text06=uicontrol(handles.panel_tab1,'Style','text',...
    'Units','pixels',...
    'position',[0 0 760 40],...
    'FontSize',12,...
    'fontname',char(data.font),...
    'ForegroundColor',[.2 .2 .2],...
    'HorizontalAlignment','center',...
    'BackgroundColor', [.784 .949 .98],...
    'String','Universitat Rovira i Virgili, Tarragona (Spain)');

handles.slider01=uicontrol(handles.panel_tab1,'Style','slider',...
    'Units','pixels',...
    'position',[200 80 360 20],...
    'min',0,...
    'max',25,...
    'value',0);

%COLOR SWITCH
% EASTERN EGG ;)
handles.colors=uicontrol(handles.panel_tab1,'Style','text',...
    'Units','pixels',...
    'position',[670 0 90 30],...
    'FontSize',14,...
    'fontname',char(data.font),...
    'ForegroundColor',[.754 .919 1],...
    'HorizontalAlignment','center',...
    'FontWeight','bold',...
    'BackgroundColor', [.784 .949 .98],...
    'enable','inactive',...
    'String','Colors');
set(handles.colors,'ButtonDownFcn',{@colors_callback, handles});

%%DATA PANEL
handles.tab(2) = uitab(handles.hTabGroup,'title','Data','Units','pixels');
handles.panel_tab2=uipanel('parent',handles.tab(2),'Units','pixels','BackgroundColor', [.941 .941 .941]);

%READ DATA PUSH BUTTON 
handles.read= uicontrol(handles.panel_tab2,'Style','Pushbutton',...
    'Units','pixels',...
    'position',[20 510 180 40],...
    'Fontname',char(data.font),...
    'Fontsize',14,...
    'ForegroundColor',[.2 .4 .6],...
    'BackgroundColor',[.911 .911 .911],...
    'String','Read Data');

%PANEL WITH INFORMATION ABOUT THE SELECTED DATA
handles.panel2_tab2=uipanel('parent',handles.tab(2),'Units','pixels',...
    'BackgroundColor', [.941 .941 .941],...
    'position',[10 440 735 50]);

%NUMBER OF PARTICIPANTS TEXT
handles.text1= uicontrol(handles.panel2_tab2,'Style','text',...
    'Units','pixels',...
    'position',[10 15 200 20],...
    'FontSize',13,...
    'fontname',char(data.font),...
    'ForegroundColor',[.2 .2 .2],...
    'HorizontalAlignment','left',...
    'FontWeight','normal',...
    'BackgroundColor', [.941 .941 .941],...
    'String','Number of participants:');

% EDITABLE BOX OF NUMBER OF PARTICIPANTS
handles.numberofparticipants= uicontrol(handles.panel2_tab2,'Style','edit',...
    'Units','pixels',...
    'position',[240 15 60 20],...
    'FontSize',13,...
    'fontname',char(data.font),...
    'TooltipString','Number of participants',...
    'max',0,...  
    'BackgroundColor',[1 1 1],...
    'ForegroundColor',[0 0 0],...  
    'Enable','off',...
    'HorizontalAlignment','right',...
    'BackgroundColor', [.941 .941 .941],...
    'String','');

%NUMBER OF VARIABLES TEXT
handles.text2= uicontrol(handles.panel2_tab2,'Style','text',...
    'Units','pixels',...
    'position',[400 15 200 20],...
    'FontSize',13,...
    'fontname',char(data.font),...
    'ForegroundColor',[.2 .2 .2],...
    'HorizontalAlignment','left',...
    'FontWeight','normal',...
    'BackgroundColor', [.941 .941 .941],...
    'String','Number of items:');

%EDITABLE BOX OF NUMBER OF PARTICIPANTS
handles.numberofvariables= uicontrol(handles.panel2_tab2,'Style','edit',...
    'Units','pixels',...
    'position',[590 15 60 20],...
    'FontSize',13,...
    'fontname',char(data.font),...
    'TooltipString','Number of variables in the matrix',...
    'max',0,...  
    'BackgroundColor',[1 1 1],...
    'ForegroundColor',[0 0 0],...  
    'Enable','off',...
    'HorizontalAlignment','right',...
    'BackgroundColor', [.941 .941 .941],...
    'String','');

%PANEL WITH INCLUDED/EXCLUDED ITEMS 
handles.panel3_tab2=uipanel('parent',handles.tab(2),'Units','pixels',...
    'BackgroundColor', [.941 .941 .941],...
    'position',[10 60 735 360]);

%INCLUDED ITEMS LABEL 
handles.text3= uicontrol(handles.panel3_tab2,'Style','text',...
    'Units','pixels',...
    'position',[20 325 140 20],...
    'FontSize',14,...
    'fontname',char(data.font),...
    'ForegroundColor',[.2 .4 .6],...
    'HorizontalAlignment','left',...
    'FontWeight','bold',...
    'BackgroundColor', [.941 .941 .941],...
    'String','Items included');

tmp1={};

% LIST OF INCLUDED ITEMS
handles.ItemsIncluded= uicontrol(handles.panel3_tab2,'Style','listbox',...
    'Units','pixels','HorizontalAlignment','left',...
    'position',[20 20 300 300],...
    'min',0,'max',2,...
    'BackgroundColor',[1 1 1],'FontSize',12,...
    'TooltipString','Items to be included in the analysis',...
    'String',tmp1);

%EXCLUDED ITEMS LABEL
handles.text4= uicontrol(handles.panel3_tab2,'Style','text',...
    'Units','pixels',...
    'position',[420 325 260 20],...
    'FontSize',14,...
    'fontname',char(data.font),...
    'ForegroundColor',[.2 .4 .6],...
    'HorizontalAlignment','left',...
    'FontWeight','bold',...
    'BackgroundColor', [.941 .941 .941],...
    'String','Items excluded');

tmp1={};
%LIST OF EXCLUDED ITEMS
handles.ItemsExcluded= uicontrol(handles.panel3_tab2,'Style','listbox',...
    'Units','pixels','HorizontalAlignment','left',...
    'position',[420 20 300 300],...
    'min',0,'max',2,...
    'BackgroundColor',[1 1 1],'FontSize',12,...
    'TooltipString','Items to be excluded in the analysis',...
    'String',tmp1);

%BUTTON FOR EXCLUDING ITEMS
handles.ButtonExcloureItems = uicontrol(handles.panel3_tab2,...
    'String','>',...
    'Style','pushbutton',...
    'FontSize',14,...
    'fontname',char(data.font),...   
    'position',[350 200 40 40],...
    'BackgroundColor', [.831 .816 .784],...
    'ForegroundColor',[.2 .4 .6]);

%BUTTON FOR INCLUDING ITEMS
handles.ButtonIncloureItems = uicontrol(handles.panel3_tab2,...
    'String','<',...
    'Style','pushbutton',...
    'FontSize',14,...
    'fontname',char(data.font),...   
    'position',[350 150 40 40],...
    'enable','off',...
    'BackgroundColor', [.831 .816 .784],...
    'ForegroundColor',[.2 .4 .6]);

%CHECKBOX FOR VISUALIZING LABELS
handles.Visualize_Labels = uicontrol(handles.panel_tab2,...
    'String','Visualize Labels',...
    'Style','checkbox',...
    'FontSize',14,...
    'fontname',char(data.font),...   
    'position',[15 10 220 40],...
    'enable','off',...
    'visible','off',...
    'BackgroundColor', [.941 .941 .941],...
    'ForegroundColor',[.2 .4 .6]);

%%VIEW DATA PANEL
handles.tab(3) = uitab(handles.hTabGroup,'title','View Data','Units','pixels');
handles.panel_tab2=uipanel('parent',handles.tab(3),'Units','pixels','BackgroundColor', [.941 .941 .941],...
    'visible','on');
handles.data_table = uitable(handles.panel_tab2,'units','normalized','Position', [.02 .02 .95 .9],'visible','off');

%%DESCRIPTIVES PANEL
handles.tab(4) = uitab(handles.hTabGroup,'title','Descriptive Statistics','Units','pixels');
handles.panel_tab4=uipanel('parent',handles.tab(4),'Units','pixels','BackgroundColor', [.941 .941 .941],...
    'visible','off');

%PANEL WITH DISPERSION MATRIX OPTIONS
handles.panel2_tab4=uipanel('parent',handles.tab(4),'Units','pixels',...
    'BackgroundColor', [.941 .941 .941],...
    'position',[10 440 735 120],...
    'FontSize',13,...
    'visible','off',...
    'fontname',char(data.font),...
    'Title','Dispersion Matrix');

%RADIOBUTTON OF COVARIANCE
handles.Matrix_Covariance = uicontrol(handles.panel2_tab4,...
    'style','radiobutton',...
    'string','Covariance Matrix',...
    'fontname',char(data.font),...
    'FontSize',13,...
    'ForegroundColor',[.2 .2 .2],...
    'HorizontalAlignment','left',...
    'value',0,...
    'BackgroundColor', [.941 .941 .941],...
    'position',[15 70 320 20]);

%RADIOBUTTON OF PEARSON
handles.Matrix_Pearson = uicontrol(handles.panel2_tab4,...
    'style','radiobutton',...
    'string','Pearson Correlation Matrix',...
    'fontname',char(data.font),...
    'FontSize',13,...
    'ForegroundColor',[.2 .2 .2],...
    'HorizontalAlignment','left',...
    'value',1,...
    'BackgroundColor', [.941 .941 .941],...
    'position',[380 70 350 20]);

%RADIOBUTTON OF POLYCHORIC
handles.Matrix_Polychoric = uicontrol(handles.panel2_tab4,...
    'style','radiobutton',...
    'string','Polychoric (tetrachoric) Matrix',...
    'fontname',char(data.font),...
    'FontSize',13,...
    'ForegroundColor',[.2 .2 .2],...
    'HorizontalAlignment','left',...
    'value',0,...
    'BackgroundColor', [.941 .941 .941],...
    'position',[15 40 320 20]);

%LABEL OF THE MINIMAL POSSIBLE SCORE 
handles.text5= uicontrol(handles.panel2_tab4,'Style','text',...
    'Units','pixels',...
    'position',[50 10 260 20],...
    'FontSize',12,...
    'fontname',char(data.font),...
    'ForegroundColor',[.2 .2 .2],...
    'HorizontalAlignment','left',...
    'Visible','off',...
    'BackgroundColor', [.941 .941 .941],...
    'String','Lowest item score');

%LABEL OF THE MAXIMUM POSSIBLE SCORE
handles.text6= uicontrol(handles.panel2_tab4,'Style','text',...
    'Units','pixels',...
    'position',[320 10 260 20],...
    'FontSize',12,...
    'fontname',char(data.font),...
    'ForegroundColor',[.2 .2 .2],...
    'HorizontalAlignment','left',...
    'Visible','off',...
    'BackgroundColor', [.941 .941 .941],...
    'String','Highest item score');

%EDITABLE BOX OF THE LOWEST SCORE
handles.lowest_score= uicontrol(handles.panel2_tab4,'Style','edit',...
    'Units','pixels',...
    'position',[220 10 60 20],...
    'FontSize',13,...
    'fontname',char(data.font),...
    'TooltipString','Lowest possible item score',...
    'max',0,...  
    'BackgroundColor',[1 1 1],...
    'ForegroundColor',[0 0 0],...  
    'Enable','off',...
    'Visible','off',...
    'HorizontalAlignment','right',...
    'BackgroundColor', [.941 .941 .941],...
    'String','');

%EDITABLE BOX OF THE HIGHEST SCORE
handles.highest_score= uicontrol(handles.panel2_tab4,'Style','edit',...
    'Units','pixels',...
    'position',[480 10 60 20],...
    'FontSize',13,...
    'fontname',char(data.font),...
    'TooltipString','Highest possible item score',...
    'max',0,...  
    'BackgroundColor',[1 1 1],...
    'ForegroundColor',[0 0 0],...  
    'Enable','off',...
    'Visible','off',...
    'HorizontalAlignment','right',...
    'BackgroundColor', [.941 .941 .941],...
    'String','');


%PANEL WITH DIMENSIONALLITY OPTIONS
handles.panel3_tab4=uipanel('parent',handles.tab(4),'Units','pixels',...
    'BackgroundColor', [.941 .941 .941],...
    'position',[10 345 735 75],...
    'FontSize',13,...
    'visible','off',...
    'fontname',char(data.font),...
    'Title','Determining the number of factors');

%CHECKBOX OF PA
handles.parallel_analysis = uicontrol(handles.panel3_tab4,...
    'style','checkbox',...
    'string','Optimal Implementation of Parallel Analysis (PA)',...
    'fontname',char(data.font),...
    'FontSize',13,...
    'ForegroundColor',[.2 .2 .2],...
    'HorizontalAlignment','left',...
    'value',0,...
    'BackgroundColor', [.941 .941 .941],...
    'position',[15 20 600 20]);

%CHECKBOX OF GRAFIC PA
handles.grafic_PA = uicontrol(handles.panel3_tab4,...
    'style','checkbox',...
    'string','Plot PA',...
    'fontname',char(data.font),...
    'FontSize',13,...
    'enable','off',...
    'Visible','off',...
    'ForegroundColor',[.2 .2 .2],...
    'HorizontalAlignment','left',...
    'value',0,...
    'BackgroundColor', [.941 .941 .941],...
    'position',[505 20 600 20]);

%LABEL OF THE OUTPUT BOX 
handles.text13= uicontrol(handles.panel_tab4,'Style','text',...
    'Units','pixels',...
    'position',[10 305 400 20],...
    'fontname',char(data.font),...
    'FontSize',13,...
    'FontWeight','bold',...
    'ForegroundColor',[.2 .2 .2],...
    'HorizontalAlignment','left',...
    'BackgroundColor', [.941 .941 .941],...
    'String','Descriptive Statistics: Output');

% - SIZING BOX
handles.ModelOutputDecrementa= uicontrol(handles.panel_tab4,'string','-',...
    'Style','pushbutton',...
    'Units','pixels',...
    'position',[675 305 30 30],...
    'FontSize',13,...
    'fontname',char(data.font),...
    'Enable','off',...
    'TooltipString','Zoom out',...
    'BackgroundColor', [.831 .816 .784],...
    'ForegroundColor',[.2 .4 .6],...  
    'HorizontalAlignment','center',...
    'FontWeight','normal');

% + SIZING BOX
handles.ModelOutputIncrementa= uicontrol(handles.panel_tab4,'string','+',...
    'Style','pushbutton',...
    'Units','pixels',...
    'position',[714 305 30 30],...
    'FontSize',13,...
    'fontname',char(data.font),...
    'Enable','off',...
    'TooltipString','Zoom out',...
    'BackgroundColor', [.831 .816 .784],...
    'ForegroundColor',[.2 .4 .6],...  
    'HorizontalAlignment','center',...
    'FontWeight','normal');

%THE LISTBOXS CONTAINING THE OUTPUT INFORMATION IN THE THREE AVAILABLE
%FORMATS (text, RTF and LaTeX)
tmp={};
global outputletter;
outputletter = 12;
handles.ModelOutput= uicontrol(handles.panel_tab4,'Style','listbox',...
    'Units','pixels',...
    'position',[10 60 735 240],...
    'FontSize',outputletter,...
    'fontname','Monospaced',...
    'BackgroundColor',[1 1 1],...
    'ForegroundColor',[0 0 0],...  
    'visible','on',...
    'HorizontalAlignment','left',...
    'min',0,'max',0,...
    'FontWeight','normal',...
    'String',tmp);

handles.ModelOutput_latex= uicontrol(handles.panel_tab4,'Style','listbox',...
    'Units','pixels',...
    'position',[10 60 735 240],...
    'FontSize',outputletter,...
    'fontname','Monospaced',...
    'Visible','off',...
    'BackgroundColor',[1 1 1],...
    'ForegroundColor',[0 0 0],...  
    'HorizontalAlignment','left',...
    'min',0,'max',0,...
    'FontWeight','normal',...
    'String',tmp);

handles.ModelOutput_rtf= uicontrol(handles.panel_tab4,'Style','listbox',...
    'Units','pixels',...
    'position',[10 60 735 240],...
    'FontSize',outputletter,...
    'fontname','Monospaced',...
    'Visible','off',...
    'BackgroundColor',[1 1 1],...
    'ForegroundColor',[0 0 0],...  
    'HorizontalAlignment','left',...
    'min',0,'max',0,...
    'FontWeight','normal',...
    'String',tmp);

%Radio button save as text
handles.save_text= uicontrol(handles.panel_tab4,'Style','radiobutton',...
    'Units','pixels',...
    'position',[15 35 120 20],...
    'FontSize',13,...
    'fontname',char(data.font),...
    'Value',1,...
    'BackgroundColor',[1 1 1],...
    'ForegroundColor',[0 0 0],...  
    'Enable','off',...
    'HorizontalAlignment','right',...
    'BackgroundColor', [.941 .941 .941],...
    'String','Text');

%Radio button save as rtf
handles.save_rtf= uicontrol(handles.panel_tab4,'Style','radiobutton',...
    'Units','pixels',...
    'position',[90 35 120 20],...
    'FontSize',13,...
    'fontname',char(data.font),...
    'Value',0,...
    'BackgroundColor',[1 1 1],...
    'ForegroundColor',[0 0 0],...  
    'Enable','off',...
    'HorizontalAlignment','right',...
    'BackgroundColor', [.941 .941 .941],...
    'String','RTF');

%Radio button save as LaTeX
handles.save_latex= uicontrol(handles.panel_tab4,'Style','radiobutton',...
    'Units','pixels',...
    'position',[150 35 120 20],...
    'FontSize',13,...
    'fontname',char(data.font),...
    'Value',0,...
    'BackgroundColor',[1 1 1],...
    'ForegroundColor',[0 0 0],...  
    'Enable','off',...
    'HorizontalAlignment','right',...
    'BackgroundColor', [.941 .941 .941],...
    'String','Latex');

%BUTTON SAVE OUTPUT
handles.ButtonSalvarOutput = uicontrol(handles.panel_tab4,'String',' Save',...
    'Style','pushbutton',...
    'FontSize',15,...
    'FontWeight','bold',...
    'fontname',char(data.font),...   
    'position',[15 10 200 25],...
    'enable','off',...
    'TooltipString','To save the output of the fitted model as a text file',...
    'BackgroundColor', [.831 .816 .784],...
    'ForegroundColor',[.2 .4 .6]);

%BUTTON LAUNCH OUTPUT
handles.ButtonLaunchOutput = uicontrol(handles.panel_tab4,'String',' Visualize Output ',...
    'Style','pushbutton',...
    'FontSize',15,...
    'FontWeight','bold',...
    'fontname',char(data.font),...   
    'position',[240 10 240 40],...
    'enable','off',...
    'TooltipString','To launch the output of the fitted model in a new window',...
    'BackgroundColor', [.831 .816 .784],...
    'ForegroundColor',[.2 .4 .6]);

%BUTTON COMPUTE DESCRIPTIVES
handles.Compute_Descriptives = uicontrol(handles.panel_tab4,...
    'String','    Compute    ',...
    'Style','pushbutton',...
    'FontSize',15,...
    'FontWeight','bold',...
    'fontname',char(data.font),...  
    'enable','off',...
    'position',[505 10 240 40],...
    'BackgroundColor', [.831 .816 .784],...
    'ForegroundColor',[.2 .4 .6]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%DS PANEL

%%SOCIAL DESIRABILITY PANEL
handles.tab(5) = uitab(handles.hTabGroup,'title','Social Desirability','Units','pixels');
handles.panel_tab5=uipanel('parent',handles.tab(5),'Units','pixels','BackgroundColor', [.941 .941 .941]);

%PANEL WITH THE ENABLE CONTROL FUNCTION
handles.panel2_tab5=uipanel('parent',handles.tab(5),'Units','pixels',...
    'BackgroundColor', [.941 .941 .941],...
    'position',[10 520 735 40],...
    'FontSize',13,...
    'fontname',char(data.font));

%CHECKBOX OF SD CONTROL
handles.DS_checkbox = uicontrol(handles.panel2_tab5,...
    'style','checkbox',...
    'string','Enable Social Desirability control function',...
    'fontname',char(data.font),...
    'FontSize',13,...
    'enable','off',...
    'ForegroundColor',[.2 .2 .2],...
    'HorizontalAlignment','left',...
    'BackgroundColor', [.941 .941 .941],...
    'position',[15 10 600 20]);

%PANEL
handles.panel3_tab5=uipanel('parent',handles.tab(5),'Units','pixels',...
    'BackgroundColor', [.941 .941 .941],...
    'Visible','off',...
    'position',[10 425 735 90]);

%TEXT NUMBER OF ITEMS EXCLUDED
handles.text14= uicontrol(handles.panel3_tab5,'Style','text',...
    'Units','pixels',...
    'position',[10 55 400 20],...
    'FontSize',13,...
    'fontname',char(data.font),...
    'ForegroundColor',[.2 .2 .2],...
    'HorizontalAlignment','left',...
    'FontWeight','normal',...
    'BackgroundColor', [.941 .941 .941],...
    'String','Number of previously excluded items:');

%EDITABLE BOX OF EXCLUDED ITEMS
handles.numberofexcludeditems= uicontrol(handles.panel3_tab5,'Style','edit',...
    'Units','pixels',...
    'position',[350 55 100 20],...
    'FontSize',13,...
    'fontname',char(data.font),...
    'TooltipString','Number of exlcuded items from the analysis',...
    'max',0,...  
    'BackgroundColor',[1 1 1],...
    'ForegroundColor',[0 0 0],...  
    'Enable','off',...
    'HorizontalAlignment','right',...
    'BackgroundColor', [.941 .941 .941],...
    'String','');

%TEXT NUMBER OF CONTENT ITEMS
handles.text14= uicontrol(handles.panel3_tab5,'Style','text',...
    'Units','pixels',...
    'position',[10 15 250 20],...
    'FontSize',13,...
    'fontname',char(data.font),...
    'ForegroundColor',[.2 .2 .2],...
    'HorizontalAlignment','left',...
    'FontWeight','normal',...
    'BackgroundColor', [.941 .941 .941],...
    'String','Number of content items:');

%EDITABLE BOX OF CONTENT ITEMS
handles.numberofcontentitems= uicontrol(handles.panel3_tab5,'Style','edit',...
    'Units','pixels',...
    'position',[240 15 100 20],...
    'FontSize',13,...
    'fontname',char(data.font),...
    'TooltipString','Number of content items',...
    'max',0,...  
    'BackgroundColor',[1 1 1],...
    'ForegroundColor',[0 0 0],...  
    'Enable','off',...
    'HorizontalAlignment','right',...
    'BackgroundColor', [.941 .941 .941],...
    'String','');

%TEXT NUMBER OF SD ITEMS
handles.text15= uicontrol(handles.panel3_tab5,'Style','text',...
    'Units','pixels',...
    'position',[400 15 200 20],...
    'FontSize',13,...
    'fontname',char(data.font),...
    'ForegroundColor',[.2 .2 .2],...
    'HorizontalAlignment','left',...
    'FontWeight','normal',...
    'BackgroundColor', [.941 .941 .941],...
    'String','Number of SD items:');

%EDITABLE BOX OF NUMBER OF SD ITEMS
handles.numberofsditems= uicontrol(handles.panel3_tab5,'Style','edit',...
    'Units','pixels',...
    'position',[590 15 100 20],...
    'FontSize',13,...
    'fontname',char(data.font),...
    'TooltipString','Number of SD items',...
    'max',0,...  
    'BackgroundColor',[1 1 1],...
    'ForegroundColor',[0 0 0],...  
    'Enable','off',...
    'HorizontalAlignment','right',...
    'BackgroundColor', [.941 .941 .941],...
    'String','');

%PANEL OF CONTENT/SD ITEMS
handles.panel4_tab5=uipanel('parent',handles.tab(5),'Units','pixels',...
    'BackgroundColor', [.941 .941 .941],...
    'Visible','off',...
    'position',[10 60 735 360]);

%LABEL CONTENT ITEMS
handles.text16= uicontrol(handles.panel4_tab5,'Style','text',...
    'Units','pixels',...
    'position',[20 325 180 20],...
    'FontSize',14,...
    'fontname',char(data.font),...
    'ForegroundColor',[.2 .4 .6],...
    'HorizontalAlignment','left',...
    'FontWeight','bold',...
    'BackgroundColor', [.941 .941 .941],...
    'String','Content Items');

tmp1={};

%LIST OF CONTENT ITEMS
handles.ContentItems= uicontrol(handles.panel4_tab5,'Style','listbox',...
    'Units','pixels','HorizontalAlignment','left',...
    'position',[20 20 300 300],...
    'min',0,'max',2,...
    'BackgroundColor',[1 1 1],'FontSize',12,...
    'TooltipString','Items that will be included in factor analysis like content items',...
    'String',tmp1);

%LABEL SD ITEMS
handles.text17= uicontrol(handles.panel4_tab5,'Style','text',...
    'Units','pixels',...
    'position',[420 325 260 20],...
    'FontSize',14,...
    'fontname',char(data.font),...
    'ForegroundColor',[.2 .4 .6],...
    'HorizontalAlignment','left',...
    'FontWeight','bold',...
    'BackgroundColor', [.941 .941 .941],...
    'String','SD Items');

tmp1={};
%LIST OF SD ITEMS
handles.SDItems= uicontrol(handles.panel4_tab5,'Style','listbox',...
    'Units','pixels','HorizontalAlignment','left',...
    'position',[420 20 300 300],...
    'min',0,'max',2,...
    'BackgroundColor',[1 1 1],'FontSize',12,...
    'TooltipString','Items that will be treated like SD Items',...
    'String',tmp1);

%BUTTON FOR EXCLUDING (FROM CONTENT TO SD)
handles.ButtonExcloureItems2 = uicontrol(handles.panel4_tab5,...
    'String','>',...
    'Style','pushbutton',...
    'FontSize',14,...
    'fontname',char(data.font),...   
    'position',[350 200 40 40],...
    'BackgroundColor', [.831 .816 .784],...
    'ForegroundColor',[.2 .4 .6]);

%BUTTON FOR INCLUDING (FROM SD TO CONTENT)
handles.ButtonIncloureItems2 = uicontrol(handles.panel4_tab5,...
    'String','<',...
    'Style','pushbutton',...
    'FontSize',14,...
    'fontname',char(data.font),...   
    'position',[350 150 40 40],...
    'enable','off',...
    'BackgroundColor', [.831 .816 .784],...
    'ForegroundColor',[.2 .4 .6]);

%CHECKBOX FOR VISUALIZING LABELS
handles.Visualize_Labels2 = uicontrol(handles.panel_tab5,...
    'String','Visualize Labels',...
    'Style','checkbox',...
    'FontSize',14,...
    'fontname',char(data.font),...   
    'position',[15 10 220 40],...
    'enable','off',...
    'visible','off',...
    'BackgroundColor', [.941 .941 .941],...
    'ForegroundColor',[.2 .4 .6]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% % ACQUIESCENCE PANEL
handles.tab(6) = uitab(handles.hTabGroup,'title','Acquiescence','Units','pixels');
handles.panel_tab6=uipanel('parent',handles.tab(6),'Units','pixels','BackgroundColor', [.941 .941 .941]);

%PANEL WITH THE OPTION TO ENABLE CONTROL FUNCTION
handles.panel2_tab6=uipanel('parent',handles.tab(6),'Units','pixels',...
    'BackgroundColor', [.941 .941 .941],...
    'position',[10 520 735 40],...
    'FontSize',13,...
    'fontname',char(data.font));

%CHECKBOX OF ACQUIESCENCE CONTROL FUNCTION
handles.AQ_checkbox = uicontrol(handles.panel2_tab6,...
    'style','checkbox',...
    'string','Enable Acquiescence control function',...
    'fontname',char(data.font),...
    'FontSize',13,...
    'enable','off',...
    'ForegroundColor',[.2 .2 .2],...
    'HorizontalAlignment','left',...
    'BackgroundColor', [.941 .941 .941],...
    'position',[15 10 600 20]);

%PANEL 
handles.panel3_tab6=uipanel('parent',handles.tab(6),'Units','pixels',...
    'BackgroundColor', [.941 .941 .941],...
    'Visible','off',...
    'position',[10 425 735 90]);

%TEXT NUMBER OF EXCLUDED ITEMS
handles.text18= uicontrol(handles.panel3_tab6,'Style','text',...
    'Units','pixels',...
    'position',[10 55 400 20],...
    'FontSize',13,...
    'fontname',char(data.font),...
    'ForegroundColor',[.2 .2 .2],...
    'HorizontalAlignment','left',...
    'FontWeight','normal',...
    'BackgroundColor', [.941 .941 .941],...
    'String','Number of previously excluded items:');

%EDITABLE BOX OF EXCLUDED ITEMS
handles.numberofexcludeditems2= uicontrol(handles.panel3_tab6,'Style','edit',...
    'Units','pixels',...
    'position',[350 55 100 20],...
    'FontSize',13,...
    'fontname',char(data.font),...
    'TooltipString','Number of exlcuded items from the analysis',...
    'max',0,...  
    'BackgroundColor',[1 1 1],...
    'ForegroundColor',[0 0 0],...  
    'Enable','off',...
    'HorizontalAlignment','right',...
    'BackgroundColor', [.941 .941 .941],...
    'String','');

%TEXT NUMBER OF BALANCED ITEMS
handles.text19= uicontrol(handles.panel3_tab6,'Style','text',...
    'Units','pixels',...
    'position',[10 15 250 20],...
    'FontSize',13,...
    'fontname',char(data.font),...
    'ForegroundColor',[.2 .2 .2],...
    'HorizontalAlignment','left',...
    'FontWeight','normal',...
    'BackgroundColor', [.941 .941 .941],...
    'String','Number of balanced items:');

%EDITABLE BOX OF BALANCED ITEMS
handles.numberofbalanceditems= uicontrol(handles.panel3_tab6,'Style','edit',...
    'Units','pixels',...
    'position',[240 15 100 20],...
    'FontSize',13,...
    'fontname',char(data.font),...
    'TooltipString','Number of balanced content items',...
    'max',0,...  
    'BackgroundColor',[1 1 1],...
    'ForegroundColor',[0 0 0],...  
    'Enable','off',...
    'HorizontalAlignment','right',...
    'BackgroundColor', [.941 .941 .941],...
    'String','');

%TEXT NUMBER OF UNBALANCED ITEMS
handles.text20= uicontrol(handles.panel3_tab6,'Style','text',...
    'Units','pixels',...
    'position',[370 15 280 20],...
    'FontSize',13,...
    'fontname',char(data.font),...
    'ForegroundColor',[.2 .2 .2],...
    'HorizontalAlignment','left',...
    'FontWeight','normal',...
    'BackgroundColor', [.941 .941 .941],...
    'String','Number of unbalanced items:');

%EDITABLE BOX OF UNBALANCED ITEMS
handles.numberofunbalanceditems= uicontrol(handles.panel3_tab6,'Style','edit',...
    'Units','pixels',...
    'position',[620 15 100 20],...
    'FontSize',13,...
    'fontname',char(data.font),...
    'TooltipString','Number of unbalanced content items',...
    'max',0,...  
    'BackgroundColor',[1 1 1],...
    'ForegroundColor',[0 0 0],...  
    'Enable','off',...
    'HorizontalAlignment','right',...
    'BackgroundColor', [.941 .941 .941],...
    'String','');

%PANEL OF BALANCED/UNBALANCED ITEMS
handles.panel4_tab6=uipanel('parent',handles.tab(6),'Units','pixels',...
    'BackgroundColor', [.941 .941 .941],...
    'Visible','off',...
    'position',[10 60 735 360]);

%LABEL OF CONTENT ITEMS
handles.text21= uicontrol(handles.panel4_tab6,'Style','text',...
    'Units','pixels',...
    'position',[20 325 280 20],...
    'FontSize',14,...
    'fontname',char(data.font),...
    'ForegroundColor',[.2 .4 .6],...
    'HorizontalAlignment','left',...
    'FontWeight','bold',...
    'BackgroundColor', [.941 .941 .941],...
    'String','Balanced Content Items');

tmp1={};

%LIST OF BALANCED ITEMS
handles.BalItems= uicontrol(handles.panel4_tab6,'Style','listbox',...
    'Units','pixels','HorizontalAlignment','left',...
    'position',[20 20 300 300],...
    'min',0,'max',2,...
    'BackgroundColor',[1 1 1],'FontSize',12,...
    'TooltipString','Items that will be included in factor analysis like balanced content items',...
    'String',tmp1);

%LABEL OF UNBALANCED ITEMS
handles.text22= uicontrol(handles.panel4_tab6,'Style','text',...
    'Units','pixels',...
    'position',[420 325 260 20],...
    'FontSize',14,...
    'fontname',char(data.font),...
    'ForegroundColor',[.2 .4 .6],...
    'HorizontalAlignment','left',...
    'FontWeight','bold',...
    'BackgroundColor', [.941 .941 .941],...
    'String','Unbalanced Content Items');

tmp1={};
%LIST OF UNBALANCED ITEMS
handles.UnbalItems= uicontrol(handles.panel4_tab6,'Style','listbox',...
    'Units','pixels','HorizontalAlignment','left',...
    'position',[420 20 300 300],...
    'min',0,'max',2,...
    'BackgroundColor',[1 1 1],'FontSize',12,...
    'TooltipString','Items that will be treated like unbalanced Items',...
    'String',tmp1);

%BUTTON FOR EXCLUDING (FROM BALANCED TO UNBALANCED)
handles.ButtonExcloureItems3 = uicontrol(handles.panel4_tab6,...
    'String','>',...
    'Style','pushbutton',...
    'FontSize',14,...
    'fontname',char(data.font),...   
    'position',[350 200 40 40],...
    'BackgroundColor', [.831 .816 .784],...
    'ForegroundColor',[.2 .4 .6]);

%BUTTON FOR INCLUDING (FROM UNBALANCED TO BALANCED)
handles.ButtonIncloureItems3 = uicontrol(handles.panel4_tab6,...
    'String','<',...
    'Style','pushbutton',...
    'FontSize',14,...
    'fontname',char(data.font),...   
    'position',[350 150 40 40],...
    'enable','off',...
    'BackgroundColor', [.831 .816 .784],...
    'ForegroundColor',[.2 .4 .6]);

%CHECKBOX FOR VISUALIZING LABELS
handles.Visualize_Labels3 = uicontrol(handles.panel_tab6,...
    'String','Visualize Labels',...
    'Style','checkbox',...
    'FontSize',14,...
    'fontname',char(data.font),...   
    'position',[15 10 220 40],...
    'enable','off',...
    'visible','off',...
    'BackgroundColor', [.941 .941 .941],...
    'ForegroundColor',[.2 .4 .6]);

%EDITABLE BOX OF INFORMATION ABOUT THE STATUS OF THE BALANCED CORE
handles.text_balanced_core= uicontrol(handles.panel_tab6,'Style','edit',...
    'Units','pixels',...
    'position',[230 5 520 45],...
    'FontSize',13,...
    'fontname',char(data.font),...
    'max',4,...  
    'BackgroundColor',[1 1 1],...
    'ForegroundColor',[0 0 0],...  
    'Enable','inactive',...
    'visible','off',...
    'HorizontalAlignment','left',...
    'BackgroundColor', [.941 .941 .941],...
    'String',' ');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% % CONTROL OF RESPONSE BIAS
handles.tab(7) = uitab(handles.hTabGroup,'title','Response Bias','Units','pixels');
handles.panel_tab7=uipanel('parent',handles.tab(7),'Units','pixels','BackgroundColor', [.941 .941 .941],'visible','off');

%PANEL OF THE SUMMARY
handles.panel2_tab7=uipanel('parent',handles.tab(7),'Units','pixels',...
    'BackgroundColor', [.941 .941 .941],...
    'position',[10 340 735 220],...
    'FontSize',13,...
    'visible','off',...
    'Title','Data Summary',...
    'fontname',char(data.font));

%TEXT EXCLUDED ITEMS
handles.text23= uicontrol(handles.panel2_tab7,'Style','text',...
    'Units','pixels',...
    'position',[10 165 290 20],...
    'FontSize',13,...
    'fontname',char(data.font),...
    'ForegroundColor',[.2 .2 .2],...
    'HorizontalAlignment','left',...
    'FontWeight','normal',...
    'BackgroundColor', [.941 .941 .941],...
    'String','Excluded items:');

%TEXT SD ITEMS
handles.text24= uicontrol(handles.panel2_tab7,'Style','text',...
    'Units','pixels',...
    'position',[10 135 280 20],...
    'FontSize',13,...
    'fontname',char(data.font),...
    'ForegroundColor',[.2 .2 .2],...
    'HorizontalAlignment','left',...
    'FontWeight','normal',...
    'BackgroundColor', [.941 .941 .941],...
    'String','SD Items:');

%TEXT CONTENT ITEMS
handles.text25= uicontrol(handles.panel2_tab7,'Style','text',...
    'Units','pixels',...
    'position',[10 105 280 20],...
    'FontSize',13,...
    'fontname',char(data.font),...
    'ForegroundColor',[.2 .2 .2],...
    'HorizontalAlignment','left',...
    'FontWeight','normal',...
    'BackgroundColor', [.941 .941 .941],...
    'String','Content items:');

%EDITABLE BOX OF EXCLUDED ITEMS
handles.excluded_list= uicontrol(handles.panel2_tab7,'Style','edit',...
    'Units','pixels',...
    'position',[160 165 550 20],...
    'FontSize',13,...
    'fontname',char(data.font),...
    'TooltipString','Number of excluded items',...
    'max',0,...  
    'BackgroundColor',[1 1 1],...
    'ForegroundColor',[0 0 0],...  
    'Enable','off',...
    'HorizontalAlignment','left',...
    'BackgroundColor', [.941 .941 .941],...
    'String','');

%EDITABLE BOX OF SD ITEMS
handles.sd_list= uicontrol(handles.panel2_tab7,'Style','edit',...
    'Units','pixels',...
    'position',[160 135 550 20],...
    'FontSize',13,...
    'fontname',char(data.font),...
    'TooltipString','Number of social desirability items',...
    'max',0,...  
    'BackgroundColor',[1 1 1],...
    'ForegroundColor',[0 0 0],...  
    'Enable','off',...
    'HorizontalAlignment','left',...
    'BackgroundColor', [.941 .941 .941],...
    'String','');

%EDITABLE BOX OF CONTENT ITEMS
handles.content_list= uicontrol(handles.panel2_tab7,'Style','listbox',...
    'Units','pixels',...
    'position',[160 80 550 45],...
    'FontSize',13,...
    'fontname',char(data.font),...
    'TooltipString','Number of Content items',...
    'max',0,...  
    'value',1,...
    'BackgroundColor',[1 1 1],...
    'ForegroundColor',[0 0 0],...  
    'Enable','on',...
    'HorizontalAlignment','left',...
    'BackgroundColor', [.941 .941 .941],...
    'String','');

%TEXT NUMBER OF CONTENT FACTORS
handles.text24= uicontrol(handles.panel2_tab7,'Style','text',...
    'Units','pixels',...
    'position',[450 11 280 20],...
    'FontSize',13,...
    'fontname',char(data.font),...
    'ForegroundColor',[.2 .2 .2],...
    'HorizontalAlignment','left',...
    'FontWeight','normal',...
    'BackgroundColor', [.941 .941 .941],...
    'String','Number of content factors:');

%EDITABLE BOX OF CONTENT FACTORS NUMBER
handles.content_factors= uicontrol(handles.panel2_tab7,'Style','edit',...
    'Units','pixels',...
    'position',[680 11 40 20],...
    'FontSize',13,...
    'fontname',char(data.font),...
    'TooltipString','Number of content factors',...
    'max',0,...  
    'BackgroundColor',[1 1 1],...
    'ForegroundColor',[0 0 0],...  
    'Enable','on',...
    'HorizontalAlignment','right',...
    'BackgroundColor', [.941 .941 .941],...
    'String','');

%TEXT DEFINING ROTATION
handles.text26= uicontrol(handles.panel2_tab7,'Style','text',...
    'Units','pixels',...
    'position',[450 40 280 20],...
    'FontSize',13,...
    'fontname',char(data.font),...
    'ForegroundColor',[.2 .2 .2],...
    'HorizontalAlignment','left',...
    'FontWeight','normal',...
    'BackgroundColor', [.941 .941 .941],...
    'String','Rotation:');

rotacio{1}='Orthogonal';
rotacio{2}='Oblique';

%Popup FOR DEFINING ROTATION METHOD
handles.tipus_rotacio= uicontrol(handles.panel2_tab7,'Style','popupmenu',...
    'Units','pixels',...
    'position',[540 45 180 20],...
    'FontSize',13,...
    'fontname',char(data.font),...
    'Value',2,...
    'BackgroundColor',[1 1 1],...
    'ForegroundColor',[0 0 0],...  
    'Enable','on',...
    'HorizontalAlignment','right',...
    'BackgroundColor', [.941 .941 .941],...
    'String',rotacio);

%CHECKBOX FACTOR SCORES FOR EACH PARTICIPANT
handles.full_analysis= uicontrol(handles.panel2_tab7,'Style','checkbox',...
    'Units','pixels',...
    'position',[10 45 425 20],...
    'FontSize',13,...
    'fontname',char(data.font),...
    'ForegroundColor',[.2 .2 .2],...
    'HorizontalAlignment','left',...
    'enable','off',...
    'FontWeight','normal',...
    'BackgroundColor', [.941 .941 .941],...
    'String','Compute all possible bias analysis combinations');

%CHECKBOX FACTOR SCORES FOR EACH PARTICIPANT
handles.pfactors= uicontrol(handles.panel2_tab7,'Style','checkbox',...
    'Units','pixels',...
    'position',[10 11 250 20],...
    'FontSize',13,...
    'fontname',char(data.font),...
    'ForegroundColor',[.2 .2 .2],...
    'HorizontalAlignment','left',...
    'FontWeight','normal',...
    'BackgroundColor', [.941 .941 .941],...
    'String','Compute factor scores');

tipus{1}='Lineal';
tipus{2}='Graded';

%Popup FOR DEFINING FACTOR SCORES CALCULATION METHOD
handles.tipus_pfactors= uicontrol(handles.panel2_tab7,'Style','popupmenu',...
    'Units','pixels',...
    'position',[220 18 130 20],...
    'FontSize',13,...
    'fontname',char(data.font),...
    'Value',1,...
    'BackgroundColor',[1 1 1],...
    'ForegroundColor',[0 0 0],...  
    'Enable','on',...
    'Visible','off',...
    'HorizontalAlignment','right',...
    'BackgroundColor', [.941 .941 .941],...
    'String',tipus);

%LABEL OF OUTPUT SECTION
handles.text27= uicontrol(handles.panel_tab7,'Style','text',...
    'Units','pixels',...
    'position',[10 305 400 20],...
    'fontname',char(data.font),...
    'FontSize',13,...
    'FontWeight','bold',...
    'ForegroundColor',[.2 .2 .2],...
    'HorizontalAlignment','left',...
    'BackgroundColor', [.941 .941 .941],...
    'String','Social Desirability and Acquiescence: Output');

% - SIZING BUTTON
handles.ModelOutputDecrementa2= uicontrol(handles.panel_tab7,'string','-',...
    'Style','pushbutton',...
    'Units','pixels',...
    'position',[675 305 30 30],...
    'FontSize',13,...
    'Enable','off',...
    'fontname',char(data.font),...
    'TooltipString','Zoom out',...
    'BackgroundColor', [.831 .816 .784],...
    'ForegroundColor',[.2 .4 .6],...  
    'HorizontalAlignment','center',...
    'FontWeight','normal');

% + SIZING BUTTON
handles.ModelOutputIncrementa2= uicontrol(handles.panel_tab7,'string','+',...
    'Style','pushbutton',...
    'Units','pixels',...
    'position',[714 305 30 30],...
    'FontSize',13,...
    'Enable','off',...
    'fontname',char(data.font),...
    'TooltipString','Zoom out',...
    'BackgroundColor', [.831 .816 .784],...
    'ForegroundColor',[.2 .4 .6],...  
    'HorizontalAlignment','center',...
    'FontWeight','normal');

% OUTPUT LISTBOXS CONTAINING THE OUTPUT OF THE ENTIRE ANALYSIS, IN THE
% THREE FORMATS AVAILABLE (text, RTF, LaTeX)
tmp={};
global outputletter2;
outputletter2 = 12;
handles.ModelOutput2= uicontrol(handles.panel_tab7,'Style','listbox',...
    'Units','pixels',...
    'position',[10 60 735 240],...
    'FontSize',outputletter2,...
    'fontname','Monospaced',...
    'visible','on',...
    'BackgroundColor',[1 1 1],...
    'ForegroundColor',[0 0 0],...  
    'HorizontalAlignment','left',...
    'min',0,'max',0,...
    'FontWeight','normal',...
    'String',tmp);

handles.ModelOutput2_latex= uicontrol(handles.panel_tab7,'Style','listbox',...
    'Units','pixels',...
    'position',[10 60 735 240],...
    'FontSize',outputletter2,...
    'fontname','Monospaced',...
    'BackgroundColor',[1 1 1],...
    'visible','off',...      
    'ForegroundColor',[0 0 0],...  
    'HorizontalAlignment','left',...
    'min',0,'max',0,...
    'FontWeight','normal',...
    'String',tmp);

handles.ModelOutput2_rtf= uicontrol(handles.panel_tab7,'Style','listbox',...
    'Units','pixels',...
    'position',[10 60 735 240],...
    'FontSize',outputletter2,...
    'fontname','Monospaced',...
    'BackgroundColor',[1 1 1],...
    'visible','off',...      
    'ForegroundColor',[0 0 0],...  
    'HorizontalAlignment','left',...
    'min',0,'max',0,...
    'FontWeight','normal',...
    'String',tmp);

% INVISIBLE OUTPUT LISTBOX CONTAINING THE PRINCIPAL MATRICES
handles.Output_matrices= uicontrol(handles.panel_tab7,'Style','listbox',...
    'Units','pixels',...
    'position',[10 60 735 240],...
    'FontSize',outputletter2,...
    'fontname','Monospaced',...
    'BackgroundColor',[1 1 1],...
    'visible','off',...      
    'ForegroundColor',[0 0 0],...  
    'HorizontalAlignment','left',...
    'min',0,'max',0,...
    'FontWeight','normal',...
    'String',tmp);

%Radio button save as text
handles.save_text2= uicontrol(handles.panel_tab7,'Style','radiobutton',...
    'Units','pixels',...
    'position',[15 35 120 20],...
    'FontSize',13,...
    'fontname',char(data.font),...
    'Value',1,...
    'BackgroundColor',[1 1 1],...
    'ForegroundColor',[0 0 0],...  
    'Enable','off',...
    'HorizontalAlignment','right',...
    'BackgroundColor', [.941 .941 .941],...
    'String','Text');

%Radio button save as text
handles.save_rtf2= uicontrol(handles.panel_tab7,'Style','radiobutton',...
    'Units','pixels',...
    'position',[90 35 120 20],...
    'FontSize',13,...
    'fontname',char(data.font),...
    'Value',0,...
    'BackgroundColor',[1 1 1],...
    'ForegroundColor',[0 0 0],...  
    'Enable','off',...
    'HorizontalAlignment','right',...
    'BackgroundColor', [.941 .941 .941],...
    'String','RTF');
 
%Radio button save as latex
handles.save_latex2= uicontrol(handles.panel_tab7,'Style','radiobutton',...
    'Units','pixels',...
    'position',[150 35 120 20],...
    'FontSize',13,...
    'fontname',char(data.font),...
    'Value',0,...
    'BackgroundColor',[1 1 1],...
    'ForegroundColor',[0 0 0],...  
    'Enable','off',...
    'HorizontalAlignment','right',...
    'BackgroundColor', [.941 .941 .941],...
    'String','Latex');

%BUTTON FOR SAVING OUTPUT 2
handles.ButtonSalvarOutput2 = uicontrol(handles.panel_tab7,'String',' Save output',...
    'Style','pushbutton',...
    'FontSize',15,...
    'FontWeight','bold',...
    'fontname',char(data.font),...   
    'position',[15 10 200 25],...
    'enable','off',...
    'TooltipString','To save the output of the fitted model as a text file',...
    'BackgroundColor', [.831 .816 .784],...
    'ForegroundColor',[.2 .4 .6]);

%BUTTON SAVE FACTOR SCORES FOR EACH PARTICIPANT
handles.save_pfactors= uicontrol(handles.panel_tab7,'Style','pushbutton',...
    'Enable','on',...
    'FontSize',15,...
    'FontWeight','bold',...
    'fontname',char(data.font),...   
    'position',[230 10 160 40],...
    'enable','off',...
    'TooltipString','To save factor scores in a .dat file',...
    'BackgroundColor', [.831 .816 .784],...
    'ForegroundColor',[.2 .4 .6],...
    'String','Save scores');

%BUTTON LAUNCH OUTPUT
handles.ButtonLaunchOutput2 = uicontrol(handles.panel_tab7,'String',' Visualize',...
    'Style','pushbutton',...
    'FontSize',15,...
    'FontWeight','bold',...
    'fontname',char(data.font),...   
    'position',[410 10 160 40],...
    'enable','off',...
    'TooltipString','To launch the output of the fitted model in a new window',...
    'BackgroundColor', [.831 .816 .784],...
    'ForegroundColor',[.2 .4 .6]);

%BUTTON COMPUTE FACTOR ANALYSIS
handles.Compute_AQiDS = uicontrol(handles.panel_tab7,...
    'String','    Compute    ',...
    'Style','pushbutton',...
    'FontSize',15,...
    'FontWeight','bold',...
    'fontname',char(data.font),...  
    'enable','off',...
    'position',[585 10 160 40],...
    'BackgroundColor', [.831 .816 .784],...
    'ForegroundColor',[.2 .4 .6]);


%%%% CALLBACK SECTION %%%%


%MENU

set(handles.MenuFile_OpenData,...
    'Callback',{@Read_Callback, handles});
set(handles.MenuFile_Save,...
    'Callback',{@SaveData_Callback, handles});
set(handles.MenuFile_Open,...
    'Callback',{@OpenData_Callback, handles});
set(handles.MenuFile_Scoring_Utility,...
    'Callback',{@Corrector_Callback, handles});
set(handles.MenuFile_Save_Matrices,...
    'Callback',{@Save_Matrices, handles});
set(handles.MenuFile_ClearData,...
    'Callback',{@ClearData_Callback, handles});
set(handles.MenuFile_Quit,...
    'Callback',{@QuitCallback, handles});

set(handles.MenuHelp_About,...
    'Callback',{@About_Callback, handles});
set(handles.MenuHelp_Help,...
    'Callback',{@HelpCallback, handles});

%%%% FRONT PAGE %%%%
set(handles.slider01,...
    'Callback',{@Slider_Callback,handles});

%%%% TAB 1 %%%%

%READ DATA
set(handles.read,...
    'Callback',{@Read_Callback, handles});
%EXCLUDING ITEMS
set(handles.ButtonExcloureItems,...
    'Callback',{@ButtonExcloureItems_Callback,handles});
%INCLUDING ITEMS
set(handles.ButtonIncloureItems,...
    'Callback',{@ButtonIncloureItems_Callback,handles});
%VISUALIZE LABELS
set(handles.Visualize_Labels,...
    'Callback',{@Visualize_Labels_Callback,handles});

%%%% TAB 2 %%%%

%RadioButtons FOR SELECTING DISPERSION MATRIX
set(handles.Matrix_Covariance,...
    'Callback',{@Matrix_Callback, handles,1});
set(handles.Matrix_Pearson,...
    'Callback',{@Matrix_Callback, handles,2});
set(handles.Matrix_Polychoric,...
    'Callback',{@Matrix_Callback, handles,3});
% - SIZING
set(handles.ModelOutputDecrementa,...
    'Callback',{@ModelOutputDecrementa_Callback, handles, 1});
% + SIZING
set(handles.ModelOutputIncrementa,...
    'Callback',{@ModelOutputDecrementa_Callback, handles, 2});

%PA SELECTED
set(handles.parallel_analysis,...
    'Callback',{@Parallel_analysis_Callback,handles});
%COMPUTE
set(handles.Compute_Descriptives,...
    'Callback',{@Compute_Descriptives_Callback, handles});

%save in text
set(handles.save_text,...
    'Callback',{@Savetype_Callback, handles,1});
%save in rtf
set(handles.save_rtf,...
    'Callback',{@Savetype_Callback, handles,2});
%save in latex
set(handles.save_latex,...
    'Callback',{@Savetype_Callback, handles,3});
%SAVING OUTPUT
set(handles.ButtonSalvarOutput,...
    'Callback',{@ButtonSalvarOutput_Callback,handles});
%LAUNCH OUTPUT
set(handles.ButtonLaunchOutput,...
    'Callback',{@ButtonLaunchOutput_Callback, handles});

%%%% TAB 3 %%%%

%SD CONTROL ENABLE
set(handles.DS_checkbox,...
    'Callback',{@DS_on_Callback, handles});
%SWITCH ITEMS TO SD ITEMS
set(handles.ButtonExcloureItems2,...
    'Callback',{@ButtonExcloureItems2_Callback, handles});
%SWITCH ITEMS TO CONTENT ITEMS
set(handles.ButtonIncloureItems2,...
    'Callback',{@ButtonIncloureItems2_Callback, handles});
%VISUALIZE LABELS
set(handles.Visualize_Labels2,...
    'Callback',{@Visualize_Labels_Callback2,handles});

%%%% TAB 4 %%%%

%ACQUIESCENCE CONTROL ENABLE
set(handles.AQ_checkbox,...
    'Callback',{@AQ_on_Callback, handles});
%SWITCH ITEMS TO UNBALANCED
set(handles.ButtonExcloureItems3,...
    'Callback',{@ButtonExcloureItems3_Callback, handles});
%SWITCH ITEMS TO BALANCED
set(handles.ButtonIncloureItems3,...
    'Callback',{@ButtonIncloureItems3_Callback, handles});
%VISUALIZE LABELS
set(handles.Visualize_Labels3,...
    'Callback',{@Visualize_Labels_Callback3,handles});

%%%% TAB 5 %%%%

%FACTOR SCORES FOR EACH PARTICIPANT
set(handles.pfactors,...
    'Callback',{@pfactorials, handles});
%COMPUTE ALL POSSIBLE ANALYSIS COMBINATIONS
set(handles.full_analysis,...
    'Callback',{@full_analysis_Callback, handles});
%popupmenu EAP METHOD
set(handles.tipus_pfactors,...
    'Callback',{@pfactors_method, handles});
%popupmenu FACTOR ROTATION
set(handles.tipus_rotacio,...
    'Callback',{@factor_rotation, handles});

%Zoom out
set(handles.ModelOutputDecrementa2,...
    'Callback',{@ModelOutputDecrementa_Callback2, handles, 1});
%Zoom in
set(handles.ModelOutputIncrementa2,...
    'Callback',{@ModelOutputDecrementa_Callback2, handles, 2});

%COMPUTE FACTOR ANALYSIS
set(handles.Compute_AQiDS,...
    'Callback',{@Compute_AQiDS_Callback, handles});
%save in text
set(handles.save_text2,...
    'Callback',{@Savetype2_Callback, handles,1});
%save in rtf
set(handles.save_rtf2,...
    'Callback',{@Savetype2_Callback, handles,2});
%save in latex
set(handles.save_latex2,...
    'Callback',{@Savetype2_Callback, handles,3});

%SAVING OUTPUT 2
set(handles.ButtonSalvarOutput2,...
    'Callback',{@ButtonSalvarOutput_Callback2, handles});
%SAVING FACTOR SCORES FOR EACH PARTICIPANT
set(handles.save_pfactors,...
    'Callback',{@ButtonSalvarpfactors_Callback, handles});
%LAUNCH OUTPUT
set(handles.ButtonLaunchOutput2,...
    'Callback',{@ButtonLaunchOutput_Callback2, handles});



return;

function Slider_Callback(hObject, eventdata,handles)
%% Slider of the main page

    pos=get(handles.slider01,'value');
    pos2=25-pos;
    logo=getappdata(0,'logo');
    logo.xv = pos2*2;
    logo.yv = pos2;
    setappdata(0,'logo',logo);
    axes(logo.axes);
    view(logo.xv,logo.yv);

return;

function About_Callback (hObject,eventdata,handles,gcf)
%% About button

text{1}='Psychological Test Toolbox (version 1.11) ';
text{2}=' ';
text{3}='David Navarro-Gonzalez, Andreu Vigil-Colet, ';
text{4}='Pere J. Ferrando & Urbano Lorenzo-Seva' ;
text{5}=' ';
text{6}='(Rovira i Virgili University, Tarragona, Spain) ';
text{7}='                            2019                 ';

msgbox(text,'About Psychological Test Toolbox');

return;

function HelpCallback (hObject,eventdata,handles,gcf)
%% Help button

web('http://psico.fcep.urv.cat/utilitats/PsychologicalTestToolbox/index.html','-browser');

return;

function QuitCallback (hObject,eventdata,handles,gcf)
%% Quit button

ButtonName=questdlg('Do you want to quit the application','Psychological Test Toolbox','Yes','No','No'); 
    switch ButtonName
    case {'No','Cancel'}
	    % take no action
    case 'Yes'
        delete(gcbf);
    end 

return;

function Corrector_Callback (hObject,eventdata,handles,gcf)
%% Scoring Utility launch button

    %Launch the Scoring Utility in a new window
    set(handles.fig,'visible','off');
    fig_new=corrector;
    waitfor(fig_new);
    
    %reset data for Psychological Test Toolbox
    data=getappdata(0,'data');
    Forced_ClearData_Callback(hObject, eventdata, handles);
    setappdata(0,'data',data);
    
    set(handles.fig,'visible','on');

return;

function ModelOutputDecrementa_Callback(hObject, eventdata, handles, accion)
%% Change the text size

global outputletter;

switch accion
    case 1
      if outputletter > 7 
         outputletter = outputletter - 1;
         set(handles.ModelOutput,'FontSize',outputletter);
      end;
    case 2
      if outputletter < 20
         outputletter = outputletter + 1;
         set(handles.ModelOutput,'FontSize',outputletter);
      end;
end;
return;

function ModelOutputDecrementa_Callback_File(hObject, eventdata, new_handle, accion)
%% Change the text size

global outputletter;

tamany=get(new_handle,'FontSize');

switch accion
    case 1
      if tamany > 7
         tamany = tamany - 1;
         set(new_handle,'FontSize',tamany);
      end;
    case 2
      if tamany < 20 
         tamany = tamany + 1;
         set(new_handle,'FontSize',tamany);
      end;
end;
return;

function ModelOutputDecrementa_Callback2(hObject, eventdata, handles, accion)
%% Change the text size

global outputletter2;

switch accion
    case 1
      if outputletter2 > 7 
         outputletter2 = outputletter2 - 1;
         set(handles.ModelOutput2,'FontSize',outputletter2);
      end;
    case 2
      if outputletter2 < 20 
         outputletter2 = outputletter2 + 1;
         set(handles.ModelOutput2,'FontSize',outputletter2);
      end;
end;
return;

function ModelOutputDecrementa_Callback_File2(hObject, eventdata, new_handle, accion)
%% Change the text size

global outputletter;

tamany=get(new_handle,'FontSize');

switch accion
    case 1
      if tamany > 7 
         tamany = tamany - 1;
         set(new_handle,'FontSize',tamany);
      end;
    case 2
      if tamany < 20
         tamany = tamany + 1;
         set(new_handle,'FontSize',tamany);
      end;
end;
return;

function Read_Callback (hObject,eventdata,handles,gcf)

%% Import data %%

    try 
        figread=import_data;
        waitfor(figread);
        
    catch
 		disp(lasterr);
    end
    
    %load data
    data=getappdata(0,'data');
    data.descriptives_done=0;
    data.forced_descriptives=0;
    data.P_done=0;
    data.th_done=0;

    % Check what kind of data has been selected by the user

    empty1=isempty(data.S); % checking if data.S is empty
    empty2=isempty(data.X); % checking if data.X is empty

    if empty1==1 && empty2==1 % if both options are empty (because the user
        %has cancelled the loading
        
        return;

    elseif empty1==1 % if data.S is empty, the user has selected raw scores
        [f1,f2] = size(data.X);
        set(handles.numberofparticipants,'string',data.N);
        set(handles.numberofvariables,'string',data.n);

        buffs={};
        set(handles.ItemsExcluded,'string',buffs);
        
        buffs={};
        for i=1:f2
            buff=sprintf('%.0f',i);
            buffs(i)=cellstr(buff);
        end;
        tmp1={};
        set(handles.ItemsIncluded,'string',tmp1);
        set(handles.ItemsIncluded,'value',1);
        set(handles.ItemsIncluded,'string',buffs);
        set(handles.ModelOutput,'value',1);
        
        %Included
        data.inclosos=(1:f2);
        data.exclosos=[];
        data.items_contingut=data.inclosos;
        data.itemsa=data.inclosos;
        data.matrix_user_on=0;

    elseif empty2==1; % if data.X is empty, the user has selected correlation matrix
        [f1,f2] = size(data.S);
        set(handles.numberofparticipants,'string',data.N);
        set(handles.numberofvariables,'string',data.n);

        buffs={};
        set(handles.ItemsExcluded,'string',buffs);
        
     
        buffs={};
        for i=1:f2
            buff=sprintf('%.0f',i);
            buffs(i)=cellstr(buff);
        end;
        tmp1={};
        set(handles.ItemsIncluded,'string',tmp1);
        set(handles.ItemsIncluded,'value',1);
        set(handles.ItemsIncluded,'string',buffs); 
        set(handles.ModelOutput,'value',1);
        
        %Included
        data.inclosos=(1:f2);
        data.exclosos=[];
        data.items_contingut=data.inclosos;
        data.itemsa=data.inclosos;
        data.matrix_user_on=1;
  
    end;

    % Checking if the user has uploaded custom labels. If not, generic
    % labels will be generated
    
    if data.labels_user_on==0;
        
        for i=1:data.n
            buff = sprintf('I%3.0f',i);
             buffs(i)=cellstr(buff);
        end;
        data.labels=buffs; % vector [1xn]
        data.labelsa=data.labels;
        set(handles.Visualize_Labels,'enable','off','value',0,'visible','off');
        set(handles.Visualize_Labels2,'enable','off','visible','off');
        set(handles.Visualize_Labels3,'enable','off','visible','off');
    else
        data.labelsa=data.labels;
        set(handles.Visualize_Labels,'enable','on','value',0,'visible','on');
        set(handles.Visualize_Labels2,'enable','on');
        set(handles.Visualize_Labels3,'enable','on');
    end;
    
    if data.missings_on==1
        X_miss = data.X == data.missings_value ;
        
        max1=max(X_miss);
        max2=max(max1);
        
        % Just in case
        if max2==0;
            warndlg('The data file does not contain any specified missing value. Data will be processed like if there were no missing values.','Psychological Test Toolbox: Warning');
            data.missings_on=0;
        end;
    end;

    % All items've been selected
    if data.matrix_user_on==0
        data.Xa=data.X;
        data.X_a=data.X;
        data.S=[];
        data.Sa=[];
        minim=min(data.Xa);
        minim2=min(minim);
        maxim=max(data.Xa);
        maxim2=max(maxim);
        if data.missings_on==1
            if data.missings_value<0
                A=unique(data.Xa);
                minim=A(2:end);
                minim2=min(minim);
            else
                A=unique(data.Xa);
                maxim=A([2,end-1]);
                maxim2=max(maxim);
            end;
        end;
        set(handles.lowest_score,'String',minim2);  
        set(handles.highest_score,'String',maxim2); 
        set(handles.Matrix_Polychoric,'enable','on',...
            'value',0);
        set(handles.Matrix_Pearson,'enable','on',...
            'value',1);
        set(handles.Matrix_Covariance,'enable','on',...
            'value',0);
        set(handles.text5,'ForegroundColor',[.2 .2 .2]);
        set(handles.text6,'ForegroundColor',[.2 .2 .2]);
        data.Matrix=2;
        set(handles.pfactors,'value',0,...
            'visible','on');
        data.pfactorial=0;
        set(handles.save_pfactors,'visible','off',...
            'enable','off');
        
    elseif data.matrix_user_on==1
        data.Sa=data.S;
        data.X=[];
        data.Xa=[];
        data.X_a=[];
        set(handles.lowest_score,'enable','off');
        set(handles.highest_score,'enable','off');
        set(handles.Matrix_Polychoric,'enable','off',...
            'value',0);
        set(handles.Matrix_Pearson,'enable','off',...
            'value',0);
        set(handles.Matrix_Covariance,'enable','off',...
            'value',0);
        set(handles.text5,'ForegroundColor',[.4 .4 .4]);
        set(handles.text6,'ForegroundColor',[.4 .4 .4]);
        set(handles.highest_score,'string','');
        set(handles.lowest_score,'string','');
        set(handles.pfactors,'value',0,...
            'visible','off');
        data.pfactorial=0;
        set(handles.save_pfactors,'visible','off',...
            'enable','off');
    end;
    
    empty1=isempty(data.H);
    if empty1 == 0
        data.Hc=data.H;
        rotacio{1}='Orthogonal';
        rotacio{2}='Oblique';
        rotacio{3}='Target oblique';
        rotacio{4}='Target orthogonal';

        %Rotatation pop up
        set(handles.tipus_rotacio,'string',rotacio,...
            'value',3);
        data.Rotation=3;
        
        %number of content factors
        [~,g2]=size(data.H);
        set(handles.content_factors,'string',num2str(g2));
    else
        rotacio{1}='Orthogonal';
        rotacio{2}='Oblique';
        %Popup rotatio
        set(handles.tipus_rotacio,'string',rotacio,...
            'value',2);
        data.Rotation=2;
    end;
    set(handles.Compute_Descriptives,'enable','on')
    set(handles.Compute_AQiDS,'enable','on');
    
    inclosos_en_ordre=sort(data.inclosos);
    [f1,f2]=size(inclosos_en_ordre);
    if f2>=16
        for i=1:f2
            buff1{i}=sprintf('%.0f, ',inclosos_en_ordre(i));
        end;
        ratio1=ceil(f2/15);
        
        for l=1:ratio1
            cof=15*(l-1);
            if cof<f2 && cof+15<f2
                buff2{l}=inclosos_en_ordre(1+cof:15+cof);                
            else
                buff2{l}=inclosos_en_ordre(1+cof:f2);
            end;
            
            buff4{l}=sprintf('%.0f, ',buff2{l});
            if l==ratio1
                [g1,g2]=size(buff4{l});
                buff4{l}=buff4{l}(1:g2-2);
            end;
        end;
                
    else
        for i=1:f2
        buff1{i}=sprintf('%.0f, ',inclosos_en_ordre(i));
        end;
        a=sprintf('%s',buff1{:});
        [f1,f2]=size(a);
        buff4=a(1:(f2-2));
    end;

     if data.matrix_user_on==0
         [f1,f2]=size(data.X);
         data.N_original=f1;
     else
         data.N_original=data.N;
     end;
     
    set(handles.content_list,'string',buff4);
    
    [f1,f2]=size(data.inclosos);
    data.G=[0 f2];
    set(handles.panel_tab4,'visible','on');
    set(handles.panel2_tab4,'visible','on');
    set(handles.panel3_tab4,'visible','on');
    set(handles.panel_tab7,'visible','on');
    set(handles.panel2_tab7,'visible','on');
    set(handles.DS_checkbox,'value',0,...
            'enable','on');
    set(handles.AQ_checkbox,'value',0,...
            'enable','on');
    tmp1={};
    
    %Descriptives tab
    set(handles.ModelOutput,'string',tmp1);
    set(handles.ModelOutput,'value',0);
    set(handles.ModelOutput_latex,'string',tmp1);
    set(handles.ModelOutput_latex,'value',0);
    set(handles.ButtonSalvarOutput,'enable','off');
    set(handles.ButtonLaunchOutput,'enable','off');
    set(handles.save_text,'value',1,...
        'enable','off');
    set(handles.save_latex,'value',0,...
        'enable','off');
    set(handles.save_rtf,'value',0,...
        'enable','off');
    set(handles.parallel_analysis,'value',0);
    set(handles.grafic_PA,'Visible','off',...
        'value',0);
     set(handles.text5,'visible','off');
     set(handles.lowest_score,'visible','off');
     set(handles.text6,'visible','off');
     set(handles.highest_score,'visible','off');
     set(handles.ModelOutputIncrementa,'enable','off');
     set(handles.ModelOutputDecrementa,'enable','off');
    
    %Social Desirability
    
    set(handles.panel3_tab5,'visible','off');
    set(handles.panel4_tab5,'visible','off');
    set(handles.numberofexcludeditems,'string',tmp1);
    set(handles.numberofsditems,'string',tmp1);
    set(handles.numberofcontentitems,'string',tmp1);
    set(handles.ContentItems,'string',tmp1);
    set(handles.SDItems,'string',tmp1);
    set(handles.ButtonIncloureItems,'enable','off');
    set(handles.Visualize_Labels2,'visible','off','value',0);

    %Acquiescence
    
    set(handles.panel3_tab6,'visible','off');
    set(handles.panel4_tab6,'visible','off');
    set(handles.numberofexcludeditems2,'string',tmp1);
    set(handles.numberofbalanceditems,'string',tmp1);
    set(handles.numberofunbalanceditems,'string',tmp1);
    set(handles.BalItems,'string',tmp1);
    set(handles.UnbalItems,'string',tmp1);
    set(handles.ButtonIncloureItems2,'enable','off');
    set(handles.content_factors,'backgroundcolor',[1 1 1]);
    set(handles.Visualize_Labels3,'visible','off','value',0);
    tmp={};
    set(handles.text_balanced_core,'visible','off','string',tmp);
    
    %SD & AC
    
    set(handles.pfactors,'value',0);
    set(handles.save_text2,'value',1,...
        'enable','off');
    set(handles.save_latex2,'value',0,...
        'enable','off');
    set(handles.save_rtf2,'value',0,...
        'enable','off');
    set(handles.ButtonSalvarOutput2,'enable','off');
    set(handles.save_pfactors,'enable','off');
    set(handles.ButtonLaunchOutput2,'enable','off');
    if empty1==0
    else
        set(handles.content_factors,'string',tmp1);
    end;
    set(handles.ModelOutput2,'string',tmp1);
    set(handles.ModelOutput2,'value',0);
    set(handles.ModelOutput2_latex,'string',tmp1);
    set(handles.ModelOutput2_latex,'value',0);
    set(handles.Output_matrices,'string',tmp1);
    set(handles.Output_matrices,'value',0);
    
    set(handles.excluded_list,'string',tmp1);
    set(handles.sd_list,'string',tmp1);
    set(handles.ModelOutputIncrementa2,'enable','off');
    set(handles.ModelOutputDecrementa2,'enable','off');
    set(handles.full_analysis,'value',0,'enable','off');
    data.full_analysis=0;
    set(handles.tipus_pfactors,'Value',1,'Visible','off')
    
    %VIEW DATA PANEL
    set(handles.panel_tab2,'visible','on');
    if data.matrix_user_on==0
        set(handles.data_table,'Data',data.X,'visible','on');
    else
        set(handles.data_table,'Data',data.S,'visible','on');        
    end;
    
    if data.labels_user_on==1
        set(handles.Visualize_Labels,'value',1);
        buff={};
        Visualize_Labels_Callback(hObject,buff,handles);
    end;
    
    %LOAD DATA
    data.items_sd=[];
    data.A=[];
    data.A1=[];
    data.ib=[];
    data.iba=[];
    data.iub=[];
    data.iuba=[];
    data.Hc=[];
    data.R=[];
    data.Rp=[];
    data.Rp_all=[];
    data.thresholds=[];
    data.labels_contingut=[];
    data.PHI=[];
    data.P=[];
    data.IRT_A=[];
    data.Ygrad=[];
    data.th=[];
    data.Lc=[];
    data.Lz=[];
    data.Xpredicted=[];
    data.WLSRegression=[];
    data.ref1=0;
    data.H_cids=[];
    data.PHIr=[];
    data.Pc=[];
    data.graded=0;
    data.n_nodes=7;
    
    data.corr_error=0;
    data.items_corr_error=[];

    data.Bandera=1;

    %%%% Output structure %%%%
    Printer=getappdata(0,'Printer');
    Printer.text=[];
    Printer.latex=[];
    Printer.rtf=[];

    Printer.text2=[];
    Printer.latex2=[];
    Printer.rtf2=[];


    %SAVE DATA 
    setappdata(0,'Printer',Printer);
    setappdata(0,'data',data);
    
    set(handles.hTabGroup,'Selectedtab',handles.tab(2))
return;

function Visualize_Labels_Callback(hObject, eventdata, handles)

%% Display labels %%

    data=getappdata(0,'data');
    
    incl=get(handles.ItemsIncluded,'string');
    excl=get(handles.ItemsExcluded,'string');
    buffs={};
    buffs2={};
    
    checking=get(handles.Visualize_Labels,'value');
    
    if checking==0
        
        %included
        [f1,f2]=size(incl);
        for i=1:f1
            str=char(incl(i));
            [g1,g2]=size(data.labels);
            for j=i:g2
                check_pos=strcmp(str,data.labels(j));
                if check_pos==1
                    number=j;
                    break;
                end;
            end;
            buff=sprintf('%.0f',number);
            buffs(i)=cellstr(buff);
        end;

        %excluded
        [f1,f2]=size(excl);
        for i=1:f1
            str=char(excl(i));
            [g1,g2]=size(data.labels);
            for j=i:g2
                check_pos=strcmp(str,data.labels(j));
                if check_pos==1
                    number=j;
                    break;
                end;
            end;
            buff2=sprintf('%.0f',number);
            buffs2(i)=cellstr(buff2);
        end;

    else
        
        %included
        [f1,f2]=size(incl);
        for i=1:f1
            number=str2num(char(incl(i)));
            buff=data.labels(number);
            buffs(i)=cellstr(buff);
        end;

        %excluded
        [f1,f2]=size(excl);
        for i=1:f1
            number=str2num(char(excl(i)));
            buff2=data.labels(number);
            buffs2(i)=cellstr(buff2);
        end;

    end;
    
    tmp1={};
    set(handles.ItemsIncluded,'string',tmp1);
    set(handles.ItemsIncluded,'value',1);
    set(handles.ItemsIncluded,'string',buffs); 

    set(handles.ItemsExcluded,'string',tmp1);
    set(handles.ItemsExcluded,'value',1);
    set(handles.ItemsExcluded,'string',buffs2); 

    set(handles.ModelOutput,'value',1);
    

return;

function ButtonExcloureItems_Callback(hObject, eventdata, handles)

%% Excluding items %%

    checking_DS=get(handles.DS_checkbox,'value');
    checking_AQ=get(handles.AQ_checkbox,'value');
    
    %LOAD DATA
        data=getappdata(0,'data');
    
    if checking_DS==1 || checking_AQ==1
        warndlg('SD and/or AC control function are enabled. For excluding or including items, these options must have been disabled previously.','Psychological Test Toolbox: Warning');
        return;
    end;
    
    %Check if the labels are visualized
    check_labels=get(handles.Visualize_Labels,'value');
    
    %Included
    ItemsIncluded=get(handles.ItemsIncluded,'string');
    [n_items,tmp] = size(ItemsIncluded);
    %Excluded
    ItemsExcluded=cellstr(get(handles.ItemsExcluded,'string'));
    [h_n,tmp] = size(ItemsExcluded);

    %Selected items in the listbox
    Seleccio = get(handles.ItemsIncluded,'value');
    [tmp,num_selected] = size(Seleccio);

    %Excluding unselected items
    nosel ={};h=1; nosel_i=[];
    if check_labels==0
        for i=1:n_items
            campana = 0;
            for j=1:num_selected
                if str2num(char(ItemsIncluded(i))) == str2num(char(ItemsIncluded(Seleccio(j))))
                    campana = 1;
                end;
            end;
            if campana == 0
                nosel(h) = ItemsIncluded(i);
                nosel_i(h)  = str2num(char(ItemsIncluded(i)));
                h=h+1;
            else
                h_n=h_n+1;
                ItemsExcluded(h_n) = ItemsIncluded(i);
            end;
        end;
    else
        for i=1:n_items
            campana = 0;
            for j=1:num_selected
                if strcmp(char(ItemsIncluded(i)),(char(ItemsIncluded(Seleccio(j)))))==1
                    campana = 1;
                end;
            end;
            if campana == 0
                nosel(h) = ItemsIncluded(i);
                [g1,g2]=size(data.labels);
                for j=i:g2
                    check_pos=strcmp(nosel(h),data.labels(j));
                    if check_pos==1
                        number=j;
                        break;
                    end;
                end;
                nosel_i(h)  = number;
                h=h+1;
            else
                h_n=h_n+1;
                ItemsExcluded(h_n) = ItemsIncluded(i);
            end;
        end;
    end;
    if (h<4)
        msgbox('At least three items are needed!','Psychological Test Toolbox: Error','error');
        return;
    end;

    set(handles.ItemsIncluded,'value',1);
    if h_n > 0
        set(handles.ItemsExcluded,'value',1);
        set(handles.ButtonIncloureItems,'enable','on');                
    else
        set(handles.ItemsExcluded,'value',0);
        set(handles.ButtonIncloureItems,'enable','off');                
    end;

    set(handles.ItemsIncluded,'string',nosel);
    if check_labels==1
        ItemsExcluded2={};
        [g1,g2]=size(data.labels);
        for i=1:h_n
            str=char(ItemsExcluded(i));
            for j=1:g2
                check_pos=strcmp(str,data.labels(j));
                if check_pos==1
                    number=j;
                    break;
                end;
            end;
            ItemsExcluded2(i)=cellstr(num2str((number)));
        end;
        ItemsExcluded2=sort(str2num(char(ItemsExcluded2)));
        set(handles.ItemsExcluded,'string',data.labels(ItemsExcluded2));
    else
        ItemsExcluded2=cellstr(num2str(sort(str2num(char(ItemsExcluded)))));
        ItemsExcluded3=sort(str2num(char(ItemsExcluded)));
        [f1,f2]=size(ItemsExcluded3);
        buffs={};
        for i=1:f1
            buff=num2str(ItemsExcluded3(i));
            buffs(i)=cellstr(buff);
        end;
        set(handles.ItemsExcluded,'string',buffs);
    end;

    empty1=isempty(data.S); 
    empty2=isempty(data.X); 

    if empty1==1 && empty2==1
        return;
    elseif empty1==1
        data.Xa=data.X(:,nosel_i);
        data.labelsa=data.labels(1,nosel_i);

        minim=min(data.Xa);
        minim2=min(minim);
        maxim=max(data.Xa);
        maxim2=max(maxim);
        if data.missings_on==1
            if data.missings_value<0
                A=unique(data.Xa);
                minim=A(2:end);
                minim2=min(minim);
            else
                A=unique(data.Xa);
                maxim=A([2,end-1]);
                maxim2=max(maxim);
            end;
        end;
        set(handles.lowest_score,'String',minim2);
        set(handles.highest_score,'String',maxim2);

    elseif empty2==1
        data.Sa=data.S(nosel_i,nosel_i);
        data.labelsa=data.labels(1,nosel_i);
    end;

    data.inclosos=nosel_i;
    if check_labels==1
        data.exclosos=ItemsExcluded2';
    else
        data.exclosos=rot90(str2num(char(ItemsExcluded2)));
    end;

    [f1,f2]=size(data.inclosos);
    set(handles.numberofvariables,'string',f2);

    data.items_contingut=data.inclosos;
    data.itemsa=data.inclosos;

    inclosos_en_ordre=sort(data.inclosos);
    [f1,f2]=size(inclosos_en_ordre);
    if f2>=16
        for i=1:f2
            buff1{i}=sprintf('%.0f, ',inclosos_en_ordre(i));
        end;
        ratio1=ceil(f2/15);

        for l=1:ratio1
            cof=15*(l-1);
            if cof<f2 && cof+15<f2
                buff2{l}=inclosos_en_ordre(1+cof:15+cof);                
            else
                buff2{l}=inclosos_en_ordre(1+cof:f2);
            end;

            buff4{l}=sprintf('%.0f, ',buff2{l});
            if l==ratio1
                [g1,g2]=size(buff4{l});
                buff4{l}=buff4{l}(1:g2-2);
            end;
        end;

    else
        for i=1:f2
        buff1{i}=sprintf('%.0f, ',inclosos_en_ordre(i));
        end;
        a=sprintf('%s',buff1{:});
        [f1,f2]=size(a);
        buff4=a(1:(f2-2));
    end;

    exclosos_en_ordre=sort(data.exclosos);
    [f1,f2]=size(exclosos_en_ordre);
    for i=1:f2
        buff3{i}=sprintf('%.0f, ',exclosos_en_ordre(i));
    end;

    c=sprintf('%s',buff3{:});
    [f1,f2]=size(c);
    d=c(1:(f2-2));
    set(handles.content_list,'string',buff4);
    set(handles.excluded_list,'string',d);

    data.Bandera=1;
    data.descriptives_done=0;
    %SAVE DATA
    setappdata(0,'data',data);
        
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
 
return;

function ButtonIncloureItems_Callback(hObject, eventdata, handles)
    
%% Including items %%

    checking_DS=get(handles.DS_checkbox,'value');
    checking_AQ=get(handles.AQ_checkbox,'value');
    
    if checking_DS==1 || checking_AQ==1
        warndlg('SD and/or AC control function are enabled. For excluding or including items, these options must have been disabled previously.','Psychological Test Toolbox: Warning');
        return;
    end;
    
    %LOAD DATA
    data=getappdata(0,'data');
    
    %Check if the labels are visualized
    check_labels=get(handles.Visualize_Labels,'value');
    
    %Excluded
    ItemsExcluded=get(handles.ItemsExcluded,'string');
    [h_n,tmp] = size(ItemsExcluded);
    
    %Included
    ItemsIncluded=get(handles.ItemsIncluded,'string');
    
 
    %Selected items in the listbox
    Seleccio = get(handles.ItemsExcluded,'value');
    [tmp,num_selected] = size(Seleccio);
    
    tmp={};hh=1;
    if check_labels==0
        for i=1:h_n
            campana = 0;
            for j=1:num_selected
                if i == Seleccio(j)
                    campana = 1;
                end;
            end;
            if campana==0
               tmp(hh)=ItemsExcluded(i);
               hh=hh+1;
            end;
        end;
    else
        for i=1:h_n
            campana = 0;
            for j=1:num_selected
                if strcmp(char(ItemsExcluded(i)),(char(ItemsExcluded(Seleccio(j)))))==1
                    campana = 1;
                end;
            end;
            if campana == 0
                tmp(hh) = ItemsExcluded(i);
                [g1,g2]=size(data.labels);
                for j=i:g2
                    check_pos=strcmp(tmp(hh),data.labels(j));
                    if check_pos==1
                        number=j;
                        break;
                    end;
                end;
                tmp_i(hh)  = number;
                hh=hh+1;
            end;
        end;
    end;
    
    
    ItemsExcluded = tmp;
    [tmp,h_n] = size(ItemsExcluded);
    
    empty1=isempty(data.S);
    empty2=isempty(data.X);
    
    if empty1==1 && empty2==1
        return;
    elseif empty1==1
        [tmp,n]=size(data.X);
    elseif empty2==1
        [tmp,n]=size(data.S);
    end;
    
    %Remove the excluded items
    nosel ={};h=1; nosel_i=[];
    if check_labels==0
        for i=1:n
            campana = 0;
            for j=1:h_n
                if i == str2num(char(ItemsExcluded(j)))
                    campana = 1;
                end;
            end;
            if campana == 0
                nosel_i(h)  = i;
                h=h+1;
            end;
        end;
    else
        for i=1:n
            campana = 0;
            for j=1:h_n
                if strcmp(char(data.labels(i)),(char(ItemsExcluded(j))))==1
                    campana = 1;
                end;
            end;
            if campana == 0
                nosel(h) = data.labels(i);
                [g1,g2]=size(data.labels);
                for j=i:g2
                    check_pos=strcmp(nosel(h),data.labels(j));
                    if check_pos==1
                        number=j;
                        break;
                    end;
                end;
                nosel_i(h)  = number;
                h=h+1;
            end;
        end;
    end;
    
    set(handles.ItemsIncluded,'value',1);
    if h_n > 0
        set(handles.ItemsExcluded,'value',1);
        set(handles.ButtonIncloureItems,'enable','on');                
    else
        set(handles.ItemsExcluded,'value',0);
        set(handles.ButtonIncloureItems,'enable','off');                
    end;
    if check_labels==0
        buffs={};
        for i=1:(h-1)
            buff=sprintf('%.0f',nosel_i(i));
            buffs(i)=cellstr(buff);
        end;

        set(handles.ItemsIncluded,'string',buffs);
    else
        set(handles.ItemsIncluded,'string',nosel);
    end;
    
    set(handles.ItemsExcluded,'string',ItemsExcluded);
    

    if empty1==1
        data.Xa=data.X(:,nosel_i);
        data.labelsa=data.labels(1,nosel_i);
    elseif empty2==1
        data.Sa=data.S(nosel_i,nosel_i);
        data.labelsa=data.labels(1,nosel_i);
    end;
    
    data.inclosos=nosel_i;
    if check_labels==1
        ItemsExcluded2={};
        [g1,g2]=size(data.labels);
        for i=1:h_n
            str=char(ItemsExcluded(i));
            for j=1:g2
                check_pos=strcmp(str,data.labels(j));
                if check_pos==1
                    number=j;
                    break;
                end;
            end;
            ItemsExcluded2(i)=cellstr(num2str((number)));
        end;
        ItemsExcluded2=sort(str2num(char(ItemsExcluded2)));
        data.exclosos=ItemsExcluded2';
    else
        data.exclosos=rot90(str2num(char(ItemsExcluded)));
    end;
    
    minim=min(data.Xa);
    minim2=min(minim);
    maxim=max(data.Xa);
    maxim2=max(maxim);
    if data.missings_on==1
        if data.missings_value<0
            A=unique(data.Xa);
            minim=A(2:end);
            minim2=min(minim);
        else
            A=unique(data.Xa);
            maxim=A([2,end-1]);
            maxim2=max(maxim);
        end;
    end;
    set(handles.lowest_score,'String',minim2);
    set(handles.highest_score,'String',maxim2);
    
    [f1,f2]=size(data.inclosos);
    set(handles.numberofvariables,'string',f2);
    
    data.items_contingut=data.inclosos;
    data.itemsa=data.inclosos;
    
    inclosos_en_ordre=sort(data.inclosos);
    [f1,f2]=size(inclosos_en_ordre);
    if f2>=16
        for i=1:f2
            buff1{i}=sprintf('%.0f, ',inclosos_en_ordre(i));
        end;
        ratio1=ceil(f2/15);
        
        for l=1:ratio1
            cof=15*(l-1);
            if cof<f2 && cof+15<f2
                buff2{l}=inclosos_en_ordre(1+cof:15+cof);                
            else
                buff2{l}=inclosos_en_ordre(1+cof:f2);
            end;
            
            buff4{l}=sprintf('%.0f, ',buff2{l});
            if l==ratio1
                [g1,g2]=size(buff4{l});
                buff4{l}=buff4{l}(1:g2-2);
            end;
        end;
                
    else
        for i=1:f2
        buff1{i}=sprintf('%.0f, ',inclosos_en_ordre(i));
        end;
        a=sprintf('%s',buff1{:});
        [f1,f2]=size(a);
        buff4=a(1:(f2-2));
    end;
    
    exclosos_en_ordre=sort(data.exclosos);
    [f1,f2]=size(exclosos_en_ordre);
    for i=1:f2
        buff3{i}=sprintf('%.0f, ',exclosos_en_ordre(i));
    end;
    if isempty(exclosos_en_ordre)
        buff3={};
    end;
    
    
    c=sprintf('%s',buff3{:});
    [f1,f2]=size(c);
    d=c(1:(f2-2));
    set(handles.content_list,'string',buff4);
    set(handles.excluded_list,'string',d);
    
    data.Bandera=1;
    data.descriptives_done=0;
    %SAVE DATA 
    setappdata(0,'data',data);
    
return;

function Matrix_Callback(gcf, eventdata, handles , radio_value)

%% Switch dispersion matrix %%

    %LOAD DATA
    data=getappdata(0,'data');

    switch radio_value       
        case 1 % Covariance
            set(handles.Matrix_Covariance,'value',1);
            set(handles.Matrix_Pearson,'value',0);
            set(handles.Matrix_Polychoric,'value',0);
            data.Matrix=1;
            set(handles.text5,'visible','off');
            set(handles.lowest_score,'visible','off');
            set(handles.text6,'visible','off');
            set(handles.highest_score,'visible','off');
            
            set(handles.tipus_pfactors,'visible','off',...
                'enable','off',...
                'value',1)
            data.graded=0;
            
        case 2 % Pearson
            set(handles.Matrix_Covariance,'value',0);
            set(handles.Matrix_Pearson,'value',1);
            set(handles.Matrix_Polychoric,'value',0);
            data.Matrix=2;
            set(handles.text5,'visible','off');
            set(handles.lowest_score,'visible','off');
            set(handles.text6,'visible','off');
            set(handles.highest_score,'visible','off');
            
            set(handles.tipus_pfactors,'visible','off',...
                'enable','off',...
                'value',1);
            data.graded=0;

        case 3 % Polychoric
            set(handles.Matrix_Covariance,'value',0);
            set(handles.Matrix_Pearson,'value',0);
            set(handles.Matrix_Polychoric,'value',1);
            data.Matrix=3;
            set(handles.text5,'visible','on');
            set(handles.lowest_score,'visible','on');
            set(handles.text6,'visible','on');
            set(handles.highest_score,'visible','on');
            
            if (get(handles.pfactors,'value'))==1
                set(handles.tipus_pfactors,'visible','on',...
                'enable','on');
            else
                set(handles.tipus_pfactors,'visible','off',...
                'enable','on');
            end

    end
    
    data.descriptives_done=0;

    %SAVE DATA 
    setappdata(0,'data',data);

return;

function Savetype_Callback(gcf, eventdata, handles, datatype_value)

%% Switch between types of saving formats %%

    data=getappdata(0,'data');

    switch datatype_value
        case 1 % Text
            set(handles.save_text,'value',1);
            set(handles.save_rtf,'value',0);
            set(handles.save_latex,'value',0);
            data.latex_on=0;
            data.rtf_on=0;
        case 2 % RTF
            set(handles.save_text,'value',0);
            set(handles.save_rtf,'value',1);
            set(handles.save_latex,'value',0);
            data.latex_on=0;
            data.rtf_on=1;
        case 3 % LaTeX
            set(handles.save_text,'value',0);
            set(handles.save_rtf,'value',0);
            set(handles.save_latex,'value',1);
            data.latex_on=1;
            data.rtf_on=0;
    end;
    
    setappdata(0,'data',data);
return;

function Compute_Descriptives_Callback (gcf, eventdata, handles)

%% Compute the descriptive analysis %%

    tmp={};
    global outputletter
    outputletter=12;
    set(handles.ModelOutput,'String',tmp,'FontSize',outputletter); % Delete previous output
    set(handles.ModelOutput,'Value',1);
    set(handles.ModelOutput_latex,'String',tmp);
    set(handles.ModelOutput_latex,'Value',1);
    set(handles.ModelOutput_rtf,'String',tmp);
    set(handles.ModelOutput_rtf,'Value',1);
    set(handles.Output_matrices,'String',tmp);
    set(handles.Output_matrices,'Value',1);
    set(handles.ButtonSalvarOutput,'enable','off');
    set(handles.ButtonLaunchOutput,'enable','off');
    set(handles.save_text,'enable','off');
    set(handles.save_latex,'enable','off');
    set(handles.save_rtf,'enable','off');
    
    %LOAD DATA
    data=getappdata(0,'data');
    
    %evalute the origin of the callback (if it's invoked from the Compute
    %button of Descriptive tab or from Response bias tab)
    if gcf==handles.Compute_Descriptives
        data.forced_descriptives=0;
    end;
    
    TicAnalysis=tic;
    
    % Cleaning the structure
    data = ClearFieldsData(data);
    
    setappdata(0,'data',data);
    
    %% STARTING ANALYSIS %%
    set(gcbf,'Pointer','watch');
    
    if data.forced_descriptives==0;
        %Start new figure with an icon meanwhile other waitbars as the 
        %polychoric computing or factor scores are not initialized
        %new figure
        finestra.N2=250;
        finestra.M2=100;
        pantalla=get(0,'ScreenSize');
        finestra.coordenades=[(pantalla(3)/2)-(finestra.N2/2)  (pantalla(4)/2)-(finestra.M2/2) finestra.N2 finestra.M2];
        handles.fig_wait = figure('Units','Pixels',...
        'color' , [0.941 0.941 0.941],...
        'Position',finestra.coordenades);
        set(handles.fig_wait, 'MenuBar','none','Name','Psychological Test Toolbox','Toolbar','none','NumberTitle','off','resize','off');
        set(handles.fig_wait,'Pointer','watch');

        jObj = waiting(handles);
    end;
    
    drawnow;
    
    if data.missings_on==1 % missing data

        X_miss = data.X == data.missings_value ;
        X_missa = data.X(:,data.inclosos) == data.missings_value ;
        da=data.X(:,data.inclosos);

        [f1,f2]=size(X_miss);
        X_sensemiss=[];
        u=0;
        v=0;

        % Exluding participants with some missing values
        for i=1:f1
            a=any(X_miss(i,:) == 1);
            b=any(X_missa(i,:) == 1);
            if a==1
                u=u+1;
            else
                X_sensemiss(i-u,:) = data.X(i,:);
            end;
            if b==1
                v=v+1;
            else
                X_sensemissa(i-v,:) = da(i,:);
            end;
        end;
            
        % Matrix only containing participats without missing data
        data.X_sensemiss=X_sensemiss;
        data.Xa=X_sensemissa;
        [f1,f2]=size(X_sensemissa);
        data.N=f1;
        
        [f1,f2]=size(data.X); 
        data.N_original=f1;
        
        setappdata(0,'data',data);

    end;
    
    if data.matrix_user_on==0
        data_calcul=data.X;
        data_calcul_a=data.Xa;
    else
        data_calcul=data.S;
        data_calcul_a=data.Sa;
    end;
    
    %% PRINT THE ANALYSIS DETAILS OF THE SELECTED DATA AND CONFIGURATION %%
    PrintAnalysisDetails(handles.ModelOutput,handles.ModelOutput_latex,handles.ModelOutput_rtf,data_calcul);
    
    if data.matrix_user_on==0 %IF THE USER HAS IMPORTED RAW DATA
        
        %COMPUTE DESCRIPTIVE STATISTICS
        ComputeItemDescriptives(handles.ModelOutput,handles.ModelOutput_latex,handles.ModelOutput_rtf);
        
        %COMPUTE DISPERSION MATRIX (COVARIANCE MATRIX/PEARSON/POLYCHORIC)
        ComputeDispersionMatrix(handles.ModelOutput,handles.ModelOutput_latex,handles.ModelOutput_rtf,handles.Output_matrices);
        
    else %IF THE USER HAS PROVIDED A DISPERSION MATRIX
        data.ref1=0;
        setappdata(0,'data',data);
        %PRINT THE MATRIX PROVIDED BY THE USER
        PrintUserMatrix(handles.ModelOutput,handles.ModelOutput_latex,handles.ModelOutput_rtf,handles.Output_matrices);
    end;
    
    data=getappdata(0,'data');
    if data.corr_error==1 %IF THERE ARE CORRELATIONS NEAR -1 OR +1
        errordlg('At least one correlation value between two variables are higher than 0.95 or lower than -0.95. Please, revise the implicated items and delete one.','Psychological Test Toolbox: Error');
        set(gcbf,'Pointer','arrow');
        
        MyPrint(handles.ModelOutput,' ');
        MyPrint(handles.ModelOutput,'At least one correlation value betwen two variables are higher than 0.95 or lower than -0.95. Please, revise the implicated items and delete one.');
        MyPrint(handles.ModelOutput,'Implicated items:');
        [f1,f2]=size(data.items_corr_error);
        for i=1:f2
            buff = sprintf('%s',data.items_corr_error{i});
            MyPrint(handles.ModelOutput,buff);
        end;
        MyPrint(handles.ModelOutput,' ');
        MyPrint(handles.ModelOutput,'-----------------------------------------------------------------------');
        MyPrint(handles.ModelOutput,' ');

        MyPrint(handles.ModelOutput_latex,' ');
        MyPrint(handles.ModelOutput_latex,'At least one correlation value betwen two variables are higher than 0.95 or lower than -0.95. Please, revise the implicated items and delete one.');
        MyPrint(handles.ModelOutput_latex,' ');

        MyPrint(handles.ModelOutput_rtf,'\par');MyPrint(handles.ModelOutput_rtf,'\par');
        MyPrint(handles.ModelOutput_rtf,'At least one correlation value betwen two variables are higher than 0.95 or lower than -0.95. Please, revise the implicated items and delete one.');
        MyPrint(handles.ModelOutput_rtf,'\par');MyPrint(handles.ModelOutput_rtf,'\par');

        Print_References(handles.ModelOutput,handles.ModelOutput_latex,handles.ModelOutput_rtf,handles);

        MyPrint(handles.ModelOutput_rtf,'}');
        set(handles.ButtonSalvarOutput,'enable','on');
        set(handles.save_latex,'enable','on');
        set(handles.save_rtf,'enable','on');
        set(handles.save_text,'enable','on');
        set(handles.ButtonLaunchOutput,'enable','on');
        
        MyPrint(handles.ModelOutput_latex,'\end{document}');

        Printer=getappdata(0,'Printer');
        Printer.text=get(handles.ModelOutput,'string');
        Printer.latex=get(handles.ModelOutput_latex,'string');
        Printer.rtf=get(handles.ModelOutput_rtf,'string');

        data.Bandera=1;
        setappdata(0,'data',data);
        setappdata(0,'Printer',Printer);
        
        if data.forced_descriptives==0;
            %all done, close the figure
            jObj.setBusyText('Analysis stopped!');jObj.stop; 

            if ishandle(handles.fig_wait)
                set(handles.fig_wait,'Pointer','arrow')
                figure(handles.fig_wait);
                t = timer;
                t.StartDelay = 5;
                t.TimerFcn = @(myTimerObj, thisEvent)close(handles.fig_wait);
                start(t);
            end;
        end;
        
        %STOP THE ANALYSIS
        return;
    end;
    
    %% Computing KMO, if it is in the last interval, cancelling forward
    %analysis %%
    if data.Matrix==3
        if isfield(data,'kmo_error');
        if data.kmo_error==1
            kmo_index=kmo(data.Rp);
            errordlg('The KMO obtained using Polychoric correlation is inacceptable. Please, select Pearson Correlation.','Psychological Test Toolbox: Error');
            set(gcbf,'Pointer','arrow');

            MyPrint(handles.ModelOutput,' ');
            MyPrint(handles.ModelOutput,'The KMO obtained using Polychoric correlation is inacceptable. Please, select Pearson Correlation.');
            buff = sprintf('KMO = %5.4f',kmo_index); MyPrint(handles.ModelOutput,buff);

            MyPrint(handles.ModelOutput,' ');
            MyPrint(handles.ModelOutput,'-----------------------------------------------------------------------');
            MyPrint(handles.ModelOutput,' ');

            MyPrint(handles.ModelOutput_latex,' ');
            MyPrint(handles.ModelOutput_latex,'The KMO obtained using Polychoric correlation is inacceptable. Please, select Pearson Correlation.');
            buff = sprintf('KMO = %5.4f',kmo_index); MyPrint(handles.ModelOutput_latex,buff);
            MyPrint(handles.ModelOutput_latex,' ');

            MyPrint(handles.ModelOutput_rtf,'\par');MyPrint(handles.ModelOutput_rtf,'\par');
            MyPrint(handles.ModelOutput_rtf,'The KMO obtained using Polychoric correlation is inacceptable. Please, select Pearson Correlation.');
            MyPrint(handles.ModelOutput_rtf,'\par');
            buff = sprintf('KMO = %5.4f',kmo_index); MyPrint(handles.ModelOutput_rtf,buff);
            MyPrint(handles.ModelOutput_rtf,'\par');MyPrint(handles.ModelOutput_rtf,'\par');

            Print_References(handles.ModelOutput,handles.ModelOutput_latex,handles.ModelOutput_rtf,handles);

            MyPrint(handles.ModelOutput_rtf,'}');
            set(handles.ButtonSalvarOutput,'enable','on');
            set(handles.save_latex,'enable','on');
            set(handles.save_rtf,'enable','on');
            set(handles.save_text,'enable','on');
            set(handles.ButtonLaunchOutput,'enable','on');

            MyPrint(handles.ModelOutput_latex,'\end{document}');

            Printer=getappdata(0,'Printer');
            Printer.text=get(handles.ModelOutput,'string');
            Printer.latex=get(handles.ModelOutput_latex,'string');
            Printer.rtf=get(handles.ModelOutput_rtf,'string');

            data.Bandera=1;
            setappdata(0,'data',data);
            setappdata(0,'Printer',Printer);
            
            if data.forced_descriptives==0;
                %all done, close the figure
                jObj.setBusyText('Analysis stopped!');jObj.stop; 

                if ishandle(handles.fig_wait)
                    set(handles.fig_wait,'Pointer','arrow')
                    figure(handles.fig_wait);
                    t = timer;
                    t.StartDelay = 5;
                    t.TimerFcn = @(myTimerObj, thisEvent)close(handles.fig_wait);
                    start(t);
                end;
            end;

            %STOP THE ANALYSIS
            return;
        end;
        end;
    end;
    
    par=get(handles.parallel_analysis,'value');
    data=getappdata(0,'data');
    
    PA_stop=0;
    
    %% PARALLEL ANALYSIS %%
    
    if par==1 % PARALLEL ANALYSIS ON
        if data.matrix_user_on==0
            if data.Matrix==3 %Polychoric
                [r,dim,realPA,meansPA] = parallelMRFA_P(data.Xa,500,95,2,2,data.Rp,handles.ModelOutput,handles.ModelOutput_latex,handles.ModelOutput_rtf); 
            else
                [r,dim,realPA,meansPA] = parallelMRFA_P(data.Xa,500,95,1,2,[],handles.ModelOutput,handles.ModelOutput_latex,handles.ModelOutput_rtf); 
            end;
            
            if isempty(r) %something goes wrong, or the analysis have been stopped by user
                MyPrint(handles.ModelOutput,' ');
                MyPrint(handles.ModelOutput,'Parallel Analysis stopped by user');
                MyPrint(handles.ModelOutput,' ');
                MyPrint(handles.ModelOutput,'-----------------------------------------------------------------------');
                MyPrint(handles.ModelOutput,' ');
                
                MyPrint(handles.ModelOutput_latex,' ');
                MyPrint(handles.ModelOutput_latex,'Parallel Analysis stopped by user');
                MyPrint(handles.ModelOutput_latex,' ');
                
                MyPrint(handles.ModelOutput_rtf,'\par');MyPrint(handles.ModelOutput_rtf,'\par');
                MyPrint(handles.ModelOutput_rtf,'Parallel Analysis stopped by user');
                MyPrint(handles.ModelOutput_rtf,'\par');MyPrint(handles.ModelOutput_rtf,'\par');
                
                if data.forced_descriptives==0
                    set(gcbf,'Pointer','arrow');
                end;

                setappdata(0,'data',data);
                PA_stop=1;
            end;
            switch data.Matrix
                case 1
                    S=cov(data.Xa);
                    R = diag(diag(S))^(-1/2) * S * diag(diag(S))^(-1/2);
                case 2
                    R=corrcoef(data.Xa);
                case 3
                    R=data.Rp;
            end;
            
        else
            R = diag(diag(data.Sa))^(-1/2) * data.Sa * diag(diag(data.Sa))^(-1/2);
            [r,dim,realPA,meansPA] = parallelPCA(R,data.N,500,95,2,handles.ModelOutput,handles.ModelOutput_latex,handles.ModelOutput_rtf);         %PA basat en la matriu definida per l'usuari
        end;
        
    else % PARALLEL ANALYSIS OFF
        if data.matrix_user_on==0
            [N,m]=size(data.Xa);
            switch data.Matrix
                case 1                    
                    S=cov(data.Xa);
                    R = diag(diag(S))^(-1/2) * S * diag(diag(S))^(-1/2);
                    r=m;
                case 2
                    R=corrcoef(data.Xa);
                    r=m;
                case 3
                    R=data.Rp;
                    r=m;
            end;
        else
            [m,m]=size(data.Sa);
            R = diag(diag(data.Sa))^(-1/2) * data.Sa * diag(diag(data.Sa))^(-1/2);
            r=m;
        end;
    end;
    
    setappdata(0,'data',data);
    
    %% COMPUTE DISPERSION MATRIX ADECUATION INDICES (KMO, BARTLETT) %%
    DispersionMatrixAdecuation(handles.ModelOutput,handles.ModelOutput_latex,handles.ModelOutput_rtf,data_calcul_a,R);

    graf=get(handles.grafic_PA,'value'); 

    if graf==1 && PA_stop==0 % IF PA GRAFIC IS On and the analysis has not been stopped
        Printgrafic(dim,realPA,meansPA,handles.ModelOutput_latex);
    end;
    
    %% Scree Test %%
    PrintScreeTest(R,r,handles.ModelOutput,handles.ModelOutput_latex);
    
    PrintScreeTest_rtf(R,r,handles.ModelOutput_rtf);
    
    if data.missings_on==1
        X_miss = data.X(:,data.inclosos) == data.missings_value;
        missings=find(X_miss == 1);
        [f1,f2]=size(missings);
        if f1>0
            % PRINT MISSING DATA REPORT
            PrintMissingData(handles.ModelOutput,handles.ModelOutput_latex,handles.ModelOutput_rtf);
        end;
    end;
    
    %% REFERENCES FOR THIS SECTION %%
    Print_References(handles.ModelOutput,handles.ModelOutput_latex,handles.ModelOutput_rtf,handles);
    
    MyPrint(handles.ModelOutput_rtf,'}');
    set(handles.ButtonSalvarOutput,'enable','on');
    set(handles.save_latex,'enable','on');
    set(handles.save_rtf,'enable','on');
    set(handles.save_text,'enable','on');
    set(handles.ButtonLaunchOutput,'enable','on');
    set(handles.ModelOutputIncrementa,'enable','on');
    set(handles.ModelOutputDecrementa,'enable','on');

    if data.forced_descriptives==0
        set(gcbf,'Pointer','arrow');
    end;

    MyPrint(handles.ModelOutput_latex,'\end{document}');
    
    compT = toc(TicAnalysis);
       
%     s_tmp=['Analysis Complete! Time elapsed = ' Time2String( compT ) ''];
%     msgbox(s_tmp,'Psychological Test Toolbox');

    if data.forced_descriptives==0;
        ButtonLaunchOutput_Callback(gcbo,eventdata,handles);
    end;
    
    drawnow;
    
    if data.forced_descriptives==0;
        %all done, close the figure
        jObj.setBusyText('Analysis complete!');jObj.stop; 
        
        if ishandle(handles.fig_wait)
            set(handles.fig_wait,'Pointer','arrow')
            figure(handles.fig_wait);
            t = timer;
            t.StartDelay = 5;
            t.TimerFcn = @(myTimerObj, thisEvent)close(handles.fig_wait);
            start(t);
        end;
        %play success sound
        [y,Fs]=audioread('success.wav');
        sound(y,Fs);
    end;
    
    Printer=getappdata(0,'Printer');
    Printer.text=get(handles.ModelOutput,'string');
    Printer.latex=get(handles.ModelOutput_latex,'string');
    Printer.rtf=get(handles.ModelOutput_rtf,'string');

    data.Bandera=1;
    data.descriptives_done=1;
    data.forced_descriptives=0;
    setappdata(0,'data',data);
    setappdata(0,'Printer',Printer);
    
    
    
return;

function PrintAnalysisDetails(Output,Output2,Output3,A)
    
%% Printing Analysis Summary %%
%
% ARGUMENTS   : Output  -> plain text listbox
%               Output2 -> LaTeX listbox
%               Output3 -> rtf listbox
%               A       -> Raw data or Matrix provided by the user
%

    data=getappdata(0,'data');    
    
    %% LATEX %%   
    
    MyPrint(Output2,'% LaTeX Report based on:');
    MyPrint(Output2,'% Example: Project Report');
    MyPrint(Output2,'% Source: http://www.howtotex.com');
    MyPrint(Output2,'\documentclass[paper=a4, fontsize=11pt]{scrartcl}');
    MyPrint(Output2,'\usepackage{float}');
    MyPrint(Output2,'\usepackage[T1]{fontenc}');
    MyPrint(Output2,'\usepackage{fourier}');
    MyPrint(Output2,'\usepackage[english]{babel}');
    MyPrint(Output2,'\usepackage[protrusion=true,expansion=true]{microtype}');
    MyPrint(Output2,'\usepackage[pdftex]{graphicx}');
    MyPrint(Output2,'\usepackage{url}');
    MyPrint(Output2,'\usepackage{longtable}');
    MyPrint(Output2,'\usepackage{pgfplots}');
    MyPrint(Output2,'\usepackage{siunitx}');
    MyPrint(Output2,'\usepackage{booktabs}');
    MyPrint(Output2,'\usepackage{array}');
    MyPrint(Output2,'\usepackage[section]{placeins}');
    MyPrint(Output2,'%%% Custom sectioning');
    MyPrint(Output2,'\usepackage{sectsty}');
    MyPrint(Output2,'\allsectionsfont{\centering \normalfont\scshape}');
    MyPrint(Output2,'%%% Custom headers/footers (fancyhdr package)');
    MyPrint(Output2,'\usepackage{fancyhdr}');
    MyPrint(Output2,'\pagestyle{fancyplain}');
    MyPrint(Output2,'\fancyhead{}');
    MyPrint(Output2,'\fancyfoot[L]{}');
    MyPrint(Output2,'\fancyfoot[C]{}');
    MyPrint(Output2,'\fancyfoot[R]{\thepage}	');
    MyPrint(Output2,'\renewcommand{\headrulewidth}{0pt}');
    MyPrint(Output2,'\renewcommand{\footrulewidth}{0pt}');
    MyPrint(Output2,'\setlength{\headheight}{13.6pt}');
    MyPrint(Output2,'%%% Maketitle metadata');
    MyPrint(Output2,'\newcommand{\horrule}[1]{\rule{\linewidth}{#1}} 	% Horizontal rule');
    MyPrint(Output2,'\title{');
    MyPrint(Output2,'		\usefont{OT1}{bch}{b}{n}');
    MyPrint(Output2,'		\normalfont \normalsize \textsc{Universitat Rovira i Virgili} \\ [5pt]');
    MyPrint(Output2,'		\horrule{0.5pt} \\[0.4cm]');
    MyPrint(Output2,'		\huge Psychological Test Toolbox Report \\');
    MyPrint(Output2,'		\horrule{2pt} \\[1pt]');
    MyPrint(Output2,'}');
    MyPrint(Output2,'\author{');
    MyPrint(Output2,'	\normalfont 		\normalsize');
    MyPrint(Output2,'	David Navarro-Gonzalez, Andreu Vigil-Colet , Pere Joan Ferrando \& Urbano Lorenzo-Seva\\[-3pt]		\normalsize');
    MyPrint(Output2,'        \today');
    MyPrint(Output2,'}');
    MyPrint(Output2,'\date{}');
    MyPrint(Output2,' ');
    MyPrint(Output2,'%%% Begin document');
    
    MyPrint(Output2,'\begin{document}');
    MyPrint(Output2,'\maketitle');
    
    MyPrint(Output2,'\section{Descriptives}');
    MyPrint(Output2,'\begin{table}[!htb]');
    MyPrint(Output2,'\caption{Data details}');
    MyPrint(Output2,'\label{taula:descriptives}');
    MyPrint(Output2,'\centering');
    MyPrint(Output2,'\begin{tabular}{l l} \toprule');
    buff2 = '\\';ve='\v'; buff0='\textbf';
    [f1,f2]=size(data.path);
    if f2>45 %if path is too long
        slashposition=findstr(data.path,'\');
        if isempty(slashposition)
            slashposition=findstr(data.path,'/');
        end;
        [g1,g2]=size(slashposition);
        for i=1:g2
            lastslash=slashposition(i);
            if lastslash<45
               %continue finding
            else
                lastslash=slashposition(i-1);
                break;
            end;
        end;
        buff = sprintf('{File path}: & %serb$%s$ ',ve,data.path(1:lastslash));
        MyPrint(Output2,[buff0 buff buff2]);
        path2=data.path(lastslash+1:f2);
        [g1,g2]=size(path2);
        if g2>45 %again
            slashposition=findstr(path2,'\');
            if isempty(slashposition)
                slashposition=findstr(path2,'/');
            end;
            [h1,h2]=size(slashposition);
            for i=1:h2
                lastslash=slashposition(i);
                if lastslash<45
                   %continue finding
                else
                    lastslash=slashposition(i-1);
                    break;
                end;
            end;
            buff = sprintf('~ & %serb$%s$ ',ve,path2(1:lastslash));
            MyPrint(Output2,[buff buff2]);
            buff = sprintf('~ & %serb$%s$ ',ve,path2(lastslash+1:g2));
            MyPrint(Output2,[buff buff2]);
            
        else
            buff = sprintf('~ & %serb$%s$ ',ve,path2);
            MyPrint(Output2,[buff buff2]);
        end;
        
    end;
%     end;
    if data.matrix_user_on==0
        buff = sprintf('{Number of participants}: & %5.0f ',data.N_original);
        MyPrint(Output2,[buff0 buff buff2]);
    end;
    if data.missings_on==1
        buff = sprintf('{Number of participants without missing data}: & %5.0f',data.N); MyPrint(Output2, [buff0 buff buff2]);
    end;
    buff = sprintf('{Number of items}: & %5.0f ',data.n);
    MyPrint(Output2,[buff0 buff buff2]);
    [tmp, m_t]=size(A);
    [m,tmp]=size(char(data.inclosos));
    if tmp > m
        m=tmp;
    end;
    if m_t == m
       MyPrint(Output2,'\textbf{Items included in the analysis}: & ALL \\');
                      
    else
        buff = '\textbf{Items included in the analysis}: & ';
        buff01='';
        for i=1:m-1
            buff1=sprintf('%.0f, ',data.inclosos(i));
            buff01 = [buff01 buff1];
            [g1,g2]=size(buff01);
            if g2>67
                MyPrint(Output2,[buff buff01 buff2]);
                buff='~ & ';
                buff01='';
            end;
        end;
        buff1=sprintf('%.0f',data.inclosos(m));
        buff = [buff buff01 buff1 buff2];
        MyPrint(Output2,buff );
        buff = '\textbf{Items excluded in the analysis}: & ';
        h=1;
        for i=1:m_t
            in=0;
            for j=1:m
                if data.inclosos(j) == i
                    in=1;
                    break;
                end;
            end;
            if in == 0
               if h==1
                  buff1=sprintf('%.0f',i);
               else
                  buff1=sprintf(', %.0f',i);
               end;
               buff = [buff buff1];
               h=2;
            end;
        end;
        MyPrint(Output2,[buff buff2]);
    end
    if data.matrix_user_on==1
        MyPrint(Output2,'\textbf{Dispersion Matrix}: & User Defined \\');
    else
        if data.Matrix==1
            MyPrint(Output2,'\textbf{Dispersion Matrix}: & Covariance Matrix \\');
        elseif data.Matrix==2
            MyPrint(Output2,'\textbf{Dispersion Matrix}: & Pearson Correlations \\');
        elseif data.Matrix==3
            MyPrint(Output2,'\textbf{Dispersion Matrix}: & Polychoric Correlations \\');
        end;
    end;
    
    %If we are doing factor analysis
    if data.forced_descriptives==1
        buff = '{Method for factor extraction}: & Unweighted Least Squares (ULS)';
        MyPrint(Output2, [buff0 buff buff2]);
        buff= ' ~ & (ten Berge and Kiers, 1991)';
        MyPrint(Output2, [buff buff2]);
        switch data.Rotation
            case 1 %Orthogonal
                buff = '{Rotation Method}: & Varimax';
                MyPrint(Output2, [buff0 buff buff2]);
            case 2 %Oblicque
                buff = '{Rotation Method}: & Promin (Lorenzo-Seva, 1999)';
                MyPrint(Output2, [buff0 buff buff2]);
            case 3
                buff = '{Rotation Method}: & Semi-specified oblique Procrustes rotation';
                MyPrint(Output2, [buff0 buff buff2]);
                buff = '~ & (Browne, 1972b)';
                MyPrint(Output2, [buff buff2]);
            case 4
                buff = '{Rotation Method}: & Semi-specified orthogonal Procrustes rotation';
                MyPrint(Output2, [buff0 buff buff2]);
                buff = '~ & (Browne, 1972a)';
                MyPrint(Output2, [buff buff2]);
        end;
    end;
    
    buff1=clock;
    c=fix(buff1);
    month={};
    month{1}='January';month{2}='February';month{3}='March';month{4}='April';month{5}='May';month{6}='June';
    month{7}='July';month{8}='August';month{9}='September';month{10}='October';month{11}='November';month{12}='December';
    actual_month=month{c(2)};
    %corrected minutes
    if c(5)<10
        buff = sprintf('{Analysis date}: & %.0f:0%.0f, %.0f-%s-%.0f',c(4),c(5),c(3),actual_month,c(1));
    else
        buff = sprintf('{Analysis date}: & %.0f:%.0f, %.0f-%s-%.0f',c(4),c(5),c(3),actual_month,c(1));
    end;
    MyPrint(Output2, [buff0 buff buff2]);
    
    MyPrint(Output2,'\bottomrule');
    MyPrint(Output2,'\end{tabular}');
    MyPrint(Output2,'\end{table}');
    
    %% TEXT %%
    
    MyPrint(Output,' ');
    MyPrint(Output,'DETAILS OF ANALYSIS');
    MyPrint(Output,' ');
    
%     buff = sprintf('Matrix analized                             : %s',data.name); MyPrint(Output, buff);
    buff = sprintf('File path                                   : %s',data.path); MyPrint(Output, buff);
    if data.matrix_user_on==0
        buff = sprintf('Number of participants                      : %5.0f',data.N_original); MyPrint(Output, buff);
    end;
    if data.missings_on==1
        buff = sprintf('Number of participants without missing data : %5.0f',data.N); MyPrint(Output, buff);
    end;
    buff = sprintf('Number of items                             : %5.0f',data.n); MyPrint(Output, buff);
    
    [tmp, m_t]=size(A);
    [m,tmp]=size(char(data.inclosos));
    if tmp > m
        m=tmp;
    end;
    if m_t == m
       MyPrint(Output,'Items included in the analysis              : ALL');
                      
    else
        buff = 'Items included in the analysis              : ';
        for i=1:m-1
            buff1=sprintf('%.0f, ',data.inclosos(i));
            buff = [buff buff1];
        end;
        buff1=sprintf('%.0f',data.inclosos(m));
        buff = [buff buff1];
        MyPrint(Output,buff);
        buff = 'Items excluded in the analysis              : ';
        
        h=1;
        for i=1:m_t
            in=0;
            for j=1:m
                if data.inclosos(j) == i
                    in=1;
                    break;
                end;
            end;
            if in == 0
               if h==1
                  buff1=sprintf('%.0f',i);
               else
                  buff1=sprintf(', %.0f',i);
               end;
               buff = [buff buff1];
               h=2;
            end;
        end;
        MyPrint(Output,buff);
    end;
    if isempty(data.items_sd)==0
        buff = 'Items selected as SD items                  : ';
        [f1,f2]=size(data.items_sd);
        h=1;
        buff_1='';
        for i=1:f2
            if h==1
                buff_2=sprintf('%.0f',data.items_sd(i));
                h=2;
            else
                buff_2=sprintf(', %.0f',data.items_sd(i));
           end;
           buff_1=[buff_1 buff_2];
        end;
        MyPrint(Output,[buff buff_1]);
    end;
    if isempty(data.iub)==0
        buff = 'Items selected as unbalanced                : ';
        [f1,f2]=size(data.iub);
        h=1;
        buff_1='';
        for i=1:f2
            if h==1
                buff_2=sprintf('%.0f',data.iub(i));
                h=2;
            else
                buff_2=sprintf(', %.0f',data.iub(i));
           end;
           buff_1=[buff_1 buff_2];
        end;
        MyPrint(Output,[buff buff_1]);
    end;
    
    if data.matrix_user_on==1
        buff = sprintf('Dispersion Matrix                           : User Defined'); MyPrint(Output, buff);
    else
        if data.Matrix==1
            buff = sprintf('Dispersion Matrix                           : Covariance Matrix'); MyPrint(Output, buff);
        elseif data.Matrix==2
            buff = sprintf('Dispersion Matrix                           : Pearson Correlations'); MyPrint(Output, buff);
        elseif data.Matrix==3
            buff = sprintf('Dispersion Matrix                           : Polychoric Correlations'); MyPrint(Output, buff);
        end;
    end;
    
    if data.forced_descriptives==1
        
        buff = 'Method for factor extraction                : Unweighted Least Squares (ULS)';
        MyPrint(Output, buff );
        switch data.Rotation
            case 1 %Orthogonal
                buff = 'Rotation Method                             : Varimax';
            case 2 %Oblicque
                buff = 'Rotation Method                             : Promin (Lorenzo-Seva, 1999)';
            case 3
                buff = 'Rotation Method                             : Semi-specified oblique Procrustes rotation (Browne, 1972b)';
            case 4
                buff = 'Rotation Method                             : Semi-specified orthogonal Procrustes rotation (Browne, 1972a)';
        end;
        MyPrint(Output, buff);
    end;
    
    buff1=clock;
    c=fix(buff1);
    month={};
    month{1}='January';month{2}='February';month{3}='March';month{4}='April';month{5}='May';month{6}='June';
    month{7}='July';month{8}='August';month{9}='September';month{10}='October';month{11}='November';month{12}='December';
    actual_month=month{c(2)};
    data.actual_date=[c(4),c(5),c(3),actual_month,c(1)];
    %corrected minutes
    if c(5)<10
        buff = sprintf('Analysis date                               : %.0f:0%.0f, %.0f-%s-%.0f',c(4),c(5),c(3),actual_month,c(1));
    else
        buff = sprintf('Analysis date                               : %.0f:%.0f, %.0f-%s-%.0f',c(4),c(5),c(3),actual_month,c(1));
    end;
    MyPrint(Output, buff);
    
    MyPrint(Output,' ');
    MyPrint(Output,'-----------------------------------------------------------------------');
    
    
    %% RTF %%
    
    %%Header

    MyPrint(Output3,'{\rtf1\ansi\ansicpg1252\uc1 \deff0\deflang3082\deflangfe3082 ');

    %%Fonts
    MyPrint(Output3,'{\fonttbl{\f0\froman\fcharset0\fprq2{\*\panose 02020603050405020304}Times New Roman;}{\f1\fswiss\fcharset0\fprq2{\*\panose 020b0604020202020204}Arial;}{\f6\fmodern\fcharset0\fprq1{\*\panose 00000000000000000000}Courier;}} ');

    %%Colours 
    MyPrint(Output3,'{\colortbl;\red0\green0\bue0;\red0\green0\blue255;\red0\green255\blue255;\red0\green255\blue0;\red255\green0\blue255;\red255\green0\blue0;\red255\green255\blue0;\red255\green255\blue255;\red0\green0\blue128;\red0\green128\blue128;\red0\green128\blue0;\red128\green0\blue128;\red128\green0\blue0;\red128\green128\blue0;\red128\green128\blue128;\red192\green192\blue192;\red179\green179\blue179;} ');

    %%Paper style
    MyPrint(Output3,'{\stylesheet{\ql \li0\ri0\widctlpar\aspalpha\aspnum\faauto\adjustright\rin0\lin0\itap0 \fs24\lang3082\langfe3082\cgrid\langnp3082\langfenp3082 \snext0 Normal;}{\*\cs10 \additive Default Paragraph Font;}{\s15\ql \li0\ri0\widctlpar \tqc\tx4153\tqr\tx8306\aspalpha\aspnum\faauto\adjustright\rin0\lin0\itap0 \fs24\lang3082\langfe3082\cgrid\langnp3082\langfenp3082 \sbasedon0 \snext15 footer;}{\*\cs16 \additive \sbasedon10 page number;}} ');

    %%Info about the propierties of the document
    titulo='Output';
    autor='Psychological Test Toolbox';
    operador='-';
    companyia='Universitat Rovira i Virgili';
    buff = ['{\info{\title ' titulo '}{\author ' autor '}{\operator ' operador '}{\creatim\yr0\mo0\dy0\hr0}{\revtim\yr0\mo0\dy0\hr0\min0}{\version1}{\edmins2}{\nofpages0}{\nofwords0}{\nofchars0}{\*\company ' companyia '}{\nofcharsws0}{\vern8249}}'];
    MyPrint(Output3, buff);

    %%Paragraph spacing
    MyPrint(Output3,'\par ');
    
    %%Printing centered
    MyPrint(Output3,'\qc \ul Details of Analysis \ul0');
    MyPrint(Output3,'\par ');
    MyPrint(Output3,'\ql');
    MyPrint(Output3,'\par ');
    interline='\trowd\trgaph144\clbrdrt\brdrs\clbrdrl\brdrs\clbrdrb\brdrs\clbrdrr\brdrs\cellx5000\clbrdrt\brdrs\clbrdrl\brdrs\clbrdrb\brdrs\clbrdrr\brdrs\cellx10000';
    MyPrint(Output3,interline);
%     buff = sprintf('\\intbl Matrix analized: \\cell %s \\cell \\row',data.name); MyPrint(Output3, buff);
    %data path contains "\" symbol, which cause conflict with rtf
    %programing language.
    [h1,h2]=size(data.path);
    buff_path=[];
    for i=1:h2
        if strcmp(data.path(i),'\')
            buff_path=[buff_path '\\'];
        else
            buff_path=[buff_path data.path(i)];
        end;
    end;
    buff = sprintf('\\intbl File path: \\cell %s \\cell \\row',buff_path); MyPrint(Output3, buff);
    if data.matrix_user_on==0
        MyPrint(Output3,interline);
        buff = sprintf('\\intbl Number of participants:\\cell %5.0f \\cell \\row',data.N_original); MyPrint(Output3, buff);
    end;
    if data.missings_on==1
        MyPrint(Output3,interline);
        buff = sprintf('\\intbl Number of participants without missing data:\\cell %5.0f \\cell \\row',data.N); MyPrint(Output3, buff);
    end;
    MyPrint(Output3,interline);
    buff = sprintf('\\intbl Number of items: \\cell %5.0f \\cell \\row',data.n); MyPrint(Output3, buff);

    cellrow=' \cell \row';
    
    [tmp, m_t]=size(A);
    [m,tmp]=size(char(data.inclosos));
    if tmp > m
        m=tmp;
    end;
    if m_t == m
        MyPrint(Output3,interline);
        MyPrint(Output3,'\intbl Items included in the analysis: \cell ALL \cell \row');
                      
    else
        buff = '\intbl Items included in the analysis: \cell ';
        for i=1:m-1
            buff1=sprintf('%.0f, ',data.inclosos(i));
            buff = [buff buff1];
        end;
        buff1=sprintf('%.0f',data.inclosos(m));
        buff = [buff buff1];
        buff=[buff cellrow];
        MyPrint(Output3,interline);
        MyPrint(Output3,buff);
        
        buff = '\intbl Items excluded in the analysis: \cell ';
        h=1;
        for i=1:m_t
            in=0;
            for j=1:m
                if data.inclosos(j) == i
                    in=1;
                    break;
                end;
            end;
            if in == 0
               if h==1
                  buff1=sprintf('%.0f',i);
               else
                  buff1=sprintf(', %.0f',i);
               end;
               buff = [buff buff1];
               h=2;
            end;
        end;
        MyPrint(Output3,interline);
        buff=[buff cellrow];
        MyPrint(Output3,buff);
    end;
    
    if isempty(data.items_sd)==0
        buff = '\intbl Items selected as SD items: \cell ';
        [f1,f2]=size(data.items_sd);
        h=1;
        for i=1:f2
            if h==1
                buff1=sprintf('%.0f',data.items_sd(i));
                buff=[buff buff1];
                h=2;
            else
                buff1=sprintf(', %.0f',data.items_sd(i));
                buff=[buff buff1];
            end;
        end;
        buff=[buff cellrow];
        MyPrint(Output3,interline);
        MyPrint(Output3,buff);
    end;
    if isempty(data.iub)==0
        buff = '\intbl Items selected as unbalanced: \cell ';
        [f1,f2]=size(data.iub);
        h=1;
        for i=1:f2
            if h==1
                buff1=sprintf('%.0f',data.iub(i));
                buff=[buff buff1];
                h=2;
            else
                buff1=sprintf(', %.0f',data.iub(i));
                buff=[buff buff1];
           end;
        end;
        buff=[buff cellrow];
        MyPrint(Output3,interline);
        MyPrint(Output3,buff);
    end;
    
    if data.matrix_user_on==1
        buff = sprintf('\\intbl Dispersion Matrix: \\cell User Defined \\cell \\row'); 
    else
        if data.Matrix==1
            buff = sprintf('\\intbl Dispersion Matrix: \\cell Covariance Matrix \\cell \\row');
        elseif data.Matrix==2
            buff = sprintf('\\intbl Dispersion Matrix: \\cell Pearson Correlations \\cell \\row');
        elseif data.Matrix==3
            buff = sprintf('\\intbl Dispersion Matrix: \\cell Polychoric Correlations \\cell \\row');
        end;
    end;
    MyPrint(Output3,interline);
    MyPrint(Output3, buff);
    
    if data.forced_descriptives==1
        
        buff = '\intbl Method for factor extraction: \cell Unweighted Least Squares (ULS) \cell \row';
        MyPrint(Output3,interline);
        MyPrint(Output3, buff );
        switch data.Rotation
            case 1 %Orthogonal
                buff = '\intbl Rotation Method: \cell Varimax \cell \row';
            case 2 %Oblicque
                buff = '\intbl Rotation Method: \cell Promin (Lorenzo-Seva, 1999) \cell \row';
            case 3
                buff = '\intbl Rotation Method: \cell Semi-specified oblique Procrustes rotation (Browne, 1972b) \cell \row';
            case 4
                buff = '\intbl Rotation Method: \cell Semi-specified orthogonal Procrustes rotation (Browne, 1972a) \cell \row';
        end;
        MyPrint(Output3,interline);
        MyPrint(Output3, buff);
    end;
    
    buff1=clock;
    c=fix(buff1);
    month={};
    month{1}='January';month{2}='February';month{3}='March';month{4}='April';month{5}='May';month{6}='June';
    month{7}='July';month{8}='August';month{9}='September';month{10}='October';month{11}='November';month{12}='December';
    actual_month=month{c(2)};
    %corrected minutes
    if c(5)<10
        buff = sprintf('\\intbl Analysis date : \\cell %.0f:0%.0f, %.0f-%s-%.0f  \\cell \\row \\pard \\par',c(4),c(5),c(3),actual_month,c(1));
    else
        buff = sprintf('\\intbl Analysis date : \\cell %.0f:%.0f, %.0f-%s-%.0f  \\cell \\row \\pard \\par',c(4),c(5),c(3),actual_month,c(1));
    end;
    MyPrint(Output3,interline);
    MyPrint(Output3, buff);
    
    buff=sprintf('%.0f-%s-%.0f, %.0f:%.0f',c(3),actual_month,c(1),c(4),c(5));
    data.actual_date=buff;
    
    setappdata(0,'data',data);    
    
return;

function ComputeItemDescriptives(Output,Output2,Output3)
    
%% Computing Item Descriptives %%
%
% ARGUMENTS   : Output  -> plain text listbox
%               Output2 -> LaTeX listbox
%               Output3 -> rtf listbox
%

    data=getappdata(0,'data');  

    if ischar(data.labelsa)
        str=cellstr(data.labelsa);
    else
        str=data.labelsa;
    end;


    [ml,tmp]=size(str);
     ml = max([ml;tmp]);

    s=0;
    for i=1:ml
        if s < length(str{i})
            s= length(str{i});
        end;
    end;
    if s>30, s=30;end;
    if s<5 , s=5;end;

    
    [N,m]=size(data.Xa);
    me  = mean(data.Xa);
    Sx2 = var(data.Xa);
    Skew = skewness(data.Xa);
    Kur  = kurtosis(data.Xa);
    
    %% LATEX %%
    
    MyPrint(Output2,'\begin{center}');
    MyPrint(Output2,'\begin{longtable}{l*{4}{S}}');
    MyPrint(Output2,'\caption{Univariate Item Descriptives}\\');
    MyPrint(Output2,'\toprule');
    MyPrint(Output2,'\multicolumn{1}{l}{\textbf{Item}} & ');
    MyPrint(Output2,'\multicolumn{1}{c}{\textbf{Mean}} & ');
    MyPrint(Output2,'\multicolumn{1}{c}{\textbf{Variance}} & ');
    MyPrint(Output2,'\multicolumn{1}{c}{\textbf{Skewness}} & ');
    MyPrint(Output2,'\multicolumn{1}{p{2.5cm}}{\textbf{Kurtosis (Zero Centered)}} \\');
    MyPrint(Output2,'\midrule');
    MyPrint(Output2,'\endfirsthead');
    MyPrint(Output2,'\multicolumn{5}{c}%');
    MyPrint(Output2,'{\tablename\ \thetable\ -- \textit{(Continued from previous page)}} \\');
    MyPrint(Output2,'\toprule');
    MyPrint(Output2,'\multicolumn{1}{l}{\textbf{Item}} & ');
    MyPrint(Output2,'\multicolumn{1}{c}{\textbf{Mean}} & ');
    MyPrint(Output2,'\multicolumn{1}{c}{\textbf{Variance}} & ');
    MyPrint(Output2,'\multicolumn{1}{c}{\textbf{Skewness}} & ');
    MyPrint(Output2,'\multicolumn{1}{p{2.5cm}}{\textbf{Kurtosis (Zero Centered)}} \\');
    MyPrint(Output2,'\midrule');
    MyPrint(Output2,'\endhead');
    MyPrint(Output2,'\bottomrule \multicolumn{5}{r}{\textit{Continued on next page}} \\');
    MyPrint(Output2,'\endfoot');
    MyPrint(Output2,'\bottomrule');
    MyPrint(Output2,'\endlastfoot');
    buff4 = '\\';
    form=sprintf('%%-%d.%ds    &    Mean    &    Variance  &  Skewness   &  Kurtosis (Zero centered)',s,s);
    buff=sprintf(form,'Item');
    for i=1:m
        form=sprintf('%%-%d.%ds        & ',s,s);   
        buff=sprintf(form,str{i});
        buff1=sprintf('%2.3f   &    %2.3f & ',me(i),Sx2(i));
        buff2 = sprintf('       % 2.3f   &   % 2.3f',Skew(i),Kur(i)-3);
        MyPrint(Output2,[buff buff1 buff2 buff4]);
    end;
    
    MyPrint(Output2,'\end{longtable}');
    MyPrint(Output2,'\end{center}');
    kurmax=max(Kur)-3;
    kurmin=min(Kur)-3;
    if kurmax>1 || kurmin<-1
        MyPrint(Output2,'');
        MyPrint(Output2,'Polychoric correlation is advised when the univariate distributions of ordinal items are asymmetric or with excess of kurtosis. If both indices are lower than one in absolute value, then Pearson correlation is advised. You can read more about this subject in:');
        MyPrint(Output2,'');
        MyPrint(Output2,'Muthen, B., and Kaplan D. (1985). A comparison of some methodologies for the factor analysis of non-normal Likert variables. British Journal of Mathematical and Statistical Psychology, 38, 171-189. \par');
        MyPrint(Output2,'Muthen, B., and Kaplan D. (1992). A comparison of some methodologies for the factor analysis of non-normal Likert variables: A note on the size of the model. British Journal of Mathematical and Statistical Psychology, 45, 19-30. ');
    end;
    
    
    %% TEXT %%

    MyPrint(Output,' ');
    MyPrint(Output,'UNIVARIATE ITEM DESCRIPTIVES');
    MyPrint(Output,' ');
    form=sprintf('%%-%d.%ds        Mean        Variance    Skewness     Kurtosis (Zero centered)',s,s);
    buff=sprintf(form,'Item');
    MyPrint(Output,buff)
    MyPrint(Output,' ');
    for i=1:m
        form=sprintf('%%-%d.%ds        ',s,s);   
        buff=sprintf(form,str{i});
        buff1=sprintf('%2.3f       %2.3f',me(i),Sx2(i));
        buff2 = sprintf('       % 2.3f      % 2.3f',Skew(i),Kur(i)-3);
        MyPrint(Output,[buff buff1 buff2]);
    end;
    
    kurmax=max(Kur)-3;
    kurmin=min(Kur)-3;
    if kurmax>1 || kurmin<-1
        MyPrint(Output,'');
        MyPrint(Output,'Polychoric correlation is advised when the univariate distributions of ordinal items are');
        MyPrint(Output,'asymmetric or with excess of kurtosis. If both indices are lower than one in absolute value,');
        MyPrint(Output,'then Pearson correlation is advised. You can read more about this subject in:');
        MyPrint(Output,'');
        MyPrint(Output,'Muthen, B., & Kaplan D. (1985).');
        MyPrint(Output,'Muthen, B., & Kaplan D. (1992).');
        data.ref1=1;
    else
        data.ref1=0;
    end;
    MyPrint(Output,' ');
    MyPrint(Output,'-----------------------------------------------------------------------');
    
    %% RTF %%
    
    MyPrint(Output3,'\par');
    MyPrint(Output3,'\qc \ul Univariate Item Descriptives \ul0');
    MyPrint(Output3,'\par');
    MyPrint(Output3,'\par');
    interline1='\trowd\trgaph144\clbrdrt\brdrs\clbrdrl\brdrs\clbrdrb\brdrs\clbrdrr\brdrs\cellx2000\clbrdrt\brdrs\clbrdrl\brdrs\clbrdrb\brdrs\clbrdrr\brdrs\cellx4000';
    interline2='\clbrdrt\brdrs\clbrdrl\brdrs\clbrdrb\brdrs\clbrdrr\brdrs\cellx6000\clbrdrt\brdrs\clbrdrl\brdrs\clbrdrb\brdrs\clbrdrr\brdrs\cellx8000\clbrdrt\brdrs\clbrdrl\brdrs\clbrdrb\brdrs\clbrdrr\brdrs\cellx10000';
    interline_cap=[interline1 interline2];
    MyPrint(Output3,interline_cap);
    
    interline_cos='\trowd\trgaph144\clbrdrl\brdrs\clbrdrr\brdrs\cellx2000\clbrdrl\brdrs\clbrdrr\brdrs\cellx4000\clbrdrl\brdrs\clbrdrr\brdrs\cellx6000\clbrdrl\brdrs\clbrdrr\brdrs\cellx8000\clbrdrl\brdrs\clbrdrr\brdrs\cellx10000';
    interline_bottom='\trowd\trgaph144\clbrdrl\brdrs\clbrdrb\brdrs\clbrdrr\brdrs\cellx2000\clbrdrl\brdrs\clbrdrr\brdrs\clbrdrb\brdrs\cellx4000\clbrdrl\brdrs\clbrdrr\brdrs\clbrdrb\brdrs\cellx6000\clbrdrl\brdrs\clbrdrr\brdrs\clbrdrb\brdrs\cellx8000\clbrdrl\brdrs\clbrdrb\brdrs\clbrdrr\brdrs\cellx10000';
    int='\intbl ';
    cell='\cell';
    row='\row';
    form=sprintf('%%-%d.%ds',s,s); 
    form2=sprintf('%s Mean %s Variance %s Skewness %s Kurtosis (Zero centered) %s %s',cell,cell,cell,cell,cell,row);
    buff=sprintf(form,'Item');
    buff=[buff form2];
    MyPrint(Output3,[int buff]);
    for i=1:m
        MyPrint(Output3,interline_cos);
        form=sprintf('%%-%d.%ds ',s,s);   
        buff=sprintf(form,str{i});
        buff=[buff cell];
        buff1=sprintf(' %2.3f %s %2.3f %s ',me(i),cell,Sx2(i),cell);
        buff2 = sprintf('% 2.3f %s % 2.3f %s %s',Skew(i),cell,Kur(i)-3,cell,row);
        if i==m
            MyPrint(Output3,[int buff buff1 buff2]);
            MyPrint(Output3,interline_bottom);
            buff5='\pard \par';
            buff4=[cell cell cell cell cell row];
            MyPrint(Output3,[int buff4 buff5]);
        else
            MyPrint(Output3,[int buff buff1 buff2]);
        end;
    end;
    MyPrint(Output3,'\par');
    kurmax=max(Kur)-3;
    kurmin=min(Kur)-3;
    if kurmax>1 || kurmin<-1
        MyPrint(Output3,'Polychoric correlation is advised when the univariate distributions of ordinal items are asymmetric or with excess of kurtosis. If both indices are lower than one in absolute value, then Pearson correlation is advised. You can read more about this subject in:');
        MyPrint(Output3,'\par');
        MyPrint(Output3,'\par');
        MyPrint(Output3,'  Muthen, B., and Kaplan D. (1985). A comparison of some methodologies for the factor analysis of non-normal Likert variables. British Journal of Mathematical and Statistical Psychology, 38, 171-189. ');
        MyPrint(Output3,'\par');
        MyPrint(Output3,'  Muthen, B., and Kaplan D. (1992). A comparison of some methodologies for the factor analysis of non-normal Likert variables: A note on the size of the model. British Journal of Mathematical and Statistical Psychology, 45, 19-30. ');
        MyPrint(Output3,'\par');
    end;
    MyPrint(Output3,'\par');
    MyPrint(Output3,'\par');
%     
    %SAVE DATA 
    setappdata(0,'data',data);
return;

function ComputeDispersionMatrix(Output,Output2,Output3,Output_matrices)

%% Computing Item Dispersion Matrix %%
%
% ARGUMENTS   : Output  -> plain text listbox
%               Output2 -> LaTeX listbox
%               Output3 -> rtf listbox
%               Output_matrics -> listbox containing relevant matrices
%

    data=getappdata(0,'data');
    
    if ischar(data.labelsa)
        str=cellstr(data.labelsa);
    else
        str=data.labelsa;
    end;
    
    [ml,tmp]=size(str);
     ml = max([ml;tmp]);

    s=0;
    for i=1:ml
        if s < length(str{i})
            s= length(str{i});
        end;
    end;
    if s>30, s=30;end;
    if s<5 , s=5;end;
    
    [f1,f2]=size(data.inclosos);
    numtables=ceil(f2/10); % Number of needed charts for correct printing the
    % correlation matrix in latex (limited by 10 items for chart)
    
    str=num2cell(data.inclosos);
    
     if data.missings_on==1
         dades=data.X_sensemiss;
     else
         dades=data.X;
     end;
    
    switch data.Matrix
        case 1 %COV
            SS = cov(dades);
            data.SS=SS(data.inclosos,data.inclosos);
            data.R=[];
            data.Rp=[];
            Dat=data.SS;
            
        case 2 %PEARSON
            R=corrcoef(dades);
            data.R=R(data.inclosos,data.inclosos);
            data.SS=[];
            data.Rp=[];
            
            Dat=data.R;
            
        case 3 %POLY
            
            % In this case, the latex printing and plaintext are mixed,
            % because it's the most efficient way to do it, using the loops
            
            %%% Output: plaintext 
            %%% Output2: Latex
            
            MyPrint(Output,'');
            MyPrint(Output','BAR CHARTS FOR ORDINAL VARIABLES');
            MyPrint(Output2','\subsection{Bar Charts for Ordinal Variables}');
            MyPrint(Output2,'\usetikzlibrary{patterns}');
            
            %Find the maximum frequence 
            maxf=0;
            [tmp1,tmp2]=size(data.Xa);
            for i=1:tmp2
                table=tabulate(data.Xa(:,i));
                tmp1 = max(table(:,2));
                if maxf < tmp1
                    maxf=tmp1;
                end;
            end;
            
            %get the range of possible scores
            minim=min(data.Xa);
            minim2=min(minim);
            maxim=max(data.Xa);
            maxim2=max(maxim);
            
            range_sco=maxim2-minim2+1;

            str=num2cell(data.inclosos);
                if ischar(data.labelsa)
                    str2=cellstr(data.labelsa);
                else
                    str2=data.labelsa;
                end;
            for i=1:f2
                MyPrint(Output,'');
                if data.labels_user_on==1  % Labels
                    form=sprintf('%%-%d.%ds        ',s,s);   
                    buff2=sprintf(form,str2{i});
                else
                    buff2={};
                end;
                buff = sprintf('Item: %.0f   ',str{i});
                MyPrint(Output,[buff buff2]);
                bartext(data.Xa(:,i),maxf,Output,Output2,range_sco);
                
                a='\caption';
                buff=sprintf('{Bar chart for Item %2.0f}',i);
                MyPrint(Output2,[a buff]);
                MyPrint(Output2,'\end{figure}');
            end;
            MyPrint(Output,' ');
            
            % POLYCHORIC MATRIX
            
            % CHECK IF THE DISPERSION MATRIX IS ALREADY COMPUTED
            
            check_Rp=isempty(data.Rp_all);
            if check_Rp==1 %si esta buida, s'ha de calcular
                
                [Rp,THRES]=polychomap(dades,min(dades),max(dades),3,0,1);
            
                data.R_NPD = 0;
                %NPD check
                D=eig(Rp);
                data.kmo_error=0;
                if min(D) <0
                %NONLINEAR TRANSFORMATION
                    Rp = sr(Rp,data.N);
                    data.R_NPD = 1;
                    data.Rp=Rp;
                    %calcular kmo, si esta en l'ultim interval cancelar
                    %l'analisis i recomanar Pearson
                    kmo_index=kmo(Rp(data.inclosos,data.inclosos));
                    if kmo_index < 0.5
                        data.kmo_error=1;
                    end;
                    
                end; 

                data.Rp=Rp(data.inclosos,data.inclosos);
                data.Rp_all=Rp;
                data.R=[];
                data.SS=[];
                data.thresholds=THRES;
                Dat=data.Rp;
                
            else % If there is already a matrix, select the items in the 
                % analysis order
                R=data.Rp_all;
                
                data.Rp=R(data.inclosos,data.inclosos);
                data.R=[];
                data.SS=[];
                
                Dat = R(data.inclosos,data.inclosos);   
            end;
    end;
    
    %Search for values higher than .95 or lower tha -.95 and remove them
    [f1,f2]=size(Dat);
    buff=[];
    %Correlation matrix without the diagonal ones
    buffs=zeros(f1-1,f1-1);
    for i=1:f1-1 
        if i==1
        else
            buff=Dat(i,1:i-1);
            buff2=zeros(1,f1-i);
            buffs(i,:)=[buff buff2];
        end;
    end;
    r_senseuns=buffs;
    [a,b]=find(r_senseuns>=0.95 | r_senseuns<=-0.95);
    
    if data.Matrix==1
        data.corr_error=0;
    else
        if isempty(a)
            [row, col] = find(isnan(r_senseuns));
            if isempty(row)
            data.corr_error=0;
            data.items_corr_error=[];
            else
                data.corr_error=1;
                [f1,f2]=size(row);
                c=sort(row);
                d=sort(col);
                buffs={};
                buff=[];
                for i=1:f1
                    buff=sprintf('%s and %s', data.labelsa{c(i)},data.labelsa{d(i)});
                    buffs(i)=cellstr(buff);
                end;
                data.corr_error=1;
                data.items_corr_error=buffs;
            end;
        else
            [f1,f2]=size(a);
            c=sort(a);
            d=sort(b);
            buffs={};
            buff=[];
            for i=1:f1
                buff=sprintf('%s and %s', data.labelsa{c(i)},data.labelsa{d(i)});
                buffs(i)=cellstr(buff);
            end;
            data.corr_error=1;
            data.items_corr_error=buffs;
        end;
       
    end;
    

    for l=1:numtables 
        
        leng=l*10;
        
        [f1,f2]=size(Dat);
        
        cof=10*(l-1);

        if cof<f2 && cof+10<f2
            str2=str(1+cof:10+cof);
            Dat2=Dat(:,1+cof:10+cof);
        else
            str2=str(1+cof:f2);
            Dat2=Dat(:,1+cof:f2);
        end;
        
        a=f2/(leng);
        if a>=1
            numb=10;
        elseif a<1
            numb=f2-(10*(l-1));
        end;
        
        %In this case, latex and rtf styles will be generated at the same
        %time, because it's the most eficient way to perform it
        %Output2 = printing in latex
        %Output3 = printing in rtf
        
        %% LATEX and RTF %%
        
        MyPrint(Output2,'\begin{table}[H]');
        switch data.Matrix
            case 1
                buff='\caption{Standarized Variance/Covariance Matrix';
                buff_rtf='\qc \ul Standarized Variance/Covariance Matrix\ul0';
                buff2=sprintf(' (Part %.0f of %.0f )}',l,numtables);
                buff2_rtf=sprintf(' (Part %.0f of %.0f )',l,numtables);
                if numtables==1
                    MyPrint(Output2,buff);
                    MyPrint(Output3,buff_rtf);
                else
                    MyPrint(Output2,[buff buff2]);
                    MyPrint(Output3,[buff_rtf buff2_rtf]);
                end;
            case 2
                buff='\caption{Standarized Variance/Covariance Matrix (Pearson Correlation)';
                buff_rtf='\qc \ul Standarized Variance/Covariance Matrix (Pearson Correlation)\ul0';
                buff2=sprintf(' (Part %.0f of %.0f )}',l,numtables);
                buff2_rtf=sprintf(' (Part %.0f of %.0f )',l,numtables);
                if numtables==1
                    MyPrint(Output2,buff);
                    MyPrint(Output3,buff_rtf);
                else
                    MyPrint(Output2,[buff buff2]);
                    MyPrint(Output3,[buff_rtf buff2_rtf]);
                end;
            case 3
                buff='\caption{Standarized Variance/Covariance Matrix (Polychoric Correlation)';
                buff_rtf='\qc \ul Standarized Variance/Covariance Matrix (Polychoric Correlation)\ul0';
                buff2=sprintf(' (Part %.0f of %.0f )}',l,numtables);
                buff2_rtf=sprintf(' (Part %.0f of %.0f )',l,numtables);
                if numtables==1
                    MyPrint(Output2,buff);
                    MyPrint(Output3,buff_rtf);
                else
                    MyPrint(Output2,[buff buff2]);
                    MyPrint(Output3,[buff_rtf buff2_rtf]);
                end;
        end;
        MyPrint(Output3,'\par');
        MyPrint(Output3,'\par');
        MyPrint(Output3,'\ql');
        interline_cap=[];
        for i=1:numb+1
            if i==1
                buff='\trowd\trgaph72\clbrdrt\brdrs\clbrdrl\brdrs\clbrdrb\brdrs\clbrdrr\brdrs\';
            else
                buff='\clbrdrt\brdrs\clbrdrl\brdrs\clbrdrb\brdrs\clbrdrr\brdrs\';
            end;
            buff2=sprintf('cellx%.0f',910*i);
            buff3=[buff buff2];
            interline_cap=[interline_cap buff3];
        end;
        int='\intbl ';
        cell='\cell';
        row='\row';
        pard='\pard \par';
        
        MyPrint(Output2,'\label{taula:matriu}');
        MyPrint(Output2,'\centering');

        buff9 = '\begin{tabular}{l*{';
        buff4 = '\\';
        buff=sprintf('%.0f}{c}}',numb);
        MyPrint(Output2,[buff9 buff buff4]);

        buff=sprintf('Item ');
        buff_rtf='Item \cell';
        dif=f2-cof;
        if dif>10
            for i=1:10
                buff2=sprintf('&  %-5.0f  ',str2{i});
                buff=[buff buff2];
                buff_rtf1=sprintf(' %-5.0f',str2{i});
                buff_rtf=[buff_rtf buff_rtf1 cell];
            end;
        else
            for i=1:f2-cof
                buff2=sprintf('&  %-3.0f  ',str2{i});
                buff=[buff buff2];
                buff_rtf1=sprintf(' %-3.0f',str2{i});
                buff_rtf=[buff_rtf buff_rtf1 cell];
            end;
        end;
        MyPrint(Output2,[buff buff4]);
        MyPrint(Output2,'');
        
        MyPrint(Output3,interline_cap);
        MyPrint(Output3,[int buff_rtf row]);
        
        [f1,f2]=size(Dat2);
        for i=1:f1-cof
            buff0=sprintf('I%3.0f ',str{i+cof}); 
            if i<=10
                buff2=sprintf('& % 5.3f   ',Dat2(i+cof,1:i));
                buff_rtf=sprintf(' % 4.3f \\cell',Dat2(i+cof,1:i)); 
                %Els 1.000 que es mostrin com a 1
                if data.Matrix==2 || data.Matrix==3
                    [h1,h2]=size(buff2);
                    buff2=buff2(1:h2-7);
                    [j1,j2]=size(buff_rtf);
                    buff_rtf=buff_rtf(1:j2-10);
                end;
                a=' \cell';
                buff_rtf=[buff_rtf a];
            else
                buff2=sprintf('& % 5.3f   ',Dat2(i+cof,1:10));
                buff_rtf=sprintf(' % 4.3f \\cell',Dat2(i+cof,1:10));
            end;
            
            emptycells=[];
            for j=1:f2-i
                buff9=sprintf('%s',cell);
                emptycells=[emptycells buff9];
            end;
            numberofemptycells=f2-i;
            numberoffullcells=f2-numberofemptycells;

            interline_fila=[];
            for k=1:numb+1
                if k==1
                    buff='\trowd\trgaph72\clbrdrt\brdrs\clbrdrl\brdrs\clbrdrb\brdrs\clbrdrr\brdrs\'; %first one, with the number of the item
                else
                    if k<=numberoffullcells+1
                        buff='\clbrdrt\brdrs\clbrdrl\brdrs\clbrdrb\brdrs\clbrdrr\brdrs\'; %Cells with names, with borders
                    else
                        buff='\';                                                       %empty cells, without borders
                    end;
                end;
                buff7=sprintf('cellx%.0f',910*k);
                buff3=[buff buff7];
                interline_fila=[interline_fila buff3];
            end;
            
            MyPrint(Output2,[buff0 buff2 buff4]);
            
            MyPrint(Output3,interline_fila);
            if i==f1-cof
                MyPrint(Output3,[int buff0 cell buff_rtf emptycells row pard]);    
            else
                MyPrint(Output3,[int buff0 cell buff_rtf emptycells row]);
            end;
        end;
        MyPrint(Output2,'\end{tabular}');
        MyPrint(Output2,'\end{table}');
        
        MyPrint(Output3,'\par');
        

    end;
    
     %% TEXT %%
            
        MyPrint(Output,'');
        switch data.Matrix
            case 1
                MyPrint(Output','STANDARIZED VARIANCE / COVARIANCE MATRIX');
            case 2
                MyPrint(Output','STANDARIZED VARIANCE / COVARIANCE MATRIX (PEARSON CORRELATION)');
            case 3
                MyPrint(Output','STANDARIZED VARIANCE / COVARIANCE MATRIX (POLYCHORIC CORRELATION)');
        end;        
        MyPrint(Output,'');
        buff=sprintf('Item        ');
        [f1,f2]=size(Dat);
        for i=1:f2
            buff2=sprintf('%-5.0f    ',str{i});
            buff=[buff buff2];
        end;
        MyPrint(Output,buff);
        MyPrint(Output,'');
        for i=1:f2
            buff=sprintf('I%3.0f     ',str{i});     
            buff2=sprintf('% 5.3f   ',Dat(i,1:i));
            % 1.000 as 1
            if data.Matrix==2 || data.Matrix==3
                [h1,h2]=size(buff2);
                buff2=buff2(1:h2-8);
                a=sprintf('%1.0f',round(Dat(i,i)));
                MyPrint(Output,[buff buff2 a]);
            else
                MyPrint(Output,[buff buff2]);
            end;
        end;
        MyPrint(Output,'');

        data.ref=0; %no additional references required
 
        if data.Matrix==3
            if data.R_NPD==1
                MyPrint(Output,'-----------------------------------------------------------------------');
                MyPrint(Output,'');
                MyPrint(Output,'WARNING: Correlation matrix is not positive-definite,');
                MyPrint(Output,'         Smothing algorithm was applied');
                MyPrint(Output,'         (Devlin, Gnanadesikan, & Kettenring, 1975; Devlin, Gnanadesikan, & Kettenring, 1981)');
                MyPrint(Output,'');
                data.ref=1; % additional references required
            end;
        end;
    MyPrint(Output,'-----------------------------------------------------------------------');
    
    PrintMatrix_apart(Output_matrices,Dat)
        
    setappdata(0,'data',data);
return

function PrintMatrix_apart (Output_matrices,Dat)

%% Print Matrix in the matrices file %%
% ARGUMENTS   : Output_matrics -> listbox containing relevant matrices
%               Dat -> Data used
%

    data=getappdata(0,'data');
    str=num2cell(data.inclosos);

    MyPrint(Output_matrices,'');
        switch data.Matrix
            case 1
                MyPrint(Output_matrices','STANDARIZED VARIANCE / COVARIANCE MATRIX');
            case 2
                MyPrint(Output_matrices','STANDARIZED VARIANCE / COVARIANCE MATRIX (PEARSON CORRELATION)');
            case 3
                MyPrint(Output_matrices','STANDARIZED VARIANCE / COVARIANCE MATRIX (POLYCHORIC CORRELATION)');
        end;        
        MyPrint(Output_matrices,'');
        buff=sprintf('Item        ');
        [f1,f2]=size(Dat);
        for i=1:f2
            buff2=sprintf('%-5.0f    ',str{i});
            buff=[buff buff2];
        end;
        MyPrint(Output_matrices,buff);
        MyPrint(Output_matrices,'');
        for i=1:f2
            buff=sprintf('I%3.0f     ',str{i});     
            buff2=sprintf('% 5.3f   ',Dat(i,1:i));
            % 1.000 as 1
            if data.Matrix==2 || data.Matrix==3
                [h1,h2]=size(buff2);
                buff2=buff2(1:h2-8);
                a=sprintf('%1.0f',round(Dat(i,i)));
                MyPrint(Output_matrices,[buff buff2 a]);
            else
                MyPrint(Output_matrices,[buff buff2]);
            end;
        end;
        MyPrint(Output_matrices,'');

return;

function PrintUserMatrix_apart (Output_matrices,Sa)

%% Printer User Matrix in the matrix file %%
% ARGUMENTS   : Output_matrics -> listbox containing relevant matrices
%               Sa -> Dispersion matrix provided by the user
%

    data=getappdata(0,'data');
    MyPrint(Output_matrices,' ');
    MyPrint(Output_matrices,'DISPERSION MATRIX (USER DEFINED)');
    MyPrint(Output_matrices,' ');
    buff=sprintf('Item        ');
    [f1,f2]=size(data.inclosos);
    str=num2cell(data.inclosos);
    for i=1:f2
        buff2=sprintf('%-5.0f    ',str{i});
        buff=[buff buff2];
    end;
    MyPrint(Output_matrices,buff);
    MyPrint(Output_matrices,'');
    [f1,f2]=size(Sa);
    for i=1:f1
        buff=sprintf('I%3.0f     ',str{i});
        buff2=sprintf('% 5.3f   ',Sa(i,1:i));
        MyPrint(Output_matrices,[buff buff2]);
    end;
    MyPrint(Output_matrices,'');
return;

function PrintUserMatrix (Output,Output2,Output3,Output_matrices)
    
%% Print User Matrix %%
% ARGUMENTS   : Output  -> plain text listbox
%               Output2 -> LaTeX listbox
%               Output3 -> rtf listbox
%               Output_matrics -> listbox containing relevant matrices
%

    data=getappdata(0,'data');
    
    [f1,f2]=size(data.inclosos);
    numtables=ceil(f2/10); %number of tables needed for printing the correlation matrix (max 10 per table)
    
    str=num2cell(data.inclosos);
        
    Dat = data.Sa;   
    
    %Search for values higher than .95 or lower than -.95 and remove them
   
    [f1,f2]=size(Dat);
    buff=[];
    %correlation matrix without the diagonal ones
    buffs=zeros(f1-1,f1-1);
    for i=1:f1-1 
        if i==1
        else
            buff=Dat(i,1:i-1);
            buff2=zeros(1,f1-i);
            buffs(i,:)=[buff buff2];
        end;
    end;
    r_senseuns=buffs;
    [a,b]=find(r_senseuns>=0.95 | r_senseuns<=-0.95);
    
    if isempty(a)
        [row, col] = find(isnan(r_senseuns));
        if isempty(row)
        data.corr_error=0;
        data.items_corr_error=[];
        else
            data.corr_error=1;
            [f1,f2]=size(row);
            c=sort(row);
            d=sort(col);
            buffs={};
            buff=[];
            for i=1:f1
                buff=sprintf('%s and %s', data.labelsa{c(i)},data.labelsa{d(i)});
                buffs(i)=cellstr(buff);
            end;
            data.corr_error=1;
            data.items_corr_error=buffs;
        end;
    else
        data.corr_error=1;
        [f1,f2]=size(a);
        c=sort(a);
        d=sort(b);
        buffs={};
        buff=[];
        for i=1:f1
            buff=sprintf('%s and %s', data.labelsa{c(i)},data.labelsa{d(i)});
            buffs(i)=cellstr(buff);
        end;
        data.corr_error=1;
        data.items_corr_error=buffs;
       
    end;
    
    for l=1:numtables
        
        leng=l*10;
        
        [f1,f2]=size(Dat);
        
        cof=10*(l-1);

        if cof<f2 && cof+10<f2
            str2=str(1+cof:10+cof);
            Dat2=Dat(:,1+cof:10+cof);
        else
            str2=str(1+cof:f2);
            Dat2=Dat(:,1+cof:f2);
        end;
        
        a=f2/(leng);
        if a>1
            numb=10;
        elseif a<1
            numb=f2-(10*(l-1));
        end;
        
        %In this case, latex and rtf styles will be generated at the same
        %time, because it's the most eficient way to perform it
        %Output2 = printing in latex
        %Output3 = printing in rtf
        
        %% LATEX and RTF %%
   
        MyPrint(Output2,'\begin{table}[!htb]');
        buff='\caption{Dispersion Matrix (User defined)}';
        MyPrint(Output3,'\par');MyPrint(Output3,'\par');
        buff_rtf='\qc \ul Dispersion Matrix (User defined) \ul0';
        buff2=sprintf('(Part %.0f of %.0f )',l,numtables);
        if numtables==1
            MyPrint(Output2,buff);
            MyPrint(Output3,buff_rtf);
        else
            MyPrint(Output2,[buff buff2]);
            MyPrint(Output3,[buff_rtf buff2]);
        end;
        MyPrint(Output3,'\par');
        MyPrint(Output3,'\par');
        MyPrint(Output3,'\ql');
        interline_cap=[];
        for i=1:numb+1
            if i==1
                buff='\trowd\trgaph72\clbrdrt\brdrs\clbrdrl\brdrs\clbrdrb\brdrs\clbrdrr\brdrs\';
            else
                buff='\clbrdrt\brdrs\clbrdrl\brdrs\clbrdrb\brdrs\clbrdrr\brdrs\';
            end;
            buff2=sprintf('cellx%.0f',910*i);
            buff3=[buff buff2];
            interline_cap=[interline_cap buff3];
        end;
        int='\intbl ';
        cell='\cell';
        row='\row';
        pard='\pard \par';
        
        MyPrint(Output2,'\label{taula:matriu}');
        MyPrint(Output2,'\centering');

        buff9 = '\begin{tabular}{l*{';
        buff4 = '\\';
        buff=sprintf('%.0f}{c}}',numb);
        MyPrint(Output2,[buff9 buff buff4]);

        buff=sprintf('Item ');
        buff_rtf='Item \cell';
        dif=f2-cof;
        if dif>10
            for i=1:10
                buff2=sprintf('&  %-5.0f  ',str2{i});
                buff=[buff buff2];
                buff_rtf1=sprintf(' %-5.0f',str2{i});
                buff_rtf=[buff_rtf buff_rtf1 cell];
            end;
        else
            for i=1:f2-cof
                buff2=sprintf('&  %-3.0f  ',str2{i});
                buff=[buff buff2];
                buff_rtf1=sprintf(' %-3.0f',str2{i});
                buff_rtf=[buff_rtf buff_rtf1 cell];
            end;
        end;
        MyPrint(Output2,[buff buff4]);
        MyPrint(Output2,'');
        
        MyPrint(Output3,interline_cap);
        MyPrint(Output3,[int buff_rtf row]);
        
        [f1,f2]=size(Dat2);
        for i=1:f1-cof
            buff0=sprintf('I%3.0f ',str{i+cof}); 
            if i<=10
                buff2=sprintf('& % 5.3f   ',Dat2(i+cof,1:i));
                buff_rtf=sprintf(' % 4.3f \\cell',Dat2(i+cof,1:i));

                a=' \cell';
                buff_rtf=[buff_rtf a];
            else
                buff2=sprintf('& % 5.3f   ',Dat2(i+cof,1:10));
                buff_rtf=sprintf(' % 4.3f \\cell',Dat2(i+cof,1:10));
            end;
            
            emptycells=[];
            for j=1:f2-i
                buff9=sprintf('%s',cell);
                emptycells=[emptycells buff9];
            end;
            numberofemptycells=f2-i;
            numberoffullcells=f2-numberofemptycells;

            interline_fila=[];
            for k=1:numb+1
                if k==1
                    buff='\trowd\trgaph72\clbrdrt\brdrs\clbrdrl\brdrs\clbrdrb\brdrs\clbrdrr\brdrs\'; 
                else
                    if k<=numberoffullcells+1
                        buff='\clbrdrt\brdrs\clbrdrl\brdrs\clbrdrb\brdrs\clbrdrr\brdrs\'; 
                    else
                        buff='\';                                                       
                    end;
                end;
                buff2=sprintf('cellx%.0f',910*k);
                buff3=[buff buff2];
                interline_fila=[interline_fila buff3];
            end;
            
            MyPrint(Output2,[buff buff2 buff4]);
            
            MyPrint(Output3,interline_fila);
            if i==f1-cof
                MyPrint(Output3,[int buff0 cell buff_rtf emptycells row pard]);    
            else
                MyPrint(Output3,[int buff0 cell buff_rtf emptycells row]);
            end;
        end;
        
        MyPrint(Output2,'\end{tabular}');
        MyPrint(Output2,'\end{table}');
        
        MyPrint(Output3,'\par');

    end; 
    
    %% TEXT %%
    
    Sa=data.Sa;
    
    MyPrint(Output,' ');
    MyPrint(Output,'DISPERSION MATRIX (USER DEFINED)');
    MyPrint(Output,' ');
    buff=sprintf('Item        ');
    [f1,f2]=size(data.inclosos);
    str=num2cell(data.inclosos);
    for i=1:f2
        buff2=sprintf('%-5.0f    ',str{i});
        buff=[buff buff2];
    end;
    MyPrint(Output,buff);
    MyPrint(Output,'');
    [f1,f2]=size(Sa);
    for i=1:f1
        buff=sprintf('I%3.0f     ',str{i});
        buff2=sprintf('% 5.3f   ',Sa(i,1:i));
        MyPrint(Output,[buff buff2]);
    end;
    MyPrint(Output,'');
    MyPrint(Output,'-----------------------------------------------------------------------');
    
    data.R=Sa;

    PrintUserMatrix_apart(Output_matrices,Sa);

    setappdata(0,'data',data);

return;

function data = ClearFieldsData(data)

%% Clear fields %%

    if (isfield(data, 'R') == 1), data = rmfield(data, 'R'); end;
%     if (isfield(data, 'Rp') == 1), data = rmfield(data, 'Rp'); end;
%     if (isfield(data, 'thresholds') == 1), data = rmfield(data, 'thresholds'); end;
%     if (isfield(data, 'R_NPD') == 1), data = rmfield(data, 'R_NPD'); end;
    if (isfield(data, 'A') == 1), data = rmfield(data, 'A'); end;
    if (isfield(data, 'Re') == 1), data = rmfield(data, 'Re'); end;
    if (isfield(data, 'As') == 1), data = rmfield(data, 'As'); end;
    if (isfield(data, 'Res') == 1), data = rmfield(data, 'Res'); end;
    if (isfield(data, 'SS') == 1), data = rmfield(data, 'SS'); end;
    if (isfield(data, 'PHI') == 1), data = rmfield(data, 'PHI'); end;
    if (isfield(data, 'mean') == 1), data = rmfield(data, 'mean'); end;
    if (isfield(data, 'Sx2') == 1), data = rmfield(data, 'Sx2'); end;
    if (isfield(data, 'Skew') == 1), data = rmfield(data, 'Skew'); end;
    if (isfield(data, 'Kur') == 1), data = rmfield(data, 'Kur'); end;
    if (isfield(data, 'th_li') == 1), data = rmfield(data, 'th_li'); end;
    if (isfield(data, 'th_ls') == 1), data = rmfield(data, 'th_ls'); end;
    if (isfield(data, 'se') == 1), data = rmfield(data, 'se'); end;
    if (isfield(data, 'reli') == 1), data = rmfield(data, 'reli'); end;
    if (isfield(data, 'ref') == 1), data = rmfield(data, 'ref'); end;
    if (isfield(data, 'Lz') == 1), data = rmfield(data, 'Lz'); end;
    if (isfield(data, 'Lc') == 1), data = rmfield(data, 'Lc'); end;
    if (isfield(data, 'WLSRegression') == 1), data = rmfield(data, 'WLSRegression'); end;
    
      
return;

function bartext(X,maxf,Output,Output2,range_sco)

%% Bar text printing %%
% ARGUMENTS   : X  -> Data to print
%               Output  -> plain text listbox
%               Output2 -> LaTeX listbox
%               range_sco -> rank of the scores
%

    ABC = 40;

    table=tabulate(X(:,1));
    
    [f1,~]=size(table(:,1));
    
    ORD = f1;

    if nargin<3 
        maxf = max(table(:,2));
    end;

    uni  = ABC/maxf;
    
    %% LATEX %%    
    
    MyPrint(Output2,'\begin{figure}[H]');
    MyPrint(Output2,'\centering');
    MyPrint(Output2,'\begin{tikzpicture}');
    MyPrint(Output2,' \begin{axis}[xbar, ');
    MyPrint(Output2,'  xlabel=$Frequency$,');
    MyPrint(Output2,'  ylabel=$Values$,');
    MyPrint(Output2,'  width=0.8\textwidth,');
    MyPrint(Output2,'  height=0.4\textwidth,');
    MyPrint(Output2,'  symbolic y coords={0,');
    for i=1:ORD
        buff = sprintf('    %2.0f,',i);
        MyPrint(Output2,buff);
    end;
    buff=sprintf('    %2.0f},',(ORD)+1);
    MyPrint(Output2,buff);
    MyPrint(Output2,'  nodes near coords, nodes near coords align={horizontal},');
    MyPrint(Output2,'  xmin=0,');
    buff=sprintf('  xmax=%4.0f,',maxf+5);
    MyPrint(Output2,buff);
    MyPrint(Output2,'  ymin=0,');
    buff=sprintf('  ymax=%2.0f,',ORD+1);
    MyPrint(Output2,buff)
    MyPrint(Output2,'  y dir=reverse,');
    if ORD<=5
        MyPrint(Output2,'  bar width=16pt,');
    else
        MyPrint(Output2,'  bar width=12pt,');
    end;
    buff = sprintf('  ytick={%2.0f,...,1}',ORD);  
    MyPrint(Output2, buff);
    MyPrint(Output2,'] ');
    MyPrint(Output2,'   \addplot [draw=blue,pattern=horizontal lines light blue] coordinates{');
    for i=1:ORD
        buff = sprintf('    (%3.0f,%1.0f)',table(i,2),i);
        MyPrint(Output2,buff);
    end;
    MyPrint(Output2,'}; ');
    MyPrint(Output2,' \end{axis}');
    MyPrint(Output2,'\end{tikzpicture}');    
    
    

    %% TEXT %%
    
    buff=sprintf('\n Value     Freq  Percent');
    MyPrint(Output,buff);
    buff=sprintf('                         |');
    MyPrint(Output,buff);
    for i=1:ORD
        if table(i,2)==0 %no frequency, no printing
        else
          str = '';
          tot = table(i,2)*uni;
          for j=1:tot
              str(j) = char('*');
          end;
          buff=sprintf(' %5.0f    %4.0f  %6.2f   | %s',table(i,1),table(i,2),table(i,3),str);
          MyPrint(Output,buff);
        end;
    end;
    buff=sprintf('                         +-----------+---------+---------+-----------+\n');
    MyPrint(Output,buff);
    buff=sprintf('                         0        %3.1f       %3.1f       %3.1f        %3.1f\n\n',maxf/4.0,maxf/2.0,maxf/4.0 + maxf/2.0,maxf);
    MyPrint(Output,buff);
    if ORD<range_sco
        MyPrint(Output,' ');
        MyPrint(Output,'     Warning: Not all the categories are observed in this item');
        MyPrint(Output,'              You should consider to remove this variable from the analysis');
    end;
        
    
return;

function DispersionMatrixAdecuation(Output,Output2,Output3,A,R)

%% Dispersion Matrix Adecuation Indices %%
% ARGUMENTS   : Output  -> plain text listbox
%               Output2 -> LaTeX listbox
%               Output3 -> rtf listbox
%               A -> Loading Matrix
%               R -> Dispersion Matrix
%

    data=getappdata(0,'data');
    
    N=data.N;
    m=data.n;
    d = det(R);
    %Bartlett Sfericity
    chi = -1.0 * (N - 1 - (2*m + 5)/6 )*log(d);
    df = m*(m-1)/2;
    pro = 1-chi2cdf(chi,df);
    
    [f1,f2]=size(A);
    if f1==f2
        A = diag(diag(A))^(-1/2) * A * diag(diag(A))^(-1/2);
    end;
    %KMO
    kmo_index = kmo(A);

    data.kmo = kmo_index;
    
    %% LATEX %%  
    
    MyPrint(Output2,'\begin{table}[H]');
    MyPrint(Output2,'\caption{Adequacy of the Dispersion Matrix}');
    MyPrint(Output2,'\label{taula:descriptives}');
    MyPrint(Output2,'\centering');
    MyPrint(Output2,'\begin{tabular}{l l} \toprule');
    buff2 = '\\';
    buff = sprintf('Determinant of the matrix= & %17.15f',d);
    MyPrint(Output2,[buff buff2]);
    buff = sprintf('Bartlett''s statistic= & %7.1f (df = %5.0f; P = %7.6f)',chi,df,pro);
    MyPrint(Output2, [buff buff2]);
    buff = sprintf('Kaiser-Meyer-Olkin (KMO) test = & %7.5f ',kmo_index);
    if     (kmo_index >= 0.9), buff1 = sprintf('(very good)');
    elseif (kmo_index >= 0.8), buff1 = sprintf('(good)');
    elseif (kmo_index >= 0.7), buff1 = sprintf('(fair)');
    elseif (kmo_index >= 0.6), buff1 = sprintf('(mediocre)');
    elseif (kmo_index >= 0.5), buff1 = sprintf('(bad)');
    else,                      buff1 = sprintf('(inaceptable)');    
    end;
    MyPrint(Output2,[buff buff1 buff2]);
    MyPrint(Output2,'\bottomrule');
    MyPrint(Output2,'\end{tabular}');
    MyPrint(Output2,'\end{table}');
    
    %% TEXT %%
    
    MyPrint(Output,' ');
    MyPrint(Output,'ADEQUACY OF THE DISPERSION MATRIX');
    MyPrint(Output,' ');
    buff = sprintf('Determinant of the matrix     = %17.15f',d);
    MyPrint(Output,buff);
    buff = sprintf('Bartlett''s statistic          = %7.1f (df = %5.0f; P = %7.6f)',chi,df,pro);
    MyPrint(Output,buff);
    buff1 = sprintf('Kaiser-Meyer-Olkin (KMO) test = %7.5f ',kmo_index);
    if     (kmo_index >= 0.9), buff2 = sprintf('(very good)');
    elseif (kmo_index >= 0.8), buff2 = sprintf('(good)');
    elseif (kmo_index >= 0.7), buff2 = sprintf('(fair)');
    elseif (kmo_index >= 0.6), buff2 = sprintf('(mediocre)');
    elseif (kmo_index >= 0.5), buff2 = sprintf('(bad)');
    else,                      buff2 = sprintf('(inaceptable)');    
    end;
    MyPrint(Output,[buff1 buff2]);
    MyPrint(Output,' ');
    MyPrint(Output,'-----------------------------------------------------------------------');
    
    %% RTF %%
    
    int='\intbl ';
    cell='\cell ';
    row='\row';
    par='\par';
    pard='\pard';
    
    MyPrint(Output3,par);
    MyPrint(Output3,'\qc \ul Adequacy of the Dispersion Matrix \ul0');
    MyPrint(Output3,par);
    MyPrint(Output3,par);
    MyPrint(Output3,'\ql');

    interline='\trowd\trgaph72\clbrdrt\brdrs\clbrdrl\brdrs\clbrdrb\brdrs\clbrdrr\brdrs\cellx4000\clbrdrt\brdrs\clbrdrl\brdrs\clbrdrb\brdrs\clbrdrr\brdrs\cellx8000';
    MyPrint(Output3,interline);
    buff = sprintf('Determinant of the matrix: %s %17.15f %s',cell,d,cell);
    MyPrint(Output3,[int buff row]);
    MyPrint(Output3,interline);
    buff = sprintf('Bartlett''s statistic: %s %5.1f (df = %4.0f; P = %7.6f )%s',cell,chi,df,pro,cell);
    MyPrint(Output3,[int buff row]);
    buff1 = sprintf('Kaiser-Meyer-Olkin (KMO) test: %s %7.5f',cell,kmo_index);
    if     (kmo_index >= 0.9), buff2 = sprintf('(very good)');
    elseif (kmo_index >= 0.8), buff2 = sprintf('(good)');
    elseif (kmo_index >= 0.7), buff2 = sprintf('(fair)');
    elseif (kmo_index >= 0.6), buff2 = sprintf('(mediocre)');
    elseif (kmo_index >= 0.5), buff2 = sprintf('(bad)');
    else                     , buff2 = sprintf('(inaceptable)');    
    end;
    MyPrint(Output3,interline);
    MyPrint(Output3,[int buff1 buff2 cell row pard par]);
    
return

function Printgrafic_residuals(A)

%% Print residuals plot %%
% ARGUMENTS   : A -> Standarized Residuals
%

    data=getappdata(0,'data');

    finestra.N=760;
    finestra.M=600;
    pantalla=get(0,'ScreenSize');
    finestra.coordenades=[(pantalla(3)/2)-(finestra.N/1.5)  (pantalla(4)/2)-(finestra.M/2) finestra.N/1.2 finestra.M/1.3];

    handles.PAfig=figure('Units','Pixels',...
        'color' , [0.941 0.941 0.941],...
        'Position',finestra.coordenades,...
        'MenuBar','none',...
        'Toolbar','figure');
    
    set(handles.PAfig,'Name','Psychological Test Toolbox: Residuals','NumberTitle','off');

    hAxes=axes('Parent',handles.PAfig,...
        'units','normalized',...
        'Position',[.1 .1 .85 .825]);

    ticks_rounded = floor(min(A)):0.5:ceil(max(A));

    [f1,f2]=size(ticks_rounded);
    if f2>20
        ticks_rounded = floor(min(A)):1:ceil(max(A));
    end;

    histogram(A,ticks_rounded);
    set(hAxes,'XLimMode','manual',...
        'XLim',[floor(min(A))-1 ceil(max(A))+1],...
        'XTickMode','manual',...
        'XTick',ticks_rounded,...
        'NextPlot','add',...
        'Fontname',char(data.font),...
        'Fontsize',12);
    
    
    title('Distribution of Standarized Residuals');
    xlabel('Value');
    ylabel('Frequency');

function Printgrafic(dim,real,mean,Output2)

%% Print plot %%
% ARGUMENTS   : dim  -> dimensions
%               real -> Real % of Explained Variance
%               mean -> Mean % of Explained Variance based on PA
%               Output2 -> LaTeX listbox
%

    data=getappdata(0,'data');

    finestra.N=760;
    finestra.M=600;
    pantalla=get(0,'ScreenSize');
    finestra.coordenades=[(pantalla(3)/2)-(finestra.N/2)  (pantalla(4)/2)-(finestra.M/2) finestra.N/1.3 finestra.M/1.4];

    handles.PAfig=figure('Units','Pixels',...
        'color' , [0.941 0.941 0.941],...
        'Position',finestra.coordenades,...
        'Toolbar','figure',...
        'Menubar','none');
    
    set(handles.PAfig,'Name','Psychological Test Toolbox: Parallel Analysis Plot','NumberTitle','off');

    hAxes=axes('Parent',handles.PAfig,...
        'units','normalized',...
        'Position',[.1 .1 .85 .825]);

    maxeig1=max(real,mean);
    maxeig=max(maxeig1);
    set(hAxes,'XLimMode','manual',...
        'YLimMode','manual',...
        'XLim',[0 (dim+1)],...
        'YLim',[0 (maxeig+1)],...
        'XTick',(1:dim),...
        'NextPlot','add',...
        'Fontname',char(data.font),...
        'Fontsize',12);
        title('Parallel Analysis');
        xlabel('Dimensions');
        ylabel('Percentage of Variance');
    
    handles.hline1 = line((1:dim),real,'LineStyle','-',...
        'LineWidth',2,...
        'Color',[.2 .4 .6]);
    handles.hline2 = line((1:dim),mean,'LineStyle','- -',...
        'LineWidth',2,...
        'Color',[.6 .2 .2]);
    handles.plot1 = plot((1:dim),real,'Marker','o');
    
    legend('Real','Parallel Analysis')

    %% LATEX %%
    
    MyPrint(Output2,'\begin{figure}[H]');
    MyPrint(Output2,'\centering');
    MyPrint(Output2,'\begin{tikzpicture}');
    MyPrint(Output2,' \begin{axis}[ ');
    MyPrint(Output2,'  xlabel=$Dimensions$,');
    MyPrint(Output2,'  ylabel=$Percentage\ of\ Variance$,');
    MyPrint(Output2,'  width=0.8\textwidth,');
    MyPrint(Output2,'  height=0.6\textwidth,');
    MyPrint(Output2,'  legend entries={$Real$,$Parallel\ Analysis$},');
    buff = sprintf('  xtick={1,...,%2.0f}',dim);  
    MyPrint(Output2, buff);
    MyPrint(Output2,'] ');
    MyPrint(Output2,'   \addplot coordinates{');
    for i=1:dim
        buff = sprintf('    (%2.0f,%4.1f)',i,real(i));
        MyPrint(Output2,buff);
    end;
    MyPrint(Output2,'}; ');
    MyPrint(Output2,'   \addplot coordinates{');
    for i=1:dim
        buff = sprintf('    (%2.0f,%4.1f)',i,mean(i));
        MyPrint(Output2,buff);
    end;
    MyPrint(Output2,'}; ');
    MyPrint(Output2,' \end{axis}');
    MyPrint(Output2,'\end{tikzpicture}');    
    MyPrint(Output2,'\caption{Parallel Analysis}');
    MyPrint(Output2,'\end{figure}');
   

return;

function PrintMissingData(Output,Output2,Output3)

%% Print missing data summary %%
% ARGUMENTS   : Output  -> plain text listbox
%               Output2 -> LaTeX listbox
%               Output3 -> rtf listbox
%

    data=getappdata(0,'data');
    
    X_miss = data.X(:,data.inclosos) == data.missings_value;
    missings=find(X_miss == 1);
    [f1,f2]=size(missings);
    missings_number=f1;
    
    
    buff=[];
    [f1,f2]=size(X_miss);
    %Per als subjectes
    for i=1:f1
        [f1,f2]=find(X_miss(i,:)==1);
        if f1>0
            [f1,f2]=size(f2);
            buff1=str2double(sprintf('%5.0f',i));
            buff2=str2double(sprintf('%5.0f',f2));
            buff=([buff;buff1,buff2]);
        end;
    end;
    missings_matrix=buff;
    
    % BY ITEMS
    buff=[];
    [f1,f2]=size(X_miss);
    for i=1:f2
        [f1,f2]=find(X_miss(:,i)==1);
        [f1,f2]=size(f1);
        buff1=str2double(sprintf('%5.0f',i));
        buff2=str2double(sprintf('%5.0f',f1));
        buff=([buff;buff1,buff2]);
    end;
    missings_matrix2=buff;


    [f1,f2]=size(data.X);
    data.N_original=f1;

    setappdata(0,'data',data);
    
    % LABELS
    if ischar(data.labelsa)
        str=cellstr(data.labelsa);
    else
        str=data.labelsa;
    end;


    [ml,tmp]=size(str);
     ml = max([ml;tmp]);

    s=0;
    for i=1:ml
        if s < length(str{i})
            s= length(str{i});
        end;
    end;
    if s>30, s=30;end;
    if s<5 , s=5;end;
    
    
    %% TEXT %%
    
    MyPrint(Output,'DESCRIPTIVES RELATED TO MISSING DATA');
    MyPrint(Output,' ');
    
    buff = sprintf('Missing value code                          : %5.0f',data.missings_value); MyPrint(Output, buff);
    buff = sprintf('Total number of missing data                : %5.0f',missings_number); MyPrint(Output, buff);
    buff = 'Method to handle missing data               : Hot-Deck Multiple Imputation in Exploratory Factor Analysis (Lorenzo-Seva & Van Ginkel, 2015)';
    MyPrint(Output, buff);
    MyPrint(Output,' ');
    MyPrint(Output,' ');
    MyPrint(Output,'NUMBER OF VARIABLES MISSING PER CASE');
    MyPrint(Output,' ');
    MyPrint(Output,'Number of variables     Frequency');
    MyPrint(Output,'            missing');
    
    buff=max(missings_matrix);
    maxmissing=buff(1,2);
    MyPrint(Output,'');
    perc=(data.N/data.N_original)*100;
    buff=sprintf('                  0  |    %5.0f (%4.1f%%)',data.N,perc);
    MyPrint(Output,buff);
    for i=1:maxmissing
        buff=histc(missings_matrix,i);
        if buff(2)==0
            buff=[];
        else
            buff1=(buff(2)/data.N_original)*100;
            buff2=sprintf('                  %.0f  |    %5.0f (%4.1f%%)',i,buff(2),buff1);
            MyPrint(Output,buff2);
        end;
    end;
    MyPrint(Output,' ');
    MyPrint(Output,'NUMBER OF CASES MISSING PER VARIABLE');
    MyPrint(Output,' ');
    form=sprintf('%%-%d.%ds                Frequency of cases missing',s,s);
    buff=sprintf(form,'Item');
    MyPrint(Output,buff)
    MyPrint(Output,' ');
    [f1,f2]=size(data.Xa);
    for i=1:f2
        if missings_matrix2(i,2)==0
        else
            form=sprintf('%%-%d.%ds                   ',s,s);   
            buff=sprintf(form,str{i});
            buff1=sprintf('%.0f  ',missings_matrix2(i,2));
            MyPrint(Output,[buff buff1]);
        end;
    end;
    MyPrint(Output,' ');
    max1=max(missings_matrix);
    max2=max(max1(2));
    if max2>1
        MyPrint(Output,' ');
        MyPrint(Output,'PARTICIPANTS WITH MORE THAN ONE MISSING VALUE');
        MyPrint(Output,' ');
        [f1,f2]=size(missings_matrix);
        MyPrint(Output,'Participant           Number of missing values');
        MyPrint(Output,' ');
        for i=1:f1
            if missings_matrix(i,2)>1
                buff=sprintf('%4.0f                   %.0f',missings_matrix(i,1),missings_matrix(i,2));
                MyPrint(Output,buff);
            end;
        end;
    end;
    MyPrint(Output,' ');
    MyPrint(Output,'-----------------------------------------------------------------------');
    MyPrint(Output,' ');
    
    
    %% LATEX %%
    
    MyPrint(Output2,'\section{Missings value descriptives}');    
    MyPrint(Output2,'\begin{table}[H]');
    MyPrint(Output2,'\caption{Descriptives related to missing data}');
    MyPrint(Output2,'\label{taula:missings1}');
    MyPrint(Output2,'\centering');
    MyPrint(Output2,'\begin{tabular}{l l} \toprule');
    buff2 = '\\';
    buff = sprintf('Missing value code: &  %.0f ',data.missings_value); MyPrint(Output2, [buff buff2]);
    buff = sprintf('Total number of missing data: & %.0f ',missings_number); MyPrint(Output2, [buff buff2]);
    buff = 'Method to handle missing data: & Hot-Deck Multiple Imputation in Exploratory'; MyPrint(Output2, [buff buff2]);
    buff = '~ & Factor Analysis (Lorenzo-Seva \& Van Ginkel, 2015)'; MyPrint(Output2, [buff buff2]);
    MyPrint(Output2,'\bottomrule');
    MyPrint(Output2,'\end{tabular}');
    MyPrint(Output2,'\end{table}');
    
    MyPrint(Output2,'\begin{table}[H]');
    MyPrint(Output2,'\caption{Number of variables missing per case}');
    MyPrint(Output2,'\label{taula:missings2}');
    MyPrint(Output2,'\centering');
    MyPrint(Output2,'\begin{tabular}{l l}');
    MyPrint(Output2,'\toprule');
    buff2 = '\\';
    MyPrint(Output2,'\multicolumn{1}{p{1.5cm}}{\textbf{Number of items missing}} &');
    MyPrint(Output2,'\multicolumn{1}{c}{\textbf{Frequency}} \\');
    MyPrint(Output2,'\midrule');
    
    buff=max(missings_matrix);
    maxmissing=buff(1,2);
    perc=(data.N/data.N_original)*100;
    buff=sprintf(' 0  & %5.0f (%4.1f\\%%)',data.N,perc);
    MyPrint(Output2,[buff buff2]);
    for i=1:maxmissing
        buff=histc(missings_matrix,i);
        if buff(2)==0
            buff=[];
        else
            buff1=(buff(2)/data.N_original)*100;
            buff3=sprintf(' %.0f & %5.0f (%4.1f\\%%)',i,buff(2),buff1);
            MyPrint(Output2,[buff3 buff2]);
        end;
    end;
    MyPrint(Output2,'\bottomrule');
    MyPrint(Output2,'\end{tabular}');
    MyPrint(Output2,'\end{table}');
    
    MyPrint(Output2,'\begin{center}');
    MyPrint(Output2,'\begin{longtable}[H]{l c}');
    MyPrint(Output2,'\caption{Number of cases missing per variable}\\');
    MyPrint(Output2,'\toprule');
    MyPrint(Output2,'\multicolumn{1}{l}{\textbf{Item}} &');
    MyPrint(Output2,'\multicolumn{1}{c}{\textbf{Frequency of cases missing}} \\');
    MyPrint(Output2,'\midrule');
    MyPrint(Output2,'\endfirsthead');
    MyPrint(Output2,'\multicolumn{2}{c}');
    MyPrint(Output2,'{\tablename\ \thetable\ -- \textit{(Continued from previous page)}} \\');
    MyPrint(Output2,'\toprule');
    MyPrint(Output2,'\multicolumn{1}{l}{\textbf{Item}} &');
    MyPrint(Output2,'\multicolumn{1}{c}{\textbf{Frequency of cases missing}} \\');
    MyPrint(Output2,'\midrule');
    MyPrint(Output2,'\endhead');
    MyPrint(Output2,'\bottomrule');
    MyPrint(Output2,'\multicolumn{2}{r}{\textit{Continued on next page}}\\');
    MyPrint(Output2,'\endfoot');
    MyPrint(Output2,'\bottomrule');
    MyPrint(Output2,'\endlastfoot');
    
    [f1,f2]=size(data.Xa);
    buff2 = '\\'; 
    for i=1:f2
        if missings_matrix2(i,2)==0
        else
            form=sprintf('%%-%d.%ds & ',s,s);   
            buff=sprintf(form,str{i});
            buff1=sprintf('%.0f  ',missings_matrix2(i,2));
            MyPrint(Output2,[buff buff1 buff2]);
        end;
    end;
    
    MyPrint(Output2,'\end{longtable}');
    MyPrint(Output2,'\end{center}');
    
    if max2>1
        MyPrint(Output2,'\begin{table}[H]');
        MyPrint(Output2,'\caption{Participants with more than one missing value}');
        MyPrint(Output2,'\label{taula:missings4}');
        MyPrint(Output2,'\centering');
        MyPrint(Output2,'\begin{tabular}{l l} \toprule');
        buff2 = '\\';
        MyPrint(Output2,'\multicolumn{1}{l}{\textbf{Participant}} &');
        MyPrint(Output2,'\multicolumn{1}{p{1.75cm}}{\textbf{Number of missing values}} \\');
        MyPrint(Output2,'\midrule');
        [f1,f2]=size(missings_matrix);
        for i=1:f1
            if missings_matrix(i,2)>1
                buff=sprintf('%4.0f  & %.0f',missings_matrix(i,1),missings_matrix(i,2));
                MyPrint(Output2,[buff buff2]);
            end;
        end;
    end;
    MyPrint(Output2,'\bottomrule');
    MyPrint(Output2,'\end{tabular}');
    MyPrint(Output2,'\end{table}');
    
    
    %% RTF %%
    
    int='\intbl ';
    cell='\cell ';
    row='\row';
    par='\par';
    pard='\pard';
    ql='\ql';
    qc='\qc';

    MyPrint(Output3,'\par');
    MyPrint(Output3,'\qc \ul Descriptives related to missing data \ul0');
    MyPrint(Output3,'\par');
    MyPrint(Output3,'\par');
    MyPrint(Output3,'\ql');
    interline='\trowd\trgaph72\clbrdrt\brdrs\clbrdrl\brdrs\clbrdrb\brdrs\clbrdrr\brdrs\cellx3600\clbrdrt\brdrs\clbrdrl\brdrs\clbrdrb\brdrs\clbrdrr\brdrs\cellx7200';
    buff = sprintf('Missing value code : %s  %.0f %s',cell,data.missings_value,cell);
    MyPrint(Output3,interline);
    MyPrint(Output3, [int buff row]);
    buff = sprintf('Total number of missing data : %s %.0f %s',cell,missings_number,cell);
    MyPrint(Output3, [int buff row]);
    buff = sprintf('Method to handle missing data: %s Hot-Deck Multiple Imputation in Exploratory Factor Analysis (Lorenzo-Seva & Van Ginkel, 2015) %s',cell,cell);
    MyPrint(Output3, [int buff row pard par]);
    
    MyPrint(Output3,'\par');
    MyPrint(Output3,'\qc \ul Number of variables missing per case \ul0');
    MyPrint(Output3,'\par');
    MyPrint(Output3,'\par');
    
    interline1='\trowd\trgaph72\clbrdrt\brdrs\clbrdrl\brdrs\clbrdrb\brdrs\clbrdrr\brdrs\cellx2000\clbrdrt\brdrs\clbrdrl\brdrs\clbrdrb\brdrs\clbrdrr\brdrs\cellx4000';
    interline2='\clbrdrt\brdrs\clbrdrl\brdrs\clbrdrb\brdrs\clbrdrr\brdrs\cellx6000';
    interline3=[interline1 interline2];
    
    cap='\int Number of variables missing \cell Frequency \cell Frequency (percent) \cell \row';
    MyPrint(Output3,interline3);
    MyPrint(Output3,cap);
    
    perc=(data.N/data.N_original)*100;
    buff=sprintf('0 %s %5.0f %s %4.1f%% %s',cell,data.N,cell,perc,cell);
    MyPrint(Output3,interline3);
    MyPrint(Output3,[int buff row]);
    for i=1:maxmissing
        buff=histc(missings_matrix,i);
        if buff(2)==0
            buff=[];
        else
            buff1=(buff(2)/data.N_original)*100;
            buff2=sprintf('%.0f %s %5.0f %s %4.1f%% %s',i,cell,buff(2),cell,buff1,cell);
            MyPrint(Output3,interline3);
            MyPrint(Output3,[int buff2 row]);
        end;
    end;
    MyPrint(Output3,[pard par]);
    
    MyPrint(Output3,'\par');
    MyPrint(Output3,'\qc \ul Number of cases missing per variable\ul0');
    MyPrint(Output3,'\par');
    MyPrint(Output3,'\par');
    
    form=sprintf('%%-%d.%ds',s,s);
    buff1=sprintf('%s Frequency of cases missing %s',cell,cell);
    buff=sprintf(form,'Item');
    tamany=s*140;
    tamany2=tamany+1300;
    buff8 = '\trowd\trgaph72\clbrdrt\brdrs\clbrdrl\brdrs\clbrdrb\brdrs\clbrdrr\brdrs\cellx';
    buff9 = '\clbrdrt\brdrs\clbrdrl\brdrs\clbrdrb\brdrs\clbrdrr\brdrs\cellx';
    interline=[buff8 num2str(tamany) buff9 num2str(tamany2)];
    MyPrint(Output3,interline);
    MyPrint(Output3,[int buff buff1 row]);
    [f1,f2]=size(data.Xa);
    for i=1:f2
        if missings_matrix2(i,2)==0
        else
            form=sprintf('%%-%d.%ds',s,s);   
            buff=sprintf(form,str{i});
            buff1=sprintf('%s %.0f %s %s',cell,missings_matrix2(i,2),qc,cell);
            MyPrint(Output3,[int buff ql buff1 row]);
        end;
    end;
    MyPrint(Output3,[pard par]);
    
    if max2>1
        MyPrint(Output3,'\par');
     	MyPrint(Output3,'\qc \ul Participants with more than one missing value\ul0');
        MyPrint(Output3,'\par');
        MyPrint(Output3,'\par');
        interline='\trowd\trgaph72\clbrdrt\brdrs\clbrdrl\brdrs\clbrdrb\brdrs\clbrdrr\brdrs\cellx1500\clbrdrt\brdrs\clbrdrl\brdrs\clbrdrb\brdrs\clbrdrr\brdrs\cellx3000';

        
        [f1,f2]=size(missings_matrix);
        MyPrint(Output3,interline);
        MyPrint(Output3,'\intbl Participant  \cell Number of missing values \cell \row');
        for i=1:f1
            if missings_matrix(i,2)>1
                buff=sprintf('%4.0f %s %s %.0f %s %s',missings_matrix(i,1),ql,cell,missings_matrix(i,2),qc,cell);
                MyPrint(Output3,[int buff row]);
            end;
        end;
    end;
    MyPrint(Output3,[pard par]);
    
return;

function Print_References(Output,Output2,Output3,handles)

%% Print references %%
% ARGUMENTS   : Output  -> plain text listbox
%               Output2 -> LaTeX listbox
%               Output3 -> rtf listbox
%

    %% TEXT %%

    MyPrint(Output,'References');
    MyPrint(Output,' ');
    
    
    par=get(handles.parallel_analysis,'value');
    data=getappdata(0,'data');
    
    if par==1
        if data.matrix_user_on==0
            MyPrint(Output,'Buja, A., & Eyuboglu, N. (1992). Remarks on parallel analysis. Multivariate Behavioral Research, 27(4), 509-540.');
            MyPrint(Output,' ');
        end;
    end;
    
    if data.Matrix==3
        if data.R_NPD==1
            MyPrint(Output,'Devlin, S. J., Gnanadesikan, R., & Kettenring, J. R. (1975). Robust estimation and outlier detection with correlation coefficients. Biometrika, 62, 531-545.');
            MyPrint(Output,' ');
            MyPrint(Output,'Devlin, S. J., Gnanadesikan, R., & Kettenring, J. R. (1981). Robust estimation of dispersion matrices and principal components. Journal of the American Statistical Association, 76, 354-362.');
            MyPrint(Output,' ');
        end;
    end;
    if par==1
        if data.matrix_user_on==1
            MyPrint(Output,'Horn, J. L. (1965). A rationale and test for the number of factors in factor analysis. Psychometrika, 30, 179-185.');
            MyPrint(Output,' ');
        end;
    end;
    if data.missings_on==1
        MyPrint(Output,'Lorenzo-Seva, U., & Van Ginkel, J. R. (2015). Multiple Imputation of missing values in exploratory factor analysis of multidimensional scales : estimating latent trait scores. Anales de Psicologia, 31. In press. \\');
        MyPrint(Output,' ');
    end;    
    if data.ref1==1
        MyPrint(Output,'Muthen, B., & Kaplan D. (1985). A comparison of some methodologies for the factor analysis of non-normal Likert variables. British Journal of Mathematical and Statistical Psychology, 38, 171-189.');
        MyPrint(Output,' ');
        MyPrint(Output,'Muthen, B., & Kaplan D. (1992). A comparison of some methodologies for the factor analysis of non-normal Likert variables: A note on the size of the model. British Journal of Mathematical and Statistical Psychology, 45, 19-30.');
        MyPrint(Output,' ');
    end;
    
    if par==1        
        MyPrint(Output,'Timmerman, M. E., & Lorenzo-Seva, U. (2011). Dimensionality Assessment of Ordered Polytomous Items with Parallel Analysis. Psychological Methods, 16, 209-220.');
        MyPrint(Output,' ');
    end;

    %% LATEX %%
    
    MyPrint(Output2,' ');
    MyPrint(Output2,'\section{References}');
    MyPrint(Output2,' ');
    MyPrint(Output2,'~ \\');
    MyPrint(Output2,' ');
    
    if par==1
        if data.matrix_user_on==0
            MyPrint(Output2,'Buja, A., \& Eyuboglu, N. (1992). Remarks on parallel analysis. Multivariate Behavioral Research, 27(4), 509-540.\\');
            MyPrint(Output2,' ');
        end;
    end;
    
    if data.Matrix==3
        if data.R_NPD==1
            MyPrint(Output2,'Devlin, S. J., Gnanadesikan, R., \& Kettenring, J. R. (1975). Robust estimation and outlier detection with correlation coefficients. Biometrika, 62, 531-545.\\');
            MyPrint(Output2,' ');
            MyPrint(Output2,'Devlin, S. J., Gnanadesikan, R., \& Kettenring, J. R. (1981). Robust estimation of dispersion matrices and principal components. Journal of the American Statistical Association, 76, 354-362.\\');
            MyPrint(Output2,' ');
        end;
    end;
    
    if par==1
        if data.matrix_user_on==1
            MyPrint(Output2,'Horn, J. L. (1965). A rationale and test for the number of factors in factor analysis. Psychometrika, 30, 179-185.\\');
            MyPrint(Output2,' ');
        end;
    end;
    if data.missings_on==1
        MyPrint(Output2,'Lorenzo-Seva, U., \& Van Ginkel, J. R. (2015). Multiple Imputation of missing values in exploratory factor analysis of multidimensional scales : estimating latent trait scores. Anales de Psicologia, 31. In press. \\');
        MyPrint(Output2,' ');
    end;    
    if data.ref1==1
        MyPrint(Output2,'Muthen, B., \& Kaplan D. (1985). A comparison of some methodologies for the factor analysis of non-normal Likert variables. British Journal of Mathematical and Statistical Psychology, 38, 171-189. \\');
        MyPrint(Output2,' ');
        MyPrint(Output2,'Muthen, B., \& Kaplan D. (1992). A comparison of some methodologies for the factor analysis of non-normal Likert variables: A note on the size of the model. British Journal of Mathematical and Statistical Psychology, 45, 19-30.\\');
        MyPrint(Output2,' ');
    end;
    if par==1
        MyPrint(Output2,'Timmerman, M. E., \& Lorenzo-Seva, U. (2011). Dimensionality Assessment of Ordered Polytomous Items with Parallel Analysis. Psychological Methods, 16, 209-220.\\');
        MyPrint(Output2,' ');
    end;
    
    %% RTF %%
    
    MyPrint(Output3,'\par');
    MyPrint(Output3,'\qc \ul References\ul0');
    MyPrint(Output3,'\par');MyPrint(Output3,'\par');MyPrint(Output3,'\ql');
    
    if par==1
        if data.matrix_user_on==0
            MyPrint(Output3,'Buja, A., & Eyuboglu, N. (1992). Remarks on parallel analysis. Multivariate Behavioral Research, 27(4), 509-540.\par');
            MyPrint(Output3,'\par');
        end;
    end;
    
    if data.Matrix==3
        if data.R_NPD==1
            MyPrint(Output3,'Devlin, S. J., Gnanadesikan, R., & Kettenring, J. R. (1975). Robust estimation and outlier detection with correlation coefficients. Biometrika, 62, 531-545.\par');
            MyPrint(Output3,'\par');
            MyPrint(Output3,'Devlin, S. J., Gnanadesikan, R., & Kettenring, J. R. (1981). Robust estimation of dispersion matrices and principal components. Journal of the American Statistical Association, 76, 354-362.\par');
            MyPrint(Output3,'\par');
        end;
    end;
    
    if par==1
        if data.matrix_user_on==1
            MyPrint(Output3,'Horn, J. L. (1965). A rationale and test for the number of factors in factor analysis. Psychometrika, 30, 179-185.\par');
            MyPrint(Output3,'\par');
        end;
    end;
    if data.missings_on==1
        MyPrint(Output3,'Lorenzo-Seva, U., & Van Ginkel, J. R. (2015). Multiple Imputation of missing values in exploratory factor analysis of multidimensional scales : estimating latent trait scores. Anales de Psicologia, 31. In press.\par');
        MyPrint(Output3,'\par');
    end;    
    if data.ref1==1
        MyPrint(Output3,'Muthen, B., & Kaplan D. (1985). A comparison of some methodologies for the factor analysis of non-normal Likert variables. British Journal of Mathematical and Statistical Psychology, 38, 171-189.\par');
        MyPrint(Output3,'\par');
        MyPrint(Output3,'Muthen, B., & Kaplan D. (1992). A comparison of some methodologies for the factor analysis of non-normal Likert variables: A note on the size of the model. British Journal of Mathematical and Statistical Psychology, 45, 19-30.\par');
        MyPrint(Output3,'\par');
    end;
    if par==1
        MyPrint(Output3,'Timmerman, M. E., & Lorenzo-Seva, U. (2011). Dimensionality Assessment of Ordered Polytomous Items with Parallel Analysis. Psychological Methods, 16, 209-220.\par');
        MyPrint(Output3,'\par');
    end;
    
   

return;

function Print_References_factorial (Output,Output2,Output3,handles,compT)

%% Print references of the FA %%
% ARGUMENTS   : Output  -> plain text listbox
%               Output2 -> LaTeX listbox
%               Output3 -> rtf listbox
%               compT   -> Total time elapsed
%

    data=getappdata(0,'data');
    checking_DS=get(handles.DS_checkbox,'value');
    checking_AQ=get(handles.AQ_checkbox,'value');
    
    
    MyPrint(Output,'-----------------------------------------------------------------------');
    MyPrint(Output,' ');
    MyPrint(Output,'References');
    MyPrint(Output,' ');
    
    %% TEXT %%
    
    MyPrint(Output,'Bentler, P.M. (1977). Factor simplicity index and transformations. Psychometrika, 59, 567-579.');
    MyPrint(Output,' ');
    
    if data.Rotation==3
        MyPrint(Output,'Browne, M. (1972b). Oblique rotation to a partially specified target. British Journal of Mathematical and Statistical Psychology, 25, 207-212.');
        MyPrint(Output,' ');
    elseif data.Rotation==4.
        MyPrint(Output,'Browne, M. (1972a). Orthogonal rotation to a partially specified target. British Journal of Mathematical and Statistical Psychology, 25, 115-120.');
        MyPrint(Output,' ');
    end;
    
    if data.pfactorial==1
        MyPrint(Output,'Ferrando, P.J., & Lorenzo-Seva, U. (2016) A note on improving EAP trait estimation in oblique factor-analytic and item response theory models. Document in ellaboration.');
        MyPrint(Output,' ');
    end;
    MyPrint(Output,'Lorenzo-Seva, U. (2003). A factor simplicity index. Psychometrika, 68, 49-60.');
    MyPrint(Output,' ');
    if checking_AQ==1
        if isempty(data.iub) %Balanced
            MyPrint(Output,'Lorenzo-Seva, U., & Rodríguez-Fornells, A. (2006). Acquiescent responding in balanced multidimensional scales and exploratory factor analysis. Psychometrika, 71, 769-77.');
            MyPrint(Output,' ');
        else %Unbalanced
            MyPrint(Output,'Lorenzo-Seva, U., & Ferrando, P. J. (2009). Acquiescent responding in partially balanced multidimensional scales. British Journal of Mathematical and Statistical Psychology, 62, 319-326.');
            MyPrint(Output,' ');
        end;
    end;
    if data.missings_on==1
        MyPrint(Output,'Lorenzo-Seva, U., & Van Ginkel, J. R. (2015). Multiple Imputation of missing values in exploratory factor analysis of multidimensional scales : estimating latent trait scores. Anales de Psicologia, 31. In press.');
        MyPrint(Output,' ');
    end;
    
    MyPrint(Output,'Lorenzo-Seva, U. & ten Berge, J.M.F. (2006). Tucker''s Congruence Coefficient as a Meaningful Index of Factor Similarity. Methodology, 2, 57-64.');
    MyPrint(Output,' ');
    MyPrint(Output3,'McDonald, R.P. (1999). Test theory: A unified treatment. Mahwah, NJ: Lawrence Erlbaum.');
    MyPrint(Output3,' ');
    if data.Matrix==3
        MyPrint(Output,'Muraki, E. (1990). Fitting a polytomous item response model to Likert-type data. Applied Psychological Measurement, 14, 59-71.');
        MyPrint(Output,' ');
    end;
    MyPrint(Output,'Ten Berge, J.M.F., Snijders, T.A.B. & Zegers, F.E. (1981). Computational aspects of the greatest lower bound to reliability and constrained minimum trace factor analysis. Psychometrika, 46, 201-213.');
    MyPrint(Output,' ');
    
    if data.Matrix==1 || data.Matrix==2
        MyPrint(Output,'Ten Berge, J.M.F., & Socan, G. (2004). The greatest lower bound to the reliability of a test and the hypothesis of unidimensionality. Psychometrika, 69, 613-625.');
        MyPrint(Output,' ');
    end;
    
    MyPrint(Output,'Tucker, L. R. (1951). A method for synthesis of factor analysis studies. Personnel Research Section Report, 984. Washington, D. C.: Department of the Army.');
    MyPrint(Output,' ');
    MyPrint(Output,'-----------------------------------------------------------------------');
    MyPrint(Output,' ');
    s_tmp=['Computing time = ' Time2String( compT )];
    MyPrint(Output,' ');
    MyPrint(Output,s_tmp);
    
    %% LATEX %%
    
    MyPrint(Output2,' ');
    MyPrint(Output2,'\section{References}');
    MyPrint(Output2,' ');
    MyPrint(Output2,'~ \\');
    MyPrint(Output2,' ');
    
    MyPrint(Output2,'Bentler, P.M. (1977). Factor simplicity index and transformations. Psychometrika, 59, 567-579.\\');
    MyPrint(Output2,' ');
    
    if data.Rotation==3
        MyPrint(Output2,'Browne, M. (1972b). Oblique rotation to a partially specified target. British Journal of Mathematical and Statistical Psychology, 25, 207-212.\\');
        MyPrint(Output2,' ');
    elseif data.Rotation==4.
        MyPrint(Output2,'Browne, M. (1972a). Orthogonal rotation to a partially specified target. British Journal of Mathematical and Statistical Psychology, 25, 115-120.\\');
        MyPrint(Output2,' ');
    end;
    
    if data.pfactorial==1
        MyPrint(Output2,'Ferrando, P.J., \& Lorenzo-Seva, U. (2016) A note on improving EAP trait estimation in oblique factor-analytic and item response theory models. Document in ellaboration.\\');
        MyPrint(Output2,' ');
    end;
    MyPrint(Output2,'Lorenzo-Seva, U. (2003). A factor simplicity index. Psychometrika, 68, 49-60.\\');
    MyPrint(Output2,' ');
    if checking_AQ==1
        if isempty(data.iub) 
            MyPrint(Output2,'Lorenzo-Seva, U., \& Rodríguez-Fornells, A. (2006). Acquiescent responding in balanced multidimensional scales and exploratory factor analysis. Psychometrika, 71, 769-77.\\');
            MyPrint(Output2,' ');
        else 
            MyPrint(Output2,'Lorenzo-Seva, U., \& Ferrando, P. J. (2009). Acquiescent responding in partially balanced multidimensional scales. British Journal of Mathematical and Statistical Psychology, 62, 319-326.\\');
            MyPrint(Output2,' ');
        end;
    end;
    if data.missings_on==1
        MyPrint(Output2,'Lorenzo-Seva, U., \& Van Ginkel, J. R. (2015). Multiple Imputation of missing values in exploratory factor analysis of multidimensional scales : estimating latent trait scores. Anales de Psicologia, 31. In press.\\');
        MyPrint(Output2,' ');
    end;
    
    MyPrint(Output2,'Lorenzo-Seva, U. \& ten Berge, J.M.F. (2006). Tucker''s Congruence Coefficient as a Meaningful Index of Factor Similarity. Methodology, 2, 57-64.\\');
    MyPrint(Output2,' ');
    MyPrint(Output2,'McDonald, R.P. (1999). Test theory: A unified treatment. Mahwah, NJ: Lawrence Erlbaum.\\');
    MyPrint(Output2,' ');
    if data.Matrix==3
        MyPrint(Output2,'Muraki, E. (1990). Fitting a polytomous item response model to Likert-type data. Applied Psychological Measurement, 14, 59-71.\\');
        MyPrint(Output2,' ');
    end;
    MyPrint(Output2,'Ten Berge, J.M.F., Snijders, T.A.B. \& Zegers, F.E. (1981). Computational aspects of the greatest lower bound to reliability and constrained minimum trace factor analysis. Psychometrika, 46, 201-213.\\');
    MyPrint(Output2,' ');
    
    if data.Matrix==1 || data.Matrix==2
        MyPrint(Output2,'Ten Berge, J.M.F., \& Socan, G. (2004). The greatest lower bound to the reliability of a test and the hypothesis of unidimensionality. Psychometrika, 69, 613-625.\\');
        MyPrint(Output2,' ');
    end;
    
    MyPrint(Output2,'Tucker, L. R. (1951). A method for synthesis of factor analysis studies. Personnel Research Section Report, 984. Washington, D. C.: Department of the Army.\\');
    MyPrint(Output2,' ');
     
    %% RTF %%
    
    MyPrint(Output3,'\par');
    MyPrint(Output3,'\qc \ul References\ul0');
    MyPrint(Output3,'\par');MyPrint(Output3,'\par');MyPrint(Output3,'\ql');
    
    MyPrint(Output3,'Bentler, P.M. (1977). Factor simplicity index and transformations. Psychometrika, 59, 567-579.\par');
    MyPrint(Output3,'\par');
    
    if data.Rotation==3
        MyPrint(Output3,'Browne, M. (1972b). Oblique rotation to a partially specified target. British Journal of Mathematical and Statistical Psychology, 25, 207-212.\par');
        MyPrint(Output3,'\par');
    elseif data.Rotation==4.
        MyPrint(Output3,'Browne, M. (1972a). Orthogonal rotation to a partially specified target. British Journal of Mathematical and Statistical Psychology, 25, 115-120.\par');
        MyPrint(Output3,'\par');
    end;
    
    if data.pfactorial==1
        MyPrint(Output3,'Ferrando, P.J., & Lorenzo-Seva, U. (2016) A note on improving EAP trait estimation in oblique factor-analytic and item response theory models. Document in ellaboration.\par');
        MyPrint(Output3,'\par');
    end;
    MyPrint(Output3,'Lorenzo-Seva, U. (2003). A factor simplicity index. Psychometrika, 68, 49-60.\par');
    MyPrint(Output3,'\par');
    if checking_AQ==1
        if isempty(data.iub) 
            MyPrint(Output3,'Lorenzo-Seva, U., & Rodríguez-Fornells, A. (2006). Acquiescent responding in balanced multidimensional scales and exploratory factor analysis. Psychometrika, 71, 769-77.\par');
            MyPrint(Output3,'\par');
        else 
            MyPrint(Output3,'Lorenzo-Seva, U., & Ferrando, P. J. (2009). Acquiescent responding in partially balanced multidimensional scales. British Journal of Mathematical and Statistical Psychology, 62, 319-326.\par');
            MyPrint(Output3,'\par');
        end;
        if data.missings_on==1
            MyPrint(Output3,'Lorenzo-Seva, U., & Van Ginkel, J. R. (2015). Multiple Imputation of missing values in exploratory factor analysis of multidimensional scales : estimating latent trait scores. Anales de Psicologia, 31. In press.\par');
            MyPrint(Output3,'\par');
        end;
    end;
    
    MyPrint(Output3,'Lorenzo-Seva, U. & ten Berge, J.M.F. (2006). Tucker''s Congruence Coefficient as a Meaningful Index of Factor Similarity. Methodology, 2, 57-64.\par');
    MyPrint(Output3,'\par');
    MyPrint(Output3,'McDonald, R.P. (1999). Test theory: A unified treatment. Mahwah, NJ: Lawrence Erlbaum.\par');
    MyPrint(Output3,'\par');
    if data.Matrix==3
        MyPrint(Output3,'Muraki, E. (1990). Fitting a polytomous item response model to Likert-type data. Applied Psychological Measurement, 14, 59-71.\par');
        MyPrint(Output3,'\par');
    end;
    MyPrint(Output3,'Ten Berge, J.M.F., Snijders, T.A.B. & Zegers, F.E. (1981). Computational aspects of the greatest lower bound to reliability and constrained minimum trace factor analysis. Psychometrika, 46, 201-213.\par');
    MyPrint(Output3,'\par');
    
    if data.Matrix==1 || data.Matrix==2
        MyPrint(Output3,'Ten Berge, J.M.F., & Socan, G. (2004). The greatest lower bound to the reliability of a test and the hypothesis of unidimensionality. Psychometrika, 69, 613-625.\par');
        MyPrint(Output3,'\par');
    end;
    
    MyPrint(Output3,'Tucker, L. R. (1951). A method for synthesis of factor analysis studies. Personnel Research Section Report, 984. Washington, D. C.: Department of the Army.\par');
    MyPrint(Output3,'\par');
    
return;

function Parallel_analysis_Callback (hObject,eventdata,handles,gcf)

%% Parallel Analysis switching %%

    t=get(handles.parallel_analysis,'value');
    data=getappdata(0,'data');
    
    if t==1
        set(handles.grafic_PA,'enable','on',...
            'Visible','on');
    else
        set(handles.grafic_PA,'Visible','off',...
            'value',0);
    end;
    data.descriptives_done=0;
    setappdata(0,'data',data);

return;

function DS_on_Callback (gcf,hObject,handles)

%% Social Desirability switch %%

    val=get(handles.DS_checkbox,'value');
    
    %load data
        data=getappdata(0,'data');
        
    if val==1
        
        checking_AQ=get(handles.AQ_checkbox,'value');
        if checking_AQ==1
            warndlg('Acquiescence control function is enabled. For enabling SD control function, this option must have been disabled previously.','Psychological Test Toolbox: Warning');
            set(handles.DS_checkbox,'value',0);
            return;
        end;
        
        set(handles.panel3_tab5,'visible','on');
        set(handles.panel4_tab5,'visible','on');
        
        tmp1={};
        set(handles.SDItems,'string',tmp1);
        set(handles.SDItems,'value',1);
        
        tmp1={};
        set(handles.ContentItems,'string',tmp1);
        set(handles.ContentItems,'value',1);
        set(handles.ContentItems,'string',data.inclosos);
        
        [f1,f2]=size(data.inclosos);
        set(handles.numberofcontentitems,'string',f2);
        [f1,f2]=size(data.exclosos);
        set(handles.numberofexcludeditems,'string',f2);
        set(handles.numberofsditems,'string','0');
        
        [f1,f2]=size(data.inclosos);
        data.G=[0 f2];
        
        set(handles.full_analysis,'enable','on');
        
        
        
        data.checking_DS=1;
        
        %visualize labels if the user had provided
        
        if data.labels_user_on==1
            set(handles.Visualize_Labels2,'value',1,'visible','on');
            buff={};
            Visualize_Labels_Callback2(hObject,buff,handles);
        else
            set(handles.Visualize_Labels2,'visible','off','value',0);
        end;
        
    else        
        set(handles.panel3_tab5,'visible','off');
        set(handles.panel4_tab5,'visible','off');
        
        tmp1={};
        set(handles.ContentItems,'string',tmp1);
        set(handles.SDItems,'string',tmp1);
        
        
        [f1,f2]=size(data.inclosos);
        data.G=[0 f2];
        
        data.items_contingut=data.inclosos;
        data.itemsa=data.items_contingut;
        data.items_sd=[];
        
        data.ib=data.items_contingut;
        data.iub=[];
        
        [f1,f2]=size(data.itemsa);
        itemsinorder=(1:f2);
        [f1,f2]=size(data.ib);
        [g1,g2]=size(data.iub);
        
        matr=[data.items_contingut;itemsinorder];

        for j=1:f2 %balanced
            a=data.ib(j);
            [b1,b2] = find(matr(1,:) == a);
            buff(j)=matr(2,b2);
        end;
        
        data.iba=buff;
        data.iuba=[];
        [f1,f2]=size(data.inclosos);
        data.G=[0 f2];
        
       inclosos_en_ordre=sort(data.inclosos);
        [f1,f2]=size(inclosos_en_ordre);
       if f2>=16
            for i=1:f2
                buff1{i}=sprintf('%.0f, ',inclosos_en_ordre(i));
            end;
            ratio1=ceil(f2/15);

            for l=1:ratio1
                cof=15*(l-1);
                if cof<f2 && cof+15<f2
                    buff2{l}=inclosos_en_ordre(1+cof:15+cof);                
                else
                    buff2{l}=inclosos_en_ordre(1+cof:f2);
                end;

                buff4{l}=sprintf('%.0f, ',buff2{l});
                if l==ratio1
                    [g1,g2]=size(buff4{l});
                    buff4{l}=buff4{l}(1:g2-2);
                end;
            end;

        else
            for i=1:f2
            buff1{i}=sprintf('%.0f, ',inclosos_en_ordre(i));
            end;
            a=sprintf('%s',buff1{:});
            [f1,f2]=size(a);
            buff4=a(1:(f2-2));
        end;
        
        set(handles.sd_list,'string',tmp1);
        set(handles.content_list,'string',buff4);
        
        % Notify that disabling SD control it also disables AQ control
        
        checking_AQ=get(handles.AQ_checkbox,'value');
        if checking_AQ==1
            warndlg('AQ control function is enabled. Disabling DS control resets the AQ control option, due to an update of the content items.','Psychological Test Toolbox: Warning');
            set(handles.AQ_checkbox,'value',0);
            AQ_on_Callback(gcf,hObject,handles);
        end;
        
        set(handles.full_analysis,'value',0);
        data.full_analysis=0;
        if checking_AQ==0 %AQ & DS off
            set(handles.full_analysis,'enable','off');
        end;  
        
        set(handles.Visualize_Labels2,'visible','off','value',0);
        
        data.checking_DS=0;
    end;
        
    %SAVE DATA 
    setappdata(0,'data',data);
    
    
return;

function Visualize_Labels_Callback2(hObject, eventdata, handles)

%% Visualize Labels Switch %%

    data=getappdata(0,'data');
    
    content=get(handles.ContentItems,'string');
    sditems=get(handles.SDItems,'string');
    buffs={};
    buffs2={};
    
    checking=get(handles.Visualize_Labels2,'value');
    
    if checking==0
        
        %Content
        [f1,f2]=size(content);
        for i=1:f1
            str=char(content(i));
            [g1,g2]=size(data.labels);
            for j=i:g2
                check_pos=strcmp(str,data.labels(j));
                if check_pos==1
                    number=j;
                    break;
                end;
            end;
            buff=sprintf('%.0f',number);
            buffs(i)=cellstr(buff);
        end;

        %SD
        [f1,f2]=size(sditems);
        for i=1:f1
            str=char(sditems(i));
            [g1,g2]=size(data.labels);
            for j=i:g2
                check_pos=strcmp(str,data.labels(j));
                if check_pos==1
                    number=j;
                    break;
                end;
            end;
            buff2=sprintf('%.0f',number);
            buffs2(i)=cellstr(buff2);
        end;

    else
        
        %Content
        [f1,f2]=size(content);
        for i=1:f1
            number=str2num(char(content(i,:)));
            buff=data.labels(number);
            buffs(i)=cellstr(buff);
        end;

        %SD
        [f1,f2]=size(sditems);
        for i=1:f1
            number=str2num(char(sditems(i,:)));
            buff2=data.labels(number);
            buffs2(i)=cellstr(buff2);
        end;

    end;
    
    tmp1={};
    set(handles.ContentItems,'string',tmp1);
    set(handles.ContentItems,'value',1);
    set(handles.ContentItems,'string',buffs); 

    set(handles.SDItems,'string',tmp1);
    set(handles.SDItems,'value',1);
    set(handles.SDItems,'string',buffs2); 
    

return;

function ButtonExcloureItems2_Callback (hObject,eventdata,handles)

%% Selecting SD items %%

    checking_AQ=get(handles.AQ_checkbox,'value');
    if checking_AQ==1
        warndlg('AC control function is enabled. For defining which items are SD markers, this option must have been disabled previously.','Psychological Test Toolbox: Warning');
        return;
    end;
    
     %LOAD DATA
    data=getappdata(0,'data');

    %Included in content items
    ContentItems=cellstr(get(handles.ContentItems,'string'));
    [n_items,tmp] = size(ContentItems);
    %SD Items
    SDItems=get(handles.SDItems,'string');
    [h_n,tmp] = size(SDItems);
    
    %Check if the labels are visualized
    check_labels=get(handles.Visualize_Labels2,'value');
 
    %Selection
    Seleccio = get(handles.ContentItems,'value');
    [tmp,num_selected] = size(Seleccio);
    
    %Remove non selected
    nosel ={};h=1; nosel_i=[];
    if check_labels==0;
        for i=1:n_items
            campana = 0;
            for j=1:num_selected
                if str2num(char(ContentItems(i))) == str2num(char(ContentItems(Seleccio(j))))
                    campana = 1;
                end;
            end;
            if campana == 0
                nosel(h) = ContentItems(i);
                nosel_i(h)  = str2num(char(ContentItems(i)));
                h=h+1;
            else
                h_n=h_n+1;
                SDItems(h_n) = ContentItems(i);
            end;
        end;
    else
        for i=1:n_items
            campana = 0;
            for j=1:num_selected
                if strcmp(char(ContentItems(i)),(char(ContentItems(Seleccio(j)))))==1
                    campana = 1;
                end;
            end;
            if campana == 0
                nosel(h) = ContentItems(i);
                [g1,g2]=size(data.labels);
                for j=i:g2
                    check_pos=strcmp(nosel(h),data.labels(j));
                    if check_pos==1
                        number=j;
                        break;
                    end;
                end;
                nosel_i(h)  = number;
                h=h+1;
            else
                h_n=h_n+1;
                SDItems(h_n) = ContentItems(i);
            end;
        end;
    end;
    
    if (h<4)
        msgbox('At least three content items are needed!','Psychological Test Toolbox','error');
        return;
    end;
    
    set(handles.ContentItems,'value',1);
    if h_n > 0
        set(handles.SDItems,'value',1);
        set(handles.ButtonIncloureItems2,'enable','on');                
    else
        set(handles.SDItems,'value',0);
        set(handles.ButtonIncloureItems2,'enable','off');                
    end;
    
    set(handles.ContentItems,'string',nosel);
    if check_labels==1
        SDItems2={};
        [g1,g2]=size(data.labels);
        for i=1:h_n
            str=char(SDItems(i));
            for j=1:g2
                check_pos=strcmp(str,data.labels(j));
                if check_pos==1
                    number=j;
                    break;
                end;
            end;
            SDItems2(i)=cellstr(num2str((number)));
        end;
        SDItems2=sort(str2num(char(SDItems2)));
        set(handles.SDItems,'string',data.labels(SDItems2));
    else
        SDItems2=cellstr(num2str(sort(str2num(char(SDItems)))));
        SDItems3=sort(str2num(char(SDItems)));
        [f1,f2]=size(SDItems3);
        buffs={};
        for i=1:f1
            buff=num2str(SDItems3(i));
            buffs(i)=cellstr(buff);
        end;
        set(handles.SDItems,'string',buffs);
    end;
    
    
    data.items_contingut=nosel_i;
    if check_labels==1
        data.items_sd=SDItems2';
     else
         data.items_sd=rot90(str2num(char(SDItems2)));
    end;
    
    data.itemsa=[data.items_sd data.items_contingut]; %ITEMS AS ADMINISTRATION                3 6 10 14  2 4 5 7 8 9  11 12 13 15 16 17 18 19
    [f1,f2]=size(data.itemsa);
    itemsinorder=(1:f2);                              %ITEMS AS IN THE ORDER OF THE ANALYSIS  1 2  3  4  5 6 7 8 9 10 11 12 13 14 15 16 17 18
    
    [f1,f2]=size(data.items_sd);
    set(handles.numberofsditems,'string',f2);
    [f1,f2]=size(data.items_contingut);
    set(handles.numberofcontentitems,'string',f2);
    
    empty1=isempty(data.S); 
    empty2=isempty(data.X); 

    if empty1==1 && empty2==1
        return;
    elseif empty1==1
        data.X_a=data.X(:,data.itemsa);
        [N,m]=size(data.X_a);
        data.labels_contingut=data.labels(1,nosel_i);
        
    elseif empty2==1
        data.S_a=data.S(data.itemsa,data.itemsa);
        data.labels_contingut=data.labels(1,nosel_i);
        [~,m]=size(data.S_a);
    end;
    
    % G:   Vector with two numbers: the number of items related to SD and the number of items related to content 
    [~,m]=size(data.inclosos);
    [tmp1,tmp2]=size(data.items_sd);
    G= [tmp2 m-tmp2];
    data.G=G;
    
    %POSITION OF ITEMS OF CONTENT IN THE ORDER OF THE ANALYSIS
    items_c = data.itemsa([(G(1)+1):m]);

    target_off=isempty(data.H);
    if target_off
    else
        %TARGET ADJUSTED A ITEMS ANALYSED (IN THE ORDER OF THE ANALYSIS)
        H=data.H(data.itemsa,:);

        %Hc: Target of contents items
        Htmp = H((G(1)+1):m,:); 
        tmp=sum(abs(Htmp));
        [tmp2,tmp3]=size(tmp);
        Hc=[];
        for i=1:tmp3
            if tmp(i)> 0
                Hc=[Hc Htmp(:,i)];
            end;
        end;
        data.Hc=Hc;
    end;
    
    inclosos_en_ordre=sort(data.items_contingut);
    [f1,f2]=size(inclosos_en_ordre);
    if f2>=16
        for i=1:f2
            buff1{i}=sprintf('%.0f, ',inclosos_en_ordre(i));
        end;
        ratio1=ceil(f2/15);
        
        for l=1:ratio1
            cof=15*(l-1);
            if cof<f2 && cof+15<f2
                buff2{l}=inclosos_en_ordre(1+cof:15+cof);                
            else
                buff2{l}=inclosos_en_ordre(1+cof:f2);
            end;
            
            buff4{l}=sprintf('%.0f, ',buff2{l});
            if l==ratio1
                [g1,g2]=size(buff4{l});
                buff4{l}=buff4{l}(1:g2-2);
            end;
        end;
                
    else
        for i=1:f2
        buff1{i}=sprintf('%.0f, ',inclosos_en_ordre(i));
        end;
        a=sprintf('%s',buff1{:});
        [f1,f2]=size(a);
        buff4=a(1:(f2-2));
    end;
    
    sd_en_ordre=sort(data.items_sd);
    [f1,f2]=size(sd_en_ordre);
    for i=1:f2
        buff3{i}=sprintf('%.0f, ',sd_en_ordre(i));
    end;
    
    
    c=sprintf('%s',buff3{:});
    [f1,f2]=size(c);
    d=c(1:(f2-2));
    set(handles.content_list,'string',buff4);
    set(handles.sd_list,'string',d);
    
    data.ib=data.itemsa;
    data.iub=[];
    
    [f1,f2]=size(data.itemsa);
    itemsinorder=(1:f2);

    matr=[data.itemsa;itemsinorder];
    
    [f1,f2]=size(data.items_contingut);

    for j=1:f2 %balanced
        a=data.items_contingut(j);
        [b1,b2] = find(matr(1,:) == a);
        buff(j)=matr(2,b2);
    end;

    data.iba=buff;
    data.iuba=[];
    
    data.Bandera=1;
    %SAVE DATA 
    setappdata(0,'data',data);


return;

function ButtonIncloureItems2_Callback (hObject, eventdata, handles)

%% Selecting Content Items %%

    checking_AQ=get(handles.AQ_checkbox,'value');
    if checking_AQ==1
        warndlg('AC control function is enabled. For defining which items are content items, this option must have been disabled previously.','Psychological Test Toolbox: Warning');
        return;
    end;

    %LOAD DATA
    data=getappdata(0,'data');
    
    %Excluded
    SDItems=get(handles.SDItems,'string');
    [h_n,tmp] = size(SDItems);
 
    %Selection
    Seleccio = get(handles.SDItems,'value');
    [tmp,num_selected] = size(Seleccio);
    
    %Check if the labels are visualized
    check_labels=get(handles.Visualize_Labels2,'value');
    
    tmp={};hh=1;
    if check_labels==0
        for i=1:h_n
            campana = 0;
            for j=1:num_selected
                if i == Seleccio(j)
                    campana = 1;
                end;
            end;
            if campana==0
               tmp(hh)=SDItems(i);
               hh=hh+1;
            end;
        end;
    else
        for i=1:h_n
            campana = 0;
            for j=1:num_selected
                if strcmp(char(SDItems(i)),(char(SDItems(Seleccio(j)))))==1
                    campana = 1;
                end;
            end;
            if campana == 0
                tmp(hh) = SDItems(i);
                [g1,g2]=size(data.labels);
                for j=i:g2
                    check_pos=strcmp(tmp(hh),data.labels(j));
                    if check_pos==1
                        number=j;
                        break;
                    end;
                end;
                tmp_i(hh)  = number;
                hh=hh+1;
            end;
        end;
    end;
    SDItems = tmp;
    [tmp,h_n] = size(SDItems);
    
    empty1=isempty(data.S);
    empty2=isempty(data.X);


    [~,f2]=size(data.inclosos);
%     itemsinorder=(1:n);                              %ITEMS AS IN THE ORDER OF THE ANALYSIS  1 2  3  4  5 6 7 8 9 10 11 12 13 14 15 16 17 18
      
    ItemsIncluded=data.inclosos;
    %Treure els items_exclosos
    nosel ={};h=1; nosel_i=[];
    if check_labels==0
        for i=1:f2
            a=ItemsIncluded(i);
                campana = 0;
                for j=1:h_n
                    if a == str2num(char(SDItems(j)))
                        campana = 1;
                    end;
                end;
                if campana == 0
                    nosel_i(h)  = (a);
                    h=h+1;
                end;
        end;
    else
        for i=1:f2
            campana = 0;
            for j=1:h_n
                if strcmp(char(data.labels(i)),(char(SDItems(j))))==1
                    campana = 1;
                end;
            end;
            if campana == 0
                a=ItemsIncluded(i);
                nosel(h) = data.labels(a);
                [g1,g2]=size(data.labels);
                for j=i:g2
                    check_pos=strcmp(nosel(h),data.labels(j));
                    if check_pos==1
                        number=j;
                        break;
                    end;
                end;
                nosel_i(h)  = number;
                h=h+1;
            end;
        end;
    end;
    
    
    set(handles.ContentItems,'value',1);
    if h_n > 0
        set(handles.SDItems,'value',1);
        set(handles.ButtonIncloureItems2,'enable','on');                
    else
        set(handles.SDItems,'value',0);
        set(handles.ButtonIncloureItems2,'enable','off');                
    end;
    
    if check_labels==0
        buffs={};
        for i=1:(h-1)
            buff=sprintf('%.0f',nosel_i(i));
            buffs(i)=cellstr(buff);
        end;
        set(handles.ContentItems,'string',buffs);
    else
        set(handles.ContentItems,'string',nosel);
    end;
    
    set(handles.SDItems,'string',SDItems);

    if empty1==1 && empty2==1 %no data.X or data.S
        return;
    elseif empty1==1
        data.X_a=data.X(:,nosel_i);
        data.labels_contingut=data.labels(1,nosel_i);
        % N: PARTICIPANTS; m: items analised
        [N,m]=size(data.X_a);
    elseif empty2==1
        data.S_a=data.S(nosel_i,nosel_i);
        data.labels_contingut=data.labels(1,nosel_i);
        % m:items analised
        [~,m]=size(data.S_a);
    end;
    
    data.items_contingut=nosel_i;
    if check_labels==1
        SDItems2={};
        [g1,g2]=size(data.labels);
        for i=1:h_n
            str=char(SDItems(i));
            for j=1:g2
                check_pos=strcmp(str,data.labels(j));
                if check_pos==1
                    number=j;
                    break;
                end;
            end;
            SDItems2(i)=cellstr(num2str((number)));
        end;
        SDItems2=sort(str2num(char(SDItems2)));
        data.items_sd=SDItems2';
    else
        data.items_sd=rot90(str2num(char(SDItems)));
    end;
    
    data.itemsa=[data.items_sd data.items_contingut]; %ITEMS AS ADMINISTRATION                3 6 10 14  2 4 5 7 8 9  11 12 13 15 16 17 18 19
    [f1,f2]=size(data.itemsa);
    itemsinorder=(1:f2);                              %ITEMS AS IN THE ORDER OF THE ANALYSIS  1 2  3  4  5 6 7 8 9 10 11 12 13 14 15 16 17 18
    
    [f1,f2]=size(data.items_sd);
    set(handles.numberofsditems,'string',f2);
    [f1,f2]=size(data.items_contingut);
    set(handles.numberofcontentitems,'string',f2);
    
    % G:   Vector with two numbers: the number of items related to SD and the number of items related to content 
    [~,m]=size(data.inclosos);
    [tmp1,tmp2]=size(data.items_sd);
    G= [tmp2 m-tmp2];
    data.G=G;
    
    %POSITION OF ITEMS OF CONTENT IN THE ORDER OF THE ANALYSIS
    items_c = data.itemsa([(G(1)+1):m]);

    target_off=isempty(data.H);
    if target_off
        
    else
        %TARGET ADJUSTED A ITEMS ANALYSED (IN THE ORDER OF THE ANALYSIS)
        H=data.H(data.itemsa,:);

        %Hc: Target of contents items
        Htmp = H((G(1)+1):m,:); 
        tmp=sum(abs(Htmp));
        [tmp2,tmp3]=size(tmp);
        Hc=[];
        for i=1:tmp3
            if tmp(i)> 0
                Hc=[Hc Htmp(:,i)];
            end;
        end;
        data.Hc=Hc;
    end;

    inclosos_en_ordre=sort(data.items_contingut);
    [f1,f2]=size(inclosos_en_ordre);
    if f2>=16
        for i=1:f2
            buff1{i}=sprintf('%.0f, ',inclosos_en_ordre(i));
        end;
        ratio1=ceil(f2/15);
        
        for l=1:ratio1
            cof=15*(l-1);
            if cof<f2 && cof+15<f2
                buff2{l}=inclosos_en_ordre(1+cof:15+cof);                
            else
                buff2{l}=inclosos_en_ordre(1+cof:f2);
            end;
            
            buff4{l}=sprintf('%.0f, ',buff2{l});
            if l==ratio1
                [g1,g2]=size(buff4{l});
                buff4{l}=buff4{l}(1:g2-2);
            end;
        end;
                
    else
        for i=1:f2
        buff1{i}=sprintf('%.0f, ',inclosos_en_ordre(i));
        end;
        a=sprintf('%s',buff1{:});
        [f1,f2]=size(a);
        buff4=a(1:(f2-2));
    end;
    
    if isempty(data.items_sd)
        buff3={};
    else
        sd_en_ordre=sort(data.items_sd);
        [f1,f2]=size(sd_en_ordre);
        for i=1:f2
            buff3{i}=sprintf('%.0f, ',sd_en_ordre(i));
        end;
    end;
    
    c=sprintf('%s',buff3{:});
    [f1,f2]=size(c);
    d=c(1:(f2-2));
    set(handles.content_list,'string',buff4);
    set(handles.sd_list,'string',d);
    

    data.ib=data.itemsa;
    data.iub=[];
    
    [f1,f2]=size(data.itemsa);
    itemsinorder=(1:f2);

    matr=[data.itemsa;itemsinorder];
    
    [f1,f2]=size(data.items_contingut);

    for j=1:f2 %balanced
        a=data.items_contingut(j);
        [b1,b2] = find(matr(1,:) == a);
        buff(j)=matr(2,b2);
    end;

    data.iba=buff;
    data.iuba=[];
    
    data.Bandera=1;
    %SAVE DATA 
    setappdata(0,'data',data);

return;

function AQ_on_Callback (gcf,hObject,handles)
   
%% Acquiescence Switch %%

    val=get(handles.AQ_checkbox,'value');
    
    %load data
        data=getappdata(0,'data');
        
    if val==1
        
        set(handles.panel3_tab6,'visible','on');
        set(handles.panel4_tab6,'visible','on');
        
        tmp1={};
        set(handles.UnbalItems,'string',tmp1);
        set(handles.UnbalItems,'value',1);
        
        tmp1={};
        set(handles.BalItems,'string',tmp1);
        set(handles.BalItems,'value',1);
        set(handles.BalItems,'string',data.items_contingut);
        
        [f1,f2]=size(data.items_contingut);
        set(handles.numberofbalanceditems,'string',f2);
        [f1,f2]=size(data.exclosos);
        [f1,g2]=size(data.items_sd);
        set(handles.numberofexcludeditems2,'string',(f2+g2));
        set(handles.numberofunbalanceditems,'string','0');
        set(handles.Compute_AQiDS,'enable','on');
        
        data.ib=data.items_contingut;
        data.iub=[];
        [f1,f2]=size(data.itemsa);
        itemsinorder=(1:f2);
        [f1,f2]=size(data.ib);
        [g1,g2]=size(data.iub);

        set(handles.numberofbalanceditems,'string',f2);
        set(handles.numberofunbalanceditems,'string',g2);  
        
        val_DS=get(handles.DS_checkbox,'value');
        if val_DS==0
            [f1,f]=size(data.items_contingut);
            data.G=[ 0 f ];
        else
            % G:   Vector with two numbers: the number of items related to SD and the number of items related to content 
            [~,m]=size(data.inclosos);
            [tmp1,tmp2]=size(data.items_sd);
            G= [tmp2 m-tmp2];
            data.G=G;
        end;


        matr=[data.itemsa;itemsinorder];

        for j=1:f2 %balanced
            a=data.ib(j);
            [b1,b2] = find(matr(1,:) == a);
            buff(j)=matr(2,b2);
        end;
        
        data.iba=buff;
        data.iuba=[];
        
        set(handles.full_analysis,'enable','on');
        
        
        
        data.checking_AQ=1;
        
        %visualize labels if the user had provided
        if data.labels_user_on==1
            set(handles.Visualize_Labels3,'value',1,'visible','on');
            buff={};
            Visualize_Labels_Callback3(hObject,buff, handles)
        else
            set(handles.Visualize_Labels3,'visible','off','value',0);
        end;
        
        if isempty(data.H)==0 %The user had provided target matrix
            signes=sign(data.H(data.items_contingut,:));
            posit=find(signes==1);
            [num_posit,~]=size(posit);
            negat=find(signes==-1);
            [num_negat,~]=size(negat);

            buff_text{1}='In this moment, the balanced core is composed by:';
            buff_text{2}=sprintf('%.0f positively worded items and %.0f negatively worded items. ',num_posit,num_negat);
            buff_text{3}='Please, remember that the balanced core has to be composed by the same number of positively worded and negatively worded items.';
            set(handles.text_balanced_core,'visible','on','string',buff_text);
        else
            %if the user has not provided a valid target matrix but
            %actually has provided the variable labels, we will check for 
            %some indicator of positive or negative items
           if data.labels_user_on==1
                positive_n = strfind(get(handles.BalItems,'string'),'+');
                index1=find(~cellfun(@isempty,positive_n));
                [num_posit,~]=size(index1);
 
                negative_n = strfind(get(handles.BalItems,'string'),'-');
                index2=find(~cellfun(@isempty,negative_n));
                [num_negat,~]=size(index2);
                
                if isempty(num_posit)==0 || isempty(num_negat)==0 % if the cells are not empty
                    if num_posit==0 && num_posit==0 %there are no + or - symbols, not display the info
                    else
                        buff_text{1}='Based on the provided labels, the balanced core is composed by:';
                        buff_text{2}=sprintf('%.0f positively worded items and %.0f negatively worded items. ',num_posit,num_negat);
                        buff_text{3}='Please, remember that the balanced core has to be composed by the same number';
                        buff_text{4}='of positively worded and negatively worded items.';
                        set(handles.text_balanced_core,'visible','on','string',buff_text);
                    end;
                end;
            end;
        end;
        
    else        
        set(handles.panel3_tab6,'visible','off');
        set(handles.panel4_tab6,'visible','off');
        
        tmp1={};
        set(handles.BalItems,'string',tmp1);
        set(handles.UnbalItems,'string',tmp1);
        
        data.ib=data.items_contingut;
        data.iub=[];
        [f1,f2]=size(data.itemsa);
        itemsinorder=(1:f2);
        [f1,f2]=size(data.ib);
        [g1,g2]=size(data.iub);

        set(handles.numberofbalanceditems,'string',f2);
        set(handles.numberofunbalanceditems,'string',g2);  
        
        val_DS=get(handles.DS_checkbox,'value');
        if val_DS==0
            [f1,f]=size(data.items_contingut);
            data.G=[ 0 f ];
        else
            % G:   Vector with two numbers: the number of items related to SD and the number of items related to content 
            [~,m]=size(data.inclosos);
            [tmp1,tmp2]=size(data.items_sd);
            G= [tmp2 m-tmp2];
            data.G=G;
        end;


        matr=[data.itemsa;itemsinorder];

        for j=1:f2 %balanced
            a=data.ib(j);
            [b1,b2] = find(matr(1,:) == a);
            buff(j)=matr(2,b2);
        end;
        
        data.iba=buff;
        data.iuba=[];
        
        set(handles.full_analysis,'value',0);
        data.full_analysis=0;
        if val_DS==0 %AQ & DS off
            set(handles.full_analysis,'enable','off');
        end;
        
        set(handles.Visualize_Labels3,'visible','off','value',0);
        tmp={};
        set(handles.text_balanced_core,'visible','off','string',tmp);
        
        data.checking_AQ=0;
    end;
    
    %SAVE DATA 
    setappdata(0,'data',data);
    
return;

function Visualize_Labels_Callback3(hObject, eventdata, handles)

%% Visualize Labels 3 %%

    data=getappdata(0,'data');
    
    BalItems=get(handles.BalItems,'string');
    UnbalItems=get(handles.UnbalItems,'string');
    buffs={};
    buffs2={};
    
    checking=get(handles.Visualize_Labels3,'value');
    
    if checking==0
        
        %Balanced Items
        [f1,f2]=size(BalItems);
        for i=1:f1
            str=char(BalItems(i));
            [g1,g2]=size(data.labels);
            for j=i:g2
                check_pos=strcmp(str,data.labels(j));
                if check_pos==1
                    number=j;
                    break;
                end;
            end;
            buff=sprintf('%.0f',number);
            buffs(i)=cellstr(buff);
        end;

        %Unbalanced Items
        [f1,f2]=size(UnbalItems);
        for i=1:f1
            str=char(UnbalItems(i));
            [g1,g2]=size(data.labels);
            for j=i:g2
                check_pos=strcmp(str,data.labels(j));
                if check_pos==1
                    number=j;
                    break;
                end;
            end;
            buff2=sprintf('%.0f',number);
            buffs2(i)=cellstr(buff2);
        end;

    else
        
        %Balanced Items
        [f1,f2]=size(BalItems);
        for i=1:f1
            number=str2num(char(BalItems(i,:)));
            buff=data.labels(number);
            buffs(i)=cellstr(buff);
        end;

        %Unbalanced Items
        [f1,f2]=size(UnbalItems);
        for i=1:f1
            number=str2num(char(UnbalItems(i,:)));
            buff2=data.labels(number);
            buffs2(i)=cellstr(buff2);
        end;

    end;
    
    tmp1={};
    set(handles.BalItems,'string',tmp1);
    set(handles.BalItems,'value',1);
    set(handles.BalItems,'string',buffs); 

    set(handles.UnbalItems,'string',tmp1);
    set(handles.UnbalItems,'value',1);
    set(handles.UnbalItems,'string',buffs2); 

return;

function ButtonExcloureItems3_Callback (hObject,eventdata,handles)

%% Selecting items as unbalanced %%

    %Included in balanced items
    BalItems=cellstr(get(handles.BalItems,'string'));
    [n_items,tmp] = size(BalItems);
    %SD Items
    UnbalItems=get(handles.UnbalItems,'string');
    [h_n,tmp] = size(UnbalItems);
 
    %Selection
    Seleccio = get(handles.BalItems,'value');
    [tmp,num_selected] = size(Seleccio);
    
    %Check if the labels are visualized
    check_labels=get(handles.Visualize_Labels3,'value');
    
    %LOAD DATA
    data=getappdata(0,'data');
    
    %Remove excluded items
    nosel ={};h=1; nosel_i=[];
    if check_labels==0
        for i=1:n_items
            campana = 0;
            for j=1:num_selected
                if str2num(char(BalItems(i))) == str2num(char(BalItems(Seleccio(j))))
                    campana = 1;
                end;
            end;
            if campana == 0
                nosel(h) = BalItems(i);
                nosel_i(h)  = str2num(char(BalItems(i)));
                h=h+1;
            else
                h_n=h_n+1;
                UnbalItems(h_n) = BalItems(i);
            end;
        end;
    else
        for i=1:n_items
            campana = 0;
            for j=1:num_selected
                if strcmp(char(BalItems(i)),(char(BalItems(Seleccio(j)))))==1
                    campana = 1;
                end;
            end;
            if campana == 0
                nosel(h) = BalItems(i);
                [g1,g2]=size(data.labels);
                for j=i:g2
                    check_pos=strcmp(nosel(h),data.labels(j));
                    if check_pos==1
                        number=j;
                        break;
                    end;
                end;
                nosel_i(h)  = number;
                h=h+1;
            else
                h_n=h_n+1;
                UnbalItems(h_n) = BalItems(i);
            end;
        end;
    end;
    
    if (h<4)
        msgbox('At least three items are needed!','Psychological Test Toolbox','error');
        return;
    end;
    
    set(handles.BalItems,'value',1);
    if h_n > 0
        set(handles.UnbalItems,'value',1);
        set(handles.ButtonIncloureItems3,'enable','on');                
    else
        set(handles.UnbalItems,'value',0);
        set(handles.ButtonIncloureItems3,'enable','off');                
    end;
    
    set(handles.BalItems,'string',nosel);
    if check_labels==1
        UnbalItems2={};
        [g1,g2]=size(data.labels);
        for i=1:h_n
            str=char(UnbalItems(i));
            for j=1:g2
                check_pos=strcmp(str,data.labels(j));
                if check_pos==1
                    number=j;
                    break;
                end;
            end;
            UnbalItems2(i)=cellstr(num2str((number)));
        end;
        UnbalItems2=sort(str2num(char(UnbalItems2)));
        set(handles.UnbalItems,'string',data.labels(UnbalItems2));
    else
        UnbalItems2=cellstr(num2str(sort(str2num(char(UnbalItems)))));
        UnbalItems3=sort(str2num(char(UnbalItems)));
        [f1,f2]=size(UnbalItems3);
        buffs={};
        for i=1:f1
            buff=num2str(UnbalItems3(i));
            buffs(i)=cellstr(buff);
        end;
        set(handles.UnbalItems,'string',buffs);
    end;
    
    data.ib=nosel_i;
    if check_labels==1
        data.iub=UnbalItems2';
    else
        data.iub=rot90(str2num(char(UnbalItems2)));
    end;
    
%     data.itemsa=[data.items_sd data.items_contingut];%ITEMS AS ADMINISTRATION                3 6 10 14  2 4 5 7 8 9  11 12 13 15 16 17 18 19
    [f1,f2]=size(data.itemsa);
    itemsinorder=(1:f2);                               %ITEMS AS IN THE ORDER OF THE ANALYSIS  1 2  3  4  5 6 7 8 9 10 11 12 13 14 15 16 17 18
%     % ib    Balanced set of content items
%                                                      ib=[                                                 6 7   9 10 11 12 13 14 15 16 17 18];
%     % iub   Unbalanced set of content items
%     iub=[                                                                                               5     8                             ];

    [f1,f2]=size(data.ib);
    [g1,g2]=size(data.iub);
    
    set(handles.numberofbalanceditems,'string',f2);
    set(handles.numberofunbalanceditems,'string',g2);  
    
    matr=[data.itemsa;itemsinorder];

    for j=1:f2 %balanced
        a=data.ib(j);
        [b1,b2] = find(matr(1,:) == a);
        buff_b(j)=matr(2,b2);
    end;
    for k=1:g2
        a=data.iub(k);
        [c1,c2] = find(matr(1,:) == a);
        buff_u(k)=matr(2,c2);
    end;
    data.iba=buff_b;
    data.iuba=buff_u;
     
    data.Bandera=1;
    data.descriptives_done=0;
    %SAVE DATA 
    setappdata(0,'data',data);
    
    if isempty(data.H)==0 %The user had provided target matrix
        signes=sign(data.H(data.ib,:));
        posit=find(signes==1);
        [num_posit,~]=size(posit);
        negat=find(signes==-1);
        [num_negat,~]=size(negat);

        buff_text{1}='In this moment, the balanced core is composed by:';
        buff_text{2}=sprintf('%.0f positively worded items and %.0f negatively worded items. ',num_posit,num_negat);
        buff_text{3}='Please, remember that the balanced core has to be composed by the same number';
        buff_text{4}='of positively worded and negatively worded items.';
        set(handles.text_balanced_core,'visible','on','string',buff_text);
    else
        %if the user has not provided a valid target matrix but
        %actually provided the variable labels, we will check for some
        %indicator of positive or negative items
       if data.labels_user_on==1
            positive_n = strfind(get(handles.BalItems,'string'),'+');
            index1=find(~cellfun(@isempty,positive_n));
            [num_posit,~]=size(index1);

            negative_n = strfind(get(handles.BalItems,'string'),'-');
            index2=find(~cellfun(@isempty,negative_n));
            [num_negat,~]=size(index2);

            if isempty(num_posit)==0 && isempty(num_negat)==0
                buff_text{1}='Based on the provided labels, the balanced core is composed by:';
                buff_text{2}=sprintf('%.0f positively worded items and %.0f negatively worded items. ',num_posit,num_negat);
                buff_text{3}='Please, remember that the balanced core has to be composed by the same number';
                buff_text{4}='of positively worded and negatively worded items.';
                set(handles.text_balanced_core,'visible','on','string',buff_text);
            end;
        end;
    end;
    
return;

function ButtonIncloureItems3_Callback (hObject, eventdata, handles)

%% Selecting Items as balanced %%

    %LOAD DATA
    data=getappdata(0,'data');
    
    %Excluded
    UnbalItems=get(handles.UnbalItems,'string');
    [h_n,tmp] = size(UnbalItems);
 
    %Selection
    Seleccio = get(handles.UnbalItems,'value');
    [tmp,num_selected] = size(Seleccio);
    
    %Check if the labels are visualized
    check_labels=get(handles.Visualize_Labels3,'value');
    
    tmp={};hh=1;
    if check_labels==0
        for i=1:h_n
            campana = 0;
            for j=1:num_selected
                if i == Seleccio(j)
                    campana = 1;
                end;
            end;
            if campana==0
               tmp(hh)=UnbalItems(i);
               hh=hh+1;
            end;
        end;
    else
        for i=1:h_n
            campana = 0;
            for j=1:num_selected
                if strcmp(char(UnbalItems(i)),(char(UnbalItems(Seleccio(j)))))==1
                    campana = 1;
                end;
            end;
            if campana == 0
                tmp(hh) = UnbalItems(i);
                [g1,g2]=size(data.labels);
                for j=i:g2
                    check_pos=strcmp(tmp(hh),data.labels(j));
                    if check_pos==1
                        number=j;
                        break;
                    end;
                end;
                tmp_i(hh)  = number;
                hh=hh+1;
            end;
        end;
    end;
    
    UnbalItems = tmp;
    [tmp,h_n] = size(UnbalItems);
    
    [~,f2]=size(data.items_contingut);
       
    ItemsIncluded=data.items_contingut;
    %Remove excluded items
    nosel ={};h=1; nosel_i=[];
    if check_labels==0
        for i=1:f2
            a=ItemsIncluded(i);
    %         if a == str2num(char(
                campana = 0;
                for j=1:h_n
                    if a == str2num(char(UnbalItems(j)))
                        campana = 1;
                    end;
                end;
                if campana == 0
                    nosel_i(h)  = (a);
                    h=h+1;
                end;
        end;
    else
        for i=1:f2
            campana = 0;
            a=ItemsIncluded(i);
            for j=1:h_n
                if strcmp(char(data.labels(a)),(char(UnbalItems(j))))==1
                    campana = 1;
                end;
            end;
            if campana == 0
                nosel(h) = data.labels(a);
                [g1,g2]=size(data.labels);
                for j=i:g2
                    check_pos=strcmp(nosel(h),data.labels(j));
                    if check_pos==1
                        number=j;
                        break;
                    end;
                end;
                nosel_i(h)  = number;
                h=h+1;
            end;
        end;
    end;
    
    set(handles.BalItems,'value',1);
    if h_n > 0
        set(handles.UnbalItems,'value',1);
        set(handles.ButtonIncloureItems3,'enable','on');                
    else
        set(handles.UnbalItems,'value',0);
        set(handles.ButtonIncloureItems3,'enable','off');                
    end;
    
    if check_labels==0
        buffs={};
        for i=1:(h-1)
            buff=sprintf('%.0f',nosel_i(i));
            buffs(i)=cellstr(buff);
        end;
        set(handles.BalItems,'string',buffs);
    else
        set(handles.BalItems,'string',nosel);
    end;
    set(handles.UnbalItems,'string',UnbalItems);

    data.ib=nosel_i;
    if check_labels==1
        UnbalItems2={};
        [g1,g2]=size(data.labels);
        for i=1:h_n
            str=char(UnbalItems(i));
            for j=1:g2
                check_pos=strcmp(str,data.labels(j));
                if check_pos==1
                    number=j;
                    break;
                end;
            end;
            UnbalItems2(i)=cellstr(num2str((number)));
        end;
        UnbalItems2=sort(str2num(char(UnbalItems2)));
        data.iub=UnbalItems2';
    else
        data.iub=rot90(str2num(char(UnbalItems)));
    end;
    
    [f1,f2]=size(data.itemsa);
    itemsinorder=(1:f2);    

    [f1,f2]=size(data.ib);
    [g1,g2]=size(data.iub);
    
    set(handles.numberofbalanceditems,'string',f2);
    set(handles.numberofunbalanceditems,'string',g2);
     
    matr=[data.itemsa;itemsinorder];
    buff3=[];
    buff4=[];
    for j=1:f2 %balanced
        a=data.ib(j);
        [b1,b2] = find(matr(1,:) == a);
        buff3(j)=matr(2,b2);
    end;
    for k=1:g2
        a=data.iub(k);
        [c1,c2] = find(matr(1,:) == a);
        buff4(k)=matr(2,c2);
    end;

    data.iba=buff3;
    
    data.iuba=buff4;

    data.Bandera=1;
    data.descriptives_done=0;
    %SAVE DATA 
    setappdata(0,'data',data);
    
    if isempty(data.H)==0 %The user had provided target matrix
        signes=sign(data.H(data.ib,:));
        posit=find(signes==1);
        [num_posit,~]=size(posit);
        negat=find(signes==-1);
        [num_negat,~]=size(negat);

        buff_text{1}='In this moment, the balanced core is composed by:';
        buff_text{2}=sprintf('%.0f positively worded items and %.0f negatively worded items. ',num_posit,num_negat);
        buff_text{3}='Please, remember that the balanced core has to be composed by the same number';
        buff_text{4}='of positively worded and negatively worded items.';
        set(handles.text_balanced_core,'visible','on','string',buff_text);
    else
        %if the user has not provided a valid target matrix but
        %actually provided the variable labels, we will check for some
        %indicator of positive or negative items
       if data.labels_user_on==1
            positive_n = strfind(get(handles.BalItems,'string'),'+');
            index1=find(~cellfun(@isempty,positive_n));
            [num_posit,~]=size(index1);

            negative_n = strfind(get(handles.BalItems,'string'),'-');
            index2=find(~cellfun(@isempty,negative_n));
            [num_negat,~]=size(index2);

            if isempty(num_posit)==0 && isempty(num_negat)==0
                buff_text{1}='Based on the provided labels, the balanced core is composed by:';
                buff_text{2}=sprintf('%.0f positively worded items and %.0f negatively worded items. ',num_posit,num_negat);
                buff_text{3}='Please, remember that the balanced core has to be composed by the same number';
                buff_text{4}='of positively worded and negatively worded items.';
                set(handles.text_balanced_core,'visible','on','string',buff_text);
            end;
        end;
    end;
    
return;

function pfactorials(gcf, eventdata, handles)

%% Select factor scores to be computed %%

    %LOAD DATA
    data=getappdata(0,'data');
    
    check=get(handles.pfactors,'value');

    if check==1
        set(handles.pfactors,'value',1);
        data.pfactorial=1; 
        set(handles.save_pfactors,'visible','on',...
            'enable','off');
        
        if data.Matrix==3
            set(handles.tipus_pfactors,'visible','on',...
                'enable','on');
        else
            set(handles.tipus_pfactors,'visible','off',...
            'enable','off',...
            'value',1');
        end
        
        %check if the other controls are on
        A=get(handles.ButtonSalvarOutput2,'enable');
        tf=strcmp(A,'off');
        
        if tf==1
            set(handles.save_pfactors,'enable','off');
        else
            set(handles.save_pfactors,'enable','on');
        end
        
    else
        set(handles.pfactors,'value',0);
        data.pfactorial=0;
        set(handles.save_pfactors,'visible','off',...
            'enable','off');
        set(handles.tipus_pfactors,'visible','off',...
            'enable','off');
    end
    
    %SAVE DATA 
    setappdata(0,'data',data);

return;

function full_analysis_Callback(gcf,eventdata,handles);

%% Full analysis Callback (computing all the possibilities %%

    %LOAD DATA
    data=getappdata(0,'data');
    
    check=get(handles.full_analysis,'value');
    check_DS=get(handles.DS_checkbox,'value');
    check_AQ=get(handles.AQ_checkbox,'value');

    if check==1
        set(handles.full_analysis,'value',1);
        data.full_analysis=1; 
        
    else
        set(handles.full_analysis,'value',0);
        data.full_analysis=0;
    end;
    
    %SAVE DATA 
    setappdata(0,'data',data);

function factor_rotation(gcf, eventdata, handles)

%% Switch factor rotation method %%

    %LOAD DATA
    data=getappdata(0,'data');
    
    rot=get(handles.tipus_rotacio,'value');
    switch rot
        case 1 %ORTHOGONAL
            data.Rotation=1;
        case 2 %OBLIQUE
            data.Rotation=2;
        case 3 %TARGET OBLIQUE
            data.Rotation=3;
        case 4 %TARGET ORTHOGONAL
            data.Rotation=4;
    end;
    
    %SAVE DATA 
    setappdata(0,'data',data);

return;

function pfactors_method(gcf, eventdata, handles)

%% Switch EAP factor scores method %%

    %LOAD DATA
    data=getappdata(0,'data');
    
    tipus_pf=get(handles.tipus_pfactors,'value');
    switch tipus_pf
        case 1 %lineal
            data.graded=0;
            data.n_nodes=[];
        case 2 %graded
            
            n_nodes=number_nodes;   
            waitfor(n_nodes);
            
            %LOAD DATA
            data=getappdata(0,'data');
            
            if data.graded == 0
                set(handles.tipus_pfactors,'value',1);
            end;
            
     end
                
    %SAVE DATA 
    setappdata(0,'data',data);

return;




function Savetype2_Callback(gcf, eventdata, handles, datatype2_value)

%% Switch between types of saving formats %%

    data=getappdata(0,'data');

    switch datatype2_value
        case 1
            set(handles.save_text2,'value',1);
            set(handles.save_rtf2,'value',0);
            set(handles.save_latex2,'value',0);
            data.latex2_on=0;
            data.rtf2_on=0;
        case 2
            set(handles.save_text2,'value',0);
            set(handles.save_rtf2,'value',1);
            set(handles.save_latex2,'value',0);
            data.latex2_on=0;
            data.rtf2_on=1;
        case 3
            set(handles.save_text2,'value',0);
            set(handles.save_rtf2,'value',0);
            set(handles.save_latex2,'value',1);
            data.latex2_on=1;
            data.rtf2_on=0;
    end;
    
    setappdata(0,'data',data);
return;

function Compute_AQiDS_Callback (gcf, eventdata, handles)

%% Main function to compute FA, please see the documentation of each subfunction %%

    data=getappdata(0,'data');
    dim=char(get(handles.content_factors,'string'));
    
    % Content dimensions
    r_c = str2num(dim);
    
    %Variables controladas (SD i AQ)
    checking_DS=get(handles.DS_checkbox,'value');
    checking_AQ=get(handles.AQ_checkbox,'value');
    
    [f1,f2]=size(data.items_contingut);
    
    %% Checkings %%
    
    if isempty(dim)
        errordlg('Please, insert the number of content factors','Psychological Test Toolbox: Error');
        set(handles.content_factors,'backgroundcolor',[.8 .2 .2]);
        return;
    elseif r_c < 0
        errordlg('Please, insert a positive number of content factors','Psychological Test Toolbox: Error');
        set(handles.content_factors,'backgroundcolor',[.8 .2 .2]);
        return;
    elseif r_c > f2/3
        errordlg('At least three content items for each Factor/Component should be included in the analysis','Psychological Test Toolbox: Error');
        set(handles.content_factors,'backgroundcolor',[.8 .2 .2]);
        return;
    end;
    
    %CHECK IF ITS NUMERICAL
    num1=isstrprop(dim,'digit'); %return 1 if are digits and 0 if not
    find1=find(num1==0); %Search if in num1 there is some 0 value
    empt=isempty(find1); %Check that there is not any 0
    
    if empt==0
        errordlg('Please, insert a valid number of dimensions','Psychological Test Toolbox: Error','Error');
        set(handles.content_factors,'backgroundcolor',[.8 .2 .2]);
        return;
    end;
    
    if checking_DS==1
        if data.items_sd<3 
            errordlg('SD control is enabled. Please, select at least three items as SD markers or uncheck this option','Psychological Test Toolbox: Error');
            return;
        end;
        if isempty(data.items_sd)
            errordlg('SD control is enabled. Please, select at least three items as SD markers or uncheck this option','Psychological Test Toolbox: Error');
            return;
        end;            
    end;
        
    
    if checking_DS==1 && checking_AQ==1
        v_controled = 2;
    elseif checking_DS==1 && checking_AQ==0
        v_controled = 1;
    elseif checking_DS==0 && checking_AQ==1;
        v_controled = 1;
    else
        v_controled = 0;
    end;
    
    if data.matrix_user_on==0
        [N,m]=size(data.Xa);
    else
        [N,m]=size(data.Sa);
    end;
    
    G=data.G;
    
    %TARGET ADJUSTED A ITEMS ANALYSED (IN THE ORDER OF THE ANALYSIS)
    if isempty(data.H)
        
    else
        if data.Rotation==3 || data.Rotation==4
            H=data.H(data.itemsa,:);
%             data.H_cids=H;
            
    %CHECK IF SOME COLUMN OF THE TARGET MATRIX IS EMPTY 
            [f1,f2]=size(H);
            for i=1:f2
                buff=find(H(:,i)==0);
                [g1,g2]=size(buff);
                if f1==g1
                    errordlg('Based on selected items, there is some column of the target matrix which all elements are zero ','Psychological Test Toolbox: Error');
                    return;
                end;
            end;
            
   
            
    %CHECK IF TARGET MATRIX IS CONSISTENT WITH CONTENT FACTORS AND/OR SD
    %ANALYSIS
         [f1,f2]=size(H);

            if f2>r_c
                errordlg('The selected target matrix have more dimensions than expected','Psychological Test Toolbox: Error');
                set(handles.content_factors,'backgroundcolor',[.8 .2 .2]);
            return;
            elseif f2<r_c
                errordlg('The selected target matrix have less dimensions than expected','Psychological Test Toolbox: Error');
                set(handles.content_factors,'backgroundcolor',[.8 .2 .2]);
            return;
            end;
            %Hc: Target of contents items
            if isempty(data.G)
                Hc=H;
            else

                Htmp = H((G(1)+1):m,:);
                tmp=sum(abs(Htmp));
                [tmp2,tmp3]=size(tmp);
                Hc=[];
                for i=1:tmp3
                    if tmp(i)> 0
                        Hc=[Hc Htmp(:,i)];
                    end;
                end;
            end;
            data.Hc=Hc;
        end;
        
    end;
    
    %% STARTING ANALYSIS %%
    set(gcbf,'Pointer','watch');
    
    %Start new figure with an icon meanwhile other waitbars as the 
    %polychoric computing or factor scores are not initialized
    %new figure
    finestra.N2=250;
    finestra.M2=100;
    pantalla=get(0,'ScreenSize');
    finestra.coordenades=[(pantalla(3)/2)-(finestra.N2/2)  (pantalla(4)/2)-(finestra.M2/2) finestra.N2 finestra.M2];
    handles.fig_wait = figure('Units','Pixels',...
    'color' , [0.941 0.941 0.941],...
    'Position',finestra.coordenades);
    set(handles.fig_wait, 'MenuBar','none','Name','Psychological Test Toolbox','Toolbar','none','NumberTitle','off','resize','off');
    set(handles.fig_wait,'Pointer','watch');
    
    jObj = waiting(handles);
    
    drawnow;
    
%     bar0 = waitbar(0,'Computing ... Please wait ');
    
    TicAnalysis=tic;
    
    set(handles.save_text2,'value',1,...
        'enable','off');
    set(handles.save_latex2,'value',0,...
        'enable','off');
    set(handles.save_rtf2,'value',0,...
        'enable','off');
    set(handles.ButtonSalvarOutput2,'enable','off');
    set(handles.save_pfactors,'enable','off');
    set(handles.ButtonLaunchOutput2,'enable','off');
    
    %% Multiple Imputation of Missing Data (if necessary) %%
    
    if data.missings_on==1 %There is some missing value

        X_miss = data.X == data.missings_value ;
        X_missa = data.X(:,data.inclosos) == data.missings_value ;
        data.Xa=data.X(:,data.inclosos);

        % X_miss is composed by 0 if the score is not missing and 1 if it is

        [f1,f2]=size(X_miss);
        X_sensemiss=[];
        X_sensemissa=[];
        u=0;

        for i=1:f1
            a=any(X_miss(i,:) == 1);
            if a==1
                u=u+1;
                % Nothing, this participant have some missing value
            else
                X_sensemiss(i-u,:) = data.X(i,:);
            end;
        end;
        u=0;
        for i=1:f1
            a=any(X_missa(i,:) == 1);
            if a==1
                u=u+1;
                % Nothing, this participant have some missing value
            else
                X_sensemissa(i-u,:) = data.Xa(i,:);
            end;
        end;
    
%         data.Xa=X_sensemiss;
        [f1,f2]=size(X_sensemiss);
        data.X_sensemiss=X_sensemiss;
        data.X_sensemissa=X_sensemissa;
        data.N_sensemiss=f1;
        
%         [f1,f2]=size(data.X); %la matriu amb tots els participants i items
%         data.N_original=f1;
        
        setappdata(0,'data',data);


        % NUMBER OF MULTIPLE IMPUTATIONS
        k = 5;

        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %  ANALYSIS OF DATA WHERE MISSING DATA IS PRESENT
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

        % Multiple imputation: To produce k samples with imputate data
        Xm = HotDeck_imputation(data.Xa,X_missa,k);
        data.Xm=Xm;
        
        Xm_all = HotDeck_imputation(data.X,X_miss,k);
        data.Xm_all=Xm_all;
    end;
    
    %% Compute dispersion matrix (if necessary) %%

    if data.matrix_user_on==0 %% raw data
        if data.missings_on==1
            X=X_sensemiss;
            Xa=X_sensemissa;
        else
            X=data.X;
            Xa=data.Xa;
        end;
        %R Matrix
        if data.Matrix==1   %Covariance

            [N,m]=size(X);
            S=cov(X);
            R = diag(diag(S))^(-1/2) * S * diag(diag(S))^(-1/2);
            data.SS=R(data.itemsa,data.itemsa);
%             data.SS=R;

            R_a = R(data.itemsa,data.itemsa);

        elseif data.Matrix==2   %Pearson

            R=corrcoef(X);
            data.R=R(data.itemsa,data.itemsa);
%             data.R=R;

            R_a = R(data.itemsa,data.itemsa);

        elseif data.Matrix==3   %Polychoric
            
            %CHECK IF THE DISPERSION MATRIX IS ALREADY COMPUTED
            % PROBLEM: If Rp has been computed previously, we can not be
            % sure if the items are in the current order. Also, if it has
            % been computed in the descriptives section, in data.itemsa the
            % items will be in the order of administration, but if it has
            % been computed previously with AC & SD, in data.itemsa the
            % items are already in the order: first SD items.
            
            check_Rp=isempty(data.Rp_all);
            
            if check_Rp==1 % If it is empty it has to be computed
                
                [Rp,THRES]=polychomap(X,min(X),max(X),3,0,1);
                D=eig(Rp);
                data.R_NPD=0;
                data.kmo_error=0;
                  if min(D) <0
                     %NONLINEAR TRANSFORMATION
                      Rp = sr(Rp,data.N);
                      data.R_NPD = 1;
                      %Compute KMO, if it is in the last interval, cancel
                      % the analysis and recommend Pearson
                      data.kmo_error=0;
                      kmo_index=kmo(Rp(data.itemsa,data.itemsa));
                      if kmo_index < 0.5
                          data.kmo_error=1;
                      end;
                  end;
                  data.thresholds=THRES;
                  data.Rp = Rp(data.itemsa,data.itemsa);
                  R=Rp;
                  data.Rp_all=Rp;
                  R_a = R(data.itemsa,data.itemsa);
                
            else % If it exist, select the items in the order of the analysis
                R=data.Rp_all;
                data.Rp=R(data.itemsa,data.itemsa);
                
                R_a = R(data.itemsa,data.itemsa);   
            end;
            
            if isfield(data,'kmo_error')
                if data.kmo_error==1
                    kmo_index=kmo(data.Rp);
                    errordlg('The KMO obtained using Polychoric correlation is inacceptable. Please, select Pearson Correlation.','Psychological Test Toolbox: Error');
                    set(gcbf,'Pointer','arrow');

                    MyPrint(handles.ModelOutput2,' ');
                    MyPrint(handles.ModelOutput2,'The KMO obtained using Polychoric correlation is inacceptable. Please, select Pearson Correlation.');
                    buff = sprintf('KMO = %5.4f',kmo_index); MyPrint(handles.ModelOutput2,buff);

                    MyPrint(handles.ModelOutput2,' ');
                    MyPrint(handles.ModelOutput2,'-----------------------------------------------------------------------');
                    MyPrint(handles.ModelOutput2,' ');

                    set(handles.ModelOutput2,'value',1);

                    Printer=getappdata(0,'Printer');
                    Printer.text=get(handles.ModelOutput2,'string');
                    
                    data.Bandera=1;
                    setappdata(0,'data',data);
                    setappdata(0,'Printer',Printer);

                    %all done, close the figure
                    jObj.setBusyText('Analysis stopped!');jObj.stop; 

                    if ishandle(handles.fig_wait)
                        set(handles.fig_wait,'Pointer','arrow')
                        figure(handles.fig_wait);
                        t = timer;
                        t.StartDelay = 5;
                        t.TimerFcn = @(myTimerObj, thisEvent)close(handles.fig_wait);
                        start(t);
                    end;

                    %STOP THE ANALYSIS
                    return;
                end;
            end;
            
        end;
            
        setappdata(0,'data',data);

    else
        S=data.S;
        R = diag(diag(S))^(-1/2) * S * diag(diag(S))^(-1/2);
        R_a = R(data.itemsa,data.itemsa);
%         R=data.Sa;
    end;
    
    setappdata(0,'data',data);

    % RESUMING: R_a matrix with the selected items in order: first SD then
    % content
    %           R entire matrix
    % in data R_a is stored
    
    set(handles.content_factors,'backgroundcolor',[1 1 1]);
    
    conv= 0.00001;
    
    %---------------------------------------------------------------------%
    
    %% Begin FA %%
    
    % FROM THIS POINT, THE ANALYSIS DIVERGE DEPENDING ON IF THE USER HAS
    % SELECTED THE NORMAL ANALYSIS OR THE OPTION OF COMPUTING ALL THE 
    % POSSIBLE COMBINATIONS, WICH WILL TURN THE ANALYSIS IN TO A 
    % ITERATIVE ONE.
    
    if data.full_analysis==1
        if checking_DS==1 && checking_AQ==0
            number_of_loops=2;
        elseif checking_DS==0 && checking_AQ==1
            number_of_loops=2;
        elseif checking_DS==1 && checking_AQ==1
            number_of_loops=4;
        end;
    else
        number_of_loops=1;
    end;
    
   
    %Printing stuff
    tmp1={};
    global outputletter2;
    outputletter2=12;    
    set(handles.ModelOutput2,'string',tmp1,'value',1,'FontSize',outputletter2);
    set(handles.ModelOutput2_latex,'string',tmp1,'value',1);
    set(handles.ModelOutput2_rtf,'string',tmp1,'value',1);
    set(handles.Output_matrices,'string',tmp1,'value',1);
    if data.matrix_user_on==0
        PrintMatrix_apart (handles.Output_matrices,R_a);
    else
        PrintUserMatrix_apart (handles.Output_matrices,R_a);
    end;
    
    if checking_DS==0 && checking_AQ==1
        whatbias=1; %only Acquiescence
    elseif checking_DS==1 && checking_AQ==0
        whatbias=2; %only SD
    elseif checking_DS==1 && checking_AQ==1
        whatbias=3; %SD and Acquiescence
    else
        whatbias=0;
    end;
    
    data.th_all=[];
    
    for u=1:number_of_loops
        
        if data.full_analysis==0
            warning_target=0;
            if checking_DS==0 && checking_AQ==0
                %IF IN THE ACTUAL LOOP, NO BIAS HAS TO BE CONTROLLED
                data = Compute_without_bias(data,R_a,r_c,conv,R,warning_target);
            else
                %IF IN THE ACTUAL LOOP, SOME BIAS HAS TO BE CONTROLLED
                data = Compute_with_bias(data,R_a,R,r_c,conv,checking_DS,checking_AQ,v_controled,warning_target);
            end;
            Pc=data.Pc;
            if data.missings_on==1
                %IF THERE ARE MISSINGS, MEFA HAS TO BE COMPUTED ALSO
                data = Compute_missings(data,r_c,checking_DS,checking_AQ,v_controled,warning_target);
            end;
        else
            
            if u==1 %primer loop, sense biaxos
                checking_DS=0;
                checking_AQ=0;
                v_controled=0;
                if whatbias==2 || whatbias==3
                    warning_target=1;
                else
                    warning_target=0;
                end;
                
            else
                %configuration in base of number of bias selected and
                %number of current loop
                if whatbias==1
                    checking_AQ=1;
                    v_controled=1;
                    warning_target=0;
                    
                elseif whatbias==2
                    checking_DS=1;
                    v_controled=1;
                    warning_target=1;
                    
                elseif whatbias==3 %vol totes les opcions
                    warning_target=1;
                    if u==2 
                        checking_AQ=1;
                        checking_DS=0;
                        v_controled=1;
                    elseif u==3
                        checking_AQ=0;
                        checking_DS=1;
                        v_controled=1;
                    elseif u==4
                        checking_AQ=1;
                        checking_DS=1;
                        v_controled=2;
                    end;
                end;
            end;
            
            %Analysis
            setappdata(0,'data',data);
            if checking_DS==0 && checking_AQ==0
                %IF IN THE ACTUAL LOOP, NO BIAS HAS TO BE CONTROLLED
                data = Compute_without_bias(data,R_a,r_c,conv,R,warning_target);
            else
                %IF IN THE ACTUAL LOOP, SOME BIAS HAS TO BE CONTROLLED
                data = Compute_with_bias(data,R_a,R,r_c,conv,checking_DS,checking_AQ,v_controled,warning_target);
            end;
            setappdata(0,'data',data);
            Pc=data.Pc;
            if data.missings_on==1
                %IF THERE ARE MISSINGS, MEFA HAS TO BE COMPUTED ALSO
                data = Compute_missings(data,r_c,checking_DS,checking_AQ,v_controled,warning_target);
            end;
        end;
        
        setappdata(0,'data',data);

        
        if u==1
            if data.descriptives_done==0
                data.forced_descriptives=1;
                setappdata(0,'data',data);
                %IF DESCRIPTIVES WERE NOT COMPUTED PREVIOUSLY, THEY HAVE TO
                %BE COMPUTED AT THIS MOMENT
                Compute_Descriptives_Callback(gcbo,eventdata,handles);
                data.descriptives_done=1;
            end;
            b=get(handles.ModelOutput,'string');
            c=get(handles.ModelOutput_latex,'string');
            [f1,f2]=size(c);
            c=c(1:f1-1,f2);
            d=get(handles.ModelOutput_rtf,'string');
            [f1,f2]=size(d);
            d=d(1:f1-1,f2);
            set(handles.ModelOutput2,'string',b);
            set(handles.ModelOutput2_latex,'string',c);
            set(handles.ModelOutput2_rtf,'string',d);
        end;

        if data.pfactorial==1
            setappdata(0,'data',data);
            if data.missings_on==0 % No missings, compute factor scores sepparately
                %FACTOR SCORES FOR EACH PARTICIPANT
                data = ComputeFactorScores(data,warning_target,checking_DS,checking_AQ,R);
            end;
        end;
        setappdata(0,'data',data);
        
        %% PRINT FACTOR ANALYSIS %%
        Print_Factor_Analysis(handles.ModelOutput2,handles.ModelOutput2_latex,handles.ModelOutput2_rtf,handles.Output_matrices,data.itemsa',data.P,data.PHI,Pc,r_c,checking_DS,checking_AQ,number_of_loops,u,warning_target,R)

        if data.pfactorial==1
            jObj.setBusyText('Printing ... Please wait');
            %PRINT FACTOR SCORES OF EACH PARTICIPANT (IF REQUIERED)
            PrintFactorScores(data,r_c,handles.ModelOutput2,checking_DS,checking_AQ,u,number_of_loops);
            setappdata(0,'data',data);
            if number_of_loops>1
                [f1,f2]=size(data.th);
                separato=cell(f1,1);
                if u==number_of_loops
                    data.th_all=[data.th_all num2cell(data.th)];
                else
                    data.th_all=[data.th_all num2cell(data.th)  separato];
                end;
            else
                data.th_all=num2cell(data.th);
            end;
        end;
        
        compT = toc(TicAnalysis);

    end;
    
    %% PRINT REFERENCES OF THIS SECTION %%
    Print_References_factorial(handles.ModelOutput2,handles.ModelOutput2_latex,handles.ModelOutput2_rtf,handles,compT);
    
    %all done, close the figure
    jObj.setBusyText('Analysis complete!');
    if ishandle(handles.fig_wait)
        set(handles.fig_wait,'Pointer','arrow');
    end;
    jObj.stop;    
%     close(handles.fig_wait);

    MyPrint(handles.ModelOutput2_rtf,'}');
    MyPrint(handles.ModelOutput2_latex,'\end{document}');
    set(handles.ButtonSalvarOutput2,'enable','on');
    set(handles.save_latex2,'enable','on');
    set(handles.save_rtf2,'enable','on');
    set(handles.save_text2,'enable','on');
    set(handles.ButtonLaunchOutput2,'enable','on');
    check=get(handles.pfactors,'value');
    if check==1
        set(handles.save_pfactors,'enable','on');
    end;
    
    set(handles.ModelOutputIncrementa2,'enable','on');
    set(handles.ModelOutputDecrementa2,'enable','on');
        
    
%     s_tmp=['Analysis Complete! Time elapsed = ' Time2String( compT ) ''];
%     msgbox(s_tmp,'Psychological Test Toolbox');

    ButtonLaunchOutput_Callback2(gcbo,eventdata,handles);
    if checking_DS==0 && checking_AQ==0
        a='Neither Acquiescence or Social Desirability control functions were enabled. ';
        b='For more exhaustive exploratory factor analysis we recommend using Factor application, ';
        c='free download avaible in http://psico.fcep.urv.cat/utilitats/factor';
        msgbox([a b c],'Psychological Test Toolbox');
        close(handles.fig_wait);
    end;
    
    drawnow;
    
    if ishandle(handles.fig_wait)
        figure(handles.fig_wait);
        t = timer;
        t.StartDelay = 5;
        t.TimerFcn = @(myTimerObj, thisEvent)close(handles.fig_wait);
        start(t);
    end;
    
    Printer=getappdata(0,'Printer');
    Printer.text2=get(handles.ModelOutput2,'string');
    Printer.latex2=get(handles.ModelOutput2_latex,'string');
    Printer.rtf2=get(handles.ModelOutput2_rtf,'string');
    setappdata(0,'Printer',Printer);
    
    data.Bandera=1;
    setappdata(0,'data',data);
    
    set(gcbf,'Pointer','arrow');
    
    %play success sound
    [y,Fs]=audioread('success.wav');
    sound(y,Fs);
    
    
return;

function data = Compute_without_bias(data,R_a,r_c,conv,R,warning_target)
%% Compute EFA when no response bias has to be controlled
%
% ARGUMENTS   : data  -> data structure
%               R_a   -> Dispersion Matrix for the selected items
%               r_c   -> Factors to be retained
%               conv  -> Convergence criterion
%               R     -> Dispersion matrix containing all the items
%

    if warning_target==1
        R_a=R(data.items_contingut,data.items_contingut);
    end;
%     end;
    
    [A, Re, chi] = my_minres (R_a,r_c,conv,data.N);

    data.A = A;    
    data.Re_cont = Re;
    data.Re_final = Re;
    Hc=data.Hc;
    H=data.H;

    switch data.Rotation
        case 1 %ORTHOGONAL
            T = varim(A,conv,10);
            Pt=A*T;
            %reverse if necessary
            data.P = Pt*diag(sign(sum(Pt))*eye(r_c));
            data.PHI=eye(r_c);
        case 2 %OBLIQUE
            [T, U, W ] = promin (A,conv,10);
            Pt = A*T;
            PHIt = U'*U;
            %reverse if necessary
            [data.P,data.PHI] = orrown(Pt*diag(sign(sum(Pt))*eye(r_c)),Pt,PHIt);
        case 3 %TARGET OBLIQUE
            [Pr, PHIr] = tarrot_obl (A, abs(Hc), 0.001);
             %reverse if necessary
            [data.P, data.PHI] = orrown(Hc,Pr,PHIr);
            
        case 4 %TARGET ORTHOGONAL
            Pr = tarrotorth (A, abs(Hc), 0.001);
            PHIr = eye(r_c);
             %reverse if necessary
            [data.P, data.PHI] = orrown(Hc,Pr,PHIr);
            
    end;

    data.Pc=data.P;
    
return;

function data = Compute_with_bias(data,R_a,R,r_c,conv,checking_DS,checking_AQ,v_controled,warning_target)
%% Compute EFA when some (or both) response bias has to be controlled
%
% ARGUMENTS   : data  -> data structure
%               R_a   -> Dispersion Matrix for the selected items
%               r_c   -> Factors to be retained
%               conv  -> Convergene criterion
%               checking_DS -> Logical value indicating if Social
%               Desirability has to be controlled
%               checking_AQ -> Logical value indicating if Acquiescence
%               has to be controlled
%               v_controled -> number of biases controlled
%
    N=data.N;
    
    if warning_target==1 && checking_DS==0
        Hc=data.Hc;
        [f1,f2]=size(data.items_contingut);
        G=[0 f2];
%         a=(1:data.G(1));
        iba=data.iba-data.G(1);
        iuba=data.iuba - data.G(1);
        R_a=R(data.items_contingut,data.items_contingut);
    else
        Hc=data.Hc;
        G=data.G;
        iba=data.iba;
        iuba=data.iuba;
    end;
    
    if checking_DS==1 && checking_AQ==1                
        [P,EV,GFI,AGFI,RMSR,kelly,Re1,Re2]=control_sda_r(R_a,N,r_c,G,iba,iuba);
    elseif checking_DS==1 && checking_AQ==0
        [P,EV,GFI,AGFI,RMSR,kelly,Re1,Re2]=control_sd_r(R_a,N,r_c,G,iba,iuba);
    elseif checking_DS==0 && checking_AQ==1
        [P,EV,GFI,AGFI,RMSR,kelly,Re1,Re2]=control_a_r(R_a,N,r_c,G,iba,iuba);
    end;
    
    data.GFI=GFI;
    data.RMSR=RMSR;
    data.Re_cont=Re1;
    data.Re_final=Re2;
    data.Kelley=kelly;
    %FACTORS DE CONTINGUT
    if warning_target==1 && checking_DS==0
        [f1,m]=size(data.items_contingut);
    else
        [f1,m]=size(data.itemsa);
    end;


    if checking_AQ==1 
        P(:,2) = P(:,2)*diag(sign(sum(P(:,2))));
    end;

    Pc = P((G(1)+1):m,(checking_DS+checking_AQ+1):r_c+v_controled);

    switch data.Rotation
        case 1 
            %% ORTHOGONAL

            %ROTATION
            T = varim(Pc,conv,10);
            Pt=Pc*T;       
            Pt = Pt*diag(sign(sum(Pt)));
            PHI=eye(r_c+checking_DS+checking_AQ);
            P((G(1)+1):m,(checking_DS+checking_AQ+1):r_c+v_controled)=Pt;
            PHIt = eye(r_c);

            data.PHIt=PHIt;

        case 2 
            %% OBLIQUE

            %ROTATION
            [T, U, W ] = promin (Pc,conv,10);
            Pt = Pc*T;
            PHIt = U'*U;
            %reverse if necessary
            [Pt,PHIt] = orrown(Pt*diag(sign(sum(Pt))*eye(r_c)),Pt,PHIt);
            P((G(1)+1):m,(checking_DS+checking_AQ+1):r_c+v_controled)=Pt;

            PHI=eye(r_c+checking_DS+checking_AQ);
            PHI((checking_DS+checking_AQ+1):(checking_DS+checking_AQ+r_c),(checking_DS+checking_AQ+1):(checking_DS+checking_AQ+r_c))=PHIt;

            data.PHIt=PHIt;

        case 3 
            %%TARGET OBL

            % Comprobation of the sizes. Hc could have less columns than Pc
            % at this point if all the values of any column are 0.
            [j1,j2]=size(Pc);
            [k1,k2]=size(Hc);
            if j2==k2
            else
                buff_error{1}='All the values of some column of the target matrix are 0';
                buff_error{2}='once the SD factor rows are extracted from the target matrix.';
                buff_error{3}='Please, make sure that the target matrix contains only the columns';
                buff_error{4}='related to content factors, and the SD items are defined with 0';
                buff_error{5}='in all the columns.';
                errordlg(buff_error,'Psychological Test Toolbox');
                figHandles = get(0,'Children');
                set(figHandles,'Pointer','arrow');
                close(figHandles(2));
                error('The target matrix related to content factors which at least one defined value has less columns than the extracted factors');
            end;
            
            %ROTATION
            [Pr, PHIr] = tarrot_obl (Pc, abs(Hc), 0.001);
             %reverse if necessary
            [Pr, PHIr] = orrown(Hc,Pr,PHIr);

            P((G(1)+1):m,(v_controled+1):(r_c+v_controled)) = Pr;
            PHI=eye(r_c+v_controled);
            PHI((v_controled+1):(r_c+v_controled),(v_controled+1):(r_c+v_controled))=PHIr;

            data.PHIr=PHIr;

         case 4 
             %% TARGET ORTH

            % Comprobation of the sizes. Hc could have less columns than Pc
            % at this point if all the values of any column are 0.
            [j1,j2]=size(Pc);
            [k1,k2]=size(Hc);
            if j2==k2
            else
                buff_error{1}='All the values of some column of the target matrix are 0';
                buff_error{2}='once the SD factor rows are extracted from the target matrix.';
                buff_error{3}='Please, make sure that the target matrix contains only the columns';
                buff_error{4}='related to content factors, and the SD items are defined with 0';
                buff_error{5}='in all the columns.';
                errordlg(buff_error,'Psychological Test Toolbox');
                figHandles = get(0,'Children');
                set(figHandles,'Pointer','arrow');
                close(figHandles(2));
                error('The target matrix related to content factors which at least one defined value has less columns than the extracted factors');
            end;
            
            %ROTATION
            Pr = tarrotorth (Pc, abs(Hc), 0.001);
            PHIr = eye(r_c+checking_DS+checking_AQ);
             %reverse if necessary
            [Pr, PHIr] = orrown(Hc,Pr,PHIr);

            P((G(1)+1):m,(v_controled+1):(r_c+v_controled)) = Pr;
            PHI=eye(r_c+v_controled);
            PHI((v_controled+1):(r_c+v_controled),(v_controled+1):(r_c+v_controled))=PHIr;

            data.PHIr=PHIr;

    end;
    data.Pc = P((G(1)+1):m,(checking_DS+checking_AQ+1):r_c+v_controled);
    data.PHI=PHI;
    data.P=P;
    data.A=[];

return;

function data = Compute_missings(data,r_c,checking_DS,checking_AQ,v_controled,warning_target)
%% Compute EFA when there are missing values
%
% ARGUMENTS   : data  -> data structure
%               r_c   -> Factors to be retained
%               conv  -> Convergene criterion
%               checking_DS -> Logical value indicating if Social
%               Desirability has to be controlled
%               checking_AQ -> Logical value indicating if Acquiescence
%               has to be controlled
%               v_controled -> number of biases controlled
%
    G=data.G;
    Hc=data.Hc;
    
    if isempty(data.H)
        checking_H=[];
    else
        if data.Rotation==3 || data.Rotation==4
%             [m,r] = size(data.H);
            checking_H=1;
%                 H=data.H(data.itemsa,:);
        else
            checking_H=[];
        end;
    end;
    r=r_c;
    
    %% Content Factors
        
    if warning_target==1 && checking_DS==0
        Xm=data.Xm_all(:,data.items_contingut,:);
        [f1,f2]=size(data.items_contingut);
        G=[0 f2];
        m=f2;
        iba=data.iba-data.G(1);
        iuba=data.iuba - data.G(1);
        items_order=data.items_contingut;
    else
        if data.matrix_user_on==1
            [f1,m]=size(data.Sa);
        else
            [f1,m]=size(data.Xa);
        end;
        Xm=data.Xm;
        G=data.G;
        iba=data.iba;
        iuba=data.iuba;
        items_order=data.itemsa;
    end;
    
    Xm_apropiat=data.Xm_all(:,items_order,:);
    
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %  ANALYSIS OF DATA WHERE MISSING DATA IS PRESENT
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    % EFA of the k samples (separate factor analysis, consensus rotation, & averaged factor scores)
    if checking_DS==0 && checking_AQ==0
        if isempty(checking_H)
            [Pi, PHIi, Fi,Fint,F_li,F_ls,Se,reli] = mefa(Xm_apropiat,r,data.Matrix,data.Rotation,data.pfactorial);
        else
            [Pi, PHIi, Fi,Fint,F_li,F_ls,Se,reli] = mefa(Xm_apropiat,r,data.Matrix,data.Rotation,data.pfactorial,Hc);
        end;
    else
        Pc_sensemiss=data.P((G(1)+1):m,(checking_DS+checking_AQ+1):r_c+v_controled);
        if isempty(checking_H)
            [Pi, PHIi, Fi,Fint,F_li,F_ls,Se,reli] = mefa_sda(Xm_apropiat,r,data.Matrix,data.Rotation,G,iba,iuba,checking_DS,checking_AQ,Pc_sensemiss,data.pfactorial);
        else
            [Pi, PHIi, Fi,Fint,F_li,F_ls,Se,reli] = mefa_sda(Xm_apropiat,r,data.Matrix,data.Rotation,G,iba,iuba,checking_DS,checking_AQ,Pc_sensemiss,data.pfactorial,Hc);
        end;
    end;
    data.P=Pi;
    
    data.Pc = data.P((G(1)+1):m,(checking_DS+checking_AQ+1):r_c+v_controled);

    % PHI computed without missing values
    data.PHIi=PHIi;data.th=Fi;
    data.th_li=F_li;
    data.th_ls=F_ls;
    data.se=Se;
    data.reli=reli;
    
return;

function data = ComputeFactorScores(data,warning_target,checking_DS,checking_AQ,R)
%% Compute Factor Scores
%
% ARGUMENTS   : data  -> data structure
%               checking_DS -> Logical value indicating if Social
%               Desirability has to be controlled
%               checking_AQ -> Logical value indicating if Acquiescence
%               has to be controlled
%               R -> Dispersion Matrix
%

    P = data.P;
    if warning_target==1 && checking_DS==0
        X= data.X(:,data.items_contingut);
        R_a=R(data.items_contingut,data.items_contingut);
    else
        X = data.X(:,data.itemsa);
        R_a=R(data.itemsa,data.itemsa);
    end;
    PHI=data.PHI;
    
   switch data.Matrix
        %Variance matrix or Pearson
        case {1,2}
            [th,th_li,th_ls,se,reli] = eap_continuous_obli(X,P,PHI,R_a);
            
        %Polychoric    
        case 3
            THRES = data.thresholds(data.itemsa,:);
            [m,k]=size(THRES);
            THRES = [THRES zeros(m,1)]';
            R_clean=R(data.items_contingut,data.items_contingut);
            
            if isempty(data.A) 
                if warning_target==1 && checking_DS==0
                    R=R(data.items_contingut,data.items_contingut);
                    sigj = diag(sqrt( diag(diag(R - P*PHI*P'))));
                else
                    sigj = diag(sqrt( diag(diag(data.Rp - P*PHI*P'))));
                end;
                
            else 
                if warning_target==1 && checking_DS==0
                    
                    R=R(data.items_contingut,data.items_contingut);
                    sigj = diag(sqrt( diag(diag(R - data.A*data.A'))));
                else
                    sigj = diag(sqrt( diag(diag(data.Rp - data.A*data.A'))));
                end;
            end;     
            
            if data.graded==1
                [th,th_li,th_ls,se,reli]  = reap_grad_obli(X, P, PHI, THRES, sigj,data.n_nodes,1); 
            else
                
                if warning_target==1 && checking_DS==0
                    [th,th_li,th_ls,se,reli] = eap_continuous_obli(X,P,PHI,R_clean);
                else
                    [th,th_li,th_ls,se,reli] = eap_continuous_obli(X,P,PHI,data.Rp);
                end;
                
            end;
%                            
                %it has to be done in separate steps:
%                 m=data.G(1)+data.G(2);
%                 [g1,g2]=size(data.P);
%                 PHI_c=data.PHI(checking_DS+checking_AQ+1:g2,checking_DS+checking_AQ+1:g2);
%                 if checking_DS==1
%                     [th1,th_li1,th_ls1,se1,reli1] = reap_grad_obli(X, P(:,1), 1, THRES, sigj,1,1);
%                     
%                     if checking_AQ==1
%                         %AC
%                         [th2,th_li2,th_ls2,se2,reli2] = reap_grad_obli(X(:,data.G(1)+1:m), P(data.G(1)+1:m,2), 1, THRES(:,data.G(1)+1:m), sigj(data.G(1)+1:m),1,2);
%                         
%                         %content
%                         [th3,th_li3,th_ls3,se3,reli3] = reap_grad_obli(X(:,data.G(1)+1:m), P(data.G(1)+1:m,3:g2), PHI_c, THRES(:,data.G(1)+1:m), sigj(data.G(1)+1:m),1,3);
%                     else
%                         %AC (NO)
%                         th2=[]; th_li2=[]; th_ls2=[]; se2=[]; reli2=[];
%                         
%                         %content
%                         [th3,th_li3,th_ls3,se3,reli3] = reap_grad_obli(X(:,data.G(1)+1:m), P(data.G(1)+1:m,2:g2), PHI_c, THRES(:,data.G(1)+1:m), sigj(data.G(1)+1:m),1,3);
%                     end;
%                 else
%                     %SD (no)
%                     th1=[]; th_li1=[]; th_ls1=[]; se1=[]; reli1=[];
%                     if checking_AQ==1
%                         [th2,th_li2,th_ls2,se2,reli2] = reap_grad_obli(X, P(:,1), 1, THRES, sigj,1,2);
%                         
%                         %content
%                         [th3,th_li3,th_ls3,se3,reli3] = reap_grad_obli(X, P(:,2:g2), PHI_c, THRES, sigj,1,3);
%                     else
%                         %AC (NO)
%                         th2=[]; th_li2=[]; th_ls2=[]; se2=[]; reli2=[];
%                         
%                         %content
%                         [th3,th_li3,th_ls3,se3,reli3] = reap_grad_obli(X, P, PHI_c, THRES, sigj,1,3);
%                     end;
%                     
%                 end;
%                 th=[th1 th2 th3];
%                 th_li=[th_li1 th_li2 th_li3];
%                 th_ls=[th_ls1 th_ls2 th_ls3];
%                 se=[se1 se2 se3];
%                 reli=[reli1 reli2 reli3];
    end;
    
    
    data.th=(th*10)+50;
    data.th_li=(th_li*10)+50;
    data.th_ls=(th_ls*10)+50;
    data.se=se;
    data.reli=reli;

return

function PrintFactorScores(data,r,Output,checking_DS,checking_AQ,current,number_of_loops)
%% Print Factor Scores
%
% ARGUMENTS   : data  -> data structure
%               r   -> Factors to be retained
%               Output  -> plain text listbox
%               checking_DS -> Logical value indicating if Social
%               Desirability has to be controlled
%               checking_AQ -> Logical value indicating if Acquiescence
%               has to be controlled
%               current -> Which index is the current one
%               number_of_loops -> Number of total loops
%
    N=data.N_original;
    th = data.th;
    se = data.se;
    th_li=data.th_li;
    th_ls=data.th_ls;
    reli=data.reli;
    
    if data.graded==1
        reli_good=[];
        [f1,f2]=size(reli);
        j=1;
        for i=1:f1
            if (sum(reli(i,:)>0.10)==f2)
                reli_good(j,:)=reli(i,:);
                j=j+1;
            end
        end
    end
    
    MyPrint(Output,'-----------------------------------------------------------------------');
    MyPrint(Output,' ');
    buff='RELIABILITY OF ';
    if data.Rotation==1 || data.Rotation==4
        buff2='EAP SCORES';
    elseif data.Rotation==2 || data.Rotation==3
        buff2='PHI-INFORMATION OBLIQUE EAP SCORES';
    end;
    MyPrint(Output,[buff buff2]);
    MyPrint(Output,'Ferrando & Lorenzo-Seva (2016)');
    MyPrint(Output,' ');
    
    [f1,f2]=size(reli);
    prov=checking_DS+checking_AQ;
    MyPrint(Output,'Factor         EAP Reliability estimate');
    MyPrint(Output,' ');
    for i=1:r+checking_DS+checking_AQ
        if i==1 && checking_DS==1
            buff='SD           :';
        elseif i==1 && checking_DS==0 && checking_AQ==1
            buff='Acquiescence :';
        elseif i==2 && checking_DS==1 && checking_AQ==1
            buff='Acquiescence :';
        else
            buff=sprintf('Factor %.0f     :',i-prov);
        end
        
        if data.Matrix==1 || data.Matrix==2
            buff2=sprintf(' %.4f',reli(i));
        else
            if data.graded==1
                buff2=sprintf(' %.4f',harmmean(reli_good(:,i)));
            else
                buff2=sprintf(' %.4f',reli(i));
            end
        end
        MyPrint(Output,[buff buff2]);
    end
        

%Calcular les linies de puntuacions a imprir

    if data.full_analysis==0
        TotalLines = N + N*(r+checking_DS+checking_AQ);
    else
        TotalLines = N;
    end
    
    %Print th score
    MyPrint(Output,' ');
    buff='PARTICIPANTS''S SCORES ON FACTORS: ';
    if data.Rotation==2 || data.Rotation==3
        buff2='PHI-Information Oblique EAP scores';
    elseif data.Rotation==1 || data.Rotation==4
        buff2='EAP scores';
    end
    MyPrint(Output,[buff buff2]);
    MyPrint(Output,'Rescaled to mean = 50 and standard deviation = 10 in the sample');
    MyPrint(Output,' ');
    MyPrint(Output,'Ferrando & Lorenzo-Seva (2016)');
    MyPrint(Output,' ');
    MyPrint(Output,' ');
   
    buff3 = 'Participant   ';
    if checking_DS==1 && checking_AQ==1
        buff2='SD         AC         ';
    elseif checking_DS==1 && checking_AQ==0
        buff2='SD          ';
    elseif checking_DS==0 && checking_AQ==1
        buff2='AC          ';
    else
        buff2='';
    end
    buff='';
    for i=1:r        
        buff1=sprintf('F%2.0f        ',i);
        buff =[buff buff1];
    end
    buff=[buff3 buff2 buff];

    MyPrint(Output,buff);
    
    BarCount = 0; bar1 = waitbar(BarCount,'Printing ... ','CreateCancelBtn','setappdata(gcbf,''canceling'',1)');
    if number_of_loops>1
        pos_1=get(bar1,'position');
        pos_2=[pos_1(1) pos_1(2) pos_1(3) (1.25*pos_1(4))];
        set(bar1,'position',pos_2);
    end
    setappdata(bar1,'canceling',0)
    TicPrint = tic;
    
    for i=1:N
        if getappdata(bar1,'canceling')
            break
        end
        buff=sprintf('%4.0f         ',i);
        for j=1:r+(checking_DS+checking_AQ)
            buff1=sprintf('% 7.3f    ',th(i,j));
            buff=[buff buff1];
        end
        MyPrint(Output,buff);
        
        BarCount = BarCount + 1; 
        compT = toc(TicPrint);
        compT = compT*(TotalLines-BarCount)/BarCount;
        s_tmp{1}=['Printing ... (Remaining time = ' Time2String( compT ) ')'];
        if number_of_loops>1
            s_tmp{2}=sprintf('Current output printing; %1.0f of %1.0f',current,number_of_loops);
        end
        waitbar(BarCount/TotalLines,bar1,s_tmp);
    end
    
    if data.full_analysis==0
        if getappdata(bar1,'canceling')
            delete(bar1);
            return;
        end
         
        MyPrint(Output,' ');
        MyPrint(Output,' ');
        for j=1:(r+checking_DS+checking_AQ)
            if j==1 && checking_DS==1
                buff='FACTOR: SOCIAL DESIRABILITY      ';
            elseif j==1 && checking_DS==0 && checking_AQ==1
                buff='FACTOR: ACQUIESCENCE      ';
            elseif j==2 && checking_DS==1 && checking_AQ==1
                buff='FACTOR: ACQUIESCENCE      ';
            else
                buff=sprintf('FACTOR: CONTENT %1.0f      ',j-prov);
            end
            MyPrint(Output,buff);
            MyPrint(Output,' ');
            
            switch data.Matrix
                case {1,2}
                      buff=sprintf('RELIABILITY OF EAP FACTOR SCORES:   % .3f          ',reli(j));
                      MyPrint(Output,buff);
                      MyPrint(Output,' ');
                      MyPrint(Output,'Participant   Estimate        Approximate 90%      Posterior');
                case 3   
                    if data.graded==1
                        buff=sprintf('RELIABILITY OF EAP FACTOR SCORES:   % .3f          ',harmmean(reli(:,j)));
                    else
                        buff=sprintf('RELIABILITY OF EAP FACTOR SCORES:   % .3f          ',reli(j));
                    end
                      MyPrint(Output,buff);
                      MyPrint(Output,' ');
                      if data.graded==1
                          MyPrint(Output,'Participant   Estimate        Approximate 90%      Posterior   Reliability');
                      else
                          MyPrint(Output,'Participant   Estimate        Approximate 90%      Posterior');
                      end
            end
            MyPrint(Output,'              factor score    confidence interval  SD (PSD)');

            MyPrint(Output,' ');
            for i=1:N
                if getappdata(bar1,'canceling')
                    break
                end

                buff=sprintf('%4.0f         % 7.3f        ',i,th(i,j));
                buff1=sprintf('(% 7.3f   % 7.3f)   ',th_li(i,j),th_ls(i,j));
                switch data.Matrix
                    case {1,2}
                          buff2=sprintf('% .3f       ',se(j,j));
                    case 3  
                        if data.graded==1
                            if reli(i,j)>0.10
                                buff2=sprintf('% .3f      % .3f',se(i,j),reli(i,j));
                            else
                                buff2=sprintf('% .3f       Inconsistent responder',se(i,j));
                            end
                        else
                            buff2=sprintf('% .3f       ',se(j,j));
                        end
                end

                buff =[ buff buff1 buff2];
                MyPrint(Output,buff);

                %Bar
                BarCount = BarCount + 1; 
                compT = toc(TicPrint);
                compT = compT*(TotalLines-BarCount)/BarCount;
                s_tmp=['Printing ... (Remaining time = ' Time2String( compT ) ')'];
                waitbar(BarCount/TotalLines,bar1,s_tmp);

            end
            MyPrint(Output,' ');
        end
       
    end
    
    delete(bar1);
    
    MyPrint(Output,' ');
    MyPrint(Output,'-----------------------------------------------------------------------');

return;

function Print_Factor_Analysis(Output,Output2,Output3,Output_matrices,items,P,PHI,Pc,r_c,checking_DS,checking_AQ,number_of_loops,current,warning_target,R)
%% Principal printing function for the EFA section
%
% ARGUMENTS   : Output  -> plain text listbox
%               Output2 -> LaTeX listbox
%               Output3 -> rtf listbox
%               Output_matrics -> listbox containing relevant matrices
%               P   -> Loading Matrix
%               PHI -> Inter-Factors Correlation
%               Pc  -> Loading Matrix of content items
%               r_c -> Number of factors to be retained
%               checking_DS -> Logical value indicating if Social
%               Desirability has to be controlled
%               checking_AQ -> Logical value indicating if Acquiescence
%               has to be controlled
%               current -> Which index is the current one
%               number_of_loops -> Number of total loops
%               R  -> Dispersion Matrix

    data=getappdata(0,'data');

    [f1,f2]=size(data.items_contingut);
    
    buff={};
    for i=1:f2
            buff{i}=sprintf('%s',data.labels{data.items_contingut(i)});
    end;
    labels_contingut=buff; % Content labels
    
    if ischar(labels_contingut)
        str=cellstr(labels_contingut);
    else
        str=labels_contingut;
    end;


    [ml,tmp]=size(str);
     ml = max([ml;tmp]);

    s=0;
    for i=1:ml
        if s < length(str{i})
            s= length(str{i});
        end;
    end;
    if s>30, s=30;end;
    if s<5 , s=5;end;
    
    [f1,f2]=size(data.itemsa);
    buff={};
    for i=1:f2
        buff{i}=sprintf('%s',data.labels{data.itemsa(i)});
    end;
    labels_tots=buff;
    labels_tots_enordre=data.labelsa;
    
    target=data.Hc; 
    target_ones=data.Hc;
    target_ones(target_ones>0)=1;
    target_ones(target_ones<0)=-1;
    Pc_correct=data.Pc;
     
    %for congruence test for each item, congru don't work if in some item,
    %every value of the target matrix=0
    if data.Rotation==3 || data.Rotation==4
        [f1,f2]=size(target_ones);
    else
        f1=[];
    end;
    target_ones_without_zeros=[];
    Pc_correct_without_zeros=[];
    str_without_zeros={};
    u=0;
    for i=1:f1
        find_ones=find(target_ones(i,:)==1|target_ones(i,:)==-1);
        if isempty(find_ones)
            u=u+1;
        else
            target_ones_without_zeros(i-u,:)=target_ones(i,:);
            Pc_correct_without_zeros(i-u,:)=Pc_correct(i,:);
            str_without_zeros{i-u}=str{i};
        end;
    end;
    
    if warning_target==1 && checking_DS==0
        R=R(data.items_contingut,data.items_contingut);
    else
        switch data.Matrix
            case 1
               R=data.SS;
            case 2
               R=data.R;
            case 3
               R=data.Rp;
        end;
    end;
    if data.matrix_user_on==1
        if warning_target==1 && checking_DS==0
            R=data.Sa(data.items_contingut,data.items_contingut);
        else
            R=data.Sa;
        end;
    end;
    if isfield(data,'Re_final');
        Re=data.Re_final;
    else
        Re=data.Re;
    end;
    [m,~]=size(Re);
    
    %GFI
    RES = R - Re;
    RES = RES - diag(diag(RES));
    R0 = R - diag(diag(R));
    GFI = 1- trace(RES*RES)/trace(R0*R0);

    data.GFI = GFI;
    
    %RMSR
     A=zeros(m*(m-1)/2,1);
     h = 1;
     for i=1:m
        for j=(i+1): m
            A(h,1)= R(j,i) - Re(j,i);
            h=h+1;
        end;
     end;

     me = mean(A);
     sd = std(A);
     RMSR = sqrt(sd*sd+me*me);
     se = sqrt(1/(data.N-1));
     if checking_DS==0 && checking_AQ==0
        data.RMSR = RMSR;
        data.Kelley = se;
     end;
     
     se=sqrt(1/(data.N-3));
     A_stand=(A/se);

    %% LATEX %%
    
    if checking_DS==0 && checking_AQ==0
        buff='\section{Exploratory Factor Analysis without controlling bias}';
    elseif checking_DS==1 && checking_AQ==0
        buff='\section{Exploratory Factor Analysis controlling Social Desirability}';
    elseif checking_DS==0 && checking_AQ==1
        buff='\section{Exploratory Factor Analysis controlling Acquiescence}';
    elseif checking_DS==1 && checking_AQ==1
        buff='\section{Exploratory Factor Analysis controlling Social Desirability and Acquiescence}';
    end;
    
    MyPrint(Output2,buff); MyPrint(Output2,'~ \');MyPrint(Output2,' ');
    buff=('Goodness of Fit Index (GFI) =');
    buff1=sprintf('% 7.4f ',data.GFI);
    buff4='\\';
    MyPrint(Output2,[buff buff1 buff4]);
    
    if data.Rotation==3 || data.Rotation==4 %Target
        MyPrint(Output2,'\begin{center}');
        [f1,f2] = size(target);
        buff='\begin{longtable}[H]';
        buff1=sprintf('{l*{%-5.0f}{S}}',f2);
        MyPrint(Output2,[buff buff1]);
        MyPrint(Output2,'\caption{Semi-Specified Target Loading Matrix}\\');
        MyPrint(Output2,'\toprule');
        buff='\multicolumn{1}{l}{\textbf{Item}}';
        for i=1:r_c
            bufff=' & \multicolumn{1}{c}{\textbf';
            buffff=sprintf(' {Factor %.0f}}',i);
            buff2{i}=[bufff buffff];
        end;
        buff3=sprintf('%s',buff2{:});
        MyPrint(Output2,[buff buff3 buff4]);
        MyPrint(Output2,'\midrule');
        MyPrint(Output2,'\endfirsthead');
        buff01='\multicolumn';
        buff02=sprintf('{%.0f}{c}',f2+1);
        MyPrint(Output2,[buff01 buff02]);
        MyPrint(Output2,'{\tablename\ \thetable\ -- \textit{(Continued from previous page)}} \\');
        MyPrint(Output2,'\toprule');
        MyPrint(Output2,[buff buff3 buff4]);
        MyPrint(Output2,'\midrule');
        MyPrint(Output2,'\endhead');
        buff01='\bottomrule \multicolumn';
        buff02=sprintf('{%.0f}',f2+1);
        buff03='{r}{\textit{Continued on next page}}';
        MyPrint(Output2,[buff01 buff02 buff03 buff4]);
        MyPrint(Output2,'\endfoot');
        MyPrint(Output2,'\bottomrule');
        MyPrint(Output2,'\endlastfoot');
        
        [f1,f2]=size(data.items_contingut);
        for i=1:f2
            form=sprintf('%%-%d.%ds  ',s,s);   
            buff=sprintf(form,str{i});
            buff2=sprintf(' & % .0f    ',target(i,:));
            MyPrint(Output2,[buff buff2 buff4]);
        end;   
        MyPrint(Output2,'\end{longtable}'); 
        MyPrint(Output2,'\end{center}');
    end;
    
    clearvars buff buff2 buff3 buff4
    
    buff4='\\';
    [f1,f2] = size(P);
    MyPrint(Output2,'\begin{center}');
    buff='\begin{longtable}[H]';
    buff1=sprintf('{l*{%-5.0f}{S}}',f2+checking_DS+checking_AQ);
    MyPrint(Output2,[buff buff1]);
    
    
    if r_c == 1
        MyPrint(Output2,'\caption{Unrotated Loading Matrix}\\');
    else
        MyPrint(Output2,'\caption{Rotated Loading Matrix}\\');
    end    
    MyPrint(Output2,'\toprule');
    
    buff_it='\multicolumn{1}{l}{\textbf';
    buff_other='\multicolumn{1}{c}{\textbf';
    if checking_DS==1 && checking_AQ==0
        buff=[buff_it '{Item}} & ' buff_other '{SD}} '];
        MyPrint(Output2,buff);
        for i=1:r_c
            buff2=sprintf(' & %s{Factor %.0f}}',buff_other,i);
            MyPrint(Output2,buff2);
        end;
        
    elseif checking_DS==0 && checking_AQ==1
        buff=[buff_it '{Item}} & ' buff_other '{AC}} '];
        MyPrint(Output2,buff);
        for i=1:r_c
            buff2=sprintf(' & %s{Factor %.0f}}',buff_other,i);
            MyPrint(Output2,buff2);
        end;
        
    elseif checking_DS==1 && checking_AQ==1
        buff=[buff_it '{Item}} & ' buff_other '{SD}} &' buff_other '{AC}} '];
        MyPrint(Output2,buff);
        for i=1:r_c
            buff2=sprintf(' &  %s{Factor %.0f}}',buff_other,i);
            MyPrint(Output2,buff2);
        end;
        
    elseif checking_DS==0 && checking_AQ==0
        buff=[buff_it '{Item}} '];
        MyPrint(Output2,buff);
        for i=1:r_c
            buff2=sprintf(' & %s{Factor %.0f}}',buff_other,i);
            MyPrint(Output2,buff2);
        end;
    end;
    MyPrint(Output2,buff4);
    MyPrint(Output2,'\midrule');
    MyPrint(Output2,'\endfirsthead');
    buff='\multicolumn';
    buff1 = sprintf('{%2.0f}{c}',checking_DS+checking_AQ+f2);
    MyPrint(Output2,[buff buff1]);
    MyPrint(Output2,'{\tablename\ \thetable\ -- \textit{(Continued from previous page)}} \\');
    MyPrint(Output2,'\toprule');
    if checking_DS==1 && checking_AQ==0
        buff=[buff_it '{Item}} & ' buff_other '{SD}} '];
        MyPrint(Output2,buff);
        for i=1:r_c
            buff2=sprintf(' & %s{Factor %.0f}}',buff_other,i);
            MyPrint(Output2,buff2);
        end;
        
    elseif checking_DS==0 && checking_AQ==1
        buff=[buff_it '{Item}} & ' buff_other '{AC}} '];
        MyPrint(Output2,buff);
        for i=1:r_c
            buff2=sprintf(' & %s{Factor %.0f}}',buff_other,i);
            MyPrint(Output2,buff2);
        end;
        
    elseif checking_DS==1 && checking_AQ==1
        buff=[buff_it '{Item}} & ' buff_other '{SD}} &' buff_other '{AC}} '];
        MyPrint(Output2,buff);
        for i=1:r_c
            buff2=sprintf(' &  %s{Factor %.0f}}',buff_other,i);
            MyPrint(Output2,buff2);
        end;
        
    elseif checking_DS==0 && checking_AQ==0
        buff=[buff_it '{Item}} '];
        MyPrint(Output2,buff);
        for i=1:r_c
            buff2=sprintf(' & %s{Factor %.0f}}',buff_other,i);
            MyPrint(Output2,buff2);
        end;
    end;
    MyPrint(Output2,buff4);
    MyPrint(Output2,'\midrule');
    MyPrint(Output2,'\endhead');
    buff='\bottomrule \multicolumn';
    buff1 = sprintf('{%2.0f}',checking_DS+checking_AQ+f2);
    buff2 = '{r}{\textit{Continued on next page}} ';
    MyPrint(Output2,[buff buff1 buff2 buff4]);
    MyPrint(Output2,'\endfoot');
    MyPrint(Output2,'\bottomrule');
    MyPrint(Output2,'\endlastfoot');

    if warning_target==1
        [~,f1]=size(data.items_contingut);
    else
        [f1,f2]=size(items);
    end;
    for i=1:f1
        form=sprintf('%%-%d.%ds  ',s,s); 
        if checking_DS==1
            buff=sprintf(form,labels_tots{i});
        else
            buff=sprintf(form,str{i});
        end;
        buff2=sprintf(' & % 5.4f    ',P(i,:));
        MyPrint(Output2,[buff buff2 buff4]);
    end;
    MyPrint(Output2,'\end{longtable}'); 
    MyPrint(Output2,'\end{center}');
    MyPrint(Output2,' ');
    
    if r_c>1
        if checking_DS==0 && checking_AQ==0
            [f1,f2] = size(PHI);
            PHIprint=PHI;
        else
            switch data.Rotation
                case 1
                    [f1,f2] = size(data.PHIt);
                    PHIprint=data.PHIt;
                case 2
                    [f1,f2] = size(data.PHIt);
                    PHIprint=data.PHIt;
                case 3
                    [f1,f2] = size(data.PHIr);
                    PHIprint=data.PHIr;
                case 4
                    [f1,f2] = size(data.PHIr);
                    PHIprint=data.PHIr;
            end;
        end;
        if r_c==2
            buff=('Correlation between the 2 Factors = \textbf');
            buff1=sprintf('{% 5.4f} ',PHIprint(1,2));
            MyPrint(Output2,[buff buff1 buff4]);
            MyPrint(Output2,' ');
        else
            if data.Rotation == 2 || data.Rotation == 3
                MyPrint(Output2,'\begin{table}[!htb]');
                MyPrint(Output2,'\caption{Inter-factor Correlations}');
                MyPrint(Output2,'\label{taula:interfactor_correlations}');
                MyPrint(Output2,'\centering');

                buff='\begin{tabular}';
                buff1=sprintf('{l*{%-2.0f}{c}}',f2);
                MyPrint(Output2,[buff buff1 buff4]);
                for i=1:f1
                    buff=sprintf('% 5.4f   &',PHIprint(i,1:i));
                    [f1,f2]=size(buff);
                    buff2=buff(1:(f2-1));
                    MyPrint(Output2,[buff2 buff4]);
                end;
                MyPrint(Output2,'\\');
                MyPrint(Output2,'\end{tabular}');
                MyPrint(Output2,'\end{table}');
            end;
       end;
    end;
    buff=sprintf('Bentler''s simplicity = % 5.4f',bentler(Pc));
    MyPrint(Output2,[buff buff4]);    
    MyPrint(Output2,' ');
    buff=sprintf('Lorenzo-Seva''s simplicity = % 5.4f',zsi(Pc));
    MyPrint(Output2,[buff buff4]);
    MyPrint(Output2,' ');
    
    if data.Rotation==3 || data.Rotation==4%Target
        MyPrint(Output2,'\begin{center}');
        MyPrint(Output2,'\begin{longtable}[H]{l S}');
        MyPrint(Output2,'\caption{Congruence between rotated loading matrix and target matrix (Tucker, 1951)}\\');
        MyPrint(Output2,'\toprule');
        
        a = diag(congru(Pc_correct_without_zeros',target_ones_without_zeros'));
        
        [f1,f2]=size(a);
        buff='\multicolumn{1}{l}{\textbf{Item}} & \multicolumn{1}{c}{\textbf{Congruence}}';
        MyPrint(Output2,[buff buff4]);
        MyPrint(Output2,'\midrule');
        MyPrint(Output2,'\endfirsthead');
        MyPrint(Output2,'\multicolumn{2}{c}%');
        MyPrint(Output2,'{\tablename\ \thetable\ -- \textit{(Continued from previous page)}} \\');
        MyPrint(Output2,'\toprule');
        MyPrint(Output2,[buff buff4]);
        MyPrint(Output2,'\midrule');
        MyPrint(Output2,'\endhead');
        MyPrint(Output2,'\bottomrule \multicolumn{2}{r}{\textit{Continued on next page}} \\');
        MyPrint(Output2,'\endfoot');
        MyPrint(Output2,'\bottomrule');
        MyPrint(Output2,'\endlastfoot');
        
        for i=1:f1
            form=sprintf('%%-%d.%ds  ',s,s);   
            buff=sprintf(form,str_without_zeros{i});
            buff2 = sprintf('& % 4.3f',a(i));
            MyPrint(Output2,[buff buff2 buff4])
        end;
        MyPrint(Output2,'\end{longtable}');
        MyPrint(Output2,'\end{center}');
        
        
        a = diag(congru(Pc_correct,target_ones));
        
        [f1,f2]=size(a);
        MyPrint(Output2,'\begin{table}[H]');
        MyPrint(Output2,'\caption{Congruence of factors}');
        
        MyPrint(Output2,'\centering');
        buff='\begin{tabular}';
        buff1=sprintf('{l*{%-2.0f}{c}}',2);
        MyPrint(Output2,[buff buff1 buff4]);
        MyPrint(Output2,'\toprule');
        
        buff='Factor & Congruence';
        MyPrint(Output2,[buff buff4]);
        MyPrint(Output2,'\midrule');
        
        for i=1:f1
            buff = sprintf('F%3.f       ', i);
            buff2 = sprintf('& % 5.4f',a(i));
            MyPrint(Output2,[buff buff2 buff4])
        end;
        MyPrint(Output2,'\bottomrule');
        MyPrint(Output2,'\end{tabular}');
        MyPrint(Output2,'\end{table}');
        
            
        a = mean(diag(congru(Pc_correct,target_ones)));
        
        MyPrint(Output2,' ');
        buff=sprintf('Overall congruence : % 4.3f',a);
        MyPrint(Output2,[buff buff4]);
        
        MyPrint(Output2,' ');
        MyPrint(Output2,'Guidelines to interpret congruence index: \\');
        MyPrint(Output2,' ');
        MyPrint(Output2,'Lorenzo-Seva \& ten Berge (2006) \\');
        MyPrint(Output2,' ');
        MyPrint(Output2,'A congruence value in the range .85-.94 corresponds to a fair similarity,\\');
        MyPrint(Output2,'while a value higher than .95 implies that the two factors (or components)\\');
        MyPrint(Output2,'compared can be considered equal.\\');
    end;
   
    
    MyPrint(Output2,'\subsection{Distribution of  Residuals}');
    MyPrint(Output2,' ');
    MyPrint(Output2,'\begin{table}[H]');
    MyPrint(Output2,'\caption{Summary Statistics for Fitted Residuals}');
    MyPrint(Output2,'\centering');
    MyPrint(Output2,'\begin{tabular}{l l}');
    MyPrint(Output2,'\toprule');
    [f1,f2]=size(A);
    buff0='\textbf';
    buff=sprintf('{Number of Residuals}: & %.0f',f1);
    MyPrint(Output2,[buff0 buff buff4]);
    buff = sprintf('{Smallest Fitted Residual}: &  % 7.4f',min(A));
    MyPrint(Output2,[buff0 buff buff4]);
    buff = sprintf('{Median Fitted Residual}: & % 7.4f',median(A));
    MyPrint(Output2,[buff0 buff buff4]);
    buff = sprintf('{Largest Fitted Residual}: & % 7.4f',max(A));
    MyPrint(Output2,[buff0 buff buff4]);
    buff = sprintf('{Mean Fitted Residual}: & % 7.4f',me);
    MyPrint(Output2,[buff0 buff buff4]);
    buff = sprintf('{Variance Fitted Residual}: & % 7.4f',sd*sd);
    MyPrint(Output2,[buff0 buff buff4]);
    buff = sprintf('{Root Mean Square of Residuals (RMSR)}: & % 7.4f',data.RMSR);
    MyPrint(Output2,[buff0 buff buff4]);
    buff = sprintf('{Expected mean value of RMSR for an acceptable model}: & % 7.4f (Kelley''s criterion)',data.Kelley);
    MyPrint(Output2,[buff0 buff buff4]);
    MyPrint(Output2,'\bottomrule');
    MyPrint(Output2,'\end{tabular}');
    MyPrint(Output2,'\end{table}');
    if RMSR > data.Kelley
       MyPrint(Output2,'(Kelley, 1935,page 146; see also Harman, 1962, page 21 of the 2nd edition)\\');
       MyPrint(Output2,'Note: if the value of RMSR is much larger than Kelley''s criterion value the model cannot be considered as good \\');
       MyPrint(Output2,' ');  
    end;


    %% TEXT %%
    
    clearvars buff buff2 buff3 buff4;
    
     
    if checking_DS==0 && checking_AQ==0
        buff='Exploratory Factor Analysis without controlling bias';
    elseif checking_DS==1 && checking_AQ==0
        buff='Exploratory Factor Analysis controlling Social Desirability';
    elseif checking_DS==0 && checking_AQ==1
        buff='Exploratory Factor Analysis controlling Acquiescence';
    elseif checking_DS==1 && checking_AQ==1
        buff='Exploratory Factor Analysis controlling Social Desirability and Acquiescence';
    end;
    
    MyPrint(Output,'-----------------------------------------------------------------------');
    MyPrint(Output,buff);
    if number_of_loops>1
        buff=sprintf('Current Output: %1.0f of %1.0f',current,number_of_loops);
        MyPrint(Output,buff);
    end;
    MyPrint(Output,'-----------------------------------------------------------------------');
    
    MyPrint(Output,' ');
    MyPrint(Output,'GOODNESS OF FIT STATISTICS');
    MyPrint(Output,' ');
    buff = sprintf('Goodness of Fit Index (GFI) = %7.4f',data.GFI);
    MyPrint(Output,buff);
    MyPrint(Output,' ');
    MyPrint(Output,'-----------------------------------------------------------------------');
    
    if data.Rotation==3 || data.Rotation==4
        MyPrint(Output,' ');
        MyPrint(Output,'Semi-Specified target loading matrix');
        MyPrint(Output,' ');
        form=sprintf('%%-%d.%ds',s,s);
        buff=sprintf(form,'Item');
        for i=1:r_c
            buff2{i}=sprintf('   Factor %.0f',i);
        end;
        buff3=sprintf('%s',buff2{:});
        
        MyPrint(Output,[buff buff3]);
        [f1,f2]=size(data.items_contingut);
        for i=1:f2
            form=sprintf('%%-%d.%ds   ',s,s);   
            buff=sprintf(form,str{i});
            buff2=sprintf('% 2.1f         ',target(i,:));
            MyPrint(Output,[buff buff2]);
        end;
        MyPrint(Output,' ');
        MyPrint(Output,'WARNING: +9 or -9 equal semi-specified values ');
        MyPrint(Output,' ');
        MyPrint(Output,'-----------------------------------------------------------------------');
        
    end;
    
    
    clearvars buff buff2 buff3 buff4;
    
    MyPrint(Output,' ');
    if r_c == 1
        MyPrint(Output,'Unrotated loading matrix');
    else
        MyPrint(Output,'Rotated loading matrix');
    end
    
    MyPrint(Output,' ');
    if checking_DS==1 && checking_AQ==0
        form=sprintf('%%-%d.%ds   SD      ',s,s);
        buff=sprintf(form,'Item');
        for i=1:r_c
            buff2{i}=sprintf('   Factor %.0f',i);
        end;
        buff3=sprintf('%s',buff2{:});
    end;
    if checking_DS==0 && checking_AQ==1;
        form=sprintf('%%-%d.%ds   AC      ',s,s);
        buff=sprintf(form,'Item');
        for i=1:r_c
            buff2{i}=sprintf('   Factor %.0f',i);
        end;
        buff3=sprintf('%s',buff2{:});
    end;
    if checking_DS==1 && checking_AQ==1
        form=sprintf('%%-%d.%ds   SD         AC      ',s,s);
        buff=sprintf(form,'Item');
        for i=1:r_c
            buff2{i}=sprintf('   Factor %.0f',i);
        end;
        buff3=sprintf('%s',buff2{:});
    end;
    if checking_DS==0 && checking_AQ==0
        form=sprintf('%%-%d.%ds',s,s);
        buff=sprintf(form,'Item');
        for i=1:r_c
            buff2{i}=sprintf('   Factor %.0f',i);
        end;
        buff3=sprintf('%s',buff2{:});
    end;
    MyPrint(Output,[buff buff3]);

    if warning_target==1 && checking_DS==0
        [~,f1]=size(data.items_contingut);
    else
        [f1,f2]=size(items);
    end;
    for i=1:f1
        form=sprintf('%%-%d.%ds  ',s,s);   
        if checking_DS==1
            buff=sprintf(form,labels_tots{i});
        else
            buff=sprintf(form,str{i});
        end;
        buff2=sprintf('% 5.4f    ',P(i,:));
        MyPrint(Output,[buff buff2]);
    end;
    
    if r_c>1
       if data.Rotation == 2 || data.Rotation == 3 
            MyPrint(Output,' ');    
            MyPrint(Output,'Inter-factor correlations');
            if checking_DS==0 && checking_AQ==0
                [f1,f2] = size(PHI);
                PHIprint=PHI;
            else
                switch data.Rotation
                    case 1
                        [f1,f2] = size(data.PHIt);
                        PHIprint=data.PHIt;
                    case 2
                        [f1,f2] = size(data.PHIt);
                        PHIprint=data.PHIt;
                    case 3
                        [f1,f2] = size(data.PHIr);
                        PHIprint=data.PHIr;
                    case 4
                        [f1,f2] = size(data.PHIr);
                        PHIprint=data.PHIr;
                end;           
            end;
            if r_c==2
                buff=sprintf('Correlation between the 2 Factors: % 5.4f ',PHIprint(1,2));
                MyPrint(Output,buff);
            else
                buff='      ';
                for i=1:f1
                    buff1=sprintf('F%2.0f       ',i);
                    buff=[buff buff1];
                end;
                MyPrint(Output,buff);
                for i=1:f1
                    buff=sprintf('F%2.0f  ',i);
                    buff1=sprintf('% 5.4f   ',PHIprint(i,1:i));
                    MyPrint(Output,[buff buff1]);
                end;    
            end;
        end;
    
        MyPrint(Output,' ')
        MyPrint(Output,'-----------------------------------------------------------------------');
        MyPrint(Output,' ')
        MyPrint(Output,'Indices of factor simplicity');
        MyPrint(Output,'Bentler (1977) & Lorenzo-Seva (2003)');
        MyPrint(Output,' ')
        MyPrint(Output,'Bentler''s simplicity');
        buff=sprintf('% 5.4f',bentler(Pc));
        MyPrint(Output,buff);    
        MyPrint(Output,' ');
        MyPrint(Output,'Lorenzo-Seva''s simplicity');
        buff=sprintf('% 5.4f',zsi(Pc));
        MyPrint(Output,buff);
        MyPrint(Output,' ');
    end;
    
    
    if data.Rotation==3 || data.Rotation==4%Target
        MyPrint(Output,'-----------------------------------------------------------------------');
        MyPrint(Output,' ');
        MyPrint(Output,'Congruence between rotated loading matrix and target matrix (Tucker, 1951)');
        MyPrint(Output,' ');
        a = diag(congru(Pc_correct_without_zeros',target_ones_without_zeros'));
        form=sprintf('%%-%d.%ds  Congruence',s,s);   
        buff=sprintf(form,'Item');
        MyPrint(Output,buff);
        [f1,f2]=size(a);
        buff2=[];
        for i=1:f1
            form=sprintf('%%-%d.%ds  ',s,s);   
            buff=sprintf(form,str_without_zeros{i});
            buff2 = sprintf('% 5.4f',a(i));
            MyPrint(Output,[buff buff2])
        end;
        
        a = diag(congru(Pc_correct,target_ones));
        MyPrint(Output,' ')
        MyPrint(Output,'Congruence of factors');
        MyPrint(Output,' ');
        MyPrint(Output,'Factor    Congruence');
        [f1,f2]=size(a);
        for i=1:f1
            buff = sprintf('F%3.f       ', i);
            buff2 = sprintf('% 5.4f',a(i));
            MyPrint(Output,[buff buff2])
        end;
        
        a = mean(diag(congru(Pc_correct,target_ones)));
        MyPrint(Output,' ');
        buff=sprintf('Overall congruence : % 4.3f',a);
        MyPrint(Output,buff);
        
        MyPrint(Output,' ');
        MyPrint(Output,'Guidelines to interpret congruence index');
        MyPrint(Output,'Lorenzo-Seva & ten Berge (2006)');
        MyPrint(Output,' ');
        MyPrint(Output,'A congruence value in the range .85-.94 corresponds to a fair similarity,');
        MyPrint(Output,'while a value higher than .95 implies that the two factors (or components)');
        MyPrint(Output,'compared can be considered equal.');
        MyPrint(Output,' ');
    end;
    
    MyPrint(Output,'-----------------------------------------------------------------------');
    MyPrint(Output,' ');
    MyPrint(Output,'DISTRIBUTION OF RESIDUALS');
    MyPrint(Output,' ');
    [f1,f2]=size(A);
    buff=sprintf('Number of Residuals = %.0f',f1);
    MyPrint(Output,buff);
    MyPrint(Output,' ');
    MyPrint(Output,'Summary Statistics for Fitted Residuals');
    MyPrint(Output,' ');
    buff = sprintf('Smallest Fitted Residual = % 7.4f',min(A));
    MyPrint(Output,buff);
    buff = sprintf('  Median Fitted Residual = % 7.4f',median(A));
    MyPrint(Output,buff);
    buff = sprintf(' Largest Fitted Residual = % 7.4f',max(A));
    MyPrint(Output,buff);
    buff = sprintf('    Mean Fitted Residual = % 7.4f',me);
    MyPrint(Output,buff);
    buff = sprintf('Variance Fitted Residual = % 7.4f',sd*sd);
    MyPrint(Output,buff);
    MyPrint(Output,' ');
    buff = sprintf('               Root Mean Square of Residuals (RMSR) = % 7.4f',data.RMSR);
    MyPrint(Output,buff);
    buff = sprintf('Expected mean value of RMSR for an acceptable model = % 7.4f (Kelley''s criterion) (Kelley, 1935,page 146; see also Harman, 1962, page 21 of the 2nd edition)',data.Kelley);
    MyPrint(Output,buff);
    MyPrint(Output,' ');
    if RMSR > data.Kelley
       MyPrint(Output,'Note: if the value of RMSR is much larger than Kelley''s criterion value the model cannot be considered as good');
       MyPrint(Output,' ');  
    end;
    
    %GRAFIC
    MyPrint(Output,'Value        Freq');
    MyPrint(Output,'                  |');
    valors = linspace(min(A),max(A),11);
    Y = discretize(A,valors);
    freq=[];
    for i=1:11;
        buff=find(Y==i);
        [f1,f2]=size(buff);
        freq(i)=f1;
    end;

    maxf=max(freq);

    for i=1:11
        buff=sprintf('  % 5.4f     %3.0f | ',valors(i),freq(i));
        ast=floor(freq(i)/maxf*40);
        if freq(i)>0
            aster='*';
        else
            aster='';
        end;
            for j=1:ast
                aster=[aster '*'];
            end;
        MyPrint(Output, [buff aster]);
    end;
    MyPrint(Output,'                  +-----------+---------+---------+----------+');
    buff=sprintf('                  0        %3.1f       %3.1f       %3.1f        %3.1f\n\n',maxf/4.0,maxf/2.0,maxf/4.0 + maxf/2.0,maxf);
    MyPrint(Output,buff);
    MyPrint(Output,' ');
    
    
    MyPrint(Output,' ');
    MyPrint(Output,'Summary Statistics for Standarized Residuals');
    MyPrint(Output,' ');
    buff = sprintf('Smallest Standarized Residual = % 7.4f',min(A_stand));
    MyPrint(Output,buff);
    buff = sprintf('  Median Standarized Residual = % 7.4f',median(A_stand));
    MyPrint(Output,buff);
    buff = sprintf(' Largest Standarized Residual = % 7.4f',max(A_stand));
    MyPrint(Output,buff);
    buff = sprintf('    Mean Standarized Residual = % 7.4f',mean(A_stand));
    MyPrint(Output,buff);
    MyPrint(Output,' ');
    MyPrint(Output,' ');
    
    %Stem and leaf plot
    MyPrint(Output,'Stemleaf Plot for Standarized Residuals');
    MyPrint(Output,' ');
    
    %metode 1
    x=A_stand*10;
    
    stemleafplot(Output,x,0);
    
    MyPrint(Output,' ');
    MyPrint(Output,' ');
    
    res_high=find(A_stand>2.50);
    res_low=find(A_stand<-2.50);
    
    [f1,f2]=size(res_high);
    [g1,g2]=size(res_low);
    
    if isempty(res_low)
    else
        MyPrint(Output,' ');
        MyPrint(Output,'Largest Negative Standarized Residuals');
        MyPrint(Output,' ');
        h=1;
         for i=1:m
            for j=(i+1): m
                A1= R(j,i) - Re(j,i);
                A2=A1/se;
                if A2<=-2.5
                    item1=j;
                    item2=i;
                    buff2=sprintf('Residual for Item %3.0f and Item %3.0f = %3.2f',item1,item2,A_stand(res_low(h)));
                    h=h+1;
                    MyPrint(Output,buff2);
                end;
            end;
         end;
        MyPrint(Output,' ');
    end;
    
    if isempty(res_high)
    else
        MyPrint(Output,' ');
        MyPrint(Output,'Largest Positive Standarized Residuals');
        MyPrint(Output,' ');
        h=1;
        for i=1:m
            for j=(i+1): m
                A1= R(j,i) - Re(j,i);
                A2=A1/se;
                if A2>=2.5
                    item1=j;
                    item2=i;
                    buff2=sprintf('Residual for Item %3.0f and Item %3.0f = %3.2f',item1,item2,A_stand(res_high(h)));
                    h=h+1;
                    MyPrint(Output,buff2);
                end;
            end;
         end;
        MyPrint(Output,' ');
    end;
    
    if number_of_loops==1
        Printgrafic_residuals(A_stand);
    end;
    
    %% Printing in the file containing the relevant matrices %%
    
    
    if checking_DS==0 && checking_AQ==0
        buff='Exploratory Factor Analysis without controlling bias';
    elseif checking_DS==1 && checking_AQ==0
        buff='Exploratory Factor Analysis controlling Social Desirability';
    elseif checking_DS==0 && checking_AQ==1
        buff='Exploratory Factor Analysis controlling Acquiescence';
    elseif checking_DS==1 && checking_AQ==1
        buff='Exploratory Factor Analysis controlling Social Desirability and Acquiescence';
    end;
    
    MyPrint(Output_matrices,'-----------------------------------------------------------------------');
    MyPrint(Output_matrices,buff);
    if number_of_loops>1
        buff=sprintf('Current Output: %1.0f of %1.0f',current,number_of_loops);
        MyPrint(Output_matrices,buff);
    end;
    MyPrint(Output_matrices,'-----------------------------------------------------------------------');
    MyPrint(Output_matrices,'');
    
    if (checking_DS==1 || checking_AQ==1) && isfield(data,'Re_cont')
        if checking_DS==1 && checking_AQ==1
            MyPrint(Output_matrices','RESIDUAL MATRIX ONCE REMOVING SD AND AC FOR CONTENT ITEMS');
        elseif checking_DS==1 && checking_AQ==0
            MyPrint(Output_matrices','RESIDUAL MATRIX ONCE REMOVING SD FOR CONTENT ITEMS');
        elseif checking_DS==0 && checking_AQ==1
            MyPrint(Output_matrices','RESIDUAL MATRIX ONCE REMOVING AC FOR CONTENT ITEMS');
        end;
        MyPrint(Output_matrices,'');

        Dat=data.Re_cont;
        [g1,g2]=size(Dat);

        str=num2cell(data.items_contingut);

        buff=sprintf('Item        ');
        for i=1:g2
            buff2=sprintf('%-5.0f    ',str{i});
            buff=[buff buff2];
        end;
        MyPrint(Output_matrices,buff);
        MyPrint(Output_matrices,'');
        for i=1:g2
            buff=sprintf('I%3.0f     ',str{i});     
            buff2=sprintf('% 5.3f   ',Dat(i,1:i));
            MyPrint(Output_matrices,[buff buff2]);
        end;
        MyPrint(Output_matrices,'');
    end;
    
    
    MyPrint(Output_matrices','FINAL RESIDUAL MATRIX');
    MyPrint(Output_matrices,'');
    
    if isfield(data,'Re_final')
        [g1,g2]=size(data.Re_final);
    else
        [g1,g2]=size(data.Re);
    end;
    
    new_order=[];
    correccio=0;
    if checking_DS==1
        for i=1:g2
            if ~any(data.items_sd==data.inclosos(i))
                %item de contingut
                new_order(i)=data.G(1)+i-correccio;

            else
                correccio=correccio+1;
                new_order(i)=correccio;
            end;
        end;
        str=num2cell(data.inclosos);
    else
        new_order=1:g2;
        str=num2cell(data.items_contingut);
    end;
    
    
    if isfield(data,'Re_final')
        Dat=data.Re_final(new_order,new_order);
    else
        Dat=data.Re(new_order,new_order);
    end;
    
    [g1,g2]=size(Dat);

    buff=sprintf('Item        ');
    for i=1:g2
        buff2=sprintf('%-5.0f    ',str{i});
        buff=[buff buff2];
    end;
    MyPrint(Output_matrices,buff);
    MyPrint(Output_matrices,'');
    for i=1:g2
        buff=sprintf('I%3.0f     ',str{i});     
        buff2=sprintf('% 5.3f   ',Dat(i,1:i));
        %Els 1.000 que es mostrin com a 1
        if data.Matrix==2 || data.Matrix==3
            [h1,h2]=size(buff2);
            buff2=buff2(1:h2-8);
            a=sprintf('%1.0f',round(Dat(i,i)));
            MyPrint(Output_matrices,[buff buff2 a]);
        else
            MyPrint(Output_matrices,[buff buff2]);
        end;
    end;
    MyPrint(Output_matrices,'');
    
    if checking_DS==1
        MyPrint(Output_matrices','FINAL RESIDUAL MATRIX: Content items only');
        MyPrint(Output_matrices,'');

        str=num2cell(data.items_contingut);
        if isfield(data,'Re_final')
            Dat_c=data.Re_final(data.G(1)+1:end,data.G(1)+1:end);
        else
            Dat_c=data.Re(data.G(1)+1:end,data.G(1)+1:end);
        end;
        
        [h1,h2]=size(Dat_c);
         buff=sprintf('Item        ');
        for i=1:h2
            buff2=sprintf('%-5.0f    ',str{i});
            buff=[buff buff2];
        end;
        MyPrint(Output_matrices,buff);
        MyPrint(Output_matrices,'');
        for i=1:h2
            buff=sprintf('I%3.0f     ',str{i});     
            buff2=sprintf('% 5.3f   ',Dat_c(i,1:i));
            %Els 1.000 que es mostrin com a 1
            if data.Matrix==2 || data.Matrix==3
                [~,j2]=size(buff2);
                buff2=buff2(1:j2-8);
                a=sprintf('%1.0f',round(Dat_c(i,i)));
                MyPrint(Output_matrices,[buff buff2 a]);
            else
                MyPrint(Output_matrices,[buff buff2]);
            end;
        end;
        MyPrint(Output_matrices,'');

    end;
    
    %% RTF %%
    
    
    int='\intbl ';
    cell1='\cell';
    row='\row';
    pard='\pard \par';
    
    clearvars buff buff2 buff3
    
    if checking_DS==0 && checking_AQ==0
        buff='\qc \ul Exploratory Factor Analysis without controlling bias\ul0';
    elseif checking_DS==1 && checking_AQ==0
        buff='\qc \ul Exploratory Factor Analysis controlling Social Desirability\ul0';
    elseif checking_DS==0 && checking_AQ==1
        buff='\qc \ul Exploratory Factor Analysis controlling Acquiescence\ul0';
    elseif checking_DS==1 && checking_AQ==1
        buff='\qc \ul Exploratory Factor Analysis controlling Social Desirability and Acquiescence\ul0';
    end;
    MyPrint(Output3,'\par');
    MyPrint(Output3,buff);
    MyPrint(Output3,'\par');
    
    if number_of_loops>1
        buff=sprintf('Current Output: %1.0f of %1.0f',current,number_of_loops);
        MyPrint(Output3,'\par');
        MyPrint(Output3,['\qc' buff]);
        MyPrint(Output3,'\par');
    end;
    
    MyPrint(Output3,'\par');MyPrint(Output3,'\par');
    buff = sprintf('Goodness of Fit Index (GFI) = %7.4f',data.GFI);
    MyPrint(Output3,['\ql ' buff]);
    MyPrint(Output3,'\par');MyPrint(Output3,'\par');
    
     if data.Rotation==3 || data.Rotation==4
        MyPrint(Output3,'\par');
        MyPrint(Output3,'\qc \ul Semi-Specified target loading matrix\ul0');
        MyPrint(Output3,'\par'); MyPrint(Output3,'\par')
        
        buff={};
        buff2={};
        buff='Item';
        for i=1:r_c
            buff2{i}=sprintf('Factor %.0f',i);
        end;
        buff3=sprintf('%s',buff2{:});
        header=[buff buff2];
        
        Print_RTF(target,Output3,header,labels_contingut);
        MyPrint(Output3,'\par');
        
    end;
    
    a=checking_DS+checking_AQ+r_c+1;
    buff=cell(a,1);
    buff{1}='Item';
    
    if checking_DS==1 && checking_AQ==0
        buff{2}='SD';
    elseif checking_DS==0 && checking_AQ==1;
        buff{2}='AC';
    elseif checking_DS==1 && checking_AQ==1
        buff{2}='SD';
        buff{3}='AC';
    elseif checking_DS==0 && checking_AQ==0
    end;
    
    for i=1:r_c
        buff{i+checking_DS+checking_AQ+1}=sprintf('Factor %.0f',i);
    end;
    
    MyPrint(Output3,'\par');
    if r_c == 1
        MyPrint(Output3,'\qc \ul Unrotated Loading Matrix \ul0');
    else
        MyPrint(Output3,'\qc \ul Rotated Loading Matrix \ul0');
    end
    MyPrint(Output3,'\par');
    MyPrint(Output3,'\par');
    if checking_DS==1
        labels_ok=labels_tots;
    else
        labels_ok=labels_contingut;
    end;
    Print_RTF_P(P,Output3,buff,labels_ok);
    
    
    if r_c>1
       if data.Rotation == 2 || data.Rotation == 3
            
            if checking_DS==0 && checking_AQ==0
                [f1,f2] = size(PHI);
                PHIprint=PHI;
            else
                switch data.Rotation
                    case 1
                        [f1,f2] = size(data.PHIt);
                        PHIprint=data.PHIt;
                    case 2
                        [f1,f2] = size(data.PHIt);
                        PHIprint=data.PHIt;
                    case 3
                        [f1,f2] = size(data.PHIr);
                        PHIprint=data.PHIr;
                    case 4
                        [f1,f2] = size(data.PHIr);
                        PHIprint=data.PHIr;
                end;           
            end;
            if r_c==2
                MyPrint(Output3,'\par');
                buff=sprintf('Correlation between the 2 Factors: % 5.4f ',PHIprint(1,2));
                MyPrint(Output3,buff);
                MyPrint(Output3,'\par');
            else
                numtables=ceil(f2/10); %nombre de taules necessaries per imprimir correctament la matriu de correlacions
                %Ho fem per facilitar la lectura de la taula, per quan tenim moltes variables. (maxim d'amplada de 10 per taula)
                
                a=(1:f2);
                str=num2cell(a);

                for l=1:numtables
                    [f1,f2] = size(PHIprint);

                    leng=l*10;
                    cof=10*(l-1);

                    if cof<f2 && cof+10<f2
                        PHI2=PHIprint(:,1+cof:10+cof);
                        str2=str(1+cof:10+cof);
                    else
                        PHI2=PHIprint(:,1+cof:f2);
                        str2=str(1+cof:f2);
                    end;

                    a=f2/(leng);
                    if a>1
                        numb=10;
                    elseif a<1
                        numb=f2-(10*(l-1));
                    end;
                
                    MyPrint(Output3,'\par');MyPrint(Output3,'\par');
                    buff_rtf='\qc \ul Inter-factor correlations\ul0';
                    buff2=sprintf(' (Part %.0f of %.0f )',l,numtables);
                    if numtables==1
                        MyPrint(Output3,buff_rtf);
                    else
                        MyPrint(Output3,[buff_rtf buff2]);
                    end;
                    MyPrint(Output3,'\par');
                    MyPrint(Output3,'\par');
                    MyPrint(Output3,'\ql');
                    interline_cap=[];
                    for i=1:numb+1
                        if i==1
                            buff='\trowd\trgaph72\clbrdrt\brdrs\clbrdrl\brdrs\clbrdrb\brdrs\clbrdrr\brdrs\';
                        else
                            buff='\clbrdrt\brdrs\clbrdrl\brdrs\clbrdrb\brdrs\clbrdrr\brdrs\';
                        end;
                        buff2=sprintf('cellx%.0f',910*i);
                        buff3=[buff buff2];
                        interline_cap=[interline_cap buff3];
                    end;
                    
                    buff_rtf='\cell';
                    dif=f2-cof;
                    if dif>10
                        for i=1:10
                            buff_rtf1=sprintf(' F %-5.0f',str2{i});
                            buff_rtf=[buff_rtf buff_rtf1 cell1];
                        end;
                    else
                        for i=1:f2-cof
                            buff_rtf1=sprintf(' F %-3.0f',str2{i});
                            buff_rtf=[buff_rtf buff_rtf1 cell1];
                        end;
                    end;
                    MyPrint(Output3,interline_cap);
                    MyPrint(Output3,[int buff_rtf row]);

                    [f1,f2]=size(PHI2);
                    
                    for i=1:f1-cof
                        buff0=sprintf('F%3.0f ',str{i+cof}); 
                        if i<=10
                            buff_rtf=sprintf(' % 4.3f \\cell',PHI2(i+cof,1:i));
                        else
                            buff_rtf=sprintf(' % 4.3f \\cell',PHI2(i+cof,1:10));
                        end;

                        emptycells=[];
                        for j=1:f2-i
                            buff9=sprintf('%s',cell1);
                            emptycells=[emptycells buff9];
                        end;
                        numberofemptycells=f2-i;
                        numberoffullcells=f2-numberofemptycells;

                        interline_fila=[];
                        for k=1:numb+1
                            if k==1
                                buff='\trowd\trgaph72\clbrdrt\brdrs\clbrdrl\brdrs\clbrdrb\brdrs\clbrdrr\brdrs\'; %la primera, la del numbero de item
                            else
                                if k<=numberoffullcells+1
                                    buff='\clbrdrt\brdrs\clbrdrl\brdrs\clbrdrb\brdrs\clbrdrr\brdrs\'; %les cel·les amb nombres, que tinguin bordes
                                else
                                    buff='\';                                                       %les cel·les buides, sense bordes
                                end;
                            end;
                            buff2=sprintf('cellx%.0f',910*k);
                            buff3=[buff buff2];
                            interline_fila=[interline_fila buff3];
                        end;

                        MyPrint(Output3,interline_fila);
                        if i==f1-cof
                            MyPrint(Output3,[int buff0 cell1 buff_rtf emptycells row pard]);    
                        else
                            MyPrint(Output3,[int buff0 cell1 buff_rtf emptycells row]);
                        end;
                    end;
                end;
                                
            end;
        end;
        
        MyPrint(Output3,'\par');
        buff1='Bentler''s simplicity = ';
        buff2=sprintf('% 5.4f',bentler(Pc));
        MyPrint(Output3,[buff1 buff2]);  
        
        MyPrint(Output3,'\par');
        MyPrint(Output3,'\par');
        
        buff1='Lorenzo-Seva''s simplicity = ';
        buff2=sprintf('% 5.4f',zsi(Pc));
        MyPrint(Output3,[buff1 buff2]);
        MyPrint(Output3,'\par');
        MyPrint(Output3,'\par');
        
        if data.Rotation==3 || data.Rotation==4 %Target 
            MyPrint(Output3,'\par');
            MyPrint(Output3,'\qc \ul Congruence between rotated loading matrix and target matrix (Tucker, 1951)\ul0');
            MyPrint(Output3,'\par');MyPrint(Output3,'\par');
            a = diag(congru(Pc_correct_without_zeros',target_ones_without_zeros'));
            header={};
            header{1}='Item';
            header{2}='Congruence';
            [f1,f2]=size(a);
            Print_RTF(a,Output3,header,str_without_zeros);

            a = diag(congru(Pc_correct,target_ones));
            MyPrint(Output3,'\par')
            MyPrint(Output3,'\qc \ul Congruence of factors\ul0');
            MyPrint(Output3,'\par');MyPrint(Output3,'\par');
            header{1}='Factor';
            [f1,f2]=size(a);
            buff={};
            for i=1:f1
                buff{i} = sprintf('F%3.f', i);
            end;
            Print_RTF(a,Output3,header,buff);
            
            a = mean(diag(congru(Pc_correct,target_ones)));
            MyPrint(Output3,'\par');MyPrint(Output3,'\par');
            buff=sprintf('Overall congruence : % 4.3f',a);
            MyPrint(Output3,buff);

            MyPrint(Output3,'\par');MyPrint(Output3,'\par');
            MyPrint(Output3,'Guidelines to interpret congruence index');
            MyPrint(Output3,'\par');
            MyPrint(Output3,'Lorenzo-Seva & ten Berge (2006)');
            MyPrint(Output3,'\par');MyPrint(Output3,'\par');
            buff1=[];buff2=[];
            buff1='A congruence value in the range .85-.94 corresponds to a fair similarity,';
            buff2='while a value higher than .95 implies that the two factors (or components) compared can be considered equal.';
            MyPrint(Output3,[buff1 buff2]);
            MyPrint(Output3,'\par');
        end;
    end;
    
    MyPrint(Output3,'\par');
    MyPrint(Output3,'\qc \ul Distribution of Residuals\ul0');
    MyPrint(Output3,'\par');MyPrint(Output3,'\par');
    [f1,f2]=size(A);
    buff=sprintf('Number of Residuals = %.0f',f1);
    MyPrint(Output3,['\ql ' buff]);
    MyPrint(Output3,'\par');MyPrint(Output3,'\par');
    buff = sprintf('Smallest Fitted Residual = % 7.4f',min(A));
    MyPrint(Output3,['\ql ' buff]);MyPrint(Output3,'\par');
    buff = sprintf('Median Fitted Residual = % 7.4f',median(A));
    MyPrint(Output3,buff);MyPrint(Output3,'\par');
    buff = sprintf('Largest Fitted Residual = % 7.4f',max(A));
    MyPrint(Output3,buff);MyPrint(Output3,'\par');
    buff = sprintf('Mean Fitted Residual = % 7.4f',me);
    MyPrint(Output3,buff);MyPrint(Output3,'\par');
    buff = sprintf('Variance Fitted Residual = % 7.4f',sd*sd);
    MyPrint(Output3,buff);MyPrint(Output3,'\par');MyPrint(Output3,'\par');
    buff = sprintf('Root Mean Square of Residuals (RMSR) = % 7.4f',data.RMSR);
    MyPrint(Output3,buff);MyPrint(Output3,'\par');
    buff = sprintf('Expected mean value of RMSR for an acceptable model = % 7.4f (Kelley''s criterion) (Kelley, 1935,page 146; see also Harman, 1962, page 21 of the 2nd edition)',data.Kelley);
    MyPrint(Output3,buff);MyPrint(Output3,'\par');
    if RMSR > data.Kelley
       MyPrint(Output3,'Note: if the value of RMSR is much larger than Kelley''s criterion value the model cannot be considered as good');MyPrint(Output3,'\par'); 
    end;
    MyPrint(Output3,'\par');
    
    MyPrint(Output3,'\par');
    MyPrint(Output3,'\qc  \ul Summary Statistics for Standarized Residuals\ul0');
    MyPrint(Output3,'\par');MyPrint(Output3,'\par');
    buff = sprintf('Smallest Standarized Residual = % 7.4f',min(A_stand));
    MyPrint(Output3,['\ql ' buff]);MyPrint(Output3,'\par');
    buff = sprintf('Median Standarized Residual = % 7.4f',median(A_stand));
    MyPrint(Output3,buff);MyPrint(Output3,'\par');
    buff = sprintf('Largest Standarized Residual = % 7.4f',max(A_stand));
    MyPrint(Output3,buff);MyPrint(Output3,'\par');
    buff = sprintf('Mean Standarized Residual = % 7.4f',mean(A_stand));
    MyPrint(Output3,buff);MyPrint(Output3,'\par');

    res_high=find(A_stand>2.50);
    res_low=find(A_stand<-2.50);
    
    [f1,f2]=size(res_high);
    [g1,g2]=size(res_low);
    
    if isempty(res_low)
    else
        MyPrint(Output3,'\par');
        MyPrint(Output3,'\qc \ul Largest Negative Standarized Residuals\ul0');
        MyPrint(Output3,'\par');MyPrint(Output3,'\par');
        MyPrint(Output3,'\ql');
        h=1;
         for i=1:m
            for j=(i+1): m
                A1= R(j,i) - Re(j,i);
                A2=A1/se;
                if A2<=-2.5
                    item1=j;
                    item2=i;
                    buff2=sprintf('Residual for Item %3.0f and Item %3.0f = %3.2f',item1,item2,A_stand(res_low(h)));
                    h=h+1;
                    MyPrint(Output3,buff2);MyPrint(Output3,'\par');
                end;
            end;
         end;
        MyPrint(Output3,'\par');
    end;
    
    if isempty(res_high)
    else
        MyPrint(Output3,'\par');
        MyPrint(Output3,'\qc \ul Largest Positive Standarized Residuals\ul0');
        MyPrint(Output3,'\par');MyPrint(Output3,'\par');
        MyPrint(Output3,'\ql');
        h=1;
        for i=1:m
            for j=(i+1): m
                A1= R(j,i) - Re(j,i);
                A2=A1/se;
                if A2>=2.5
                    item1=j;
                    item2=i;
                    buff2=sprintf('Residual for Item %3.0f and Item %3.0f = %3.2f',item1,item2,A_stand(res_high(h)));
                    h=h+1;
                    MyPrint(Output3,buff2);MyPrint(Output3,'\par');
                end;
            end;
         end;
        MyPrint(Output3,'\par');
    end;
    MyPrint(Output3,'\par');
    
return;

function ClearData_Callback(hObject, eventdata, handles)
%% Clear Data from the application
%

data=getappdata(0,'data');

if isempty(data.X) && isempty(data.S)
        errordlg('There are no data avaible','Psychological Test Toolbox: Error');
        return;
else
    ButtonName=questdlg('Do you want to clear the data in memory','Psychological Test Toolbox','Yes','No','No'); 
    switch ButtonName
    case 'No'
	    % take no action
    case 'Yes'
        evalin('base','clear');
        evalin('base','clc');
        tmp1={};
        set(handles.ButtonExcloureItems,'string','>'); 
        set(handles.ButtonIncloureItems,'enable','off');        
        set(handles.numberofvariables,'string','');
        set(handles.numberofparticipants,'string','');
        set(handles.ItemsIncluded,'string',tmp1);
        set(handles.ItemsExcluded,'string',tmp1);        
        set(handles.ItemsIncluded,'value',0);
        set(handles.ItemsExcluded,'value',0);
        set(handles.Visualize_Labels,'enable','off','value',0,'visible','off');
        
        %VIEW DATA
        set(handles.data_table,'visible','off');
        
        %DESCRIPTIVES
        set(handles.lowest_score,'string',tmp1,...
            'enable','off');
        set(handles.highest_score,'string',tmp1,...
            'enable','off');
        text={};
        set(handles.ModelOutput,'string',text);
        set(handles.ModelOutput,'value',0);
        set(handles.ModelOutput_latex,'string',text);
        set(handles.ModelOutput_latex,'value',0);
        set(handles.Compute_Descriptives,'enable','off');
        set(handles.ButtonSalvarOutput,'enable','off');
        set(handles.ButtonLaunchOutput,'enable','off');
        set(handles.Matrix_Polychoric,'enable','on',...
            'value',0);
        set(handles.Matrix_Pearson,'enable','on',...
            'value',1);
        set(handles.Matrix_Covariance,'enable','on',...
            'value',0);
        set(handles.text5,'ForegroundColor',[.2 .2 .2]);
        set(handles.text6,'ForegroundColor',[.2 .2 .2]);
        set(handles.save_text,'value',1,...
            'enable','off');
        set(handles.save_latex,'value',0,...
            'enable','off');
        set(handles.save_rtf,'value',0,...
            'enable','off');
        set(handles.parallel_analysis,'value',0);
        set(handles.grafic_PA,'Visible','off',...
            'value',0);
        set(handles.text5,'visible','off');
        set(handles.lowest_score,'visible','off');
        set(handles.text6,'visible','off');
        set(handles.highest_score,'visible','off');
            
        set(handles.panel_tab4,'visible','off');
        set(handles.panel2_tab4,'visible','off');
        set(handles.panel3_tab4,'visible','off');
        
        
        set(handles.ModelOutputIncrementa,'enable','off');
        set(handles.ModelOutputDecrementa,'enable','off');
        
        %DS
        set(handles.DS_checkbox,'value',0,...
            'enable','off');
        set(handles.panel3_tab5,'visible','off');
        set(handles.panel4_tab5,'visible','off');
        set(handles.numberofexcludeditems,'string',tmp1);
        set(handles.numberofsditems,'string',tmp1);
        set(handles.numberofcontentitems,'string',tmp1);
        set(handles.ContentItems,'string',tmp1);
        set(handles.SDItems,'string',tmp1);
        set(handles.ButtonIncloureItems,'enable','off');
        set(handles.Visualize_Labels2,'enable','off','visible','off','value',0);
        
        %AQ
        set(handles.AQ_checkbox,'value',0,...
            'enable','off');
        set(handles.panel3_tab6,'visible','off');
        set(handles.panel4_tab6,'visible','off');
        set(handles.numberofexcludeditems2,'string',tmp1);
        set(handles.numberofbalanceditems,'string',tmp1);
        set(handles.numberofunbalanceditems,'string',tmp1);
        set(handles.BalItems,'string',tmp1);
        set(handles.UnbalItems,'string',tmp1);
        set(handles.ButtonIncloureItems2,'enable','off');
        set(handles.Visualize_Labels3,'enable','off','visible','off','value',0);
        tmp={};
        set(handles.text_balanced_core,'visible','off','string',tmp);
        
        
        %AQiDS
        set(handles.excluded_list,'string',tmp1);
        set(handles.sd_list,'string',tmp1);
        set(handles.content_list,'string',tmp1);
        set(handles.pfactors,'value',0);
        set(handles.content_factors,'string',tmp1);
        rotacio{1}='Orthogonal';
        rotacio{2}='Oblique';
        set(handles.tipus_rotacio,'string',rotacio,...
            'value',2);
        set(handles.save_text2,'value',1,...
            'enable','off');
        set(handles.save_latex2,'value',0,...
            'enable','off');
        set(handles.save_rtf2,'value',0,...
            'enable','off');
        set(handles.ButtonSalvarOutput2,'enable','off');
        set(handles.save_pfactors,'enable','off',...
            'visible','off');
        set(handles.ButtonLaunchOutput2,'enable','off');
        set(handles.Compute_AQiDS,'enable','off');
        set(handles.ModelOutput2,'string',text);
        set(handles.ModelOutput2,'value',1);
        set(handles.ModelOutput2_latex,'string',text);
        set(handles.ModelOutput2_latex,'value',1);
        set(handles.Output_matrices,'string',tmp1);
        set(handles.Output_matrices,'value',0);
        set(handles.panel_tab7,'visible','off');
        set(handles.panel2_tab7,'visible','off');
        set(handles.content_factors,'backgroundcolor',[1 1 1]);
        
        set(handles.panel_tab2,'visible','on');
        set(handles.tipus_pfactors,'visible','off',...
            'enable','off',...
            'value',1);
        
        set(handles.ModelOutputIncrementa,'enable','off');
        set(handles.ModelOutputDecrementa,'enable','off');
    
        %LOAD DATA
        Printer=getappdata(0,'Printer');
        data=getappdata(0,'data');
        data.N=[];
        data.N_sensemiss=[];
        data.N_original=[];
        data.missings_value=[];
        data.missings_position=[];
        data.n=[];
        data.name=[];
        data.path=[];
        data.path_X=[];
        data.path_S=[];
        data.path_labels=[];
        data.path_target=[];
        data.missings_on=0;
        data.matrix_user_on=[];
        data.X=[];
        data.Xa=[];
        data.X_a=[];
        data.X_sensemiss=[];
        data.X_sensemissa=[];
        data.Xm=[];
        data.S=[];
        data.Sa=[];
        data.S_a=[];
        data.labels=[];
        data.labelsa=[];
        data.labels_user_on=[];
        data.H=[];
        data.inclosos=[];
        data.exclosos=[];
        data.Matrix=2;
        data.Rotation=2;
        data.items_contingut=[];
        data.exclosos=[];
        data.items_sd=[];
        data.G=[];
        data.A=[];
        data.A1=[];
        data.Re_cont=[];
        data.Re_final=[];
        data.latex_on=0;
        data.rtf_on=0;
        data.latex2_on=0;
        data.rtf2_on=0;
        data.itemsa=[];
        data.ib=[];
        data.iba=[];
        data.iub=[];
        data.iuba=[];
        data.Hc=[];
        data.R=[];
        data.SS=[];
        data.Rp=[];
        data.thresholds=[];
        data.R_NPD=0;
        data.kmo_error=0;
        data.time_poly=[];
        data.labels_contingut=[];
        data.full_analysis=0;
        data.PHI=[];
        data.PHIt=[];
        data.PHIi=[];
        data.PHIr=[];
        data.P=[];
        data.Pc=[];
        data.IRT_A=[];
        data.IRT_D=[];
        data.IRT_MDISC=[];
        data.IRT_B=[];
        data.IRT_A1=[];
        data.Ygrad=[];
        data.th=[];
        data.Lc=[];
        data.Lz=[];
        data.th_li=[];
        data.th_ls=[];
        data.se=[];
        data.reli=[];
        data.th=[];
        data.th_all=[];
        data.Xpredicted=[];
        data.WLSRegression=[];
        data.corr_error=0;
        data.items_corr_error=[];
        data.descriptives_done=0;
        data.forced_descriptives=0;
        data.Rotation=2;
        data.n_nodes=7;
        
        data.Bandera=0;
        
        %%%% Output structure %%%%
        Printer.text=[];
        Printer.latex=[];
        Printer.rtf=[];

        Printer.text2=[];
        Printer.latex2=[];
        Printer.rtf2=[];
        
        %SAVE DATA 
        setappdata(0,'Printer',Printer);
        setappdata(0,'data',data);
        
        set(gcbf,'Pointer','arrow')
        
        evalin('base','clc');
        drawnow;
        
        set(handles.hTabGroup,'Selectedtab',handles.tab(2))
    end;   
end;
return

function Forced_ClearData_Callback (hObject, eventdata, handles)
%% Clear data from the application (forced)
%

    tmp1={};
    set(handles.ButtonExcloureItems,'string','>'); 
    set(handles.ButtonIncloureItems,'enable','off');        
    set(handles.numberofvariables,'string','');
    set(handles.numberofparticipants,'string','');
    set(handles.ItemsIncluded,'string',tmp1);
    set(handles.ItemsExcluded,'string',tmp1);        
    set(handles.ItemsIncluded,'value',0);
    set(handles.ItemsExcluded,'value',0);
    set(handles.Visualize_Labels,'enable','off','value',0,'visible','off');

    %VIEW DATA
    set(handles.data_table,'visible','off');

    %DESCRIPTIVES
    set(handles.lowest_score,'string',tmp1,...
        'enable','off');
    set(handles.highest_score,'string',tmp1,...
        'enable','off');
    text={};
    set(handles.ModelOutput,'string',text);
    set(handles.ModelOutput,'value',0);
    set(handles.ModelOutput_latex,'string',text);
    set(handles.ModelOutput_latex,'value',0);
    set(handles.Compute_Descriptives,'enable','off');
    set(handles.ButtonSalvarOutput,'enable','off');
    set(handles.ButtonLaunchOutput,'enable','off');
    set(handles.Matrix_Polychoric,'enable','on',...
        'value',0);
    set(handles.Matrix_Pearson,'enable','on',...
        'value',1);
    set(handles.Matrix_Covariance,'enable','on',...
        'value',0);
    set(handles.text5,'ForegroundColor',[.2 .2 .2]);
    set(handles.text6,'ForegroundColor',[.2 .2 .2]);
    set(handles.save_text,'value',1,...
        'enable','off');
    set(handles.save_latex,'value',0,...
        'enable','off');
    set(handles.save_rtf,'value',0,...
        'enable','off');
    set(handles.parallel_analysis,'value',0);
    set(handles.grafic_PA,'Visible','off',...
        'value',0);
    set(handles.text5,'visible','off');
    set(handles.lowest_score,'visible','off');
    set(handles.text6,'visible','off');
    set(handles.highest_score,'visible','off');

    set(handles.panel_tab4,'visible','off');
    set(handles.panel2_tab4,'visible','off');
    set(handles.panel3_tab4,'visible','off');


    set(handles.ModelOutputIncrementa,'enable','off');
    set(handles.ModelOutputDecrementa,'enable','off');

    %SD
    set(handles.DS_checkbox,'value',0,...
        'enable','off');
    set(handles.panel3_tab5,'visible','off');
    set(handles.panel4_tab5,'visible','off');
    set(handles.numberofexcludeditems,'string',tmp1);
    set(handles.numberofsditems,'string',tmp1);
    set(handles.numberofcontentitems,'string',tmp1);
    set(handles.ContentItems,'string',tmp1);
    set(handles.SDItems,'string',tmp1);
    set(handles.ButtonIncloureItems,'enable','off');
    set(handles.Visualize_Labels2,'enable','off','visible','off','value',0);

    %AC
    set(handles.AQ_checkbox,'value',0,...
        'enable','off');
    set(handles.panel3_tab6,'visible','off');
    set(handles.panel4_tab6,'visible','off');
    set(handles.numberofexcludeditems2,'string',tmp1);
    set(handles.numberofbalanceditems,'string',tmp1);
    set(handles.numberofunbalanceditems,'string',tmp1);
    set(handles.BalItems,'string',tmp1);
    set(handles.UnbalItems,'string',tmp1);
    set(handles.ButtonIncloureItems2,'enable','off');
    set(handles.Visualize_Labels3,'enable','off','visible','off','value',0);
    tmp={};
    set(handles.text_balanced_core,'visible','off','string',tmp);


    %AC & SD
    set(handles.excluded_list,'string',tmp1);
    set(handles.sd_list,'string',tmp1);
    set(handles.content_list,'string',tmp1);
    set(handles.pfactors,'value',0);
    set(handles.content_factors,'string',tmp1);
    rotacio{1}='Orthogonal';
    rotacio{2}='Oblique';
    set(handles.tipus_rotacio,'string',rotacio,...
        'value',2);
    set(handles.save_text2,'value',1,...
        'enable','off');
    set(handles.save_latex2,'value',0,...
        'enable','off');
    set(handles.save_rtf2,'value',0,...
        'enable','off');
    set(handles.ButtonSalvarOutput2,'enable','off');
    set(handles.save_pfactors,'enable','off',...
        'visible','off');
    set(handles.ButtonLaunchOutput2,'enable','off');
    set(handles.Compute_AQiDS,'enable','off');
    set(handles.ModelOutput2,'string',text);
    set(handles.ModelOutput2,'value',1);
    set(handles.ModelOutput2_latex,'string',text);
    set(handles.ModelOutput2_latex,'value',1);
    set(handles.Output_matrices,'string',tmp1);
    set(handles.Output_matrices,'value',0);
    set(handles.panel_tab7,'visible','off');
    set(handles.panel2_tab7,'visible','off');
    set(handles.content_factors,'backgroundcolor',[1 1 1]);

    set(handles.panel_tab2,'visible','on');
    set(handles.tipus_pfactors,'visible','off',...
            'enable','off',...
            'value',1);

    set(handles.ModelOutputIncrementa,'enable','off');
    set(handles.ModelOutputDecrementa,'enable','off');

    %LOAD DATA
    Printer=getappdata(0,'Printer');
    data=getappdata(0,'data');
    data.N=[];
    data.N_sensemiss=[];
    data.N_original=[];
    data.missings_value=[];
    data.missings_position=[];
    data.n=[];
    data.name=[];
    data.path=[];
    data.missings_on=0;
    data.matrix_user_on=[];
    data.X=[];
    data.Xa=[];
    data.X_a=[];
    data.X_sensemiss=[];
    data.X_sensemissa=[];
    data.Xm=[];
    data.S=[];
    data.Sa=[];
    data.S_a=[];
    data.labels=[];
    data.labelsa=[];
    data.labels_user_on=[];
    data.H=[];
    data.inclosos=[];
    data.exclosos=[];
    data.Matrix=2;
    data.Rotation=2;
    data.items_contingut=[];
    data.exclosos=[];
    data.items_sd=[];
    data.G=[];
    data.A=[];
    data.A1=[];
    data.Re_cont=[];
    data.Re_final=[];
    data.latex_on=0;
    data.rtf_on=0;
    data.latex2_on=0;
    data.rtf2_on=0;
    data.itemsa=[];
    data.ib=[];
    data.iba=[];
    data.iub=[];
    data.iuba=[];
    data.Hc=[];
    data.R=[];
    data.SS=[];
    data.Rp=[];
    data.thresholds=[];
    data.time_poly=[];
    data.labels_contingut=[];
    data.full_analysis=0;
    data.PHI=[];
    data.PHIt=[];
    data.PHIi=[];
    data.PHIr=[];
    data.P=[];
    data.Pc=[];
    data.IRT_A=[];
    data.IRT_D=[];
    data.IRT_MDISC=[];
    data.IRT_B=[];
    data.IRT_A1=[];
    data.Ygrad=[];
    data.th=[];
    data.Lc=[];
    data.Lz=[];
    data.th_li=[];
    data.th_ls=[];
    data.se=[];
    data.reli=[];
    data.th=[];
    data.th_all=[];
    data.Xpredicted=[];
    data.WLSRegression=[];
    data.corr_error=0;
    data.items_corr_error=[];
    data.descriptives_done=0;
    data.forced_descriptives=0;
    data.Rotation=2;
    data.n_nodes=7;

    data.Bandera=0;

    %%%% Output structure %%%%
    Printer.text=[];
    Printer.latex=[];
    Printer.rtf=[];

    Printer.text2=[];
    Printer.latex2=[];
    Printer.rtf2=[];

    %SAVE DATA 
    setappdata(0,'Printer',Printer);
    setappdata(0,'data',data);

    set(gcbf,'Pointer','arrow')

    evalin('base','clc');
    drawnow;

    set(handles.hTabGroup,'Selectedtab',handles.tab(2))

return;

function Paste_Data(handles)
%% Apply the loaded state of the application
%

    data=getappdata(0,'data');
    set(handles.numberofparticipants,'string',data.N_original);
    
    buffs={};
    set(handles.ItemsIncluded,'string',buffs);
    set(handles.ItemsExcluded,'string',buffs);

    [f1,f2] = size(data.inclosos);
    for i=1:f2
        buff=sprintf('%.0f',data.inclosos(i));
        buffs(i)=cellstr(buff);
    end;
    set(handles.ItemsIncluded,'value',1);
    set(handles.ItemsIncluded,'string',buffs);
    set(handles.numberofvariables,'string',f2);
    buffs={};
    [f1,f2] = size(data.exclosos);
    for i=1:f2
        buff=sprintf('%.0f',data.exclosos(i));
        buffs(i)=cellstr(buff);
    end;
    set(handles.ItemsExcluded,'value',1);
    set(handles.ItemsExcluded,'string',buffs);
    
    if f2>0
        set(handles.ButtonIncloureItems,'enable','on');
    else
        set(handles.ButtonIncloureItems,'enable','off');
    end;
    
    setappdata(0,'data',data);

return;
    
function Paste_Descriptives(handles)
%% Apply the loaded state of the application in the descriptives section
%

    data=getappdata(0,'data');
    config=getappdata(0,'config'); 
    Printer=getappdata(0,'Printer');
    
    % All items've been selected
    if data.matrix_user_on==0
        minim=min(data.Xa);
        minim2=min(minim);
        maxim=max(data.Xa);
        maxim2=max(maxim);
        if data.missings_on==1
            if data.missings_value<0
                A=unique(data.Xa);
                minim=A(2:end);
                minim2=min(minim);
            else
                A=unique(data.Xa);
                maxim=A([2,end-1]);
                maxim2=max(maxim);
            end;
        end;
        set(handles.lowest_score,'String',minim2);  
        set(handles.highest_score,'String',maxim2);
        
        set(handles.Matrix_Polychoric,'enable','on',...
            'value',0);
        set(handles.Matrix_Pearson,'enable','on',...
            'value',0);
        set(handles.Matrix_Covariance,'enable','on',...
            'value',0);
        switch data.Matrix
            case 1 % covariance
                set(handles.Matrix_Covariance,'value',1);
            case 2 % Pearson
                set(handles.Matrix_Pearson,'value',1);
            case 3 % Polychoric
                set(handles.Matrix_Polychoric,'value',1);
        end;
        set(handles.text5,'ForegroundColor',[.2 .2 .2]);
        set(handles.text6,'ForegroundColor',[.2 .2 .2]);
        %%%%%%%%
        set(handles.pfactors,'value',0,...
            'visible','on');
        data.pfactorial=0;
        set(handles.save_pfactors,'visible','off',...
            'enable','off');
        
    elseif data.matrix_user_on==1
        set(handles.lowest_score,'enable','off');
        set(handles.highest_score,'enable','off');
        set(handles.Matrix_Polychoric,'enable','off',...
            'value',0);
        set(handles.Matrix_Pearson,'enable','off',...
            'value',0);
        set(handles.Matrix_Covariance,'enable','off',...
            'value',0);
        set(handles.text5,'ForegroundColor',[.4 .4 .4]);
        set(handles.text6,'ForegroundColor',[.4 .4 .4]);
        set(handles.highest_score,'string','');
        set(handles.lowest_score,'string','');
        %%%%%%%%%%%
        set(handles.pfactors,'value',0,...
            'visible','off');
        data.pfactorial=0;
        set(handles.save_pfactors,'visible','off',...
            'enable','off');
    end;
    
    %save button
    set(handles.save_text,'value',0);
    set(handles.save_rtf,'value',0);
    set(handles.save_latex,'value',0);
    if data.rtf_on==1 && data.latex_on==0
        set(handles.save_rtf,'value',1);
    elseif data.rtf_on==0 && data.latex_on==1
        set(handles.save_latex,'value',1);
    else
        set(handles.save_text,'value',1);
    end;
    
    %PA
    set(handles.parallel_analysis,'value',config.PA_on);
    set(handles.grafic_PA,'value',config.grafic_PA);
    set(handles.grafic_PA,'visible',config.grafic_PA_vis);
    
    set(handles.Compute_Descriptives,'enable','on')
    
    set(handles.ButtonSalvarOutput,'enable',config.ButtonSalvarOutput_e)
    set(handles.ButtonLaunchOutput,'enable',config.ButtonLaunchOutput_e);
    set(handles.text5,'visible',config.text5_v);
    set(handles.lowest_score,'visible',config.lowest_score_v);
    set(handles.text6,'visible',config.text6_v);
    set(handles.highest_score,'visible',config.highest_score_v);
    
    set(handles.ModelOutput,'string',Printer.text,'value',1);
    set(handles.ModelOutput_latex,'string',Printer.latex,'value',1);
    set(handles.ModelOutput_rtf,'string',Printer.rtf,'value',1);
    
    set(handles.panel_tab4,'visible','on');
    set(handles.panel2_tab4,'visible','on');
    set(handles.panel3_tab4,'visible','on');
    
    set(handles.save_text,'enable',config.saves_enabled);
    set(handles.save_rtf,'enable',config.saves_enabled);
    set(handles.save_latex,'enable',config.saves_enabled);
    
    set(handles.ModelOutputIncrementa,'enable',config.ModelOutputIncrementa);
    set(handles.ModelOutputDecrementa,'enable',config.ModelOutputDecrementa);
    
    set(handles.Visualize_Labels,'value',config.Visualize_Labels_value);
    set(handles.Visualize_Labels,'visible',config.Visualize_Labels_visible);
    set(handles.Visualize_Labels,'enable',config.Visualize_Labels_enable);
    
    setappdata(0,'data',data);
    
return;

function Paste_DS (handles)
%% Apply the loaded state of the application in the SD section
%

    data=getappdata(0,'data');
    config=getappdata(0,'config');
    
    set(handles.DS_checkbox,'value',config.DS_checkbox);
    set(handles.DS_checkbox,'enable',config.DS_checkbox_e);
    
    if config.DS_checkbox==1
        
        set(handles.panel3_tab5,'visible','on');
        set(handles.panel4_tab5,'visible','on');
        
        tmp1={};
        set(handles.SDItems,'string',tmp1);
        set(handles.SDItems,'value',1);
        [f1,f2]=size(data.items_sd);
        buffs={};
        for i=1:f2
            buff=num2str(data.items_sd(i));
            buffs(i)=cellstr(buff);
        end;
        set(handles.SDItems,'string',buffs);
        
        
        tmp1={};
        set(handles.ContentItems,'string',tmp1);
        set(handles.ContentItems,'value',1);
        [f1,f2]=size(data.items_contingut);
        buffs={};
        for i=1:f2
            buff=num2str(data.items_contingut(i));
            buffs(i)=cellstr(buff);
        end;
        set(handles.ContentItems,'string',buffs);
        
        [f1,f2]=size(data.items_contingut);
        set(handles.numberofcontentitems,'string',f2);
        [f1,f2]=size(data.exclosos);
        [g1,g2]=size(data.items_sd);
        set(handles.numberofexcludeditems,'string',f2);
        set(handles.numberofsditems,'string',g2);
        
        if g2>0
            set(handles.ButtonIncloureItems2,'enable','on');
        else
            set(handles.ButtonIncloureItems2,'enable','off');
        end;
        
        
    else        
        set(handles.panel3_tab5,'visible','off');
        set(handles.panel4_tab5,'visible','off');
        
        tmp1={};
        set(handles.ContentItems,'string',tmp1);
        set(handles.SDItems,'string',tmp1);
        
    set(handles.numberofexcludeditems,'string',tmp1);
    set(handles.numberofsditems,'string',tmp1);
    set(handles.numberofcontentitems,'string',tmp1);
    set(handles.ButtonIncloureItems,'enable','off');
    end;

    set(handles.Visualize_Labels2,'value',config.Visualize_Labels2_value);
    set(handles.Visualize_Labels2,'visible',config.Visualize_Labels2_visible);
    set(handles.Visualize_Labels2,'enable',config.Visualize_Labels2_enable);
    
return;

function Paste_AQ (handles)
%% Apply the loaded state of the application in the AC section
%

    data=getappdata(0,'data');
    config=getappdata(0,'config');
    
    set(handles.AQ_checkbox,'value',config.AQ_checkbox);
    set(handles.AQ_checkbox,'enable',config.AQ_checkbox_e);
    
    if config.AQ_checkbox==1
        
        set(handles.panel3_tab6,'visible','on');
        set(handles.panel4_tab6,'visible','on');
        
        tmp1={};
        set(handles.UnbalItems,'string',tmp1);
        set(handles.UnbalItems,'value',1);
         [f1,f2]=size(data.iub);
        buffs={};
        for i=1:f2
            buff=num2str(data.iub(i));
            buffs(i)=cellstr(buff);
        end;
        set(handles.UnbalItems,'string',buffs);
        
        tmp1={};
        set(handles.BalItems,'string',tmp1);
        set(handles.BalItems,'value',1);
         [f1,f2]=size(data.ib);
        buffs={};
        for i=1:f2
            buff=num2str(data.ib(i));
            buffs(i)=cellstr(buff);
        end;
        set(handles.BalItems,'string',buffs);
        
        [f1,f2]=size(data.ib);
        set(handles.numberofbalanceditems,'string',f2);
        [f1,f2]=size(data.exclosos);
        [f1,g2]=size(data.items_sd);
        set(handles.numberofexcludeditems2,'string',(f2+g2));
        [f1,f2]=size(data.iub);
        set(handles.numberofunbalanceditems,'string',f2);
          
        if f2>0
            set(handles.ButtonIncloureItems3,'enable','on');
        else
            set(handles.ButtonIncloureItems3,'enable','off');
        end;
        
    else        
        set(handles.panel3_tab6,'visible','off');
        set(handles.panel4_tab6,'visible','off');
        
        tmp1={};
        set(handles.BalItems,'string',tmp1);
        set(handles.UnbalItems,'string',tmp1);
        set(handles.ButtonIncloureItems3,'enable','off');
    end;
    
    set(handles.Visualize_Labels3,'value',config.Visualize_Labels3_value);
    set(handles.Visualize_Labels3,'visible',config.Visualize_Labels3_visible);
    set(handles.Visualize_Labels3,'enable',config.Visualize_Labels3_enable);
    
    set(handles.text_balanced_core,'visible',config.text_balanced_core_visible);
    set(handles.text_balanced_core,'string',config.text_balanced_core_string);
    
return;

function Paste_advanced (handles)
%% Apply the loaded state of the application in the EFA section
%

    data=getappdata(0,'data');
    config=getappdata(0,'config');
    Printer=getappdata(0,'Printer');
    
    set(handles.panel_tab7,'visible',config.panel_tab6);
    set(handles.panel2_tab7,'visible',config.panel2_tab6);
    
    set(handles.pfactors,'value',config.pfactors);
    set(handles.pfactors,'enable',config.pfactors_e);
    set(handles.save_text2,'enable',config.saves_enabled2);
    set(handles.save_latex2,'enable',config.saves_enabled2);
    set(handles.save_rtf2,'enable',config.saves_enabled2);
    set(handles.ButtonSalvarOutput2,'enable',config.ButtonSalvarOutput2);
    set(handles.save_pfactors,'enable',config.save_pfactors_e);
    set(handles.save_pfactors,'visible',config.save_pfactors_v);
    set(handles.ButtonLaunchOutput2,'enable',config.ButtonLaunchOutput2);
    set(handles.content_factors,'string',config.content_factors);
    set(handles.tipus_rotacio,'string',config.tipus_rotacio);
    set(handles.tipus_rotacio,'value',config.tipus_rotacio_value);
    set(handles.Compute_AQiDS,'enable',config.Compute_AQiDS);
    
    %save button
    set(handles.save_text2,'value',0);
    set(handles.save_rtf2,'value',0);
    set(handles.save_latex2,'value',0);
    if data.rtf2_on==1 && data.latex2_on==0
        set(handles.save_rtf2,'value',1);
    elseif data.rtf2_on==0 && data.latex2_on==1
        set(handles.save_latex2,'value',1);
    else
        set(handles.save_text2,'value',1);
    end;
    
    tmp1={};
    set(handles.ModelOutput2,'string',tmp1);
    set(handles.ModelOutput2,'value',1);
    set(handles.ModelOutput2_latex,'string',tmp1);
    set(handles.ModelOutput2_latex,'value',1);
    set(handles.Output_matrices,'string',tmp1);
    set(handles.Output_matrices,'value',0);
    
    set(handles.excluded_list,'string',tmp1);
    set(handles.sd_list,'string',tmp1);
    set(handles.content_list,'string',tmp1);

    set(handles.ModelOutput2,'string',Printer.text2);
    set(handles.ModelOutput2_latex,'string',Printer.latex2);
    set(handles.ModelOutput2_rtf,'string',Printer.rtf2);
    
    set(handles.excluded_list,'string',config.excluded_list);
    set(handles.sd_list,'string',config.sd_list);
    set(handles.content_list,'string',config.content_list);
    
    
    set(handles.full_analysis,'value',config.handles.full_analysis);
    set(handles.full_analysis,'enable',config.handles.full_analysis_e);
    
    
    set(handles.ModelOutputIncrementa2,'enable',config.ModelOutputIncrementa2);
    set(handles.ModelOutputDecrementa2,'enable',config.ModelOutputDecrementa2);
    
    if isfield(config,'tipus_pfactors_visible')
        set(handles.tipus_pfactors,'visible',config.tipus_pfactors_visible);
        set(handles.tipus_pfactors,'enable',config.tipus_pfactors_enable);
        set(handles.tipus_pfactors,'value',config.tipus_pfactors_value);
    end
    
return;

function SaveData_Callback(hObject, eventdata, handles)
%% Save the current state of the application
%
    evalin('base','clc');
    evalin('base','clear');
   
    
    config=getappdata(0,'config');
    
    config.Visualize_Labels_value=get(handles.Visualize_Labels,'value');
    config.Visualize_Labels_visible=get(handles.Visualize_Labels,'visible');
    config.Visualize_Labels_enable=get(handles.Visualize_Labels,'enable');
    
    %Descriptives tab
    config.ButtonSalvarOutput_e=get(handles.ButtonSalvarOutput,'enable');
    config.ButtonLaunchOutput_e=get(handles.ButtonLaunchOutput,'enable');
    config.text5_v=get(handles.text5,'visible');
    config.lowest_score_v=get(handles.lowest_score,'visible');
    config.text6_v=get(handles.text6,'visible');
    config.highest_score_v=get(handles.highest_score,'visible');
    config.PA_on=get(handles.parallel_analysis,'value');
    config.grafic_PA=get(handles.grafic_PA,'value');
    config.grafic_PA_vis=get(handles.grafic_PA,'visible');
    config.saves_enabled=get(handles.save_text,'enable');
    config.ModelOutputIncrementa=get(handles.ModelOutputIncrementa,'enable');
    config.ModelOutputDecrementa=get(handles.ModelOutputDecrementa,'enable');
    
    %View data
    config.data_table_v=get(handles.data_table,'visible');
    config.data_table_d=get(handles.data_table,'Data');
    
    %SD
    config.DS_checkbox=get(handles.DS_checkbox,'value');
    config.DS_checkbox_e=get(handles.DS_checkbox,'enable');
    config.Visualize_Labels2_value=get(handles.Visualize_Labels2,'value');
    config.Visualize_Labels2_visible=get(handles.Visualize_Labels2,'visible');
    config.Visualize_Labels2_enable=get(handles.Visualize_Labels2,'enable');
    
    %AQ
    config.AQ_checkbox=get(handles.AQ_checkbox,'value');
    config.AQ_checkbox_e=get(handles.AQ_checkbox,'enable');
    config.Visualize_Labels3_value=get(handles.Visualize_Labels3,'value');
    config.Visualize_Labels3_visible=get(handles.Visualize_Labels3,'visible');
    config.Visualize_Labels3_enable=get(handles.Visualize_Labels3,'enable');
    config.text_balanced_core_visible=get(handles.text_balanced_core,'visible');
    config.text_balanced_core_string=get(handles.text_balanced_core,'string');
    
    %Factorial
    config.pfactors=get(handles.pfactors,'value');
    config.pfactors_e=get(handles.pfactors,'enable');
    config.saves_enabled2=get(handles.save_text2,'enable');
    config.ButtonSalvarOutput2=get(handles.ButtonSalvarOutput2,'enable');
    config.save_pfactors_e=get(handles.save_pfactors,'enable');
    config.save_pfactors_v=get(handles.save_pfactors,'visible');
    config.ButtonLaunchOutput2=get(handles.ButtonLaunchOutput2,'enable');
    config.content_factors=get(handles.content_factors,'string');
    config.tipus_rotacio=get(handles.tipus_rotacio,'string');
    config.tipus_rotacio_value=get(handles.tipus_rotacio,'value');
    config.Compute_AQiDS=get(handles.Compute_AQiDS,'enable');
    config.panel_tab6=get(handles.panel_tab7,'visible');
    config.panel2_tab6=get(handles.panel2_tab7,'visible');
    config.excluded_list=get(handles.excluded_list,'string');
    config.sd_list=get(handles.sd_list,'string');
    config.content_list=get(handles.content_list,'string');
    config.handles.full_analysis=get(handles.full_analysis,'value');
    config.handles.full_analysis_e=get(handles.full_analysis,'enable');
    config.ModelOutputIncrementa2=get(handles.ModelOutputIncrementa2,'enable');
    config.ModelOutputDecrementa2=get(handles.ModelOutputDecrementa2,'enable');
    
    if isfield(handles,'tipus_pfactors')
        config.tipus_pfactors_visible=get(handles.tipus_pfactors,'visible');
        config.tipus_pfactors_enable=get(handles.tipus_pfactors,'enable');
        config.tipus_pfactors_value=get(handles.tipus_pfactors,'value');
    end
    
    
     % MANERA RAPIDA: NO ELIMINIS LA ESTRUCTURA handles, pero et crea una
     % nova figura quan facis l'open
%     clear hObject eventdata
    % MANERA MES NETA: ELIMINA EL handles i quedat una estructura nomes amb
    % els valors que necessitis
    clear handles hObject eventdata
    
    data=getappdata(0,'data');
    data.Bandera=0; %Posem 0 de moment
    setappdata(0,'data',data);
    Printer=getappdata(0,'Printer');
    
    if isempty(data.X) && isempty(data.S)
        errordlg('There are no data avaible!','Psychological Test Toolbox: Error');
        return;
    else
        try
            comaposition=findstr(data.actual_date,',');
            buff= sprintf('analysis_%s.mat',data.actual_date(1:comaposition-1));
            [fn,pn,filterindex] = uiputfile({'*.mat','MAT-files (*.mat)'}, getString(message('MATLAB:uistring:filedialogs:SaveWorkspaceVariables')), buff);
            
            if ~isequal(fn,0) % fn will be zero if user hits cancel
                save([pn fn]);
                msgbox('Analysis saving complete','Psychological Test Toolbox');
            else
                data.Bandera=1;
                setappdata(0,'data',data);
            end;
        catch
            disp(lasterr);
            return;
        end
    end;

    
return;

function OpenData_Callback(hObject, eventdata, handles)
%% Load a state of the application
%    
    evalin('base','clc');
    evalin('base','clear');
    data=getappdata(0,'data');
    if data.Bandera==1
        choice = questdlg('Loading new analysis will cause the loss of the actual analysis configuration. Continue loading? ','Psychological Test Toolbox: Warning','Yes','No','No');
        switch choice
            case 'Yes'
                try 
%                    uiload;
                   
                    [fn,pn] = uigetfile({'*.mat';'*.*'},getString(message('MATLAB:uistring:uiopen:DialogOpen')));
                    
                    
                    if isequal(fn,0)
                        return;
                    else
                        load([pn fn]);
                        setappdata(0,'data',data);
                        setappdata(0,'Printer',Printer);
                        setappdata(0,'config',config);

                        %%%% DATA %%%%
                        Paste_Data (handles);
                        
                        %%%% VIEW DATA %%%%
                        set(handles.data_table,'visible',config.data_table_v,...
                            'Data',config.data_table_d);

                        %%%% DESCRIPTIVES %%%%
                        Paste_Descriptives (handles);

                        %%%% SOCIAL DESIRABILITY %%%%
                        Paste_DS (handles);

                        %%%% ACQUIESCENCE %%%%
                        Paste_AQ (handles);

                        %%%% FACTOR ANALYSIS %%%%
                        Paste_advanced (handles)
                        
                        data.Bandera=0;
                        setappdata(0,'data',data);
                        
                        msgbox('          Successful Loading!          ','Psychological Test Toolbox');
                    end
                   
               catch
                   disp(lasterr);
               end;

               
            case 2
                return;
        end;
    else
        try 
%            uiload;

            [fn,pn] = uigetfile({'*.mat';'*.*'},getString(message('MATLAB:uistring:uiopen:DialogOpen')));
                    
            if isequal(fn,0)
                
                return;
            else
                load([pn fn]);
                
                setappdata(0,'data',data);
                setappdata(0,'Printer',Printer);
                setappdata(0,'config',config);

                %Anem per pestanyes

                %%%% DATA %%%%
                Paste_Data (handles);
                
                %%%% VIEW DATA %%%%
                set(handles.data_table,'visible',config.data_table_v,...
                     'Data',config.data_table_d);

                %%%% DESCRIPTIVES %%%%
                Paste_Descriptives (handles);

                %%%% SOCIAL DESIRABILITY %%%%
                Paste_DS (handles);

                %%%% ACQUIESCENCE %%%%
                Paste_AQ (handles);

                %%%% FACTOR ANALYSIS %%%%
                Paste_advanced (handles)

                data.Bandera=0;
                setappdata(0,'data',data);
                msgbox('          Successful Loading!          ','Psychological Test Toolbox');
            end
           
       catch
           disp(lasterr);
       end;

       
    end;
    
return;

function Save_Matrices (hObject, eventdata, handles)
%% Save matrices file
%
    buff=get(handles.Output_matrices,'string');
    if isempty(buff)
        errordlg('There are no data avaible!','Psychological Test Toolbox: Error');
        return;
    end;

    [FILENAME, PATHNAME] = uiputfile({'*.txt','Text files'}, 'Psychological Test Toolbox: Save output as a text file');

    if FILENAME == 0
        return;
    end;

    try 
        FileDiary=[PATHNAME FILENAME];
        f=exist(FileDiary,'file');
        if f == 2
           eval(['delete ' FileDiary]);
        end;

        buff = get(handles.Output_matrices,'string');
        c=cellstr(buff);
        fileID = fopen(FileDiary,'w');
        fprintf(fileID,'%s\r\n',c{:});

        msgbox('File save complete','Psychological Test Toolbox');
        fclose(fileID);
    catch 
        msgbox(lasterr,'error');
    end;

function ButtonSalvarOutput_Callback(hObject, eventdata, handles)
%% Save the output 1
%    
    data=getappdata(0,'data');
    comaposition=findstr(data.actual_date,',');

    if data.latex_on==0 && data.rtf_on==0;
        buff= sprintf('output_%s.txt',data.actual_date(1:comaposition-1));
        [FILENAME, PATHNAME] = uiputfile({'.txt','Text files'}, 'Psychological Test Toolbox: Save output as a text file',buff);

        if FILENAME == 0
            return;
        end;

        try 
            FileDiary=[PATHNAME FILENAME];
            f=exist(FileDiary,'file');
            if f == 2
               eval(['delete ' FileDiary]);
            end;
            
            buff = get(handles.ModelOutput,'string');
            c=cellstr(buff);
            fileID = fopen(FileDiary,'w');
            fprintf(fileID,'%s\r\n',c{:});

            msgbox('File save complete','Psychological Test Toolbox');
            fclose(fileID);
        catch 
            msgbox(lasterr,'error');
        end;
        
    elseif data.latex_on==1 && data.rtf_on==0;
        buff= sprintf('output_%s.tex',data.actual_date(1:comaposition-1));
        [FILENAME, PATHNAME] = uiputfile({'*.tex','.tex files'}, 'Psychological Test Toolbox: Save output as a .tex file',buff);

        if FILENAME == 0
            return;
        end;

        try 
            FileDiary=[PATHNAME FILENAME];
            f=exist(FileDiary,'file');
            if f == 2
               eval(['delete ' FileDiary]);
            end;
            
            buff = get(handles.ModelOutput_latex,'string');
            c=cellstr(buff);
            fileID = fopen(FileDiary,'w');
            fprintf(fileID,'%s\r\n',c{:});

            msgbox('File save complete','Psychological Test Toolbox');
            fclose(fileID);
        catch 
            msgbox(lasterr,'error');
        end;
    elseif data.latex_on==0 && data.rtf_on==1
        buff= sprintf('output_%s.rtf',data.actual_date(1:comaposition-1));
        [FILENAME, PATHNAME] = uiputfile({'*.rtf','Rich Text Format files'}, 'Psychological Test Toolbox: Save output as a RTF text file',buff);

        if FILENAME == 0
            return;
        end;

        try 
            FileDiary=[PATHNAME FILENAME];
            f=exist(FileDiary,'file');
            if f == 2
               eval(['delete ' FileDiary]);
            end;
            
            buff = get(handles.ModelOutput_rtf,'string');
            c=cellstr(buff);
            fileID = fopen(FileDiary,'w');
            fprintf(fileID,'%s\r\n',c{:});

            msgbox('File save complete','Psychological Test Toolbox');
            fclose(fileID);
        catch 
            msgbox(lasterr,'error');
        end;
    end;
return;

function ButtonLaunchOutput_Callback(hObject,eventdata,handles)
%% Launch the output 1
%
    h=handles.ModelOutput;

    finestra.N=760;
    finestra.M=600;
    pantalla=get(0,'ScreenSize');
    
    fig2=figure('units','pixels',...
        'position',[(pantalla(3)/2)-(finestra.N/2)-50  (pantalla(4)/2)-(finestra.M/2)-50 (finestra.N) (finestra.M)],...
        'MenuBar','none',...
        'Toolbar','none');
    
    
    handles.MenuFile2 = uimenu('Label','File');
    handles.MenuFile_Save_text = uimenu(handles.MenuFile2,...
                                        'Label','Save as text',...
                                        'checked','off',...
                                        'Callback',{@File_save_text, handles});
    handles.MenuFile_Save_rtf = uimenu(handles.MenuFile2,...
                                        'Label','Save as RTF',...
                                        'checked','off',...
                                        'Callback',{@File_save_rtf, handles});
    handles.MenuFile_Save_latex = uimenu(handles.MenuFile2,...
                                        'Label','Save as latex',...
                                        'checked','off',...
                                        'Callback',{@File_save_latex, handles}); 

    new_handle= copyobj(h,fig2);

    set(new_handle,'units','normalized',...
        'position',[.025 .025 .95 .95]);   
    
    t=uitoolbar;    
    pr=uipushtool(t,'TooltipString','Print the output',...
                'ClickedCallback',{@print_output, new_handle});
    icon=imread('printer.png');
    pr.CData=icon;
    
    decrement=uipushtool(t,'TooltipString','Zoom out',...
                'ClickedCallback',{@ModelOutputDecrementa_Callback_File, new_handle, 1});
    icon=imread('minus.png');
    decrement.CData=icon;
    
    increment=uipushtool(t,'TooltipString','Zoom in',...
                'ClickedCallback',{@ModelOutputDecrementa_Callback_File, new_handle, 2});
    icon=imread('plus.png');
    increment.CData=icon;

    set(fig2,'Name','Psychological Test Toolbox','NumberTitle','off');

return

function print_output(hObject,eventdata,newhandle)
%% Send to the default printing service the displaying text
%
    print;

return;

function File_save_text(hObject, eventdata, handles)
%% Select text file to be saved
%
    set(handles.save_text,'value',1);
    set(handles.save_latex,'value',0);
    set(handles.save_rtf,'value',0);

    data=getappdata(0,'data');
    data.rtf_on=0;
    data.latex_on=0;

    setappdata(0,'data',data);

    ButtonSalvarOutput_Callback(gcbo,eventdata,handles);

return;

function File_save_rtf(hObject, eventdata, handles)
%% Select rtf file to be saved
%
    set(handles.save_text,'value',0);
    set(handles.save_latex,'value',0);
    set(handles.save_rtf,'value',1);

    data=getappdata(0,'data');
    data.rtf_on=1;
    data.latex_on=0;

    setappdata(0,'data',data);

    ButtonSalvarOutput_Callback(gcbo,eventdata,handles);

return;

function File_save_latex(hObject, eventdata, handles)
%% Select LaTeX file to be saved
%
    set(handles.save_text,'value',0);
    set(handles.save_latex,'value',1);
    set(handles.save_rtf,'value',0);

    data=getappdata(0,'data');
    data.rtf_on=0;
    data.latex_on=1;

    setappdata(0,'data',data);

    ButtonSalvarOutput_Callback(gcbo,eventdata,handles);

return;

function ButtonSalvarOutput_Callback2(hObject, eventdata, handles)
%% Save the output 2
%   
    data=getappdata(0,'data');
    comaposition=findstr(data.actual_date,',');    

    if data.latex2_on==1 && data.rtf2_on==0;
        buff= sprintf('output_%s.tex',data.actual_date(1:comaposition-1));
        [FILENAME, PATHNAME] = uiputfile({'*.tex','.tex files'}, 'Psychological Test Toolbox: Save output as a .tex file',buff);

        if FILENAME == 0
            return;
        end;

        try 
            FileDiary=[PATHNAME FILENAME];
            f=exist(FileDiary,'file');
            if f == 2
               eval(['delete ' FileDiary]);
            end;
            
            buff = get(handles.ModelOutput2_latex,'string');
            c=cellstr(buff);
            fileID = fopen(FileDiary,'w');
            fprintf(fileID,'%s\r\n',c{:});

            msgbox('File save complete','Psychological Test Toolbox');
            fclose(fileID);
        catch 
            msgbox(lasterr,'error');
        end;
    elseif data.latex2_on==0 && data.rtf2_on==0
        buff= sprintf('output_%s.txt',data.actual_date(1:comaposition-1));
        [FILENAME, PATHNAME] = uiputfile({'*.txt','Text files'}, 'Psychological Test Toolbox: Save output as a text file',buff);

        if FILENAME == 0
            return;
        end;

        try 
            FileDiary=[PATHNAME FILENAME];
            f=exist(FileDiary,'file');
            if f == 2
               eval(['delete ' FileDiary]);
            end;
            
            buff = get(handles.ModelOutput2,'string');
            c=cellstr(buff);
            fileID = fopen(FileDiary,'w');
            fprintf(fileID,'%s\r\n',c{:});

            msgbox('File save complete','Psychological Test Toolbox');
            fclose(fileID);
        catch 
            msgbox(lasterr,'error');
        end;
    elseif data.latex2_on==0 && data.rtf2_on==1
        buff= sprintf('output_%s.rtf',data.actual_date(1:comaposition-1));
        [FILENAME, PATHNAME] = uiputfile({'*.rtf','Rich Text Format files'}, 'Psychological Test Toolbox: Save output as a RTF text file',buff);

        if FILENAME == 0
            return;
        end;

        try 
            FileDiary=[PATHNAME FILENAME];
            f=exist(FileDiary,'file');
            if f == 2
               eval(['delete ' FileDiary]);
            end;
            
            buff = get(handles.ModelOutput2_rtf,'string');
            c=cellstr(buff);
            fileID = fopen(FileDiary,'w');
            fprintf(fileID,'%s\r\n',c{:});

            msgbox('File save complete','Psychological Test Toolbox');
            fclose(fileID);
        catch 
            msgbox(lasterr,'error');
        end;
    end;
    
  return;

function ButtonSalvarpfactors_Callback(hObject,eventdata,handles)
%% Save the factor scores
%
    data=getappdata(0,'data');

    [FILENAME, PATHNAME] = uiputfile({'*.dat','Data file'}, 'Psychological Test Toolbox: Save factorial scores as a data file');

    if FILENAME == 0
        return;
    end;

    try 
        File_1=[PATHNAME FILENAME];
        f=exist(File_1,'file');
        if f == 2
           eval(['delete ' File_1]);
        end;

        A=num2cell(data.th_all);
        dlmwrite(File_1, data.th_all, 'delimiter','\t','precision',3);
    %             dlmwrite(FILENAME,A,'delimiter','\t');
        msgbox('File save complete','Psychological Test Toolbox');
    catch 
        msgbox(lasterr,'error');
    end;
        
        
return;

function ButtonLaunchOutput_Callback2(hObject,eventdata,handles)
%% Launch the output 2
%
    h=handles.ModelOutput2;

    finestra.N=760;
    finestra.M=600;
    pantalla=get(0,'ScreenSize');
    
    fig2=figure('units','pixels',...
        'position',[(pantalla(3)/2)-(finestra.N/2)-50  (pantalla(4)/2)-(finestra.M/2)-50 (finestra.N) (finestra.M)],...
        'MenuBar','none',...
        'Toolbar','none');
    
    
    handles.MenuFile2 = uimenu('Label','File');
    handles.MenuFile_Save_text = uimenu(handles.MenuFile2,...
                                        'Label','Save as text',...
                                        'checked','off',...
                                        'Callback',{@File_save_text2, handles});
    handles.MenuFile_Save_rtf = uimenu(handles.MenuFile2,...
                                        'Label','Save as RTF',...
                                        'checked','off',...
                                        'Callback',{@File_save_rtf2, handles});
    handles.MenuFile_Save_latex = uimenu(handles.MenuFile2,...
                                        'Label','Save as latex',...
                                        'checked','off',...
                                        'Callback',{@File_save_latex2, handles}); 
    handles.MenuFile_Save_scores = uimenu(handles.MenuFile2,...
                                    'Label','Save scores',...
                                    'checked','off',...
                                    'Separator','on',...
                                    'Callback',{@ButtonSalvarpfactors_Callback, handles});

    set(fig2,'Name','Psychological Test Toolbox','NumberTitle','off');

    new_handle= copyobj(h,fig2);

    set(new_handle,'units','normalized',...
        'position',[.025 .025 .95 .95]);
    
    t=uitoolbar;    
    pr=uipushtool(t,'TooltipString','Print the output',...
                'ClickedCallback',{@print_output2, new_handle});
    icon=imread('printer.png');
    pr.CData=icon;
    
    decrement=uipushtool(t,'TooltipString','Zoom out',...
                'ClickedCallback',{@ModelOutputDecrementa_Callback_File2, new_handle, 1});
    icon=imread('minus.png');
    decrement.CData=icon;
    
    increment=uipushtool(t,'TooltipString','Zoom in',...
                'ClickedCallback',{@ModelOutputDecrementa_Callback_File2, new_handle, 2});
    icon=imread('plus.png');
    increment.CData=icon;

return

function print_output2(hObject,eventdata,new_handle)
%% Send to the default printing service the displaying text 2
%
    print;

return;

function File_save_text2(hObject, eventdata, handles)
%% Select text file to be saved 2
%
    set(handles.save_text2,'value',1);
    set(handles.save_latex2,'value',0);
    set(handles.save_rtf2,'value',0);

    data=getappdata(0,'data');
    data.rtf2_on=0;
    data.latex2_on=0;

    setappdata(0,'data',data);

    ButtonSalvarOutput_Callback2(gcbo,eventdata,handles);

return;

function File_save_rtf2(hObject, eventdata, handles)
%% Select rtf file to be saved 2
%
    set(handles.save_text2,'value',0);
    set(handles.save_latex2,'value',0);
    set(handles.save_rtf2,'value',1);

    data=getappdata(0,'data');
    data.rtf2_on=1;
    data.latex2_on=0;

    setappdata(0,'data',data);

    ButtonSalvarOutput_Callback2(gcbo,eventdata,handles);

return;

function File_save_latex2(hObject, eventdata, handles)
%% Select LaTeX file to be saved 2
%
    set(handles.save_text2,'value',0);
    set(handles.save_latex2,'value',1);
    set(handles.save_rtf2,'value',0);

    data=getappdata(0,'data');
    data.rtf2_on=0;
    data.latex2_on=1;

    setappdata(0,'data',data);

    ButtonSalvarOutput_Callback2(gcbo,eventdata,handles);

return;

function colors_callback(hObject, eventdata, handles)
%% EASTER EGG : change the colour of the background
%
    num_color=round(unifrnd(1,5,1));
    
    switch num_color
        case 1
            color_selected=[1 .8 .6];            
        case 2
            color_selected=[.2 .8 .6];            
        case 3
            color_selected=[1 1 1];            
        case 4
            color_selected=[.6 .7 .9];            
        case 5
            color_selected=[.9 .9 .5];
    end;
    
    actual_color=get(handles.panel_tab1,'BackgroundColor');
    if actual_color~=color_selected;
    else
        if num_color==5
            num_color=1;
        else
            num_color=num_color+1;
        end;
            switch num_color
                case 1
                    color_selected=[1 .8 .6];            
                case 2
                    color_selected=[.2 .8 .6];            
                case 3
                    color_selected=[1 1 1];            
                case 4
                    color_selected=[.6 .7 .9];            
                case 5
                    color_selected=[.9 .9 .5];
            end;
    end;

    set(handles.panel_tab1,'backgroundcolor',color_selected);
    set(handles.text01,'backgroundcolor',color_selected);
    set(handles.text021,'backgroundcolor',color_selected);
    set(handles.text022,'backgroundcolor',color_selected);
    set(handles.text03,'backgroundcolor',color_selected);
    set(handles.text04,'backgroundcolor',color_selected);
    set(handles.text05,'backgroundcolor',color_selected);
    set(handles.text06,'backgroundcolor',color_selected);
    set(handles.colors,'backgroundcolor',color_selected,'foregroundcolor',[.2 .2 .2]);


return;

function jObj = waiting (handles,jObj,eventdata,hObject)
%% Java display of the computing window
%
    % Call the icon by java
    iconsClassName = 'com.mathworks.widgets.BusyAffordance$AffordanceSize';
    iconsSizeEnums = javaMethod('values',iconsClassName);
    SIZE_32x32 = iconsSizeEnums(2);  % (1) = 16x16,  (2) = 32x32
    jObj = com.mathworks.widgets.BusyAffordance(SIZE_32x32, 'Computing ... Please wait');  % icon, label
    
    jObj.setPaintsWhenStopped(true);  % default = false
    jObj.useWhiteDots(false);         % default = false
    javacomponent(jObj.getComponent, [0,0,250,100], handles.fig_wait);
    jObj.getComponent.setBackground(java.awt.Color(.941, .941, .941));  % orange
    jObj.start;
    
return;
